INSERT INTO contests (contest_id, descrption) VALUES
('070-160M-SPRINT', 'PODXS Great Pumpkin Sprint'),
('070-3-DAY', 'PODXS Three Day Weekend'),
('070-31-FLAVORS', 'PODXS 31 Flavors'),
('070-40M-SPRINT', 'PODXS 40m Firecracker Sprint'),
('070-80M-SPRINT', 'PODXS 80m Jay Hudak Memorial Sprint'),
('070-PSKFEST', 'PODXS PSKFest'),
('070-ST-PATS-DAY', 'PODXS St. Patricks Day'),
('070-VALENTINE-SPRINT', 'PODXS Valentine Sprint'),
('10-RTTY', 'Ten-Meter RTTY Contest (2011 onwards)'),
('1010-OPEN-SEASON', 'Open Season Ten Meter QSO Party'),
('7QP', '7th-Area QSO Party'),
('AL-QSO-PARTY', 'Alabama QSO Party'),
('ALL-ASIAN-DX-CW', 'JARL All Asian DX Contest (CW)'),
('ALL-ASIAN-DX-PHONE', 'JARL All Asian DX Contest (PHONE)'),
('ANARTS-RTTY', 'ANARTS WW RTTY'),
('ANATOLIAN-RTTY', 'Anatolian WW RTTY'),
('AP-SPRINT', 'Asia - Pacific Sprint'),
('AR-QSO-PARTY', 'Arkansas QSO Party'),
('ARI-DX', 'ARI DX Contest'),
('ARRL-10', 'ARRL 10 Meter Contest'),
('ARRL-160', 'ARRL 160 Meter Contest'),
('ARRL-222', 'ARRL 222 MHz and Up Distance Contest'),
('ARRL-DX-CW', 'ARRL International DX Contest (CW)'),
('ARRL-DX-SSB', 'ARRL International DX Contest (Phone)'),
('ARRL-EME', 'ARRL EME contest'),
('ARRL-FIELD-DAY', 'ARRL Field Day'),
('ARRL-RR-CW', 'ARRL Rookie Roundup (CW)'),
('ARRL-RR-RTTY', 'ARRL Rookie Roundup (RTTY)'),
('ARRL-RR-SSB', 'ARRL Rookie Roundup (Phone)'),
('ARRL-RTTY', 'ARRL RTTY Round-Up'),
('ARRL-SCR', 'ARRL School Club Roundup'),
('ARRL-SS-CW', 'ARRL November Sweepstakes (CW)'),
('ARRL-SS-SSB', 'ARRL November Sweepstakes (Phone)'),
('ARRL-UHF-AUG', 'ARRL August UHF Contest'),
('ARRL-VHF-JAN', 'ARRL January VHF Sweepstakes'),
('ARRL-VHF-JUN', 'ARRL June VHF QSO Party'),
('ARRL-VHF-SEP', 'ARRL September VHF QSO Party'),
('AZ-QSO-PARTY', 'Arizona QSO Party'),
('BARTG-RTTY', 'BARTG Spring RTTY Contest'),
('BARTG-SPRINT', 'BARTG Sprint Contest'),
('BC-QSO-PARTY', 'British Columbia QSO Party'),
('CA-QSO-PARTY', 'California QSO Party'),
('CIS-DX', 'CIS DX Contest'),
('CO-QSO-PARTY', 'Colorado QSO Party'),
('CQ-160-CW', 'CQ WW 160 Meter DX Contest (CW)'),
('CQ-160-SSB', 'CQ WW 160 Meter DX Contest (SSB)'),
('CQ-M', 'CQ-M International DX Contest'),
('CQ-VHF', 'CQ World-Wide VHF Contest'),
('CQ-WPX-CW', 'CQ WW WPX Contest (CW)'),
('CQ-WPX-RTTY', 'CQ/RJ WW RTTY WPX Contest'),
('CQ-WPX-SSB', 'CQ WW WPX Contest (SSB)'),
('CQ-WW-CW', 'CQ WW DX Contest (CW)'),
('CQ-WW-RTTY', 'CQ/RJ WW RTTY DX Contest'),
('CQ-WW-SSB', 'CQ WW DX Contest (SSB)'),
('CT-QSO-PARTY', 'Connecticut QSO Party'),
('CVA-DX-CW', 'Concurso Verde e Amarelo DX CW Contest'),
('CVA-DX-SSB', 'Concurso Verde e Amarelo DX CW Contest'),
('CWOPS-CW-OPEN', 'CWops CW Open Competition'),
('CWOPS-CWT', 'CWops Mini-CWT Test'),
('DARC-WAEDC-CW', 'WAE DX Contest (CW)'),
('DARC-WAEDC-RTTY', 'WAE DX Contest (RTTY)'),
('DARC-WAEDC-SSB', 'WAE DX Contest (SSB)'),
('DARC-WAG', 'DARC Worked All Germany'),
('DE-QSO-PARTY', 'Delaware QSO Party'),
('DL-DX-RTTY', 'DL-DX RTTY Contest'),
('DMC-RTTY', 'DMC RTTY Contest'),
('EA-CNCW', 'Concurso Nacional de Telegrafía'),
('EA-DME', 'Municipios Españoles'),
('EA-PSK63', 'EA PSK63'),
('EA-SMRE-CW', 'Su Majestad El Rey de España - CW'),
('EA-SMRE-SSB', 'Su Majestad El Rey de España - SSB'),
('EA-VHF-ATLANTIC', 'Atlántico V-UHF'),
('EA-VHF-COM', 'Combinado de V-UHF'),
('EA-VHF-COSTA-SOL', 'Costa del Sol V-UHF'),
('EA-VHF-EA', 'Nacional VHF'),
('EA-VHF-EA1RCS', 'Segovia EA1RCS V-UHF'),
('EA-VHF-QSL', 'QSL V-UHF & 50MHz'),
('EA-VHF-SADURNI', 'Sant Sadurni V-UHF'),
('EA-WW-RTTY', 'Unión de Radioaficionados Españoles RTTY Contest'),
('EPC-PSK63', 'PSK63 QSO Party'),
('EU Sprint', 'EU Sprint'),
('EU-HF', 'EU HF Championship'),
('EU-PSK-DX', 'EU PSK DX Contest'),
('EUCW160M', 'European CW Association 160m CW Party'),
('FALL SPRINT', 'FISTS Fall Sprint'),
('FL-QSO-PARTY', 'Florida QSO Party'),
('GA-QSO-PARTY', 'Georgia QSO Party'),
('HA-DX', 'Hungarian DX Contest'),
('HELVETIA', 'Helvetia Contest'),
('HI-QSO-PARTY', 'Hawaiian QSO Party'),
('HOLYLAND', 'IARC Holyland Contest'),
('IA-QSO-PARTY', 'Iowa QSO Party'),
('IARU-FIELD-DAY', 'DARC IARU Region 1 Field Day'),
('IARU-HF', 'IARU HF World Championship'),
('ID-QSO-PARTY', 'Idaho QSO Party'),
('IL QSO Party', 'Illinois QSO Party'),
('IN-QSO-PARTY', 'Indiana QSO Party'),
('JARTS-WW-RTTY', 'JARTS WW RTTY'),
('JIDX-CW', 'Japan International DX Contest (CW)'),
('JIDX-SSB', 'Japan International DX Contest (SSB)'),
('JT-DX-RTTY', 'Mongolian RTTY DX Contest'),
('KS-QSO-PARTY', 'Kansas QSO Party'),
('KY-QSO-PARTY', 'Kentucky QSO Party'),
('LA-QSO-PARTY', 'Louisiana QSO Party'),
('LDC-RTTY', 'DRCG Long Distance Contest (RTTY)'),
('LZ DX', 'LZ DX Contest'),
('MAR-QSO-PARTY', 'Maritimes QSO Party'),
('MD-QSO-PARTY', 'Maryland QSO Party'),
('ME-QSO-PARTY', 'Maine QSO Party'),
('MI-QSO-PARTY', 'Michigan QSO Party'),
('MIDATLANTIC-QSO-PARTY', 'Mid-Atlantic QSO Party'),
('MN-QSO-PARTY', 'Minnesota QSO Party'),
('MO-QSO-PARTY', 'Missouri QSO Party'),
('MS-QSO-PARTY', 'Mississippi QSO Party'),
('MT-QSO-PARTY', 'Montana QSO Party'),
('NA-SPRINT-CW', 'North America Sprint (CW)'),
('NA-SPRINT-RTTY', 'North America Sprint (RTTY)'),
('NA-SPRINT-SSB', 'North America Sprint (Phone)'),
('NAQP-CW', 'North America QSO Party (CW)'),
('NAQP-RTTY', 'North America QSO Party (RTTY)'),
('NAQP-SSB', 'North America QSO Party (Phone)'),
('NC-QSO-PARTY', 'North Carolina QSO Party'),
('ND-QSO-PARTY', 'North Dakota QSO Party'),
('NE-QSO-PARTY', 'Nebraska QSO Party'),
('NEQP', 'New England QSO Party'),
('NH-QSO-PARTY', 'New Hampshire QSO Party'),
('NJ-QSO-PARTY', 'New Jersey QSO Party'),
('NM-QSO-PARTY', 'New Mexico QSO Party'),
('NRAU-BALTIC-CW', 'NRAU-Baltic Contest (CW)'),
('NRAU-BALTIC-SSB', 'NRAU-Baltic Contest (SSB)'),
('NV-QSO-PARTY', 'Nevada QSO Party'),
('NY-QSO-PARTY', 'New York QSO Party'),
('OCEANIA-DX-CW', 'Oceania DX Contest (CW)'),
('OCEANIA-DX-SSB', 'Oceania DX Contest (SSB)'),
('OH-QSO-PARTY', 'Ohio QSO Party'),
('OK-DX-RTTY', 'Czech Radio Club OK DX Contest'),
('OK-OM-DX', 'Czech Radio Club OK-OM DX Contest'),
('OK-QSO-PARTY', 'Oklahoma QSO Party'),
('OMISS-QSO-PARTY', 'Old Man International Sideband Society QSO Party'),
('ON-QSO-PARTY', 'Ontario QSO Party'),
('OR-QSO-PARTY', 'Oregon QSO Party'),
('PA-QSO-PARTY', 'Pennsylvania QSO Party'),
('PACC', 'Dutch PACC Contest'),
('QC-QSO-PARTY', 'Quebec QSO Party'),
('RAC-CANADA-DAY', 'RAC Canada Day Contest'),
('RAC-CANADA-WINTER', 'RAC Canada Winter Contest'),
('RDAC', 'Russian District Award Contest'),
('RDXC', 'Russian DX Contest'),
('REF-160M', 'Reseau des Emetteurs Francais 160m Contest'),
('REF-CW', 'Reseau des Emetteurs Francais Contest (CW)'),
('REF-SSB', 'Reseau des Emetteurs Francais Contest (SSB)'),
('REP-PORTUGAL-DAY-HF', 'Rede dos Emissores Portugueses Portugal Day HF Contest'),
('RI-QSO-PARTY', 'Rhode Island QSO Party'),
('RSGB-160', '1.8MHz Contest'),
('RSGB-21/28-CW', '21/28 MHz Contest (CW)'),
('RSGB-21/28-SSB', '21/28 MHz Contest (SSB)'),
('RSGB-80M-CC', '80m Club Championships'),
('RSGB-AFS-CW', 'Affiliated Societies Team Contest (CW)'),
('RSGB-AFS-SSB', 'Affiliated Societies Team Contest (SSB)'),
('RSGB-CLUB-CALLS', 'Club Calls'),
('RSGB-COMMONWEALTH', 'Commonwealth Contest'),
('RSGB-IOTA', 'IOTA Contest'),
('RSGB-LOW-POWER', 'Low Power Field Day'),
('RSGB-NFD', 'National Field Day'),
('RSGB-ROPOCO', 'RoPoCo'),
('RSGB-SSB-FD', 'SSB Field Day'),
('RUSSIAN-RTTY', 'Russian Radio RTTY Worldwide Contest'),
('SAC-CW', 'Scandinavian Activity Contest (CW)'),
('SAC-SSB', 'Scandinavian Activity Contest (SSB)'),
('SARTG-RTTY', 'SARTG WW RTTY'),
('SC-QSO-PARTY', 'South Carolina QSO Party'),
('SCC-RTTY', 'SCC RTTY Championship'),
('SD-QSO-PARTY', 'South Dakota QSO Party'),
('SMP-AUG', 'SSA Portabeltest'),
('SMP-MAY', 'SSA Portabeltest'),
('SP-DX-RTTY', 'PRC SPDX Contest (RTTY)'),
('SPAR-WINTER-FD', 'SPAR Winter Field Day'),
('SPDXContest', 'SP DX Contest'),
('SPRING SPRINT', 'FISTS Spring Sprint'),
('SR-MARATHON', 'Scottish-Russian Marathon'),
('STEW-PERRY', 'Stew Perry Topband Distance Challenge'),
('SUMMER SPRINT', 'FISTS Summer Sprint'),
('TARA-GRID-DIP', 'TARA Grid Dip PSK-RTTY Shindig'),
('TARA-RTTY', 'TARA RTTY Mêlée'),
('TARA-RUMBLE', 'TARA Rumble PSK Contest'),
('TARA-SKIRMISH', 'TARA Skirmish Digital Prefix Contest'),
('TMC-RTTY', 'The Makrothen Contest'),
('TN-QSO-PARTY', 'Tennessee QSO Party'),
('TX-QSO-PARTY', 'Texas QSO Party'),
('UBA-DX-CW', 'UBA Contest (CW)'),
('UBA-DX-SSB', 'UBA Contest (SSB)'),
('UK-DX-BPSK63', 'European PSK Club BPSK63 Contest'),
('UK-DX-RTTY', 'UK DX RTTY Contest'),
('UKR-CHAMP-RTTY', 'Open Ukraine RTTY Championship'),
('UKRAINIAN DX', 'Ukrainian DX'),
('UKSMG-6M-MARATHON', 'UKSMG 6m Marathon'),
('UKSMG-SUMMER-ES', 'UKSMG Summer Es Contest'),
('US-COUNTIES-QSO', 'Mobile Amateur Awards Club'),
('UT-QSO-PARTY', 'Utah QSO Party'),
('VA-QSO-PARTY', 'Virginia QSO Party'),
('VENEZ-IND-DAY', 'RCV Venezuelan Independence Day Contest'),
('VIRGINIA QSO PARTY (import-only)', 'Virginia QSO Party'),
('VOLTA-RTTY', 'Alessandro Volta RTTY DX Contest'),
('WA-QSO-PARTY', 'Washington QSO Party'),
('WI-QSO-PARTY', 'Wisconsin QSO Party'),
('WIA-HARRY ANGEL', 'WIA Harry Angel Memorial 80m Sprint'),
('WIA-JMMFD', 'WIA John Moyle Memorial Field Day'),
('WIA-OCDX', 'WIA Oceania DX (OCDX) Contest'),
('WIA-REMEMBRANCE', 'WIA Remembrance Day'),
('WIA-ROSS HULL', 'WIA Ross Hull Memorial VHF/UHF Contest'),
('WIA-TRANS TASMAN', 'WIA Trans Tasman Low Bands Challenge'),
('WIA-VHF/UHF FD', 'WIA VHF UHF Field Days'),
('WIA-VK SHIRES', 'WIA VK Shires'),
('WINTER SPRINT', 'FISTS Winter Sprint'),
('WV-QSO-PARTY', 'West Virginia QSO Party'),
('WY-QSO-PARTY', 'Wyoming QSO Party'),
('XE-INTL-RTTY', 'Mexico International Contest (RTTY)'),
('YOHFDX', 'YODX HF contest'),
('YUDXC', 'YU DX Contest');
