// Stub implementations for testing RigctldManager

#include <QLoggingCategory>

// Debug category definitions
Q_LOGGING_CATEGORY(runtime, "qlog.runtime")
Q_LOGGING_CATEGORY(function_parameters, "qlog.function_parameters")
