result["Europe/Andorra"] = "AD";
result["Asia/Dubai"] = "AE";
result["Asia/Kabul"] = "AF";
result["America/Antigua"] = "AG";
result["America/Anguilla"] = "AI";
result["Europe/Tirane"] = "AL";
result["Asia/Yerevan"] = "AM";
result["Africa/Luanda"] = "AO";
result["Antarctica/McMurdo"] = "AQ";
result["Antarctica/Casey"] = "AQ";
result["Antarctica/Davis"] = "AQ";
result["Antarctica/DumontDUrville"] = "AQ";
result["Antarctica/Mawson"] = "AQ";
result["Antarctica/Palmer"] = "AQ";
result["Antarctica/Rothera"] = "AQ";
result["Antarctica/Syowa"] = "AQ";
result["Antarctica/Troll"] = "AQ";
result["Antarctica/Vostok"] = "AQ";
result["America/Argentina/Buenos_Aires"] = "AR";
result["America/Argentina/Cordoba"] = "AR";
result["America/Argentina/Salta"] = "AR";
result["America/Argentina/Jujuy"] = "AR";
result["America/Argentina/Tucuman"] = "AR";
result["America/Argentina/Catamarca"] = "AR";
result["America/Argentina/La_Rioja"] = "AR";
result["America/Argentina/San_Juan"] = "AR";
result["America/Argentina/Mendoza"] = "AR";
result["America/Argentina/San_Luis"] = "AR";
result["America/Argentina/Rio_Gallegos"] = "AR";
result["America/Argentina/Ushuaia"] = "AR";
result["America/Nuuk"] = "GL";
result["Pacific/Pago_Pago"] = "AS";
result["Europe/Vienna"] = "AT";
result["Australia/Lord_Howe"] = "AU";
result["Antarctica/Macquarie"] = "AU";
result["Australia/Hobart"] = "AU";
result["Australia/Currie"] = "AU";
result["Australia/Melbourne"] = "AU";
result["Australia/Sydney"] = "AU";
result["Australia/Broken_Hill"] = "AU";
result["Australia/Brisbane"] = "AU";
result["Australia/Lindeman"] = "AU";
result["Australia/Adelaide"] = "AU";
result["Australia/Darwin"] = "AU";
result["Australia/Perth"] = "AU";
result["Australia/Eucla"] = "AU";
result["America/Aruba"] = "AW";
result["Europe/Mariehamn"] = "AX";
result["Asia/Baku"] = "AZ";
result["Europe/Sarajevo"] = "BA";
result["America/Barbados"] = "BB";
result["Asia/Dhaka"] = "BD";
result["Europe/Brussels"] = "BE";
result["Africa/Ouagadougou"] = "BF";
result["Europe/Sofia"] = "BG";
result["Asia/Bahrain"] = "BH";
result["Africa/Bujumbura"] = "BI";
result["Africa/Porto-Novo"] = "BJ";
result["America/St_Barthelemy"] = "BL";
result["Atlantic/Bermuda"] = "BM";
result["Asia/Brunei"] = "BN";
result["America/La_Paz"] = "BO";
result["America/Kralendijk"] = "BQ";
result["America/Noronha"] = "BR";
result["America/Belem"] = "BR";
result["America/Fortaleza"] = "BR";
result["America/Recife"] = "BR";
result["America/Araguaina"] = "BR";
result["America/Maceio"] = "BR";
result["America/Bahia"] = "BR";
result["America/Sao_Paulo"] = "BR";
result["America/Campo_Grande"] = "BR";
result["America/Cuiaba"] = "BR";
result["America/Santarem"] = "BR";
result["America/Porto_Velho"] = "BR";
result["America/Boa_Vista"] = "BR";
result["America/Manaus"] = "BR";
result["America/Eirunepe"] = "BR";
result["America/Rio_Branco"] = "BR";
result["America/Nassau"] = "BS";
result["Asia/Thimphu"] = "BT";
result["Africa/Gaborone"] = "BW";
result["Europe/Minsk"] = "BY";
result["America/Belize"] = "BZ";
result["America/St_Johns"] = "CA";
result["America/Halifax"] = "CA";
result["America/Glace_Bay"] = "CA";
result["America/Moncton"] = "CA";
result["America/Goose_Bay"] = "CA";
result["America/Blanc-Sablon"] = "CA";
result["America/Toronto"] = "CA";
result["America/Nipigon"] = "CA";
result["America/Thunder_Bay"] = "CA";
result["America/Iqaluit"] = "CA";
result["America/Pangnirtung"] = "CA";
result["America/Atikokan"] = "CA";
result["America/Winnipeg"] = "CA";
result["America/Rainy_River"] = "CA";
result["America/Resolute"] = "CA";
result["America/Rankin_Inlet"] = "CA";
result["America/Regina"] = "CA";
result["America/Swift_Current"] = "CA";
result["America/Edmonton"] = "CA";
result["America/Cambridge_Bay"] = "CA";
result["America/Yellowknife"] = "CA";
result["America/Inuvik"] = "CA";
result["America/Creston"] = "CA";
result["America/Dawson_Creek"] = "CA";
result["America/Fort_Nelson"] = "CA";
result["America/Vancouver"] = "CA";
result["America/Whitehorse"] = "CA";
result["America/Dawson"] = "CA";
result["Indian/Cocos"] = "CC";
result["Africa/Kinshasa"] = "CD";
result["Africa/Lubumbashi"] = "CD";
result["Africa/Bangui"] = "CF";
result["Africa/Brazzaville"] = "CG";
result["Europe/Zurich"] = "CH";
result["Africa/Abidjan"] = "CI";
result["Pacific/Rarotonga"] = "CK";
result["America/Santiago"] = "CL";
result["America/Punta_Arenas"] = "CL";
result["Pacific/Easter"] = "CL";
result["Africa/Douala"] = "CM";
result["Asia/Shanghai"] = "CN";
result["Asia/Urumqi"] = "CN";
result["America/Bogota"] = "CO";
result["America/Costa_Rica"] = "CR";
result["America/Havana"] = "CU";
result["Atlantic/Cape_Verde"] = "CV";
result["America/Curacao"] = "CW";
result["Indian/Christmas"] = "CX";
result["Asia/Nicosia"] = "CY";
result["Asia/Famagusta"] = "CY";
result["Europe/Prague"] = "CZ";
result["Europe/Berlin"] = "DE";
result["Europe/Busingen"] = "DE";
result["Africa/Djibouti"] = "DJ";
result["Europe/Copenhagen"] = "DK";
result["America/Dominica"] = "DM";
result["America/Santo_Domingo"] = "DO";
result["Africa/Algiers"] = "DZ";
result["America/Guayaquil"] = "EC";
result["Pacific/Galapagos"] = "EC";
result["Europe/Tallinn"] = "EE";
result["Africa/Cairo"] = "EG";
result["Africa/El_Aaiun"] = "EH";
result["Africa/Asmara"] = "ER";
result["Europe/Madrid"] = "ES";
result["Africa/Ceuta"] = "ES";
result["Atlantic/Canary"] = "ES";
result["Africa/Addis_Ababa"] = "ET";
result["Europe/Helsinki"] = "FI";
result["Pacific/Fiji"] = "FJ";
result["Atlantic/Stanley"] = "FK";
result["Pacific/Chuuk"] = "FM";
result["Pacific/Pohnpei"] = "FM";
result["Pacific/Kosrae"] = "FM";
result["Atlantic/Faroe"] = "FO";
result["Europe/Paris"] = "FR";
result["Africa/Libreville"] = "GA";
result["Europe/London"] = "GB";
result["America/Grenada"] = "GD";
result["Asia/Tbilisi"] = "GE";
result["America/Cayenne"] = "GF";
result["Europe/Guernsey"] = "GG";
result["Africa/Accra"] = "GH";
result["Europe/Gibraltar"] = "GI";
result["America/Godthab"] = "GL";
result["America/Danmarkshavn"] = "GL";
result["America/Scoresbysund"] = "GL";
result["America/Thule"] = "GL";
result["Africa/Banjul"] = "GM";
result["Africa/Conakry"] = "GN";
result["America/Guadeloupe"] = "GP";
result["Africa/Malabo"] = "GQ";
result["Europe/Athens"] = "GR";
result["Atlantic/South_Georgia"] = "GS";
result["America/Guatemala"] = "GT";
result["Pacific/Guam"] = "GU";
result["Africa/Bissau"] = "GW";
result["America/Guyana"] = "GY";
result["Asia/Hong_Kong"] = "HK";
result["America/Tegucigalpa"] = "HN";
result["Europe/Zagreb"] = "HR";
result["America/Port-au-Prince"] = "HT";
result["Europe/Budapest"] = "HU";
result["Asia/Jakarta"] = "ID";
result["Asia/Pontianak"] = "ID";
result["Asia/Makassar"] = "ID";
result["Asia/Jayapura"] = "ID";
result["Europe/Dublin"] = "IE";
result["Asia/Jerusalem"] = "IL";
result["Europe/Isle_of_Man"] = "IM";
result["Asia/Kolkata"] = "IN";
result["Indian/Chagos"] = "IO";
result["Asia/Baghdad"] = "IQ";
result["Asia/Tehran"] = "IR";
result["Atlantic/Reykjavik"] = "IS";
result["Europe/Rome"] = "IT";
result["Europe/Jersey"] = "JE";
result["America/Jamaica"] = "JM";
result["Asia/Amman"] = "JO";
result["Asia/Tokyo"] = "JP";
result["Africa/Nairobi"] = "KE";
result["Asia/Bishkek"] = "KG";
result["Asia/Phnom_Penh"] = "KH";
result["Pacific/Tarawa"] = "KI";
result["Pacific/Enderbury"] = "KI";
result["Pacific/Kiritimati"] = "KI";
result["Indian/Comoro"] = "KM";
result["America/St_Kitts"] = "KN";
result["Asia/Pyongyang"] = "KP";
result["Asia/Seoul"] = "KR";
result["Asia/Kuwait"] = "KW";
result["America/Cayman"] = "KY";
result["Asia/Almaty"] = "KZ";
result["Asia/Qyzylorda"] = "KZ";
result["Asia/Aqtobe"] = "KZ";
result["Asia/Aqtau"] = "KZ";
result["Asia/Atyrau"] = "KZ";
result["Asia/Oral"] = "KZ";
result["Asia/Vientiane"] = "LA";
result["Asia/Beirut"] = "LB";
result["America/St_Lucia"] = "LC";
result["Europe/Vaduz"] = "LI";
result["Asia/Colombo"] = "LK";
result["Africa/Monrovia"] = "LR";
result["Africa/Maseru"] = "LS";
result["Europe/Vilnius"] = "LT";
result["Europe/Luxembourg"] = "LU";
result["Europe/Riga"] = "LV";
result["Africa/Tripoli"] = "LY";
result["Africa/Casablanca"] = "MA";
result["Europe/Monaco"] = "MC";
result["Europe/Chisinau"] = "MD";
result["Europe/Podgorica"] = "ME";
result["America/Marigot"] = "MF";
result["Indian/Antananarivo"] = "MG";
result["Pacific/Majuro"] = "MH";
result["Pacific/Kwajalein"] = "MH";
result["Europe/Skopje"] = "MK";
result["Africa/Bamako"] = "ML";
result["Asia/Yangon"] = "MM";
result["Asia/Ulaanbaatar"] = "MN";
result["Asia/Hovd"] = "MN";
result["Asia/Choibalsan"] = "MN";
result["Asia/Macau"] = "MO";
result["Pacific/Saipan"] = "MP";
result["America/Martinique"] = "MQ";
result["Africa/Nouakchott"] = "MR";
result["America/Montserrat"] = "MS";
result["Europe/Malta"] = "MT";
result["Indian/Mauritius"] = "MU";
result["Indian/Maldives"] = "MV";
result["Africa/Blantyre"] = "MW";
result["America/Mexico_City"] = "MX";
result["America/Cancun"] = "MX";
result["America/Merida"] = "MX";
result["America/Monterrey"] = "MX";
result["America/Matamoros"] = "MX";
result["America/Mazatlan"] = "MX";
result["America/Chihuahua"] = "MX";
result["America/Ojinaga"] = "MX";
result["America/Hermosillo"] = "MX";
result["America/Tijuana"] = "MX";
result["America/Bahia_Banderas"] = "MX";
result["Asia/Kuala_Lumpur"] = "MY";
result["Asia/Qostanay"] = "KZ";
result["Asia/Kuching"] = "MY";
result["Africa/Maputo"] = "MZ";
result["Africa/Windhoek"] = "NA";
result["Pacific/Noumea"] = "NC";
result["Africa/Niamey"] = "NE";
result["Pacific/Norfolk"] = "NF";
result["Africa/Lagos"] = "NG";
result["America/Managua"] = "NI";
result["Europe/Amsterdam"] = "NL";
result["Europe/Oslo"] = "NO";
result["Asia/Kathmandu"] = "NP";
result["Pacific/Nauru"] = "NR";
result["Pacific/Niue"] = "NU";
result["Pacific/Auckland"] = "NZ";
result["Pacific/Chatham"] = "NZ";
result["Asia/Muscat"] = "OM";
result["America/Panama"] = "PA";
result["America/Lima"] = "PE";
result["Pacific/Tahiti"] = "PF";
result["Pacific/Marquesas"] = "PF";
result["Pacific/Gambier"] = "PF";
result["Pacific/Port_Moresby"] = "PG";
result["Pacific/Bougainville"] = "PG";
result["Asia/Manila"] = "PH";
result["Asia/Karachi"] = "PK";
result["Europe/Warsaw"] = "PL";
result["America/Miquelon"] = "PM";
result["Pacific/Pitcairn"] = "PN";
result["America/Puerto_Rico"] = "PR";
result["Asia/Gaza"] = "PS";
result["Asia/Hebron"] = "PS";
result["Europe/Lisbon"] = "PT";
result["Atlantic/Madeira"] = "PT";
result["Atlantic/Azores"] = "PT";
result["Pacific/Palau"] = "PW";
result["America/Asuncion"] = "PY";
result["Asia/Qatar"] = "QA";
result["Indian/Reunion"] = "RE";
result["Europe/Bucharest"] = "RO";
result["Europe/Belgrade"] = "RS";
result["Europe/Kaliningrad"] = "RU";
result["Europe/Moscow"] = "RU";
result["Europe/Simferopol"] = "RU";
result["Europe/Volgograd"] = "RU";
result["Europe/Kirov"] = "RU";
result["Europe/Astrakhan"] = "RU";
result["Europe/Saratov"] = "RU";
result["Europe/Ulyanovsk"] = "RU";
result["Europe/Samara"] = "RU";
result["Asia/Yekaterinburg"] = "RU";
result["Asia/Omsk"] = "RU";
result["Asia/Novosibirsk"] = "RU";
result["Asia/Barnaul"] = "RU";
result["Asia/Tomsk"] = "RU";
result["Asia/Novokuznetsk"] = "RU";
result["Asia/Krasnoyarsk"] = "RU";
result["Asia/Irkutsk"] = "RU";
result["Asia/Chita"] = "RU";
result["Asia/Yakutsk"] = "RU";
result["Asia/Khandyga"] = "RU";
result["Asia/Vladivostok"] = "RU";
result["Asia/Ust-Nera"] = "RU";
result["Asia/Magadan"] = "RU";
result["Asia/Sakhalin"] = "RU";
result["Asia/Srednekolymsk"] = "RU";
result["Asia/Kamchatka"] = "RU";
result["Asia/Anadyr"] = "RU";
result["Africa/Kigali"] = "RW";
result["Asia/Riyadh"] = "SA";
result["Pacific/Guadalcanal"] = "SB";
result["Indian/Mahe"] = "SC";
result["Africa/Khartoum"] = "SD";
result["Europe/Stockholm"] = "SE";
result["Asia/Singapore"] = "SG";
result["Atlantic/St_Helena"] = "SH";
result["Europe/Ljubljana"] = "SI";
result["Arctic/Longyearbyen"] = "SJ";
result["Europe/Bratislava"] = "SK";
result["Africa/Freetown"] = "SL";
result["Europe/San_Marino"] = "SM";
result["Africa/Dakar"] = "SN";
result["Africa/Mogadishu"] = "SO";
result["America/Paramaribo"] = "SR";
result["Africa/Juba"] = "SS";
result["Africa/Sao_Tome"] = "ST";
result["America/El_Salvador"] = "SV";
result["America/Lower_Princes"] = "SX";
result["Asia/Damascus"] = "SY";
result["Africa/Mbabane"] = "SZ";
result["America/Grand_Turk"] = "TC";
result["Africa/Ndjamena"] = "TD";
result["Indian/Kerguelen"] = "TF";
result["Africa/Lome"] = "TG";
result["Asia/Bangkok"] = "TH";
result["Asia/Dushanbe"] = "TJ";
result["Pacific/Fakaofo"] = "TK";
result["Asia/Dili"] = "TL";
result["Asia/Ashgabat"] = "TM";
result["Africa/Tunis"] = "TN";
result["Pacific/Tongatapu"] = "TO";
result["Europe/Istanbul"] = "TR";
result["America/Port_of_Spain"] = "TT";
result["Pacific/Funafuti"] = "TV";
result["Asia/Taipei"] = "TW";
result["Africa/Dar_es_Salaam"] = "TZ";
result["Europe/Kyiv"] = "UA";
result["Europe/Uzhgorod"] = "UA";
result["Europe/Zaporozhye"] = "UA";
result["Africa/Kampala"] = "UG";
result["Pacific/Midway"] = "UM";
result["Pacific/Wake"] = "UM";
result["America/New_York"] = "US";
result["America/Detroit"] = "US";
result["America/Kentucky/Louisville"] = "US";
result["America/Kentucky/Monticello"] = "US";
result["America/Indiana/Indianapolis"] = "US";
result["America/Indiana/Vincennes"] = "US";
result["America/Indiana/Winamac"] = "US";
result["America/Indiana/Marengo"] = "US";
result["America/Indiana/Petersburg"] = "US";
result["America/Indiana/Vevay"] = "US";
result["America/Chicago"] = "US";
result["America/Indiana/Tell_City"] = "US";
result["America/Indiana/Knox"] = "US";
result["America/Menominee"] = "US";
result["America/North_Dakota/Center"] = "US";
result["America/North_Dakota/New_Salem"] = "US";
result["America/North_Dakota/Beulah"] = "US";
result["America/Denver"] = "US";
result["America/Boise"] = "US";
result["America/Phoenix"] = "US";
result["America/Los_Angeles"] = "US";
result["America/Anchorage"] = "US";
result["America/Juneau"] = "US";
result["America/Sitka"] = "US";
result["America/Metlakatla"] = "US";
result["America/Yakutat"] = "US";
result["America/Nome"] = "US";
result["America/Adak"] = "US";
result["Pacific/Honolulu"] = "US";
result["America/Montevideo"] = "UY";
result["Asia/Samarkand"] = "UZ";
result["Asia/Tashkent"] = "UZ";
result["Europe/Vatican"] = "VA";
result["America/St_Vincent"] = "VC";
result["America/Caracas"] = "VE";
result["America/Tortola"] = "VG";
result["America/St_Thomas"] = "VI";
result["Asia/Ho_Chi_Minh"] = "VN";
result["Pacific/Efate"] = "VU";
result["Pacific/Wallis"] = "WF";
result["Pacific/Apia"] = "WS";
result["Asia/Aden"] = "YE";
result["Indian/Mayotte"] = "YT";
result["Africa/Johannesburg"] = "ZA";
result["Africa/Lusaka"] = "ZM";
result["Africa/Harare"] = "ZW";
