QLineEdit#callsignEdit {
        text-transform: uppercase;
}

QLineEdit#stationCallsignEdit{
        text-transform: uppercase;
}

QLineEdit#stationOperatorCallsignEdit{
        text-transform: uppercase;
}

QLineEdit#stationDarcDokEdit{
        text-transform: uppercase;
}

QLineEdit#stationDarcDokEdit{
        text-transform: uppercase;
}

QLineEdit#myDOKEdit{
        text-transform: uppercase;
}

QLineEdit#dokEdit{
        text-transform: uppercase;
}

QLineEdit#myCallsignEdit{
        text-transform: uppercase;
}

QLineEdit#myOperatorCallsignEdit{
        text-transform: uppercase;
}

QLineEdit#gridEdit {
        text-transform: uppercase;
}

QLabel#chatUnread {
         padding: 2px;
         min-height: 10px;
         border-radius: 10px;
         background: orange;
}

QLabel#chatValuableMsg {
         padding: 2px;
         min-height: 10px;
         border-radius: 10px;
         background: red;
}

QLabel.kindex-0, QLabel.kindex-1, QLabel.kindex-2, QLabel.kindex-3 {
        color: #0000ff;
}

QMenu::item:disabled{background-color:transparent; color:palette(disabled);}
