ALTER TABLE rig_profiles ADD COLUMN share_rigctld INTEGER DEFAULT 0;
ALTER TABLE rig_profiles ADD COLUMN rigctld_port INTEGER DEFAULT 4532;
ALTER TABLE rig_profiles ADD COLUMN rigctld_path TEXT;
ALTER TABLE rig_profiles ADD COLUMN rigctld_args TEXT;
ALTER TABLE contacts_qsl_cards ADD COLUMN favorite INTEGER DEFAULT 0;

ALTER TABLE cwkey_profiles ADD paddle_only_sidetone INTEGER DEFAULT 0;
ALTER TABLE cwkey_profiles ADD sidetone_frequency INTEGER DEFAULT 800;

INSERT INTO qso_filter_operators VALUES(7, "regexp");

UPDATE log_param
   SET value = REPLACE(value, '|FT8', '|FTx')
 WHERE name = 'dxc/filter/dx/moderegexp'
   AND value LIKE '%|FT8%';

UPDATE alert_rules
   SET mode = REPLACE(mode, '|FT8', '|FTx')
 WHERE mode LIKE '%|FT8%';

CREATE TABLE IF NOT EXISTS adif_enum_secondary_subdivision (
    dxcc             INTEGER NOT NULL,
    code             TEXT NOT NULL,
    subdivision_name TEXT NOT NULL,
    primary_code     TEXT,
    aux              TEXT,
    PRIMARY KEY (dxcc, code)
);

CREATE INDEX IF NOT EXISTS contacts_dxcc_upper_cnty_idx ON contacts(dxcc, upper(cnty)) WHERE cnty IS NOT NULL AND cnty != '';

INSERT INTO adif_enum_secondary_subdivision VALUES (6,'AK,Haines','Haines','AK','Alaska First Judicial District');
INSERT INTO adif_enum_secondary_subdivision VALUES (6,'AK,Juneau','Juneau','AK','Alaska First Judicial District');
INSERT INTO adif_enum_secondary_subdivision VALUES (6,'AK,Ketchikan Gateway','Ketchikan Gateway','AK','Alaska First Judicial District');
INSERT INTO adif_enum_secondary_subdivision VALUES (6,'AK,Prince of Wales-Hyder','Prince of Wales-Hyder','AK','Alaska First Judicial District');
INSERT INTO adif_enum_secondary_subdivision VALUES (6,'AK,Sitka','Sitka','AK','Alaska First Judicial District');
INSERT INTO adif_enum_secondary_subdivision VALUES (6,'AK,Skagway','Skagway','AK','Alaska First Judicial District');
INSERT INTO adif_enum_secondary_subdivision VALUES (6,'AK,Yakutat','Yakutat','AK','Alaska First Judicial District');
INSERT INTO adif_enum_secondary_subdivision VALUES (6,'AK,Wrangell','Wrangell','AK','Alaska First Judicial District');
INSERT INTO adif_enum_secondary_subdivision VALUES (6,'AK,Hoonah-Angoon','Hoonah-Angoon','AK','Alaska First Judicial District');
INSERT INTO adif_enum_secondary_subdivision VALUES (6,'AK,Petersburg','Petersburg','AK','Alaska First Judicial District');
INSERT INTO adif_enum_secondary_subdivision VALUES (6,'AK,Wales-Hyder','Wales-Hyder','AK','Alaska First Judicial District');
INSERT INTO adif_enum_secondary_subdivision VALUES (6,'AK,Denali','Denali','AK','Alaska Fourth Judicial District');
INSERT INTO adif_enum_secondary_subdivision VALUES (6,'AK,Bethel','Bethel','AK','Alaska Fourth Judicial District');
INSERT INTO adif_enum_secondary_subdivision VALUES (6,'AK,Fairbanks North Star','Fairbanks North Star','AK','Alaska Fourth Judicial District');
INSERT INTO adif_enum_secondary_subdivision VALUES (6,'AK,Southeast Fairbanks','Southeast Fairbanks','AK','Alaska Fourth Judicial District');
INSERT INTO adif_enum_secondary_subdivision VALUES (6,'AK,Saint Matthew Island','Saint Matthew Island','AK','Alaska Fourth Judicial District');
INSERT INTO adif_enum_secondary_subdivision VALUES (6,'AK,Yukon-Koyukuk','Yukon-Koyukuk','AK','Alaska Fourth Judicial District');
INSERT INTO adif_enum_secondary_subdivision VALUES (6,'AK,Fairbanks','Fairbanks','AK','Alaska Fourth Judicial District');
INSERT INTO adif_enum_secondary_subdivision VALUES (6,'AK,Nome','Nome','AK','Alaska Second Judicial District');
INSERT INTO adif_enum_secondary_subdivision VALUES (6,'AK,North Slope','North Slope','AK','Alaska Second Judicial District');
INSERT INTO adif_enum_secondary_subdivision VALUES (6,'AK,Northwest Arctic','Northwest Arctic','AK','Alaska Second Judicial District');
INSERT INTO adif_enum_secondary_subdivision VALUES (6,'AK,Kusilvak','Kusilvak','AK','Alaska Second Judicial District');
INSERT INTO adif_enum_secondary_subdivision VALUES (6,'AK,Aleutians East','Aleutians East','AK','Alaska Third Judicial District');
INSERT INTO adif_enum_secondary_subdivision VALUES (6,'AK,Aleutians West','Aleutians West','AK','Alaska Third Judicial District');
INSERT INTO adif_enum_secondary_subdivision VALUES (6,'AK,Lake and Peninsula','Lake and Peninsula','AK','Alaska Third Judicial District');
INSERT INTO adif_enum_secondary_subdivision VALUES (6,'AK,Pribilof Islands','Pribilof Islands','AK','Alaska Third Judicial District');
INSERT INTO adif_enum_secondary_subdivision VALUES (6,'AK,Anchorage','Anchorage','AK','Alaska Third Judicial District');
INSERT INTO adif_enum_secondary_subdivision VALUES (6,'AK,Bristol Bay','Bristol Bay','AK','Alaska Third Judicial District');
INSERT INTO adif_enum_secondary_subdivision VALUES (6,'AK,Dillingham','Dillingham','AK','Alaska Third Judicial District');
INSERT INTO adif_enum_secondary_subdivision VALUES (6,'AK,Kenai Peninsula','Kenai Peninsula','AK','Alaska Third Judicial District');
INSERT INTO adif_enum_secondary_subdivision VALUES (6,'AK,Kodiak Island','Kodiak Island','AK','Alaska Third Judicial District');
INSERT INTO adif_enum_secondary_subdivision VALUES (6,'AK,Matanuska-Susitna','Matanuska-Susitna','AK','Alaska Third Judicial District');
INSERT INTO adif_enum_secondary_subdivision VALUES (6,'AK,Valdez-Cordova','Valdez-Cordova','AK','Alaska Third Judicial District');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'AL-01','Zheleznodorozhny district of Barnaul','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'AL-02','Industrial district of Barnaul','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'AL-03','Leninsky district of Barnaul','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'AL-04','Oktyabrsky district of Barnaul','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'AL-05','Centalny District of Barnaul','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'AL-08','Rubtsovsk','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'AL-09','Slavgorod','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'AL-10','Yarovoye','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'AL-11','Aleysky Area ','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'AL-12','Altaisky Area ','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'AL-13','Bayevsky Area','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'AL-14','Biysky Area ','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'AL-15','Blagoveshchensk Area','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'AL-16','Burlinsky Area','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'AL-17','Bystroistoksky Area','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'AL-18','Volchihinsky Area','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'AL-19','Egoryevsky Area ','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'AL-20','Eltsovsky Area ','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'AL-21','Zavyalovsky Area ','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'AL-22','Zalesovsky Area','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'AL-23','Zarinsky Area','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'AL-24','Zmeinogorsky Area','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'AL-25','Zonalnyj Area','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'AL-26','Kalmansky Area','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'AL-27','Kamensky Area','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'AL-28','Klyuchevsky Area','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'AL-29','Kosihinsky Area','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'AL-30','Krasnogorsky Area','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'AL-31','Krasnoshchekovsky Area','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'AL-32','Krutikhinsky Area','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'AL-33','Kulundinsky Area','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'AL-34','Kuryinsky Area','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'AL-35','Kytmanovsky Area','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'AL-36','Loktevsky Area','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'AL-37','Mamontovsky Area','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'AL-38','Mihajlovsky Area','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'AL-39','Nemetsky National Area','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'AL-40','Novichihinsky Area','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'AL-41','Pavlovsky Area','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'AL-42','Pankrushihinsky Area','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'AL-43','Pervomajsky Area','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'AL-44','Petropavlovsky Area','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'AL-45','Pospelihinsky Area','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'AL-46','Rebrihinsky Area','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'AL-47','Rodinsky Area','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'AL-48','Romanovsky Area','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'AL-49','Rubtsovsky Area','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'AL-50','Slavgorodsky Area','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'AL-51','Smolensky Area','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'AL-52','Sovietsky Area','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'AL-53','Soloneshensky Area','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'AL-54','Soltonsky Area','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'AL-55','Suyetsky Area','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'AL-56','Tabunsky Area','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'AL-57','Talmensky Area','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'AL-58','Togulsky Area','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'AL-59','Topchihinsky Area','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'AL-60','Tretyakovsky Area','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'AL-61','Troitsky Area','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'AL-62','Tyumentsevsky Area','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'AL-63','Uglovsky Area','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'AL-64','Ust-Kalmanskiy Area','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'AL-65','Ust-Pristanskiy Area','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'AL-66','Habarsky Area','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'AL-67','Tselinny Area','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'AL-68','Charyshsky Area','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'AL-69','Shelabolihinsky Area','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'AL-70','Shipunovsky Area','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'AL-71','Alejsk','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'AL-72','Belokurikha','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'AL-73','Biysk','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'AL-74','Gornjak','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'AL-75','Zarinsk','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'AL-76','Zmeinogorsk','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'AL-77','Kamen-Na-Obi','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'AL-78','Novoaltaisk','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'AL-79','Zato Sibirsky','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'AM-01','Blagoveshchensk','AM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'AM-02','Belogorsk','AM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'AM-03','Zeya','AM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'AM-04','Raychikhinsk','AM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'AM-05','Svobodnyj','AM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'AM-06','Tynda','AM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'AM-07','Shimanovsk','AM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'AM-08','Arharinsky Area','AM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'AM-09','Belogorsky Area','AM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'AM-10','Blagoveshchensky Area','AM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'AM-11','Bureysky Area','AM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'AM-12','Zavitinsky Area / incl. Zavitinsk','AM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'AM-13','Zeysky Area','AM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'AM-14','Ivanovsky Area','AM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'AM-15','Konstantinovsky Area','AM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'AM-16','Magdagachinsky Area','AM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'AM-17','Mazanovsky Area','AM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'AM-18','Mikhaylovsky Area','AM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'AM-19','Oktyabrsky district','AM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'AM-20','Romnensky Area','AM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'AM-21','Svobodnensky Area','AM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'AM-22','Selemdzhinsky Area','AM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'AM-23','Seryshevsky Area','AM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'AM-24','Skovorodinsky Area / incl. Skovorodino','AM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'AM-25','Tambovsky Area','AM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'AM-26','Tyndinsky Area','AM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'AM-27','Shimanowski Area','AM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'AM-28','Zato Tsiolkovsky','AM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'AM-29','Progress','AM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'BA-01','Demsky District of Ufa','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'BA-02','Kalininsky District of Ufa','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'BA-03','Kirovsky District of Ufa','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'BA-04','Leninsky District of Ufa','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'BA-05','Oktyabrsky District of Ufa','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'BA-06','Ordzhonikidzevsky District of Ufa','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'BA-07','Sovetsky District of Ufa','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'BA-08','Agidel','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'BA-17','Kumertau','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'BA-19','Neftekamsk','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'BA-20','Oktjabrsky','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'BA-21','Salavat','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'BA-22','Sibaj','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'BA-23','Sterlitamak','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'BA-27','Zato Mezhgore','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'BA-28','Abzelilovsky Area','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'BA-29','Alsheevsky Area','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'BA-30','Arkhangelsk Area','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'BA-31','Askinsky Area','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'BA-32','Aurgazinsky Area','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'BA-33','Baymaksky Area / incl. Baymak','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'BA-34','Bakalinsky Area','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'BA-35','Baltachevsky Area','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'BA-36','Belebeevsky Area / incl. Belebey','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'BA-37','Belokatajsky Area','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'BA-38','Beloretsky Area / incl. Beloreck','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'BA-39','Bizhbulyaksky Area','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'BA-40','Birsky Area / incl. Birsk','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'BA-41','Blagovarsky Area','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'BA-42','Blagoveshchensk Area / incl. Blagoveshchensk','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'BA-43','Buzdjaksky Area','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'BA-44','Buraevsky Area','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'BA-45','Burzjansky Area','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'BA-46','Gafurijsky Area','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'BA-47','Davlekanovsky / incl. Davlekanovo','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'BA-48','Duvansky Area','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'BA-49','Dyurtyulinsky Area / incl. Dyurtyuli','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'BA-50','Ermekeevsky Area','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'BA-51','Zianchurinsky Area','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'BA-52','Zilairsky Area','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'BA-53','Iglinsky Area','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'BA-54','Ilishevsky Area','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'BA-55','Ishimbaysky Area / incl. Ishimbay','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'BA-56','Kaltasinsky Area','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'BA-57','Karaidelsky Area','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'BA-58','Karmaskalinsky Area','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'BA-59','Kiginsky Area','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'BA-60','Krasnokamsky Area','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'BA-61','Kugarchinsky Area','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'BA-62','Kuyurgazinsky Area','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'BA-63','Kushnarenkovsky Area','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'BA-64','Meleuzovsky Area / incl. Meleuz','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'BA-65','Mechetlinsky Area','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'BA-66','Mishkinsky Area','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'BA-67','Miyakinsky Area','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'BA-68','Nurimanovsky Area','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'BA-69','Salavatsky Area','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'BA-70','Sterlibashevsky Area','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'BA-71','Sterlitamaksky Area','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'BA-72','Tatyshlinsky Area','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'BA-73','Tuymazinsky Area / incl. Tuymazy','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'BA-74','Ufimsky Area','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'BA-75','Uchalinsky Area / incl. Uchaly','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'BA-76','Fedorovsky Area','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'BA-77','Haybullinsky Area','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'BA-78','Chekmagushevsky Area','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'BA-79','Chishminsky Area','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'BA-80','Sharansky Area','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'BA-81','Yanaulsky Area / incl. Yanaul','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'BU-01','Zheleznodorozhny districd of Ulan-Ude','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'BU-02','Oktyabrsky district of Ulan-Ude','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'BU-03','Sovetsky district of Ulan-Ude','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'BU-04','Severobaykalsk','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'BU-05','Barguzinsky Area','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'BU-06','Bauntovsky Area','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'BU-07','Bichursky Area','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'BU-08','Dzhidinsky Area','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'BU-09','Eravninsky Area','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'BU-10','Zaigrayevsky Area','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'BU-11','Zakamensky Area / incl. Zakamensk','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'BU-12','Ivolginsky Area','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'BU-13','Kabansky Area / incl. Grandmother''s','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'BU-14','Kizhinginsky Area','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'BU-15','Kurumkansky Area','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'BU-16','Kyakhtinsky district / incl. Kyakhta','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'BU-17','Muysky Area','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'BU-18','Okinsky Area','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'BU-19','Pribaykalsky Area','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'BU-20','Severo-Baikaljsky Area','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'BU-21','Selenginsky Area / incl. Gusinoozersk','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'BU-22','Tarbagatajsky Area','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'BU-23','Tunkinsky Area','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'BU-24','Horinsky Area','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'BU-25','Mukhorshibirsky Area','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'CB-01','Kalininsky district of Chelyabinsk','CB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'CB-02','Kurchatovsky district of Chelyabinsk','CB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'CB-03','Leninsky district of Chelyabinsk','CB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'CB-04','Metallurgichesky district of Chelyabinsk','CB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'CB-05','Sovetsky district of Chelyabinsk','CB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'CB-06','Traktorozavodsky district of Chelyabinsk','CB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'CB-07','Centralny district of Chelyabinsk','CB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'CB-08','Leninsky district of Magnitogorsk','CB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'CB-09','Ordzhonikidzevsky district of Magnitogorsk','CB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'CB-10','Pravoberezhny district of Magnitogorsk','CB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'CB-12','Verhnij Ufalej','CB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'CB-13','Emanzhelinsky Area / incl. Emanzhelinsk','CB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'CB-14','Zlatoust','CB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'CB-15','Karabash','CB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'CB-19','Kopeysk','CB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'CB-20','Korkinsky Area / incl. Korkino','CB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'CB-21','Kyshtym','CB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'CB-22','Miass','CB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'CB-23','Plastovsky Area / incl. Plast','CB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'CB-25','Troitsk','CB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'CB-26','Ust-Katav','CB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'CB-27','Tchebarkul','CB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'CB-28','Yuzhnouralsk','CB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'CB-29','Zato Snezhinsk','CB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'CB-30','Zato Trehgorny','CB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'CB-31','Zato Ozersk','CB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'CB-32','Agapovsky Area','CB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'CB-33','Argajashsky Area','CB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'CB-34','Ashinsky Area / incl. Asha, Minyar, Sim','CB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'CB-35','Bredinsky Area','CB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'CB-36','Varnensky Area','CB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'CB-37','Verkhneuralsky Area / incl. Verhneuralsk','CB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'CB-38','Etkulsky Area','CB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'CB-39','Kartalinsky Area / incl. Kartaly','CB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'CB-40','Kaslinsky Area / incl. Kasli','CB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'CB-41','Katav-Ivanovskiy Area / incl. Katav-Ivanovsk, Yuryuzan','CB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'CB-42','Kizilsky Area','CB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'CB-43','Krasnoarmeisky Area','CB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'CB-44','Kunashaksky Area','CB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'CB-45','Kusinsky Area / incl. Kusa','CB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'CB-46','Nagajbaksky Area','CB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'CB-47','Nyazepetrovsky Area / incl. Nyazepetrovsk','CB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'CB-48','Oktyabrsky Area','CB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'CB-49','Satkinsky Area / incl. Satka, Bakal','CB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'CB-50','Sosnovsky Area','CB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'CB-51','Troitsk Area','CB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'CB-52','Uvelsky Area','CB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'CB-53','Uysky Area','CB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'CB-54','Chebarkulsky Area','CB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'CB-55','Chesmensky Area','CB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'CB-56','Locomotivny','CB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'CK-01','Anadyr','CK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'CK-02','Anadyrsky Area','CK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'CK-04','Bilibinsky Area / incl. Bilibino','CK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'CK-05','Iultinsky Area','CK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'CK-06','Providensky Area','CK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'CK-07','Chaunsky Area / incl. Pevek','CK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'CK-08','Chukotsky Area','CK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'EA-01','Birobidzhan','EA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'EA-02','Birobidzhansky Area','EA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'EA-03','Leninsky Area','EA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'EA-04','Obluchensky Area / incl. Obluche','EA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'EA-05','Oktyabrsky Area','EA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'EA-06','Smidovichsky Area','EA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'GA-01','Gorno-Altaysk','GA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'GA-02','Kosh-Agachskiy Area','GA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'GA-03','Mayminsky Area','GA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'GA-04','Ongudaysky Area','GA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'GA-05','Turochaksky Area','GA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'GA-06','Ulagansky Area','GA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'GA-07','Ust-Kansky Area','GA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'GA-08','Ust-Koksinskiy Area','GA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'GA-09','Chemalsky Area','GA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'GA-10','Choysky Area','GA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'GA-11','Shebalinsky Area','GA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'HA-01','Abakan','HA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'HA-02','Sayanogorsk','HA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'HA-03','Chernogorsk','HA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'HA-04','Altaysky Area','HA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'HA-05','Askizsky Area','HA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'HA-06','Biysky Area','HA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'HA-07','Bogradsky Area','HA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'HA-08','Ordzhonikidzevsky Area','HA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'HA-09','Tashtypsky Area','HA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'HA-10','Ust-Abakanskiy Area','HA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'HA-11','Shirinsky Area','HA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'HA-12','Abaza','HA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'HA-13','Sorsk','HA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'HK-01','Zheleznodorozhny district of Khabarovsk','HK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'HK-02','Industrialny district of Khabarovsk','HK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'HK-03','Kirovsky district of Khabarovsk','HK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'HK-04','Krasnoflotsky district of Khabarovsk','HK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'HK-05','Centralny district of Khabarovsk','HK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'HK-06','Leninsky district of Komsomolsk-on-Amur','HK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'HK-07','Centralny district of Komsomolsk-on-Amur','HK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'HK-08','Amursk','HK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'HK-09','Bikin','HK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'HK-10','Nikolaevsk-na-Amure','HK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'HK-11','Sovietskaya Gavan','HK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'HK-12','Amursky Area','HK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'HK-13','Ayano-Mayskiy Area','HK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'HK-14','Bikinsky Area','HK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'HK-15','Vaninsky Area','HK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'HK-16','Verhnebureinsky Area','HK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'HK-17','Vyazemsky Area / incl. Vyazemsky','HK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'HK-18','Komsomolsky Area','HK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'HK-19','Lazo Area','HK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'HK-20','Nanajsky Area','HK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'HK-21','Nikolaevsky Area','HK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'HK-22','Okhotsky Area / incl. Okhotck','HK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'HK-23','Poliny Osipenko Area','HK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'HK-24','Sovetsko-Havansky Area','HK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'HK-25','Solnechny Area','HK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'HK-26','Tuguro-Chumikansky Area','HK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'HK-27','Ulchsky Area','HK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'HK-28','Khabarovsky Area','HK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'HM-01','Hanty-Mansijsk','HM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'HM-02','Beloyarsky','HM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'HM-03','Kogalym','HM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'HM-04','Langepas','HM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'HM-05','Megion','HM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'HM-06','Nefteyugansk','HM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'HM-07','Nizhnevartovsk','HM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'HM-08','Nyagan','HM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'HM-09','Pokachi','HM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'HM-10','Pyt-Yakh','HM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'HM-11','Raduzhny','HM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'HM-12','Surgut','HM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'HM-13','Uraj','HM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'HM-14','Yugorsk','HM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'HM-15','Beloyarsky Area / incl. Beloyarsk','HM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'HM-16','Berezovsky Area','HM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'HM-17','Kondinsky Area','HM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'HM-18','Nefteyugansky Area','HM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'HM-19','Nizhnevartovsk Area','HM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'HM-20','Oktyabrsky Area','HM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'HM-21','Sovetsky Area  / incl. Sovietsky','HM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'HM-22','Surgutsky Area / incl. Lyantor','HM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'HM-23','Khanty-Mansijsky Area','HM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'IR-01','Leninsky District of Irkutsk','IR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'IR-02','Oktiabrsky District of Irkutsk','IR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'IR-03','Pravoberezhny District of Irkutsk','IR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'IR-04','Sverdlovsky District of Irkutsk','IR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'IR-05','Padunsky District of Bratsk','IR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'IR-06','Centralny District of Bratsk','IR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'IR-09','Zima','IR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'IR-11','Sayansk','IR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'IR-13','Tulun','IR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'IR-14','Usolye-Sibirskoye','IR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'IR-15','Ust-Ilimsk','IR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'IR-17','Cheremhovo','IR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'IR-19','Zalarinsky Area','IR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'IR-20','Angarsky Area / incl. Angarsk','IR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'IR-21','Balagansky Area','IR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'IR-22','Bodaybinsky Area / incl. Bodaibo','IR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'IR-23','Bratsky Area / incl. Vihorevka','IR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'IR-24','Zhigalovsky Area','IR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'IR-25','Ziminsky Area','IR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'IR-26','Irkutsky Area','IR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'IR-27','Kazachinsko-Lensky Area','IR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'IR-28','Katangsky Area','IR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'IR-29','Kachugsky Area','IR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'IR-30','Kirensky Area / incl. Kirensk','IR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'IR-31','Kuytunsky Area','IR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'IR-32','Mamsko-Chujsky Area','IR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'IR-33','Nizhneilimsky Area / incl. Zheleznogorsk-Ilimskiy','IR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'IR-34','Nizhneudinsk Area / incl. Nizhneudinsk, Alzamay','IR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'IR-35','Olkhonsky Area','IR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'IR-36','Slyudyansky Area / incl. Slyudyanka, Baikalsk','IR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'IR-37','Tayshetsky Area / incl. Tayshet, Biryusinsk','IR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'IR-38','Tulunsky Area','IR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'IR-39','Usolsky Area','IR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'IR-40','Ust-Ilimsky Area','IR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'IR-41','Ust-Kutskiy Area / incl. Ust-Kut','IR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'IR-42','Ust-Udinskiy Area','IR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'IR-43','Cheremhovsky Area','IR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'IR-44','Chunsky Area','IR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'IR-45','Shelekhovsky Area / incl. Shelehov','IR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'IR-46','Pravoberezhny District of Bratsk','IR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'IR-47','Svirsk','IR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'IR-48','Alarsky Area','IR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'IR-49','Bayandayevsky Area','IR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'IR-50','Bokhansky Area','IR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'IR-51','Nukutsky Area','IR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'IR-52','Osinsky Area','IR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'IR-53','Ehirit-Bulagatsky Area','IR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'IR-54','Kirovsky Area','IR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'IR-55','Kujbyshevsky Area','IR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'KE-01','Zavodskoj District of Kemerovo','KE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'KE-02','Kirovsky District of Kemerovo','KE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'KE-03','Leninsky District of Kemerovo','KE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'KE-04','Rudnichny District of Kemerovo','KE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'KE-05','Centralny District of Kemerovo','KE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'KE-06','Zavodsky District of Novokuznetsk','KE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'KE-07','Kuznetsky District of Novokuznetsk','KE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'KE-08','Kuibyshevsky District of Novokuznetsk','KE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'KE-09','Ordzhonikidzevsky District of Novokuznetsk','KE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'KE-10','Centralny District of Novokuznetsk','KE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'KE-11','Zenkovsky District of Prokopevsk','KE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'KE-12','Rudnichny District of Prokopevsk','KE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'KE-13','Centralny District of Prokopevsk','KE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'KE-14','Anzhero-Sudzhensk','KE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'KE-15','Belovo','KE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'KE-16','Berezovsky','KE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'KE-17','Gur`evsk','KE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'KE-18','Kaltan','KE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'KE-19','Kiselevsk','KE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'KE-20','Leninsk-Kuznetskiy','KE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'KE-21','Krasnobrodsky Okrug','KE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'KE-22','Mezhdurechensky District','KE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'KE-23','Myski','KE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'KE-24','Osinniki','KE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'KE-25','Taiga','KE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'KE-26','Tashtagol','KE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'KE-27','Polysaevo','KE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'KE-28','Yurga','KE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'KE-29','Belovsky Area','KE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'KE-30','Guryevsky Area / incl. Gurevsk, Salair','KE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'KE-31','Izhmorsky Area','KE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'KE-32','Kemerovsky Area','KE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'KE-33','Krapivinsky Area','KE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'KE-34','Leninsk-Kuznetsky Area','KE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'KE-35','Mariinsky Area','KE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'KE-36','Mezhdurechensky Area','KE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'KE-37','Novokuznetsky Area','KE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'KE-38','Prokopyevsky Area','KE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'KE-39','Promyshlennovsky Area','KE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'KE-40','Tashtagolsky Area / incl. G.Tashtagol','KE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'KE-41','Tisulsky Area','KE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'KE-42','Topkinsky Area','KE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'KE-43','Tjazhinsky Area','KE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'KE-44','Chebulinsky Area','KE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'KE-45','Yurginsky Area','KE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'KE-46','Jajsky Area','KE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'KE-47','Yashkinsky Area','KE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'KE-48','Novoilyinsky District of Novokuznetsk','KE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'KE-49','Lesnaja Polyana of Kemerovo','KE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'KE-50','Kedrovka and Promyshlennovsky District of Kemerovo','KE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'KE-51','Yagunovskiy and Pioner District Kemerovo','KE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'KE-52','Mariinsk','KE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'KE-53','Topki','KE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'KK-01','Zheleznodorozhny District of Krasnoyarsk','KK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'KK-02','Kirovsky District of Krasnoyarsk','KK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'KK-03','Leninsky District of Krasnoyarsk','KK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'KK-04','Oktyabrsky District of Krasnoyarsk','KK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'KK-05','Sverdlovsky District of Krasnoyarsk','KK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'KK-06','Sovetsky District of Krasnoyarsk','KK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'KK-07','Centralny District of Krasnoyarsk','KK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'KK-08','Achinsk','KK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'KK-09','Bogotol','KK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'KK-10','Borodino','KK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'KK-11','Divnogorsk','KK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'KK-12','Yeniseisk','KK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'KK-15','Kansk','KK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'KK-16','Lesosibirsk','KK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'KK-17','Minusinsk','KK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'KK-18','Nazarovo','KK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'KK-19','Sosnovoborsk','KK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'KK-20','Sharypovo','KK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'KK-21','Zato Zheleznogorsk','KK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'KK-22','Zato Zelenogorsk','KK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'KK-23','Abansky Area','KK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'KK-24','Achinsky Area','KK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'KK-25','Balakhtinsky Area','KK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'KK-26','Berezovsky Area','KK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'KK-27','Biriljussky Area','KK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'KK-28','Bogotolsky Area','KK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'KK-29','Boguchansky Area','KK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'KK-30','Bolshemurtinsky Area','KK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'KK-31','Bolsheuluysky Area','KK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'KK-32','Dzerzhinsky Area','KK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'KK-33','Emelyanovsky Area','KK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'KK-34','Yeniseisky Area','KK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'KK-35','Yermakovsky Area','KK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'KK-36','Idrinsky Area','KK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'KK-37','Ilansky Area / incl. Ilanskiy','KK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'KK-38','Irbeysky Area','KK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'KK-39','Kazachinsky Area','KK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'KK-40','Kansky Area','KK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'KK-41','Karatuzsky Area','KK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'KK-42','Kezhemsky Area / incl. Kodinskiy','KK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'KK-43','Kozulsky Area','KK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'KK-44','Krasnoturansky Area','KK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'KK-45','Kuraginsky Area / incl. Artemivsk','KK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'KK-46','Mansky Area','KK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'KK-47','Minusinsky Area','KK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'KK-48','Motyginsky Area','KK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'KK-49','Nazarovsky Area','KK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'KK-50','Nizhneingashsky Area','KK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'KK-51','Novoselovsky Area','KK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'KK-52','Partizansky Area','KK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'KK-53','Pirovsky Area','KK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'KK-54','Rybinsky Area / incl. Zaozyorny','KK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'KK-55','Sayansky Area','KK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'KK-56','Severo-Yeniseisky Area','KK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'KK-57','Suhobuzimsky Area','KK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'KK-58','Taseevsky Area','KK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'KK-59','Turukhansky Area / incl. Igarka','KK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'KK-60','Tyukhtetsky Area','KK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'KK-61','Uzhursky Area / incl. Uzhur','KK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'KK-62','Uyarsky Area / incl. Uyar','KK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'KK-63','Sharypovsky Area','KK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'KK-64','Shushensky Area','KK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'KK-65','Tajmysky Area','KK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'KK-66','Norilsk','KK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'KK-67','Evenkijsky Area','KK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'KK-68','Zato Solnechny','KK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'KK-69','Kedrovy','KK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'KK-70','Kajerkan Area','KK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'KK-71','Talnakh Area','KK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'KN-01','Kurgan','KN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'KN-02','Shadrinsk','KN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'KN-03','Almenevsky Area','KN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'KN-04','Belozersky Area','KN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'KN-05','Vargashinsky Area','KN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'KN-06','Dalmatovsky Area / incl. Dalmatovo','KN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'KN-07','Zverinogolovsky Area','KN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'KN-08','Kargapol''Sky Area','KN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'KN-09','Kataysky Area / incl. Kataysk','KN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'KN-10','Ketovsky Area','KN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'KN-11','Kurtamyshsky Area / incl. Kurtamysh','KN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'KN-12','Lebyazhyevsky Area','KN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'KN-13','Makushinsky Area / incl. Makushino','KN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'KN-14','Mishkinsky Area','KN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'KN-15','Mokrousovsky Area','KN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'KN-16','Petukhovsky Area / incl. Petuhovo','KN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'KN-17','Polovinsky Area','KN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'KN-18','Pritobolny Area','KN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'KN-19','Safakulevsky Area','KN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'KN-20','Tselinny Area','KN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'KN-21','Chastoozersky Area','KN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'KN-22','Shatrovsky Area','KN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'KN-23','Shumikhinsky Area / incl. Sensation','KN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'KN-24','Shchuchansky Area / incl. Pike','KN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'KN-25','Yurgamyshsky Area','KN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'KN-26','Shadrinsky Area','KN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'KO-01','Syktyvkar','KO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'KO-02','Vorkuta','KO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'KO-03','Vuktylsky Area / incl. Vuktyl','KO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'KO-04','Inta','KO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'KO-05','Pechorsky Area / incl. Pechora','KO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'KO-06','Sosnogorsky Area / incl. Sosnogorsk and suburbs','KO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'KO-07','Usinsk / incl. suburb','KO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'KO-08','Ukhta','KO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'KO-09','Izhemsky Area','KO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'KO-10','Knyazhpogostsky Area / incl. Emba','KO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'KO-11','Kojgorodsky Area','KO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'KO-12','Kortkerossky Area','KO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'KO-13','Priluzsky Area','KO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'KO-14','Syktyvdinsky Area','KO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'KO-15','Sysolsky Area','KO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'KO-16','Troitsko-Pechorsky Area','KO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'KO-17','Udorsky Area','KO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'KO-18','Ust-Vymskiy Area / incl. Mikun','KO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'KO-19','Ust-Kulomskiy Area','KO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'KO-20','Ust-Tsilemskiy Area','KO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'KO-21','Ezhvinsky Syktyvkar','KO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'KT-01','Petropavlovsk-Kamchatsky','KT','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'KT-02','Elizovo','KT','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'KT-03','Zato Vilyuchinsk','KT','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'KT-04','Aleutsky Area','KT','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'KT-05','Bystrinsky Area','KT','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'KT-06','Elizovsky Area / incl. Elizovo','KT','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'KT-07','Milkovsky Area','KT','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'KT-08','Sobolevsky Area','KT','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'KT-09','Ust-Bolsheretskiy Area','KT','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'KT-10','Ust-Kamchatskiy Area / incl. Kljuchi','KT','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'KT-11','Karaginsky Area','KT','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'KT-12','Oljutorsky Area','KT','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'KT-13','Penzhinsky Area','KT','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'KT-14','Tigilsky Area','KT','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'KT-15','Palana','KT','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'MG-01','Magadan','MG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'MG-02','Olsky Area','MG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'MG-03','Omsukchansky Area','MG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'MG-04','Severo-Evensky Area','MG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'MG-05','Srednekansky Area','MG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'MG-06','Susuman Area / incl. Susuman','MG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'MG-07','Tenkinsky Area','MG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'MG-08','Hasynsky Area','MG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'MG-09','Yagodninsky Area','MG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'NS-01','Dzerzhinsky District of Novosibirsk','NS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'NS-02','Zheleznodorozhny District of Novosibirsk','NS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'NS-03','Zayeltsovsky District of Novosibirsk','NS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'NS-04','Kalininsky District of Novosibirsk','NS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'NS-05','Kirovsky District of Novosibirsk','NS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'NS-06','Leninsky District of Novosibirsk','NS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'NS-07','Oktyabrsky District of Novosibirsk','NS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'NS-08','Pervomajsky District of Novosibirsk','NS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'NS-09','Sovetsky District of Novosibirsk','NS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'NS-10','Centralny District of Novosibirsk','NS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'NS-11','Bagansky Area','NS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'NS-12','Barabinsk Area / incl. Barabinsk','NS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'NS-13','Bolotninsky Area / incl. Bolotnoe','NS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'NS-14','Vengerovsky Area','NS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'NS-15','Dovolensky Area','NS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'NS-16','Zdvinsky Area','NS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'NS-17','Iskitimsky Area','NS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'NS-18','Karasuksky Area / incl. Karasuk','NS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'NS-19','Kargatsky Area / incl. Kargat','NS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'NS-20','Kolyvansky Area','NS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'NS-21','Kochenevsky Area','NS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'NS-22','Kochkovsky Area','NS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'NS-23','Krasnozersky Area','NS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'NS-24','Kuibyshevsky Area / incl. Kuibyshev','NS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'NS-25','Kupinsky Area Including Kupino','NS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'NS-26','Kyshtovsky Area','NS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'NS-27','Maslyaninsky Area','NS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'NS-28','Moszkowski Area','NS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'NS-29','Novosibirsky Area','NS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'NS-30','Ordynsky Area','NS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'NS-31','Severny Area','NS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'NS-32','Suzunsky Area','NS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'NS-33','Tatarsky Area / incl. Tatarsk','NS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'NS-34','Toguchinsky Area / incl. Toguchin','NS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'NS-35','Ubinsky Area','NS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'NS-36','Ust-Tarkskiy Area','NS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'NS-37','Chanovsky Area','NS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'NS-38','Cherepanovsky Area / incl. Cherepanovo','NS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'NS-39','Chistoozerny Area','NS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'NS-40','Chulym Area / incl. Chulym','NS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'NS-41','Barabinsk','NS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'NS-42','Berdsk','NS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'NS-43','Iskitim','NS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'NS-44','Kujbyshev','NS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'NS-45','Ob','NS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'NS-46','Tatarsk','NS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'NS-47','Koltsovo','NS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'OB-01','Dzerzhinsky District of Orenburg ','OB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'OB-02','Leninsky District of Orenburg','OB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'OB-03','Industrialny District of Orenburg','OB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'OB-04','Centralny District of Orenburg','OB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'OB-05','Leninsky District of Orsk','OB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'OB-06','Oktyabrsky District of Orsk','OB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'OB-07','Sovetsky District of Orsk','OB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'OB-08','Abdulino','OB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'OB-09','Buguruslan','OB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'OB-10','Buzuluk','OB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'OB-11','Guy','OB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'OB-12','Kuvandyk','OB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'OB-13','Mednogorsk','OB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'OB-14','Novotroitsk','OB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'OB-15','Sorochinsk','OB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'OB-16','Yasny','OB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'OB-17','Abdulinsky Area / incl. Abdulino','OB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'OB-18','Adamovsky Area','OB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'OB-19','Akbulaksky Area','OB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'OB-20','Aleksandrovsky Area','OB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'OB-21','Asekeevsky Area','OB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'OB-22','Beljaevsky Area','OB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'OB-23','Buguruslansky Area','OB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'OB-24','Buzuluksky Area','OB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'OB-25','Gaysky Area','OB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'OB-26','Grachevsky Area','OB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'OB-27','Dombarovsky Area','OB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'OB-28','Ileksky Area','OB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'OB-29','Kvarkensky Area','OB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'OB-30','Krasnogvardeisky Area','OB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'OB-31','Kuvandyksky Area / incl. Kuvandyk','OB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'OB-32','Kurmanayevsky Area','OB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'OB-33','Matveevsky Area','OB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'OB-34','Novoorsky Area','OB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'OB-35','Novosergievsky Area','OB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'OB-36','Oktyabrsky Area','OB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'OB-37','Orenburgsky Area','OB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'OB-38','Pervomajsky Area','OB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'OB-39','Perevolotsky Area','OB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'OB-40','Ponomarevsky Area','OB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'OB-41','Sakmarsky Area','OB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'OB-42','Saraktashsky Area','OB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'OB-43','Svetlinsky Area','OB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'OB-44','Severny Area','OB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'OB-45','Sol-Iletskiy Area / incl. Sol-Iletsk','OB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'OB-46','Sorochinsky City District','OB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'OB-47','Tashlinsky Area','OB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'OB-48','Totsky Area','OB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'OB-49','Tyulgansky Area','OB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'OB-50','Sharlyksky Area','OB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'OB-51','Yasnensky Area / incl. Yasny','OB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'OB-52','Sol`-Iletsk','OB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'OB-53','Zato Komarovskiy','OB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'OM-01','Kirovsky District of Omsk','OM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'OM-02','Leninsky District of Omsk','OM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'OM-03','Oktiabrsky District of Omsk','OM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'OM-04','Sovetsky District of Omsk','OM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'OM-05','Centralny District of Omsk','OM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'OM-06','Isil`Kul`','OM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'OM-07','Kalachinsk','OM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'OM-08','Nazyvaevsk','OM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'OM-09','Tara','OM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'OM-10','Tyukalinsk','OM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'OM-11','Azovsky Nemetsky National Area','OM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'OM-12','Bol''Sherechensky Area','OM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'OM-13','Bolsheukovsky Area','OM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'OM-14','Gor''Kovsky Area','OM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'OM-15','Znamensky Area','OM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'OM-16','Isilkulsky Area / incl. Isilkul','OM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'OM-17','Kalachinsky Area / incl. Kalachinsk','OM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'OM-18','Kolosovsky Area','OM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'OM-19','Kormilovsky Area','OM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'OM-20','Krutinsky Area','OM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'OM-21','Ljubinsky Area','OM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'OM-22','Mar''Janovsky Area','OM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'OM-23','Moskalensky Area','OM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'OM-24','Muromtsevsky Area','OM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'OM-25','Nazyvayevsky Area / incl. Nazyvaevsk','OM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'OM-26','Nizhneomsky Area','OM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'OM-27','Novovarshavsky Area','OM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'OM-28','Odessky Area','OM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'OM-29','Okoneshnikovsky Area','OM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'OM-30','Omsky Area','OM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'OM-31','Pavlogradsky Area','OM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'OM-32','Poltavsky Area','OM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'OM-33','Russko-Polyansky Area','OM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'OM-34','Sargatsky Area','OM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'OM-35','Sedelnikovsky Area','OM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'OM-36','Tavrichesky Area','OM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'OM-37','Tarsky Area / incl. Tara ','OM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'OM-38','Tevrizsky Area','OM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'OM-39','Tyukalinsky Area / incl. Tyukalinsk','OM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'OM-40','Ust-Ishimskiy Area','OM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'OM-41','Cherlaksky Area','OM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'OM-42','Sherbakulsky Area','OM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'PK-01','Leninsky District of Vladivostok','PK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'PK-02','Pervomajsky District of Vladivostok','PK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'PK-03','Pervorechensky District of Vladivostok','PK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'PK-04','Sovetsky District of Vladivostok','PK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'PK-05','Frunzensky District of Vladivostok','PK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'PK-06','Arsenyev','PK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'PK-07','Artem','PK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'PK-08','Dalnegorsk','PK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'PK-09','Dalnerechensk','PK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'PK-10','Lesozavodsk','PK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'PK-11','Nahodka','PK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'PK-12','Partizansk','PK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'PK-13','Spassk-Dalniy','PK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'PK-14','Ussuriisk District','PK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'PK-15','Bolshoy Kamen','PK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'PK-16','Zato Fokino','PK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'PK-17','Anuchinsky Area','PK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'PK-18','Dalnerechensky Area','PK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'PK-19','Kavalerovsky Area','PK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'PK-20','Kirovsky Area','PK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'PK-21','Krasnoarmeisky District','PK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'PK-22','Lazovsky Area','PK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'PK-24','Mikhaylovsky Area','PK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'PK-25','Nadezhdinsky Area','PK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'PK-26','Oktyabrsky Area','PK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'PK-27','Olginsky Area','PK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'PK-28','Partizansky Area','PK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'PK-29','Pogranichny Area','PK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'PK-30','Pozharsky Area','PK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'PK-31','Spassky Area','PK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'PK-32','Terneysky Area','PK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'PK-34','Hankaysky Area','PK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'PK-35','Hasansky Area','PK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'PK-36','Horolsky Area','PK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'PK-37','Chernigovsky Area','PK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'PK-38','Chuguyevsky Area','PK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'PK-39','Shkotovsky Area','PK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'PK-40','Yakovlevsky Area','PK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'PM-01','Dzerzhinsky District of Perm','PM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'PM-02','Industrialny District of Perm','PM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'PM-03','Kirovsky District of Perm','PM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'PM-04','Leninsky District of Perm','PM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'PM-05','Motovilikhinsky District of Perm','PM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'PM-06','Ordzhonikidzevsky District of Perm','PM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'PM-07','Sverdlovsky District of Perm','PM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'PM-08','Aleksandrovsky Area / incl. Aleksandrovsk','PM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'PM-09','Berezniki','PM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'PM-10','Gremyachinsky Area / incl. Gremyachinsk','PM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'PM-11','Gubakhinsky Area / incl. Gubakha','PM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'PM-13','Kizelovsky Area / incl. Kizel','PM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'PM-14','Krasnokamsky Area / incl. Krasnokamsk','PM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'PM-15','Kungur','PM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'PM-17','Solikamsk','PM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'PM-20','Bardymsky Area','PM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'PM-21','Berezovsky Area','PM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'PM-22','Bolshesosnovsky Area','PM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'PM-23','Vereshchaginsky Area / incl. Vereshchagino','PM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'PM-24','Gornozavodsky Area / incl. Gornozavodsk','PM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'PM-25','Elovsky Area','PM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'PM-26','Ilyinsky Area / incl. Chermoz','PM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'PM-27','Karagaysky Area','PM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'PM-28','Kishertsky Area','PM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'PM-29','Krasnovishersky District / incl. Krasnovishersk','PM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'PM-30','Kuedinsky Area','PM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'PM-31','Kungursky Area','PM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'PM-32','Lysvensky Area / incl. Lysva','PM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'PM-33','Nytvensky Area / incl. Nytva','PM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'PM-34','Oktyabrsky Area','PM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'PM-35','Ordinsky Area','PM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'PM-36','Osinsky Area / incl. Osa','PM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'PM-37','Ohansky Area Including Ohansk','PM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'PM-38','Ochersky Area Including Ocher','PM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'PM-39','Permsky Area','PM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'PM-40','Sivinsky Area','PM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'PM-41','Solikamsky Area','PM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'PM-42','Suksunsky Area','PM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'PM-43','Uinsky Area','PM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'PM-44','Usolsky Area / incl. Usolye','PM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'PM-45','Chastinsky Area','PM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'PM-46','Cherdynsky Area / incl. Cherdyn','PM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'PM-47','Chernushinsky Area / incl. Chernushka','PM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'PM-48','Chusovskoy Area / incl. Chusovoy','PM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'PM-49','Tchaikovsky Area','PM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'PM-50','Dobryansky Area / incl. Dobryanka','PM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'PM-51','Zato Zvezdny ','PM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'PM-52','Kudymkar','PM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'PM-53','Gaynsky Area','PM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'PM-54','Kosinsky Area','PM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'PM-55','Kochevsky Area','PM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'PM-56','Kudymkarsky Area','PM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'PM-57','Yurlinsky Area','PM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'PM-58','Yusvinsky Area','PM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'SL-01','Yuzhno-Sakhalinsk','SL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'SL-10','Aleksandrovsk-Sahalinskiy Area / incl. Aleksandrov-Sahalinskiy','SL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'SL-11','Anivsky Area / incl. Aniva','SL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'SL-12','Dolinsky Area / incl. Dolinsk','SL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'SL-13','Korsakovsky Area / incl. Korsakov','SL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'SL-14','Kurilsky Area / incl. Kurilsk','SL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'SL-15','Makarovsky Area / incl. Makarov ','SL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'SL-16','Nevelsky Area / incl. Nevelsk, Gornozavodsk','SL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'SL-17','Nogliksky Area','SL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'SL-18','Okhinsky Area / incl. Okha','SL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'SL-19','Poronaysky Area / incl. Poronaysk, Vahrushev','SL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'SL-20','Severo-Kurilsky Area / incl. Severo-Kurilsk','SL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'SL-21','Tomarinsky Area / incl. Tomari, Krasnogorsk','SL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'SL-22','Tymovsky Area','SL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'SL-23','Uglegorsk Area / incl. Uglegorsk, Shahtersk','SL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'SL-24','Holmsky Area / incl. Kholmsk, Chekhov','SL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'SL-25','Juzhno-Kurilsky Area','SL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'SL-26','Smirnykhovsky Area','SL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'SV-01','Verkh-Isetsky District of Ekaterinburg','SV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'SV-02','Zheleznodorozhny District of Ekaterinburg','SV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'SV-03','Kirovsky District of Ekaterinburg','SV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'SV-04','Leninsky District of Ekaterinburg','SV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'SV-05','Oktyabrsky District of Ekaterinburg','SV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'SV-06','Ordzhonikidzevsky District of Ekaterinburg','SV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'SV-07','Chkalovsky District of Ekaterinburg','SV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'SV-08','Krasnogorsk District of Kamensk-Uralsky','SV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'SV-09','Sinarsky District of Kamensk-Uralsky','SV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'SV-10','Dzerzhinsky District of Nizhni Tagil','SV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'SV-11','Leninsky District of Nizhni Tagil','SV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'SV-12','Tagilstroyevsky District of Nizhni Tagil','SV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'SV-13','City of Alapaevsk','SV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'SV-14','Artemovsky City District','SV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'SV-15','Asbestovsky City District','SV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'SV-16','Berezovsky City District','SV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'SV-17','Bogdanovich City District','SV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'SV-18','Verhnaya Pyshma City District','SV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'SV-19','Verkhnesaldinsky City District','SV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'SV-20','City District Zarechny','SV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'SV-21','Ivdel City District','SV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'SV-22','City of Irbit','SV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'SV-23','Kamyshlovsky City District','SV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'SV-24','Karpinsk City District','SV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'SV-25','Kachkanar City District','SV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'SV-26','Kirovgrad City District','SV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'SV-27','Krasnoturyinsk City District','SV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'SV-28','Krasnouralsk City District','SV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'SV-29','Kushva City District','SV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'SV-30','Nevyansk City District','SV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'SV-31','Nizhnyaya Salda City District','SV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'SV-32','Nizhnyaya Tura City District','SV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'SV-33','Pervouralsk City District','SV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'SV-34','Polevskoy City District','SV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'SV-35','Revda City District','SV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'SV-36','Rezhevsky City District','SV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'SV-37','Severouralsk City District','SV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'SV-38','Serov City District','SV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'SV-39','Suhoj Log City District','SV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'SV-40','Tavdinsky City District','SV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'SV-41','Zato Lesnoj','SV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'SV-42','Novouralsk City District','SV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'SV-43','Alapaevsk City District','SV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'SV-45','Artinsky City District','SV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'SV-46','Achitsky City District','SV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'SV-47','Bajkalovsky Area','SV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'SV-48','Beloyarsk City District','SV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'SV-51','Verkhotursky City District','SV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'SV-52','Garinsky City District','SV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'SV-53','Irbitsky Education','SV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'SV-54','Kamensk City District','SV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'SV-55','Kamyshlovsky Area','SV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'SV-56','Krasnoufimsky District','SV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'SV-58','Nizhneserginsky Area','SV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'SV-60','Gornouralsky City District','SV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'SV-61','Pyshminsky City District','SV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'SV-63','Servosky Area','SV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'SV-64','Slobodo-Turinsky Area','SV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'SV-66','Sysertsky City District','SV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'SV-67','Taborinsky Area','SV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'SV-69','Talitsky City District','SV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'SV-70','Tugulymsky City District','SV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'SV-71','Turinsky City District','SV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'SV-72','Shalinsky City District','SV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'SV-74','Krasnoufimsk City District','SV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'SV-76','Novaya Lyalya City District','SV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'SV-77','Zato Svobodny','SV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'SV-78','Zato Ural`Sky','SV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'SV-79','Aramilsky City District','SV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'SV-80','Bisertsky City District','SV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'SV-81','Verhnee Dubrovo City District','SV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'SV-82','Verh-Neyvinskiy City District','SV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'SV-83','Verhny Tagil City District','SV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'SV-84','Verhnyaya Tura City District','SV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'SV-85','Volchansky City District','SV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'SV-86','Degtyarsk City District','SV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'SV-87','Malyshevsky City District','SV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'SV-88','Makhnevsky Municipality','SV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'SV-89','Pelym City District','SV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'SV-90','Reftinsky City District','SV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'SV-91','Sosvinsky City District','SV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'SV-92','Sredneuralsk City District','SV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'SV-93','Staroutkinsk City District','SV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'SV-94','Academic District Of Ekaterinburg','SV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'TN-01','Kalininsky District of Tyumen','TN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'TN-02','Leninsky District of Tyumen','TN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'TN-03','Centralny District of Tyumen','TN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'TN-04','Zavodoukovsky City District','TN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'TN-05','Ishim','TN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'TN-06','Tobolsk','TN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'TN-07','Yalutorovsk','TN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'TN-08','Abatsky Area','TN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'TN-09','Armizonsky Area','TN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'TN-10','Aromashevsky Area','TN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'TN-11','Berdjuzhsky Area','TN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'TN-12','Vagaysky Area','TN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'TN-13','Vikulovsky Area','TN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'TN-14','Golyshmanovsky Area','TN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'TN-15','Zavodoukovsky Area','TN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'TN-16','Isetsky Area','TN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'TN-17','Ishimsky Area','TN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'TN-18','Kazansky Area','TN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'TN-19','Nizhnetavdinsky Area','TN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'TN-20','Omutinsky Area','TN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'TN-21','Sladkovsky Area','TN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'TN-22','Sorokinsky Area','TN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'TN-23','Tobolsky Area','TN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'TN-24','Tyumensky Area','TN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'TN-25','Uvatsky Area','TN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'TN-26','Uporovsky Area','TN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'TN-27','Yurginsky Area','TN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'TN-28','Yalutorovsky Area','TN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'TN-29','Yarkovsky Area','TN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'TN-30','East District of Tyumen','TN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'TO-01','Kirovsky District of Tomsk','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'TO-02','Leninsky District of Tomsk','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'TO-03','Oktyabrsky District of Tomsk','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'TO-04','Sovetsky District of Tomsk','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'TO-07','Kedrovy','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'TO-08','Strezhevoj','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'TO-09','Seversk','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'TO-10','Aleksandrovsky Area','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'TO-11','Asinovsky Area / incl. Asino','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'TO-12','Bakcharsky Area','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'TO-13','Verhneketsky Area','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'TO-14','Zyrjansky Area','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'TO-15','Kargasoksky Area','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'TO-16','Kozhevnikovsky Area','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'TO-17','Kolpashevsky Area / incl. Kolpashevo','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'TO-18','Krivosheinsky Area','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'TO-19','Molchanovsky Area','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'TO-20','Parabel''Sky Area','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'TO-21','Pervomajsky Area','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'TO-22','Tegul''Detsky Area','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'TO-23','Tomsky Area','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'TO-24','Chainsky Area','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'TO-25','Shegarsky Area','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'TU-01','Kyzyl','TU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'TU-02','Ak-Dovurak','TU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'TU-03','Bay-Tayginskiy Kozhuun','TU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'TU-04','Barun-Hemchiksky Kozhuun','TU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'TU-05','Dzun-Hemchiksky Kozhuun / incl. Chadan','TU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'TU-06','Kaa-Hemsky Kozhuun','TU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'TU-07','Kyzylsky Kozhuun','TU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'TU-08','Mongun-Tayginsky Kozhuun','TU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'TU-09','Ovyursky Kozhuun','TU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'TU-10','Piy-Hemskiy Kozhuun / incl. Turan','TU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'TU-11','Sut-Holskiy Kozhuun','TU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'TU-12','Tandinsky Kozhuun','TU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'TU-13','Tes-Hemsky Kozhuun','TU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'TU-14','Todzhinsky Kozhuun','TU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'TU-15','Ulug-Hemsky Area Including Shagonar','TU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'TU-16','Ulug- Hemsky Kozhuun / incl. Shagonar','TU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'TU-17','Chedi- Holsky Kozhuun','TU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'TU-18','Erzinsky Kozhuun','TU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'TU-19','Tere - Holsky Kozhuun','TU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'YA-01','Yakutsk','YA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'YA-06','Abyjsky Area','YA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'YA-07','Aldansky Area / incl. Aldan, Tommot','YA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'YA-08','Allaihovsky Area','YA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'YA-09','Amginsky Area','YA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'YA-10','Anabarsky National Area','YA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'YA-11','Bulunsky Area','YA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'YA-12','Verhneviljujsky Area','YA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'YA-13','Verhnekolymsky Area','YA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'YA-14','Verkhoyansky Area / incl. Verkhoyansk','YA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'YA-15','Vilyuysky Area / incl. Vilyuysk','YA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'YA-16','Gorny Area','YA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'YA-17','Zhigansky Area','YA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'YA-18','Kobyaysky Area','YA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'YA-19','Lensky Area / incl. Lensk','YA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'YA-20','Megino-Kangalassky Area','YA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'YA-21','Mirninsky Area / incl. Mirny, Udachny','YA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'YA-22','Momsky National Area','YA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'YA-23','Namsky Area','YA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'YA-24','Nizhnekolymsky Area','YA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'YA-25','Nyurbinsky Area / incl. Nyurba','YA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'YA-26','Ojmjakonsky Area','YA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'YA-27','Olekminsky Area / incl. Olekminsk','YA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'YA-28','Oleneksky National Area','YA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'YA-29','Srednekolymsky Area / incl. Srednekolymsk','YA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'YA-30','Suntarsky Area','YA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'YA-31','Tattinsky Area','YA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'YA-32','Tomponsky Area','YA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'YA-33','Ust''-Aldansky Area','YA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'YA-34','Ust''-Majsky Area','YA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'YA-35','Ust''-Jansky Area','YA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'YA-36','Hangalassky Area / incl. Pokrovsk','YA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'YA-37','Churapchinsky Area','YA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'YA-38','Eveno-Bytantajsky National Area','YA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'YA-39','Neryungrinsky Area / incl. Neryungri','YA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'YA-40','Zhataj','YA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'YN-01','Salekhard','YN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'YN-02','Gubkinsky','YN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'YN-03','Labytnangi','YN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'YN-04','Muravlenko','YN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'YN-05','Nadym','YN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'YN-06','Novyj Urengoy','YN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'YN-07','Noyabrsk','YN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'YN-08','Krasnoselkupsky Area','YN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'YN-09','Nadymsky Area / incl. Nadym ','YN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'YN-10','Priural''Sky Area','YN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'YN-11','Purovsky Area','YN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'YN-12','Tazovsky District','YN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'YN-13','Shuryshkarsky Area','YN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'YN-14','Yamalsky Area','YN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'ZK-01','Zheleznodorozhny District of Chita','ZK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'ZK-02','Ingodinsky District of Chita','ZK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'ZK-03','Centralny District of Chita','ZK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'ZK-04','Chernovsky District of Chita','ZK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'ZK-05','Balej','ZK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'ZK-06','Borzja','ZK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'ZK-07','Krasnokamensk','ZK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'ZK-08','Petrovsk-Zabajkalsky','ZK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'ZK-09','Akshinsky Area','ZK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'ZK-10','Aleksandrovo-Zavodsky Area','ZK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'ZK-11','Balejsky Area / incl. Baley','ZK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'ZK-12','Borzinsky Area / incl. Borzya','ZK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'ZK-13','Gazimuro-Zavodsky Area','ZK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'ZK-14','Transbaikalsky Area','ZK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'ZK-15','Kalarsky Area','ZK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'ZK-16','Kalgansky Area','ZK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'ZK-17','Karymsky Area','ZK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'ZK-18','Krasnokamensky Area / incl. Krasnokamensk','ZK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'ZK-19','Krasnochikoysky Area','ZK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'ZK-20','Kyrinsky Area','ZK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'ZK-21','Mogochinsky Area / incl. Mogocha','ZK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'ZK-22','Nerchinsky Area / incl. Nerchinsk ','ZK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'ZK-23','Nerchinsko-Zavodsky Area','ZK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'ZK-24','Olovyanninsky Area','ZK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'ZK-25','Ononsky Area','ZK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'ZK-26','Petrovsk-Zabaykalsky Area','ZK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'ZK-27','Priargunsky Area','ZK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'ZK-28','Sretensky Area / incl. Sretensk','ZK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'ZK-29','Tungiro-Olekminsky Area','ZK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'ZK-30','Uletovsky Area','ZK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'ZK-31','Hiloksky Area / incl. Hilok','ZK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'ZK-32','Chernyshevsky Area','ZK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'ZK-33','Chitinsky Area','ZK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'ZK-34','Shelopuginsky Area','ZK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'ZK-35','Shilkinsky Area / incl. Shilka','ZK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'ZK-36','Tungokochensky Area','ZK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'ZK-37','Aginsky Area','ZK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'ZK-38','Duldurginsky Area','ZK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'ZK-39','Mogojtujsky Area','ZK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'ZK-40','Aginskoe','ZK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (15,'ZK-41','Zato Poselok Gorny','ZK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (21,'07001','Alaró','IB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (21,'07002','Alaior','IB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (21,'07003','Alcúdia','IB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (21,'07004','Algaida','IB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (21,'07005','Andratx','IB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (21,'07006','Artà','IB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (21,'07007','Banyalbufar','IB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (21,'07008','Binissalem','IB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (21,'07009','Búger','IB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (21,'07010','Bunyola','IB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (21,'07011','Calvià','IB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (21,'07012','Campanet','IB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (21,'07013','Campos','IB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (21,'07014','Capdepera','IB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (21,'07015','Ciutadella de Menorca','IB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (21,'07016','Consell','IB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (21,'07017','Costitx','IB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (21,'07018','Deiá','IB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (21,'07019','Escorca','IB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (21,'07020','Esporles','IB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (21,'07021','Estellencs','IB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (21,'07022','Felanitx','IB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (21,'07023','Ferreries','IB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (21,'07024','Formentera','IB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (21,'07025','Fornalutx','IB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (21,'07026','Eivissa','IB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (21,'07027','Inca','IB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (21,'07028','Lloret de Vistalegre','IB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (21,'07029','Lloseta','IB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (21,'07030','Llubí','IB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (21,'07031','Llucmajor','IB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (21,'07032','Maó-Mahón','IB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (21,'07033','Manacor','IB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (21,'07034','Mancor de la Vall','IB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (21,'07035','Maria de la Salut','IB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (21,'07036','Marratxí','IB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (21,'07037','Mercadal (Es)','IB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (21,'07038','Montuïri','IB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (21,'07039','Muro','IB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (21,'07040','Palma','IB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (21,'07041','Petra','IB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (21,'07042','Pollença','IB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (21,'07043','Porreres','IB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (21,'07044','Pobla (Sa)','IB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (21,'07045','Puigpunyent','IB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (21,'07046','Sant Antoni de Portmany','IB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (21,'07047','Sencelles','IB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (21,'07048','Sant Josep de sa Talaia','IB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (21,'07049','Sant Joan','IB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (21,'07050','Sant Joan de Labritja','IB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (21,'07051','Sant Llorenç des Cardassar','IB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (21,'07052','Sant Lluís','IB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (21,'07053','Santa Eugènia','IB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (21,'07054','Santa Eulària des Riu','IB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (21,'07055','Santa Margalida','IB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (21,'07056','Santa María del Camí','IB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (21,'07057','Santanyí','IB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (21,'07058','Selva','IB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (21,'07059','Salines (Ses)','IB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (21,'07060','Sineu','IB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (21,'07061','Sóller','IB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (21,'07062','Son Servera','IB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (21,'07063','Valldemossa','IB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (21,'07064','Castell (Es)','IB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (21,'07065','Vilafranca de Bonany','IB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (21,'07901','Ariany','IB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (21,'07902','Migjorn Gran (Es)','IB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (29,'35001','Agaete','GC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (29,'35002','Agüimes','GC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (29,'35003','Antigua','GC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (29,'35004','Arrecife','GC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (29,'35005','Artenara','GC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (29,'35006','Arucas','GC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (29,'35007','Betancuria','GC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (29,'35008','Firgas','GC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (29,'35009','Gáldar','GC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (29,'35010','Haría','GC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (29,'35011','Ingenio','GC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (29,'35012','Mogán','GC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (29,'35013','Moya','GC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (29,'35014','Oliva (La)','GC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (29,'35015','Pájara','GC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (29,'35016','Palmas de Gran Canaria (Las)','GC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (29,'35017','Puerto del Rosario','GC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (29,'35018','San Bartolomé','GC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (29,'35019','San Bartolomé de Tirajana','GC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (29,'35020','Aldea de San Nicolás (La)','GC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (29,'35021','Santa Brígida','GC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (29,'35022','Santa Lucía de Tirajana','GC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (29,'35023','Santa María de Guía de Gran Canaria','GC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (29,'35024','Teguise','GC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (29,'35025','Tejeda','GC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (29,'35026','Telde','GC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (29,'35027','Teror','GC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (29,'35028','Tías','GC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (29,'35029','Tinajo','GC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (29,'35030','Tuineje','GC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (29,'35031','Valsequillo de Gran Canaria','GC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (29,'35032','Valleseco','GC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (29,'35033','Vega de San Mateo','GC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (29,'35034','Yaiza','GC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (29,'38001','Adeje','TF','');
INSERT INTO adif_enum_secondary_subdivision VALUES (29,'38002','Agulo','TF','');
INSERT INTO adif_enum_secondary_subdivision VALUES (29,'38003','Alajeró','TF','');
INSERT INTO adif_enum_secondary_subdivision VALUES (29,'38004','Arafo','TF','');
INSERT INTO adif_enum_secondary_subdivision VALUES (29,'38005','Arico','TF','');
INSERT INTO adif_enum_secondary_subdivision VALUES (29,'38006','Arona','TF','');
INSERT INTO adif_enum_secondary_subdivision VALUES (29,'38007','Barlovento','TF','');
INSERT INTO adif_enum_secondary_subdivision VALUES (29,'38008','Breña Alta','TF','');
INSERT INTO adif_enum_secondary_subdivision VALUES (29,'38009','Breña Baja','TF','');
INSERT INTO adif_enum_secondary_subdivision VALUES (29,'38010','Buenavista del Norte','TF','');
INSERT INTO adif_enum_secondary_subdivision VALUES (29,'38011','Candelaria','TF','');
INSERT INTO adif_enum_secondary_subdivision VALUES (29,'38012','Fasnia','TF','');
INSERT INTO adif_enum_secondary_subdivision VALUES (29,'38013','Frontera','TF','');
INSERT INTO adif_enum_secondary_subdivision VALUES (29,'38014','Fuencaliente de la Palma','TF','');
INSERT INTO adif_enum_secondary_subdivision VALUES (29,'38015','Garachico','TF','');
INSERT INTO adif_enum_secondary_subdivision VALUES (29,'38016','Garafía','TF','');
INSERT INTO adif_enum_secondary_subdivision VALUES (29,'38017','Granadilla de Abona','TF','');
INSERT INTO adif_enum_secondary_subdivision VALUES (29,'38018','Guancha (La)','TF','');
INSERT INTO adif_enum_secondary_subdivision VALUES (29,'38019','Guía de Isora','TF','');
INSERT INTO adif_enum_secondary_subdivision VALUES (29,'38020','Güímar','TF','');
INSERT INTO adif_enum_secondary_subdivision VALUES (29,'38021','Hermigua','TF','');
INSERT INTO adif_enum_secondary_subdivision VALUES (29,'38022','Icod de los Vinos','TF','');
INSERT INTO adif_enum_secondary_subdivision VALUES (29,'38023','San Cristóbal de La Laguna','TF','');
INSERT INTO adif_enum_secondary_subdivision VALUES (29,'38024','Llanos de Aridane (Los)','TF','');
INSERT INTO adif_enum_secondary_subdivision VALUES (29,'38025','Matanza de Acentejo (La)','TF','');
INSERT INTO adif_enum_secondary_subdivision VALUES (29,'38026','Orotava (La)','TF','');
INSERT INTO adif_enum_secondary_subdivision VALUES (29,'38027','Paso (El)','TF','');
INSERT INTO adif_enum_secondary_subdivision VALUES (29,'38028','Puerto de la Cruz','TF','');
INSERT INTO adif_enum_secondary_subdivision VALUES (29,'38029','Puntagorda','TF','');
INSERT INTO adif_enum_secondary_subdivision VALUES (29,'38030','Puntallana','TF','');
INSERT INTO adif_enum_secondary_subdivision VALUES (29,'38031','Realejos (Los)','TF','');
INSERT INTO adif_enum_secondary_subdivision VALUES (29,'38032','Rosario (El)','TF','');
INSERT INTO adif_enum_secondary_subdivision VALUES (29,'38033','San Andrés y Sauces','TF','');
INSERT INTO adif_enum_secondary_subdivision VALUES (29,'38034','San Juan de la Rambla','TF','');
INSERT INTO adif_enum_secondary_subdivision VALUES (29,'38035','San Miguel de Abona','TF','');
INSERT INTO adif_enum_secondary_subdivision VALUES (29,'38036','San Sebastián de la Gomera','TF','');
INSERT INTO adif_enum_secondary_subdivision VALUES (29,'38037','Santa Cruz de la Palma','TF','');
INSERT INTO adif_enum_secondary_subdivision VALUES (29,'38038','Santa Cruz de Tenerife','TF','');
INSERT INTO adif_enum_secondary_subdivision VALUES (29,'38039','Santa Úrsula','TF','');
INSERT INTO adif_enum_secondary_subdivision VALUES (29,'38040','Santiago del Teide','TF','');
INSERT INTO adif_enum_secondary_subdivision VALUES (29,'38041','Sauzal (El)','TF','');
INSERT INTO adif_enum_secondary_subdivision VALUES (29,'38042','Silos (Los)','TF','');
INSERT INTO adif_enum_secondary_subdivision VALUES (29,'38043','Tacoronte','TF','');
INSERT INTO adif_enum_secondary_subdivision VALUES (29,'38044','Tanque (El)','TF','');
INSERT INTO adif_enum_secondary_subdivision VALUES (29,'38045','Tazacorte','TF','');
INSERT INTO adif_enum_secondary_subdivision VALUES (29,'38046','Tegueste','TF','');
INSERT INTO adif_enum_secondary_subdivision VALUES (29,'38047','Tijarafe','TF','');
INSERT INTO adif_enum_secondary_subdivision VALUES (29,'38048','Valverde','TF','');
INSERT INTO adif_enum_secondary_subdivision VALUES (29,'38049','Valle Gran Rey','TF','');
INSERT INTO adif_enum_secondary_subdivision VALUES (29,'38050','Vallehermoso','TF','');
INSERT INTO adif_enum_secondary_subdivision VALUES (29,'38051','Victoria de Acentejo (La)','TF','');
INSERT INTO adif_enum_secondary_subdivision VALUES (29,'38052','Vilaflor de Chasna','TF','');
INSERT INTO adif_enum_secondary_subdivision VALUES (29,'38053','Villa de Mazo','TF','');
INSERT INTO adif_enum_secondary_subdivision VALUES (29,'38901','Pinar de El Hierro (El)','TF','');
INSERT INTO adif_enum_secondary_subdivision VALUES (32,'51001','Ceuta','CE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (32,'52001','Melilla','ML','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'AD-01','Maikop','AD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'AD-02','Adygeysk','AD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'AD-03','Giaginsky Area','AD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'AD-04','Koshekhablsky Area','AD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'AD-05','Krasnogvardeisky district','AD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'AD-06','Maikopsky Area','AD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'AD-07','Takhtamukaysky Area','AD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'AD-08','Teuchezhsky Area','AD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'AD-09','Shovgenovsky Area','AD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'AO-01','Kirovsky district of Astrakhan','AO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'AO-02','Leninsky district of Astrakhan','AO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'AO-03','Sovetsky district of Astrakhan','AO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'AO-04','Trusovsky district of Astrakhan','AO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'AO-06','Zato Znamensk','AO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'AO-07','Akhtubinsky Area / incl. Akhtubinsk','AO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'AO-08','Volodarsky Area','AO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'AO-09','Enotayevsky Area','AO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'AO-10','Ikryaninsky Area','AO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'AO-11','Kamyzyaksky Area / incl. Kamyzyak','AO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'AO-12','Krasnoyarsk Area','AO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'AO-13','Limansky Area','AO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'AO-14','Narimanovsky Area / incl. Narimanov','AO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'AO-15','Privolzhsky Area','AO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'AO-16','Harabalinsky Area / incl. Harabali','AO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'AO-17','Chernoyarsky Area','AO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'AR-01','Isakogorsky district of Arkhangelsk','AR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'AR-02','Lomonosov District of Arkhangelsk','AR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'AR-03','Oktiabrsky District of Arkhangels','AR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'AR-04','Solombalsky District of Arkhangelsk','AR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'AR-05','Koryazhma','AR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'AR-06','Kotlas','AR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'AR-07','Novodvinsk','AR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'AR-08','Onega','AR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'AR-09','Severodvinsk','AR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'AR-10','Zato Mirny','AR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'AR-11','Velsky Area / incl. Velsk','AR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'AR-12','Verhnetoemsky Area','AR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'AR-13','Vilegodsky Area','AR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'AR-14','Vinogradovsky Area','AR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'AR-15','Kargopolsky Area / incl. Kargopol','AR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'AR-16','Konoshsky Area','AR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'AR-17','Kotlassky Area / incl. Solvychegodsk','AR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'AR-18','Krasnoborsky Area','AR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'AR-19','Lensky Area','AR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'AR-20','Leshukonsky Area','AR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'AR-21','Mezensky Area / incl. Mezen','AR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'AR-22','Nyandomsky Area / incl. Nyandoma','AR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'AR-23','Onezhsky Area / incl. Onega','AR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'AR-24','Pinezhsky Area','AR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'AR-25','Plesetsky Area','AR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'AR-26','Primorsky Area','AR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'AR-27','Solovetsky Area','AR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'AR-28','Ustyansky Area','AR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'AR-29','Holmogorsky Area','AR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'AR-30','Shenkursky Area / incl. Shenkursk','AR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'AR-31','Novaya Zemlya','AR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'AR-32','Franz Joseph Land','AR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'AR-33','Varavino-Factoriya District of Arkhangelsk','AR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'AR-34','Maymaksansky District of Arkhangelsk','AR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'AR-35','Majskaya Gorka District of Arkhangelsk','AR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'AR-36','Severny District of Arkhangelsk ','AR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'AR-37','Tsiglomensky District of Arkhangelsk ','AR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'BO-01','Vostochny District of Belgorod','BO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'BO-02','Zapadny District of Belgorod','BO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'BO-05','Gubkinsky District','BO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'BO-06','Stary Oskol District','BO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'BO-08','Alexeyevsky District / incl. Alekseyevka','BO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'BO-09','Belgorodsky Area','BO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'BO-10','Borisovsky Area','BO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'BO-11','Valuysky Area / incl. Valuyki','BO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'BO-12','Veydelevsky Area','BO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'BO-13','Volokonovsky Area','BO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'BO-14','Grayvoronsky Area / incl. Grayvoron','BO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'BO-16','Ivnyansky Area','BO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'BO-17','Korochansky Area / incl. Korocha','BO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'BO-18','Krasnensky Area','BO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'BO-19','Krasnogvardeisky District','BO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'BO-20','Krasnoyaruzhsky Area','BO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'BO-21','Novooskolsky Area / incl. New Oskol','BO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'BO-22','Prohorov''s Area','BO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'BO-23','Rakityansky Area','BO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'BO-24','Rovensky Area','BO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'BO-26','Chernyansky Area','BO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'BO-27','Shebekinsky Area / incl. Shebekino','BO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'BO-28','Yakovlevsky Area','BO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'BO-29','Alekseevka','BO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'BO-30','Valuiki','BO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'BO-31','Gubkin','BO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'BO-32','Stary Oskol','BO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'BR-01','Bezhitsky District of Bryansk','BR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'BR-02','Volodarsky District of Bryansk','BR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'BR-03','Sovetsky District of Bryansk','BR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'BR-04','Fokinsky District of Bryansk','BR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'BR-06','Klintsy','BR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'BR-07','Novozybkov','BR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'BR-08','Seltso','BR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'BR-09','Brasovsky Area','BR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'BR-10','Bryansky Area','BR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'BR-11','Vygonichsky Area','BR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'BR-12','Gordeevsky Area','BR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'BR-13','Dubrovsky Area','BR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'BR-14','Dyatkovsky Area / incl. Dyatkovo','BR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'BR-15','Zhiryatinsky Area','BR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'BR-16','Zhukovsky Area / incl. Zhukovka','BR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'BR-17','Zlynkovsky Area / incl. Zlynka','BR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'BR-18','Karachevsky Area / incl. Karachev','BR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'BR-19','Kletnyansky Area','BR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'BR-20','Klimovsky Area','BR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'BR-21','Klintsovsky Area','BR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'BR-22','Komarichsky Area','BR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'BR-23','Krasnogorsky Area','BR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'BR-24','Mglinsky Area / incl. Mglin','BR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'BR-25','Navlinsky Area','BR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'BR-26','Novozybkovsky Area','BR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'BR-27','Pogarsky Area','BR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'BR-28','Pochepsky Area / incl. Pochep','BR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'BR-29','Rognedinsky Area','BR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'BR-30','Sevsky Area / incl. Cebck','BR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'BR-31','Starodubsky Area','BR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'BR-32','Suzemsky Area','BR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'BR-33','Surazhsky Area / incl. Surazh','BR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'BR-34','Trubchevsky Area / incl. Trubchevsk','BR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'BR-35','Unechsky Area / incl. Unecha','BR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'BR-36','Starodub','BR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'BR-37','Fokino','BR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'CN-01','Zavodskoy district of Grozny','CN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'CN-02','Leninsky district of Grozny','CN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'CN-03','Oktyabrsky district of Grozny','CN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'CN-04','Staropromyslovsky Area of Grozny','CN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'CN-06','Argun','CN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'CN-07','Gudermes','CN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'CN-10','Sunzhensky Area','CN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'CN-11','Achhoy-Martanovskiy Area','CN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'CN-12','Vedensky Area','CN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'CN-13','Groznensky Area','CN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'CN-14','Gudermessky Area / incl. Gudermes','CN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'CN-15','Itum-Kalinskiy Area','CN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'CN-16','Kurchaloyevsky Area','CN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'CN-17','Nadterechny Area','CN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'CN-18','Naursky Area','CN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'CN-19','Nozhay-Yurtovskiy Area','CN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'CN-20','Urus-Martanovskiy Area / incl. Urus-Martan','CN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'CN-21','Shatoysky Area','CN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'CN-22','Shalinsky Area / incl. Shawls','CN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'CN-23','Sharoysky Area','CN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'CN-24','Shchelkovskoj Area','CN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'CU-01','Kalininsky district Cheboksary','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'CU-02','Leninsky district Cheboksary','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'CU-03','Moskovsky district Cheboksary','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'CU-04','Alatyr','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'CU-05','Kanash','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'CU-06','Novocheboksarsk','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'CU-07','Shumerlya','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'CU-08','Alatyrsky Area','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'CU-09','Alikovsky Area','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'CU-10','Batyrevsky Area','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'CU-11','Vurnarsky Area','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'CU-12','Ibresinsky Area','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'CU-13','Kanashsky Area','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'CU-14','Kozlowski Area / incl. Kozlovka','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'CU-15','Komsomolsky Area','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'CU-16','Krasnoarmeisky district','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'CU-17','Krasnochetaysky Area','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'CU-18','Mariinsko-Posadsky Area / incl. Mariinsky Posad','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'CU-19','Morgaushsky Area','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'CU-20','Poretsky Area','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'CU-21','Urmarsky Area','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'CU-22','Tsivilsky Area / incl. Tsivilsk','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'CU-23','Cheboksary Area','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'CU-24','Shemurshinsky Area','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'CU-25','Shumerlinsky Area','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'CU-26','Yadrinsky Area / incl. Yadrin','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'CU-27','Yalchiksky Area','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'CU-28','Yantikovsky Area','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'DA-01','Kirovsky district of Makhachkala','DA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'DA-02','Leninsky district of Makhachkala','DA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'DA-03','Sovetsky district of Makhachkala','DA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'DA-04','Buinaksk','DA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'DA-05','Dagestanskie Ogni','DA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'DA-06','Derbent','DA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'DA-07','Izberbash','DA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'DA-08','Kaspiysk','DA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'DA-09','Kizilyurt','DA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'DA-10','Kizlyar','DA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'DA-11','Khasavyurt','DA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'DA-12','Juzhno-Suhokumsk','DA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'DA-13','Agulsky Area','DA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'DA-14','Akushinsky Area','DA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'DA-15','Akhvakhsky Area','DA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'DA-16','Akhtynsky Area','DA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'DA-17','Babayurtovsky Area','DA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'DA-18','Botlikhsky Area','DA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'DA-19','Buinaksky Area','DA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'DA-20','Gergebilsky Area','DA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'DA-21','Gumbetovsky Area','DA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'DA-22','Gunibsky Area','DA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'DA-23','Dakhadayevsky Area','DA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'DA-24','Derbentsky Area','DA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'DA-25','Dokuzparinsky Area','DA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'DA-26','Kazbekovsky Area','DA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'DA-27','Kaitagsky Area','DA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'DA-28','Kayakentsky Area','DA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'DA-29','Kizilyurtovsky Area','DA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'DA-30','Kizlyarsky Area','DA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'DA-31','Kulinsky Area','DA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'DA-32','Kurakhsky Area','DA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'DA-33','Laksky Area','DA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'DA-34','Levashinsky Area','DA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'DA-35','Karabudahkentsky Area','DA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'DA-36','Kumtorkalinsky Area','DA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'DA-37','Magaramkentsky Area','DA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'DA-38','Novolaksky Area','DA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'DA-39','Nogaisky Area','DA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'DA-40','Rutulsky Area','DA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'DA-41','Sergokalinsky Area','DA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'DA-42','Soulieyman-Stalsky Area','DA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'DA-43','Tabasaransky Area','DA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'DA-44','Tarumovsky Area','DA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'DA-45','Tlyaratinsky Area','DA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'DA-46','Untsukulsky Area','DA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'DA-47','Khasavyurtovsky Area','DA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'DA-48','Hivsky Area','DA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'DA-49','Hunzahsky Area','DA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'DA-50','Tsumadinsky Area','DA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'DA-51','Tsuntinsky Area','DA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'DA-52','Charodinsky Area','DA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'DA-53','Shamilsky Area','DA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'IN-01','Magas','IN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'IN-02','Karabulak','IN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'IN-03','Nazran','IN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'IN-04','Malgobek','IN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'IN-05','Dzhayrakhsky Area','IN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'IN-06','Malgobeksky Area','IN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'IN-07','Nazransky Area ','IN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'IN-08','Sunzhensky Area','IN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'IN-09','Sunzha','IN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'IN-10','Altievsky District of the City of Nazran','IN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'IN-11','Gamurzievsky District of the City of Nazran','IN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'IN-12','Nasyr-Kortsky District of the City of Nazran','IN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'IN-13','Central District of the City of Nazran','IN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'IV-01','Leninsky District of Ivanovo','IV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'IV-02','Oktyabrsky District of Ivanovo','IV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'IV-03','Sovetsky District of Ivanovo','IV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'IV-04','Frunzensky District of Ivanovo','IV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'IV-05','Vichuga','IV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'IV-06','Kineshma','IV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'IV-07','Teykovo','IV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'IV-10','Verhnelandehovsky Area','IV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'IV-11','Vichugsky Area','IV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'IV-12','Gavrilovo-Posadskiy Area / incl. Gavrilov Posad','IV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'IV-13','Zavolzhye Area / incl. Zavolzhsk','IV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'IV-14','Ivanovsky Area','IV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'IV-15','Ilyinsky Area','IV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'IV-16','Kineshma Area / incl. Navoloki','IV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'IV-17','Komsomolsky / incl. Komsomolsk','IV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'IV-18','Lezhnevsky Area','IV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'IV-19','Lukhsky Area','IV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'IV-20','Palehsky Area','IV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'IV-21','Pestyakovsky Area','IV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'IV-22','Privolzhsky Area / incl. Ples, Privolzhsk','IV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'IV-23','Puchezhsky Area / incl. Puchezh','IV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'IV-24','Rodnikovsky Area / incl. Rodniki','IV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'IV-25','Savinsky Area','IV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'IV-26','Teykovsky Area','IV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'IV-27','Furmanovsky Area / incl. Furmanov ','IV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'IV-28','Shuysky Area','IV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'IV-29','Yuzhskiy Area / incl. Yuzha','IV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'IV-30','Yuryevetsky Area / incl. Yuryevets','IV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'IV-31','Kohma','IV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KB-01','Nalchik','KB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KB-02','Baksansky Area','KB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KB-03','Zolsky Area','KB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KB-04','Majsky Area / incl. Majsky','KB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KB-05','Prohladnensky Area','KB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KB-06','Tersky Area / incl. Terek','KB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KB-07','Urvansky Area / incl. Nartkala','KB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KB-08','Chegemsky Area','KB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KB-09','Chereksky Area','KB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KB-10','Elbrussky Area / incl. Tyrnyauz','KB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KB-11','Leskemsky Area','KB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KB-12','Baksan','KB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KB-13','Prokhladny','KB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KC-01','Cherkessk','KC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KC-02','Karachaevsk / incl. Teberda','KC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KC-03','Adyge-Hablskiy Area','KC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KC-04','Zelenchuksky Area','KC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KC-05','Karachaevsky Area','KC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KC-06','Malokarachaevsky Area','KC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KC-07','Prikubansky Area','KC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KC-08','Urupsky Area','KC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KC-09','Ust-Dzhegutinskiy Area / incl. Ust-Dzheguta','KC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KC-10','Habezsky Area','KC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KC-11','Abazinsky Area','KC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KC-12','Nogaisky Area','KC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KG-01','Leninsky District of Kaluga','KG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KG-02','Moskovsky District of Kaluga','KG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KG-03','Oktyabrsky District of Kaluga','KG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KG-05','Lyudinovo','KG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KG-06','Obninsk','KG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KG-07','Babyninsky Area','KG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KG-08','Barjatinsky Area','KG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KG-09','Borovsky Area / incl. Borovsk','KG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KG-10','Dzerzhinsky District / incl. Kondrovo','KG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KG-11','Duminichsky Area','KG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KG-12','Zhizdrinsky Area / incl. Zhizdra','KG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KG-13','Zhukovsky Area / incl. Bugs','KG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KG-14','Iznoskovsky Area','KG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KG-15','City of Kirov and Kirovsky District','KG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KG-16','Kozelsky Area / incl. Kozelsk, Sosenskiy','KG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KG-17','Kuibyshevsky District','KG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KG-18','Lyudinovsky Area','KG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KG-19','Maloyaroslavetsky Area / incl. Maloyaroslavets','KG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KG-20','Medynsky Area / incl. Medyn','KG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KG-21','Meshchovsky Area / incl. Meshchovsk','KG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KG-22','Mosalsky Area / incl. Mosalsk','KG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KG-23','Peremyshlsky Area','KG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KG-24','Spas-Demenskiy Area / incl. Spas-Demensk','KG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KG-25','Sukhinichsky Area / incl. Suhinichi','KG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KG-26','Tarussky Area / incl. Tarusa','KG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KG-27','Ulyanovsky Area','KG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KG-28','Ferzikovsky Area','KG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KG-29','Hvastovichsky Area','KG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KG-30','Yukhnovsky Area / incl. Yuhnov','KG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KG-31','Balabanovo','KG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KI-01','Leninsky District of Kirov','KI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KI-02','Novovjatsky District of Kirov','KI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KI-03','Oktyabrsky District of Kirov','KI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KI-04','Pervomajsky District of Kirov','KI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KI-05','Vjatskie Polyany','KI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KI-06','Kirovo-Chepetsk','KI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KI-07','Kotelnich','KI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KI-08','Slobodskoj','KI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KI-09','Arbazhsky Area','KI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KI-10','Afanasyevsky Area','KI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KI-11','Belokholunitsky Area / incl. Belaja Kholunitsa','KI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KI-12','Bogorodsky Area','KI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KI-13','Verkhnekamsky Area / incl. Kirs','KI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KI-14','Verhoshizhemsky Area','KI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KI-15','Vyatskopolyansky Area / incl. Sosnovka','KI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KI-16','Darovsky Area','KI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KI-17','Zuyevsky Area / incl. Zuevka','KI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KI-18','Kiknursky Area','KI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KI-19','Kilmezsky Area','KI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KI-20','Kirovo-Chepetsky Area','KI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KI-21','Kotelnichsky Area','KI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KI-22','Kumensky Area','KI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KI-23','Lebjazhsky Area','KI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KI-24','Luzsky Area / incl. Luza','KI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KI-25','Malmyzhsky Area / incl. Malmyzh','KI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KI-26','Murashinsky Area / incl. Murashi','KI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KI-27','Nagorsky Area','KI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KI-28','Nemsky Area','KI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KI-29','Nolinsky Area / incl. Nolinsk','KI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KI-30','Omutninsky Area / incl. Omutninsk','KI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KI-31','Oparinsky Area','KI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KI-32','Orichevsky Area','KI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KI-33','Orlovsky Area / incl. Orlov','KI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KI-34','Pizhansky Area','KI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KI-35','Podosinovsky Area','KI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KI-36','Sanchursky Area','KI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KI-37','Svechinsky Area','KI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KI-38','Slobodskoj Area','KI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KI-39','Sovetsky Area  / incl. Sovetsk','KI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KI-40','Sunsky Area','KI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KI-41','Tuzhinsky Area','KI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KI-42','Uninsky Area','KI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KI-43','Urzhumsky Area / incl. Urzhum','KI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KI-44','Falensky Area','KI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KI-45','Shabalinsky Area','KI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KI-46','Yuryansky Area','KI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KI-47','Yaransky Area / incl. Yaransk','KI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KI-48','Zato Pervomajsky','KI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KL-01','Petrozavodsk','KL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KL-04','Kostomuksha','KL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KL-07','Sortavalsky Area / incl. Sortavala','KL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KL-08','Belomorsky Area / incl. Belomorsk','KL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KL-10','Kalevalsky National Area','KL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KL-11','Kemsky Area / incl. Kem','KL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KL-12','Kondopozhsky Area / incl. Kondopoga','KL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KL-13','Lakhdenpokhsky Area / incl. Lahdenpohya','KL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KL-14','Louhsky Area','KL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KL-15','Medvezhyegorsky Area / incl. Medvezhyegorsk','KL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KL-16','Muyezersky Area','KL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KL-17','Olonetsky Area / incl. Olonets','KL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KL-18','Pitkyarantsky Area / incl. Pitkyaranta','KL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KL-19','Prionezhsky Area','KL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KL-20','Pryazhinsky Area','KL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KL-21','Pudozhsky Area / incl. Pudozh','KL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KL-22','Segezhsky Area / incl. Segezha','KL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KL-23','Suoyarvsky Area / incl. Suoyarvi','KL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KM-01','Elista','KM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KM-02','Gorodovikovsky Area / incl. Gorodovikovsk','KM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KM-03','Iki-Burulskiy Area','KM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KM-04','Ketchenerovsky Area','KM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KM-05','Lagansky Area / incl. Lagan','KM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KM-06','Maloderbetovsky Area','KM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KM-07','Oktyabrsky District','KM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KM-08','Priyutnensky Area','KM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KM-09','Sarpinsky Area','KM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KM-10','Tselinny Area','KM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KM-11','Chernozemelsky Area','KM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KM-12','Yustinsky Area','KM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KM-13','Yashaltinsky Area','KM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KM-14','YashkulskY Area','KM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KO-01','Syktyvkar','KO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KO-02','Vorkuta','KO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KO-03','Vuktylsky Area / incl. Vuktyl','KO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KO-04','Inta','KO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KO-05','Pechorsky Area / incl. Pechora','KO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KO-06','Sosnogorsky Area / incl. Sosnogorsk and suburbs','KO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KO-07','Usinsk / incl. suburb','KO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KO-08','Ukhta','KO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KO-09','Izhemsky Area','KO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KO-10','Knyazhpogostsky Area / incl. Emba','KO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KO-11','Koygorodsky Area','KO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KO-12','Kortkerossky Area','KO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KO-13','Priluzsky Area','KO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KO-14','Syktyvdinsky Area','KO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KO-15','Sysolsky Area','KO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KO-16','Troitsko-Pechorsky Area','KO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KO-17','Udorsky Area','KO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KO-18','Ust-Vymskiy Area / incl. Mikun','KO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KO-19','Ust-Kulomskiy Area','KO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KO-20','Ust-Tsilemskiy Area','KO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KR-01','Zapadny District of Krasnodar','KR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KR-02','Karasunsky District of Krasnodar','KR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KR-03','Prikubansky District of Krasnodar','KR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KR-04','Centralny District of Krasnodar','KR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KR-05','Vostochny District of Novorossiysk','KR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KR-06','Primorsky District of Novorossiysk','KR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KR-07','Centralny District of Novorossiysk','KR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KR-08','Adlersky District OF Sochi','KR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KR-09','Lazarevsky District of Sochi','KR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KR-10','Khostinsky District of Sochi','KR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KR-11','Centralny District of Sochi','KR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KR-12','Anapa','KR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KR-13','Armavir','KR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KR-14','Belorechensk','KR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KR-15','Gelendzhik','KR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KR-16','Goryachij Kljuch','KR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KR-17','Yeysk','KR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KR-18','Kropotkin','KR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KR-19','Krymsk','KR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KR-20','Labinsk','KR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KR-21','Slavyansk-On-Kuban','KR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KR-22','Tikhoretsk','KR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KR-23','Tuapse','KR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KR-24','Abinsky Area / incl. Abinsk','KR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KR-26','Apsheronsky Area / incl. Apsheronsk, Hadyzhensk','KR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KR-27','Beloglinsky Area','KR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KR-28','Belorechensky Area / incl. Belorechensk','KR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KR-29','Bryukhovetsky Area ','KR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KR-30','Vyselkovsky Area','KR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KR-31','Gulkevichsky Area / incl. Gulkevichi','KR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KR-32','Dinskoj Area','KR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KR-33','Yeysk Area / incl. Yeysk','KR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KR-34','Kavkazsky Area / incl. Kropotkin','KR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KR-35','Kalininsky District','KR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KR-36','Kanevsky Area','KR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KR-37','Korenovsky Area / incl. Kopehobck','KR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KR-38','Krasnoarmejsky Area','KR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KR-39','Krylovsky Area','KR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KR-40','Krymsky Area / incl. Krymsk','KR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KR-41','Kurganinsky Area / incl. Kurganinsk','KR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KR-42','Kushchevsky Area','KR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KR-43','Labinsky Area / incl. Labinsk','KR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KR-44','Leningradsky Area','KR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KR-45','Mostovsky Area','KR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KR-46','Novokubansky Area / incl. Novokubansk','KR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KR-47','Novopokrovsky Area','KR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KR-48','Otradnensky Area','KR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KR-49','Pavlovsky Area','KR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KR-50','Primorsko-Ahtarsky Area / incl. Primorsko-Ahtarsk','KR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KR-51','Seversky Area','KR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KR-52','Slavyansky Area / incl. Slavyansk-na-Kubani','KR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KR-53','Starominsky Area','KR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KR-54','Tbilissky Area','KR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KR-55','Temryuksky Area / incl. Temryuk','KR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KR-56','Timashevsky Area / incl. Timashevsk','KR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KR-57','Tikhoretsky Area / incl. Tihoretsk','KR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KR-58','Tuapsinsky Area / incl. Tuapse','KR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KR-59','Uspensky Area','KR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KR-60','Ust-Labinsky Area / incl. Ust-Labinsk','KR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KR-61','Shcherbinovsky Area','KR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KR-65','Novorossijsky District of Novorossiysk','KR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KR-66','Yuzhny District of Novorossiysk','KR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KR-67','Sirius Regional Center','KR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KR-68','Anapsky Area','KR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KS-01','Kostroma','KS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KS-02','Buj','KS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KS-03','Volgorechensk','KS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KS-04','Galich','KS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KS-05','Manturovo','KS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KS-06','Nerekhta','KS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KS-07','Neya','KS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KS-08','Sharja','KS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KS-09','Antropovsky Area','KS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KS-10','Buysky Area','KS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KS-11','Vohomsky Area','KS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KS-12','Galichsky Area','KS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KS-13','Kadyysky Area','KS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KS-14','Kologrivsky Area / incl. Kologriv','KS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KS-15','Kostromskoj Area','KS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KS-16','Krasnoselsky District','KS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KS-17','Makaryevsky Area / incl. Makarev','KS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KS-18','Manturovsky Area','KS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KS-19','Mezhevsky Area','KS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KS-20','Neysky Area','KS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KS-21','Nerehtsky Area','KS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KS-22','Oktyabrsky Area','KS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KS-23','Ostrovsky Area','KS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KS-24','Pavinsky Area','KS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KS-25','Parfenyevsky Area','KS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KS-26','Ponazyrevsky Area','KS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KS-27','Pyshchugsky Area','KS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KS-28','Soligalichsky Area / incl. Soligalich','KS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KS-29','Sudislavsky Area','KS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KS-30','Susaninsky Area','KS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KS-31','Chukhlomsky Area / incl. Chuhloma','KS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KS-32','Shar''Insky Area','KS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KU-01','Zheleznodorozhny District of Kursk','KU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KU-02','Seymsky District of Kursk','KU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KU-03','Centralny District of Kursk','KU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KU-04','Zheleznogorsk','KU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KU-05','Kurchatov','KU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KU-06','Lgov','KU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KU-07','Shchigry','KU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KU-08','Belovsky Area','KU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KU-09','Bolshesoldatsky Area','KU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KU-10','Glushkovsky Area','KU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KU-11','Gorshechensky Area','KU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KU-12','Dmitriyevsky Area / incl. Dmitriev-Lgovskiy','KU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KU-13','Zheleznogorsky Area','KU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KU-14','Zolotuhinsky Area','KU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KU-15','Kastorensky Area','KU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KU-16','Konyshevsky Area','KU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KU-17','Korenevsky Area','KU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KU-18','Kursky Area','KU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KU-19','Kurchatovsky Area','KU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KU-20','Lgovsky Area','KU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KU-21','Manturovsky Area','KU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KU-22','Medvensky Area','KU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KU-23','Oboyansky Area / incl. Oboyan','KU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KU-24','Oktyabrsky Area','KU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KU-25','Ponyrovsky Area','KU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KU-26','Pristensky Area','KU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KU-27','Rylsky Area / incl. Rylsk','KU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KU-28','Sovetsky Area','KU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KU-29','Solntsevsky Area','KU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KU-30','Sudzhansky Area / incl. Sudzha','KU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KU-31','Timsky Area','KU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KU-32','Fatezhsky Area / incl. Fatezh','KU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KU-33','Homutovsky Area','KU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KU-34','Cheremisinovsky Area','KU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'KU-35','Shchigrovsky Area','KU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'LO-16','Sosnovy Bor','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'LO-20','Boksitogorsky Area / incl. Boksitogorsk, Pikalyovo','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'LO-21','Volosovsky Area','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'LO-22','Volkhovsky Area / incl. Volkhov, Novaja Ladoga','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'LO-23','Vsevolozhsky Area / incl. Vsevolzhsk, Sertolovo','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'LO-24','Vyborgsky Area / incl. Vyborg, Vysotsk, Kamennogorsk','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'LO-25','Gatchina Area / incl. Gatchina, Communar','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'LO-26','Kingiseppsky Area / incl. Kingisepp, Ivangorod','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'LO-27','Kirishinsky Area / incl. Kirishi','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'LO-28','Kirovsky Area / incl. Kirovsk, Otradnoye','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'LO-29','Lodeynopolsky Area / incl. Lodeynoye Pole, Svirstroy','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'LO-30','Lomonosovsky Area','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'LO-31','Luzhsky Area / incl. Luga','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'LO-32','Podporozhsky Area / incl. Podporozhe','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'LO-33','Priozersky Area / incl. Priozersk','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'LO-34','Slantsevsky Area / incl. Slantsy','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'LO-35','Tikhvinsky Area / incl. Tikhvin','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'LO-36','Tosnensky Area / incl. Tosno, Lyuban, Nikolskoe','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'LP-01','Levoberezhny District of Lipetsk','LP','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'LP-02','Oktiabrsky District of Lipetsk','LP','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'LP-03','Pravoberezhny District of Lipetsk','LP','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'LP-04','Sovetsky District of Lipetsk','LP','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'LP-05','Yelets','LP','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'LP-06','Volovsky Area','LP','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'LP-07','Gryazinsky Area / incl. Grjazi','LP','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'LP-08','Dankovsky Area / incl. Dankov','LP','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'LP-09','Dobrinsky Area','LP','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'LP-10','Dobrovsky Area','LP','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'LP-11','Dolgorukovsky Area','LP','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'LP-12','Yeletsky Area','LP','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'LP-13','Zadonsky Area / incl. Zadonsk','LP','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'LP-14','Izmalkovsky Area','LP','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'LP-15','Krasninsky Area','LP','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'LP-16','Lebedyansky Area / incl. Lebedyan','LP','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'LP-17','Lev - Tolstovsky Area','LP','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'LP-18','Lipetsky Area','LP','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'LP-19','Stanovlyansky Area','LP','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'LP-20','Terbunsky Area','LP','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'LP-21','Usmansky Area / incl. Usman','LP','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'LP-22','Hlevensky Area','LP','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'LP-23','Chaplyginsky Area / incl. Chaplygin','LP','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'MA-01','Vostochny District of Moscow','MA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'MA-02','Zapadny District of Moscow','MA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'MA-03','City Of Zelenograd','MA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'MA-04','Severny District of Moscow','MA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'MA-05','Severovostochny District of Moscow','MA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'MA-06','Severozapadny District of Moscow','MA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'MA-07','Centralny District of Moscow','MA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'MA-08','Jugovostochny District of Moscow','MA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'MA-09','Jugozapadny District of Moscow','MA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'MA-10','Juzhny District of Moscow','MA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'MA-11','Novomoskovsky District of Moscow','MA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'MA-12','Troitsky District of Moscow','MA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'MD-01','Leninsky District of Saransk','MD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'MD-02','Oktyabrsky District of Saransk','MD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'MD-03','Proletarsky District of Saransk','MD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'MD-04','Kovylkino','MD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'MD-05','Ruzaevka','MD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'MD-06','Ardatovsky Area / incl. Ardatov','MD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'MD-07','Atyuryevsky Area','MD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'MD-08','Atyashevsky Area','MD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'MD-09','Bolshebereznikovsky Area','MD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'MD-10','Bolsheignatovsky Area','MD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'MD-11','Dubensky Area','MD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'MD-12','Elnikovsky Area','MD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'MD-13','Zubovo-Polyansky Area','MD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'MD-14','Insarsky Area / incl. Insar','MD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'MD-15','Ichalkovsky Area','MD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'MD-16','Kadoshkinsky Area','MD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'MD-17','Kovylkinsky Area / incl. Kovylkino','MD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'MD-18','Kochkurovsky Area','MD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'MD-19','Krasnoslobodsky Area / incl. Krasnoslobodsk','MD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'MD-20','Lyambirsky Area','MD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'MD-21','Romodanovsky Area','MD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'MD-22','Ruzayevsky Area / incl. Ruzaevka ','MD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'MD-23','Staroshajgovsky Area','MD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'MD-24','Temnikovsky Area / incl. Temnikov','MD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'MD-25','Tengushevsky Area','MD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'MD-26','Torbeevsky Area','MD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'MD-27','Chamzinsky Area','MD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'MO-01','City District of Balashikha','MO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'MO-02','Bronnitsy','MO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'MO-03','Zato Vlasiha','MO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'MO-04','Zato Voskhod','MO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'MO-06','City District of Dzerzhinsk','MO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'MO-08','Dolgoprudnyj','MO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'MO-09','City District of Domodedovo','MO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'MO-10','Dubna','MO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'MO-13','Zhukovsky','MO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'MO-14','Zato Zvezdny Gorodok','MO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'MO-15','City District of Zvenigorod','MO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'MO-16','City District of Ivanteevka','MO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'MO-21','City District of Kolomna','MO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'MO-22','City District of Korolev','MO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'MO-23','City District of Kotelniki','MO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'MO-24','City District of Lobnya','MO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'MO-25','City District of Lytkarino','MO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'MO-26','City District Losino-Petrovsky','MO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'MO-27','Krasnoarmejsk','MO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'MO-28','Zato Molodezhny','MO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'MO-34','Orekhovo-Zuyevo','MO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'MO-36','City District of Podolsk','MO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'MO-37','Protvino','MO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'MO-39','Pushchino','MO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'MO-41','City District of Reutov','MO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'MO-42','Roshal','MO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'MO-44','Serpukhov','MO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'MO-48','City District of Fryazino','MO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'MO-49','City District of Khimki','MO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'MO-50','City District of Chernogolovka','MO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'MO-52','Elektrogorsk','MO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'MO-54','City District of Electrostal','MO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'MO-56','City District of Krasnoznamensk','MO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'MO-58','Volokolamsk Area / incl. Volokolamsk','MO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'MO-59','Voskresensky Area / incl. Voskresensk','MO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'MO-60','Dmitrovsky Area / incl. Yakhroma, Dmitrov','MO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'MO-62','Egoryevsky Area / incl. Egoryevsk','MO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'MO-63','Zaraysky Area / incl. Zaraysk','MO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'MO-64','Istrinsky Area / incl. Dedovsk, Istra','MO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'MO-65','City District of Kashira','MO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'MO-66','Klinsky Area / incl. Vysokovsk, Klin','MO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'MO-67','Kolomensky Area','MO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'MO-68','Krasnogorsky Area / incl. Krasnogorsk','MO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'MO-69','Leninsky Area / incl. Vidnoe','MO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'MO-70','Lotoshinsky Area','MO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'MO-71','Lukhovitsky Area / incl. Lukhovitsy','MO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'MO-72','Lyubertsy Area / incl. Lyubertsy','MO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'MO-73','Mozhaisky Area / incl. Mozhaisk ','MO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'MO-74','Mytishchi Area / incl. Mytischi ','MO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'MO-75','Naro-Fominskiy Area / incl. Aprelevka, Naro-Fominsk','MO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'MO-76','Noginsky Area / incl. Elektrougli, Noginsk','MO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'MO-77','Odintsovsky Area / incl. Odintsovo','MO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'MO-78','Ozyorsky Area / incl. Ozery','MO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'MO-79','Orekhovo-Zuyevsky Area / incl. Drezna, Likino-Dulevo','MO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'MO-80','Pavlovo-Posadsky Area / incl. Pavlovsk Posad','MO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'MO-82','Pushkinsky Area / incl. Pushkino','MO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'MO-83','Ramensky Area / incl. Ramenskoje','MO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'MO-84','Ruzsky Area / incl. Ruza','MO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'MO-85','Sergiyevo-Posadsky Area / incl. Krasnozavodsk','MO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'MO-86','Serebrjano-Prudsky Area','MO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'MO-87','Serpukhovsky Area','MO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'MO-88','Solnechnogorsky Area / incl. Solnechnogorsk','MO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'MO-89','Stupinsky Area / incl. Stupino','MO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'MO-90','Taldomsky Area / incl. Taldom','MO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'MO-92','Shatursky Area / incl. Shatura','MO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'MO-93','Shakhovskoy Area','MO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'MO-94','Shchelkovo Area / incl. Shchelkovo','MO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'MO-95','Chekhovskt Area /incl.Chekhov','MO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'MR-01','Yoshkar-Ola','MR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'MR-02','Volzhsk','MR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'MR-03','Kozmodemyansk','MR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'MR-04','Volzhsky Area','MR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'MR-05','Gornomariysky Area','MR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'MR-06','Zvenigovsky Area / incl. Zvenigovo','MR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'MR-07','Kilemarsky Area','MR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'MR-08','Kuzhenersky Area','MR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'MR-09','Mari-Turekskiy Area','MR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'MR-10','Medvedevsky Area','MR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'MR-11','Morkinsky Area','MR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'MR-12','Novotor''yalskiy Area','MR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'MR-13','Orshansky Area','MR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'MR-14','Paranginsky Area','MR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'MR-15','Sernursky Area','MR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'MR-16','Sovetsky Area','MR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'MR-17','Yurinsky Area','MR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'MU-01','Leninsky District of Murmansk','MU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'MU-02','Oktiabrsky District of Murmansk','MU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'MU-03','Pervomajsky District of Murmansk','MU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'MU-04','Apatity','MU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'MU-05','Kandalaksha Area / incl. Kandalaksha','MU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'MU-06','Kirovsk','MU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'MU-07','Monchegorsk','MU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'MU-08','Olenegorsk','MU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'MU-09','Polyarnye Zori','MU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'MU-10','Zato Zaozersk','MU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'MU-11','Zato Ostrovnoj','MU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'MU-13','Zato Severomorsk','MU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'MU-16','Kovdorsky Area','MU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'MU-17','Kolsky Area / incl. Kola','MU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'MU-18','Lovozersky Area','MU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'MU-19','Pechengsky Area / incl. Zapolyarny','MU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'MU-20','Tersky Area','MU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'MU-21','Zato Vidyayevo','MU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'MU-22','Zato Aleksandrovsk / incl. Gadzhievo','MU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'NN-01','Avtozavodsky District of Nizhny Novgorod','NN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'NN-02','Kanavinsky District of Nizhny Novgorod','NN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'NN-03','Leninsky District of Nizhny Novgorod','NN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'NN-04','Moskovsky District of Nizhny Novgorod','NN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'NN-05','Nizhegorodsky District of Nizhny Novgorod','NN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'NN-06','Prioksky District of Nizhny Novgorod','NN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'NN-07','Sovetsky District of Nizhny Novgorod','NN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'NN-08','Sormovsky District of Nizhny Novgorod','NN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'NN-09','City District Arzamas','NN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'NN-15','City District Dzerzhinsk','NN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'NN-19','City District Sarov','NN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'NN-20','Ardatovsky Area','NN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'NN-21','Arzamas Area','NN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'NN-22','Balakhna Area / incl. Balakhna','NN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'NN-23','Bogorodsky Area / incl. Bogorodsk','NN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'NN-24','Bolsheboldinsky Area','NN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'NN-25','Bol''Shemurashkinsky Area','NN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'NN-26','City District of Bor','NN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'NN-27','Buturlinsky Area','NN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'NN-28','Vadsky Area','NN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'NN-29','Varnavinsky Area','NN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'NN-30','Vachsky Area','NN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'NN-31','Vetluzhsky Area / incl. Vetluga','NN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'NN-32','Voznesensky Area','NN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'NN-33','Volodarsky Area / incl. Volodarsk','NN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'NN-34','Vorotynsky Area','NN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'NN-35','Voskresensky Area','NN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'NN-36','City District Vyksa','NN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'NN-37','Gaginsky Area','NN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'NN-38','Gorodetsky Area / incl. Gorodets, Zavolzhe','NN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'NN-39','Dalnekonstantinovsky Area','NN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'NN-40','Diveevsky Area','NN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'NN-41','Knyagininsky Area / incl. Knyaginino','NN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'NN-42','Koverninsky Area','NN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'NN-43','Krasnobakovsky Area','NN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'NN-44','Krasnooktyabrsky Area','NN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'NN-45','Kstovsky Area / incl. Kstovo','NN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'NN-46','City District of Kulebaki','NN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'NN-47','Lukoyanovsky Area / incl. Lukoyanov','NN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'NN-48','Lyskovsky Area / incl. Lyskovo','NN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'NN-49','City District Navashinsky','NN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'NN-50','Pavlovsk Area / incl. Pavlovo, Vorsma, Gorbatov','NN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'NN-51','City District Pervomajsk','NN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'NN-52','Perevozsky Area','NN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'NN-53','Pilninsky Area','NN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'NN-54','Pochinkovsky Area','NN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'NN-55','City District Semenovsky','NN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'NN-56','Sergachsky Area / incl. Sergach','NN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'NN-57','Sechenovsky Area','NN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'NN-58','City District Sokolsky','NN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'NN-59','Sosnovsky Area','NN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'NN-60','Spassky Area','NN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'NN-61','Tonkinsky Area','NN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'NN-62','Tonshaevsky Area','NN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'NN-63','Urensky Area / incl. Uren','NN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'NN-64','City District of Chkalovsk','NN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'NN-65','Sharangsky Area','NN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'NN-66','Shatkovsky Area','NN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'NN-67','City District of Shakhunya','NN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'NO-01','Naryan-Mar','NO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'NO-02','Zapoljarny Area / incl. Iskatelej','NO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'NO-03','Amderma','NO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'NV-01','Novgorod','NV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'NV-02','Borovichi','NV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'NV-03','Staraja Russa','NV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'NV-04','Batetsky Area','NV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'NV-05','Borovichsky Area / incl. Borovichi','NV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'NV-06','Valdaisky Area / incl. Valdai','NV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'NV-07','Volotovsky Area','NV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'NV-08','Demyansky Area','NV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'NV-09','Krestetsky Area','NV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'NV-10','Lyubytinsky Area','NV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'NV-11','Malovishersky Area / incl. Malaya Vishera','NV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'NV-12','Marevsky Area','NV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'NV-13','Moshenskoy Area','NV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'NV-14','Novgorodsky Area','NV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'NV-15','Okulovsky Area / incl. Okulovka','NV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'NV-16','Parfinsky Area','NV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'NV-17','Pestovsky Area / incl. Pestovo','NV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'NV-18','Poddorsky Area','NV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'NV-19','Soletsky Area / incl. Soltsy','NV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'NV-20','Starorussky Area / incl. Staraja Russa','NV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'NV-21','Hvoyninsky Area','NV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'NV-22','Holmsky Area / incl. Holm','NV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'NV-23','Chudovsky Area / incl. Chudovo','NV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'NV-24','Shimsky Area','NV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'OR-01','Zheleznodorozhny District of Orel','OR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'OR-02','Zavodskoj Area of Orel','OR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'OR-03','Severny Area of Orel','OR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'OR-04','Sovetsky District of Orel','OR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'OR-05','Livny','OR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'OR-06','Mtsensk','OR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'OR-07','Bolkhovsky Area / incl. Bolhov','OR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'OR-08','Verhovsky Area','OR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'OR-09','Glazunovsky Area','OR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'OR-10','Dmitrovsky District / incl. Dmitrovsk-Orlovskiy','OR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'OR-11','Dolzhansky Area','OR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'OR-12','Zalegoshchensky Area','OR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'OR-13','Znamensky Area','OR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'OR-14','Kolpnjansky Area','OR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'OR-15','Korsakovsky Area','OR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'OR-16','Krasnozorensky Area','OR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'OR-17','Kromsky Area','OR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'OR-18','Livensky Area','OR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'OR-19','Maloarkhangel''sky Area / incl. Maloarhangelsk','OR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'OR-20','Mtsensky Area','OR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'OR-21','Novoderevenkovsky Area','OR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'OR-22','Novosilsky Area / incl. Novosil','OR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'OR-23','Orlovsky Area','OR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'OR-24','Pokrovsky Area','OR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'OR-25','Sverdlovsky Area','OR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'OR-26','Soskovsky Area','OR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'OR-27','Trosnyansky Area','OR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'OR-28','Uritsky Area','OR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'OR-29','Hotynetsky Area','OR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'OR-30','Shablykinsky Area','OR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'PE-01','Zheleznodorozhny District of Penza','PE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'PE-02','Leninsky District of Penza','PE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'PE-03','Oktyabrsky District of Penza','PE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'PE-04','Pervomajsky District of Penza','PE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'PE-06','Kuznetsk','PE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'PE-08','Zato Zarechny','PE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'PE-09','Bashmakovsky Area','PE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'PE-10','Bednodemyanovsky Area / incl. Bednodemyanovsk','PE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'PE-11','Bekovsky Area','PE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'PE-12','Belinsky Area / incl. Belinsky','PE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'PE-13','Bessonovsky Area','PE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'PE-14','Vadinsky Area','PE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'PE-15','Gorodishchensky Area / incl. Gorodische, Sursk','PE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'PE-16','Zemetchinsky Area','PE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'PE-17','Issinsky Area','PE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'PE-18','Kamensky Area / incl. Kamenka','PE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'PE-19','Kameshkirsky Area','PE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'PE-20','Kolyshleysky Area','PE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'PE-22','Kuznetsky Area','PE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'PE-23','Lopatinsky Area','PE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'PE-24','Luninsky Area','PE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'PE-25','Maloserdobinsky Area','PE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'PE-26','Mokshansky Area','PE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'PE-27','Narovchatsky Area','PE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'PE-28','Neverkinsky Area','PE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'PE-29','Nizhnelomovsky Area / incl. Nizhny Lomov','PE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'PE-30','Nikolsky Area / incl. Nikolsk','PE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'PE-31','Pachelmsky Area','PE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'PE-32','Penzensky Area','PE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'PE-33','Serdobsky Area / incl. Serdobsk','PE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'PE-34','Sosnovoborsk Area','PE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'PE-35','Tamalinsky Area','PE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'PE-36','Shemysheysky Area','PE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'PM-01','Dzerzhinsky District of Perm','PM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'PM-02','Industrialny District of Perm','PM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'PM-03','Kirovsky District of Perm','PM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'PM-04','Leninsky District of Perm','PM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'PM-05','Motovilikhinsky District of Perm','PM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'PM-06','Ordzhonikidzevsky District of Perm','PM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'PM-07','Sverdlovsky District of Perm','PM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'PM-08','Aleksandrovsky Area / incl. Aleksandrovsk','PM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'PM-09','Berezniki','PM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'PM-10','Gremyachinsky Area / incl. Gremyachinsk','PM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'PM-11','Gubakhinsky Area / incl. Gubakha','PM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'PM-13','Kizelovsky Area / incl. Kizel','PM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'PM-14','Krasnokamsky Area / incl. Krasnokamsk','PM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'PM-15','Kungur','PM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'PM-17','Solikamsk','PM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'PM-20','Bardymsky Area','PM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'PM-21','Berezovsky Area','PM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'PM-22','Bolshesosnovsky Area','PM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'PM-23','Vereshchaginsky Area / incl. Vereshchagino','PM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'PM-24','Gornozavodsky Area / incl. Gornozavodsk','PM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'PM-25','Elovsky Area','PM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'PM-26','Ilyinsky Area / incl. Chermoz','PM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'PM-27','Karagaysky Area','PM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'PM-28','Kishertsky Area','PM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'PM-29','Krasnovishersky District / incl. Krasnovishersk','PM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'PM-30','Kuyedinsky Area','PM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'PM-31','Kungursky Area','PM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'PM-32','Lysvensky Area / incl. Lysva','PM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'PM-33','Nytvensky Area / incl. Nytva','PM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'PM-34','Oktyabrsky Area','PM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'PM-35','Ordinsky Area','PM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'PM-36','Osinsky Area / incl. Osa','PM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'PM-37','Okhansky Area / incl. Okhansk','PM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'PM-38','Ochersky Area / incl. Ocher','PM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'PM-39','Permsky Area','PM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'PM-40','Sivinsky Area','PM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'PM-41','Solikamsky Area','PM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'PM-42','Suksunsky Area','PM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'PM-43','Uinsky Area','PM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'PM-44','Usolsky Area / incl. Usolye','PM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'PM-45','Chastinsky Area','PM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'PM-46','Cherdynsky Area / incl. Cherdyn','PM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'PM-47','Chernushinsky Area / incl. Chernushka','PM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'PM-48','Chusovskoy Area / incl. Chusovoy','PM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'PM-49','Tchaikovsky Area','PM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'PM-50','Dobryansky Area / incl. Dobryanka','PM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'PM-51','Zato Zvezdny ','PM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'PM-52','Kudymkar','PM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'PM-53','Gaynsky Area','PM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'PM-54','Kosinsky Area','PM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'PM-55','Kochevsky Area','PM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'PM-56','Kudymkarsky Area','PM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'PM-57','Yurlinsky Area','PM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'PM-58','Yusvinsky Area','PM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'PS-01','Pskov','PS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'PS-02','Velikiye Luki','PS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'PS-03','Bezhanitsky Area','PS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'PS-04','Velikiye Luki Area','PS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'PS-05','Gdovsky Area / incl. Gdov','PS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'PS-06','Dedovichsky Area','PS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'PS-07','Dnovsky Area / incl. Dno','PS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'PS-08','Krasnogorodsky Area','PS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'PS-09','Kunyinsky Area','PS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'PS-10','Loknjansky Area','PS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'PS-11','Nevelsky Area / incl. Nevel','PS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'PS-12','Novorzhevsky Area / incl. Novorzhev','PS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'PS-13','Novosokolnichesky Area / incl. Novosokolniki','PS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'PS-14','Opochetsky Area / incl. Opochka','PS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'PS-15','Ostrovsky Area / incl. Ostrov','PS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'PS-16','Palkinsky Area','PS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'PS-17','Pechorsky Area / incl. Pechora','PS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'PS-18','Pljussky Area','PS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'PS-19','Porkhovsky Area / incl. Porhov','PS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'PS-20','Pskovsky Area','PS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'PS-21','Pustoshkinsky Area / incl. Pustoshka','PS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'PS-22','Pushkinogorsky Area','PS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'PS-23','Pytalovsky Area / incl. Pytalovo','PS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'PS-24','Sebezhsky Area / incl. Sebezh','PS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'PS-25','Strugo-Krasnenskiy Area','PS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'PS-26','Usvyatsky Area','PS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'RA-01','Zheleznodorozhny District of Ryazan','RA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'RA-02','Moskovsky District of Ryazan','RA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'RA-03','Oktiabrsky District of Ryazan','RA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'RA-04','Sovetsky District of Ryazan','RA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'RA-05','Kasimov','RA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'RA-06','Sasovo','RA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'RA-07','Skopin','RA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'RA-08','Ermishinsky Area','RA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'RA-09','Zaharovsky Area','RA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'RA-10','Kadomsky Area','RA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'RA-11','Kasimovsky Area','RA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'RA-12','Klepikovsky Area / incl. Spas-Klepiki','RA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'RA-13','Korablinsky Area / incl. Korablino','RA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'RA-14','Miloslavsky Area','RA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'RA-15','Mikhaylovsky Area / incl. Mikhaylov','RA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'RA-16','Aleksandro-Nevskiy Area','RA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'RA-17','Pitelinsky Area','RA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'RA-18','Pronsky Area / incl. Novomichurinsk','RA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'RA-19','Putjatinsky Area','RA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'RA-20','Rybnovsky Area / incl. Rybnoe','RA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'RA-21','Ryazhsky Area / incl. Ryazhsk','RA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'RA-22','Ryazansky Area','RA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'RA-23','Sapozhkovsky Area','RA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'RA-24','Saraevsky Area','RA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'RA-25','Sasovsky Area','RA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'RA-26','Skopinsky Area','RA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'RA-27','Spassky Area / incl. Spassk-Ryazanskiy','RA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'RA-28','Starozhilovsky Area','RA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'RA-29','Uholovsky Area','RA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'RA-30','Chuchkovsky Area','RA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'RA-31','Shatsky Area / incl. Shatsk','RA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'RA-32','Shilovsky Area','RA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'RO-01','Voroshilovsky District of Rostov-on-Don','RO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'RO-02','Zheleznodorozhny District of Rostov-on-Don','RO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'RO-03','Kirovsky District of Rostov-on-Don','RO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'RO-04','Leninsky District of Rostov-on-Don','RO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'RO-05','Oktyabrsky District of Rostov-on-Don','RO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'RO-06','Pervomajsky District of Rostov-on-Don','RO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'RO-07','Proletarsky District of Rostov-on-Don','RO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'RO-08','Sovetsky District of Rostov-on-Don','RO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'RO-09','Azov','RO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'RO-10','Batajsk','RO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'RO-12','Volgodonsk','RO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'RO-13','Gukovo','RO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'RO-14','Donetsk','RO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'RO-15','Zverevo','RO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'RO-16','Kamensk-Shahtinsky','RO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'RO-19','Novocherkassk','RO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'RO-20','Novoshakhtinsk','RO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'RO-22','Taganrog','RO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'RO-23','Shahty','RO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'RO-24','Azovsky Area','RO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'RO-25','Aksaysky Area / incl. Aksay','RO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'RO-26','Bagaevsky Area','RO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'RO-27','Belokalitvinsky Area / incl. Belaja Kalitva','RO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'RO-28','Bokovsky Area','RO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'RO-29','Verhnedonskoj Area','RO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'RO-30','Veselovsky Area','RO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'RO-31','Volgodonsky Area','RO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'RO-32','Dubovsky Area','RO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'RO-33','Egorlyksky Area','RO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'RO-34','Zavetinsky Area','RO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'RO-35','Zernogradsky Area / incl. Zernograd','RO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'RO-36','Zimovnikovsky Area','RO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'RO-37','Kagalnitsky Area','RO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'RO-38','Kamensky Area','RO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'RO-39','Kasharsky Area','RO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'RO-40','Konstantinovsky Area / incl. Konstantinovsk','RO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'RO-41','Krasnosulinsky Area / incl. Krasny Sulin','RO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'RO-42','Kuibyshevsky Area','RO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'RO-43','Martynovsky Area','RO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'RO-44','Matveevo-Kurganskiy Area','RO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'RO-45','Millerovsky Area / incl. Millerovo','RO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'RO-46','Milyutinsky Area','RO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'RO-47','Morozovsky Area / incl. Morozovsk','RO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'RO-48','Myasnikovsky Area','RO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'RO-49','Neklinovsky Area','RO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'RO-50','Oblivsky Area','RO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'RO-51','Oktyabrsky Area','RO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'RO-52','Orlovsky Area','RO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'RO-53','Peschanokopsky Area','RO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'RO-54','Proletarsky Area / incl. Proletarsk','RO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'RO-55','Remontnensky Area','RO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'RO-56','Rodionovo-Nesvetayskiy Area','RO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'RO-57','Salsky Area / incl. Salsk','RO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'RO-58','Semikarakorsky Area / incl. Semikarakorsk','RO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'RO-59','Sovetsky Area ','RO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'RO-60','Tarasovsky Area','RO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'RO-61','Tatsinsky Area','RO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'RO-62','Ust-Donetskiy Area','RO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'RO-63','Tselinsky Area','RO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'RO-64','Tsimlyansk Area / incl. Tsimlyansk','RO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'RO-65','Chertkovsky Area','RO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'RO-66','Sholohovsky Area','RO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'SA-01','Volzhsky District of Saratov','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'SA-02','Zavodskoj District of Saratov','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'SA-03','Kirovsky District of Saratov','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'SA-04','Leninsky District of Saratov','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'SA-05','Oktyabrsky District of Saratov','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'SA-06','Frunzensky District of Saratov','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'SA-17','Shikhany','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'SA-19','Aleksandrovo-Gayskiy Area','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'SA-20','Arkadaksky Area / incl. Arkadak','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'SA-21','Atkarsky Area','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'SA-22','Bazarno-Karabulaksky Area','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'SA-23','Balakovsky Area','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'SA-24','Balashovsky Area','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'SA-25','Baltaysky Area','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'SA-26','Volsky Area','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'SA-27','Voskresensky Area','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'SA-28','Dergachevsky Area','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'SA-29','Duhovnitsky Area','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'SA-30','Ekaterinovsky Area','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'SA-31','Ershovsky Area / incl. Ershov','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'SA-32','Ivanteevsky Area','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'SA-33','Kalininsky Area / incl. Kalininsk','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'SA-34','Krasnoarmejsky Area','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'SA-35','Krasnokutsky Area / incl. Krasny Kut','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'SA-36','Krasnopartizansky Area','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'SA-37','Lysogorsky Area','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'SA-38','Markssky Area','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'SA-39','Novoburassky Area','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'SA-40','Novouzensky Area / incl. Novouzensk','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'SA-41','Ozinsky Area','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'SA-42','Perelyubsky Area','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'SA-43','Petrovsky Area','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'SA-44','Pitersky Area','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'SA-45','Pugachevsky Area','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'SA-46','Rovensky Area','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'SA-47','Romanovsky Area','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'SA-48','Rtishchevsky Area','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'SA-49','Samoylovsky Area','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'SA-50','Saratovsky Area','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'SA-51','Sovetsky Area ','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'SA-52','Tatishchevsky Area','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'SA-53','Turkovsky Area','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'SA-54','Fedorovsky Area','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'SA-55','Hvalynsky Area','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'SA-56','Engel''Ssky Area','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'SA-57','Zato Mihaylovskiy','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'SA-58','Zato Svetly','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'SM-01','Zadneprovsky District of Smolensk','SM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'SM-02','Leninsky District of Smolensk','SM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'SM-03','Industrialny District of Smolensk','SM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'SM-04','Desnogorsk','SM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'SM-05','Velizhsky Area / incl. Velizh','SM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'SM-06','Vyazemsky Area / incl. Vyazma','SM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'SM-07','Gagarinsky Area / incl. Gagarin','SM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'SM-08','Glinkovsky Area','SM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'SM-09','Demidovsky Area / incl. Demidov','SM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'SM-10','Dorogobuzhsky Area / incl. Dorogobuzh','SM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'SM-11','Dukhovshchinsky Area / incl. Duhovshchina','SM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'SM-12','Yelninsky Area / incl. Yelnya','SM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'SM-13','Ershichsky Area','SM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'SM-14','Kardymovsky Area','SM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'SM-15','Krasninsky Area','SM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'SM-16','Monastyrshchinsky Area','SM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'SM-17','Novoduginsky Area','SM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'SM-18','Pochinkovsky Area / incl. Pochinok','SM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'SM-19','Roslavlsky Area / incl. Roslavl','SM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'SM-20','Rudnyansky Area / incl. Rudnya','SM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'SM-21','Safonovsky Area / incl. Safonovo','SM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'SM-22','Smolensky Area','SM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'SM-23','Sychevsky Area / incl. Sychevka','SM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'SM-24','Temkinsky Area','SM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'SM-25','Ugransky Area','SM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'SM-26','Hislavichsky Area','SM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'SM-27','Holm-Zhirkovskiy Area','SM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'SM-28','Shumyachsky Area','SM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'SM-29','Yartsevsky Area / incl. Yartsevo','SM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'SO-01','Zaterechny District of Vladikavkaz','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'SO-02','Iristonsky District of Vladikavkaz','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'SO-03','Industrialny District of Vladikavkaz','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'SO-04','Severozapadny District of Vladikavkaz','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'SO-05','Alagirsky Area / incl. Alagir','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'SO-06','Ardonsky Area / incl. Ardon','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'SO-07','Digorsky Area / incl. Digora','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'SO-08','Irafsky Area','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'SO-09','Kirovsky Area','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'SO-10','Mozdoksky Area / incl. Mozdok','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'SO-11','Pravoberezhny Area / incl. Beslan','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'SO-12','Prigorodny Area','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'SP-01','Admiralteisky District St. Petersburg','SP','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'SP-02','Vasileostrovsky District St. Petersburg','SP','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'SP-03','Vyborgsky District St. Petersburg','SP','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'SP-04','Kalininsky District St. Petersburg','SP','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'SP-05','Kirovsky District St. Petersburg','SP','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'SP-06','Kolpino District St. Petersburg','SP','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'SP-07','Krasnogvardeisky District St. Petersburg','SP','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'SP-08','Krasnoselsky District St. Petersburg','SP','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'SP-09','Kronshtadt District St. Petersburg','SP','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'SP-10','Kurortny District St. Petersburg','SP','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'SP-12','Primorsky District St. Petersburg','SP','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'SP-13','Moskovsky District St. Petersburg','SP','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'SP-14','Nevsky District St. Petersburg','SP','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'SP-16','Petrogradsky District St. Petersburg','SP','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'SP-17','Petrodvorets District St. Petersburg','SP','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'SP-18','Pushkin District St. Petersburg','SP','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'SP-19','Frunzensky District St. Petersburg','SP','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'SP-20','Centralny District St. Petersburg','SP','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'SR-01','Zheleznodorozhny District of Samara','SR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'SR-02','Kirovsky District of Samara','SR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'SR-03','Krasnoglinsky District of Samara','SR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'SR-04','Kuibyshevsky District of Samara','SR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'SR-05','Leninsky District of Samara','SR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'SR-06','Oktyabrsky District of Samara','SR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'SR-07','Industrialny District of Samara','SR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'SR-08','Samarsky District of Samara','SR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'SR-09','Sovetsky District of Samara','SR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'SR-10','Avtozavodsky District of Tolyatti','SR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'SR-11','Komsomolsky District of Tolyatti','SR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'SR-12','Centralny District of Tolyatti','SR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'SR-13','Zhigulyovsk','SR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'SR-14','Kinel','SR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'SR-15','Novokujbyshevsk','SR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'SR-16','Oktyabrsk','SR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'SR-17','Otradny','SR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'SR-18','Pokhvistnevo','SR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'SR-19','Syzran','SR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'SR-20','Chapaevsk','SR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'SR-21','Alexeyevsky Area','SR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'SR-22','Bezenchuksky Area','SR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'SR-23','Bogatovsky Area','SR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'SR-24','Bolsheglushitsky Area','SR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'SR-25','Bolshechernigovsky Area','SR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'SR-26','Borsky Area','SR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'SR-27','Volzhsky Area','SR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'SR-28','Elhovsky Area','SR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'SR-29','Isaklinsky Area','SR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'SR-30','Kamyshlinsky Area','SR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'SR-31','Kinelsky Area','SR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'SR-32','Kinel-Cherkasskiy Area','SR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'SR-33','Klyavlinsky Area','SR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'SR-34','Koshkinsky Area','SR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'SR-35','Krasnoarmeisky Area','SR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'SR-36','Krasnoyarsky Area','SR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'SR-37','Neftegorsky Area / incl. Neftegorsk','SR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'SR-38','Pestravsky Area','SR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'SR-39','Pohvistnevsky Area','SR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'SR-40','Privolzhsky Area','SR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'SR-41','Sergiyevsky Area','SR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'SR-42','Stavropolsky Area','SR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'SR-43','Syzransky Area','SR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'SR-44','Hvorostyansky Area','SR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'SR-45','Chelno-Vershinsky Area','SR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'SR-46','Shentalinsky Area','SR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'SR-47','Shigonsky Area','SR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'ST-01','Leninsky District of Stavropol','ST','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'ST-02','Oktyabrsky District of Stavropol','ST','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'ST-03','Industrialny Region of Stavropol','ST','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'ST-04','Budyonovsk','ST','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'ST-05','Georgievsk','ST','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'ST-06','Essentuki','ST','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'ST-07','Zheleznovodsk','ST','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'ST-08','Kislovodsk','ST','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'ST-09','Lermontov','ST','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'ST-10','Mineralnye Vody','ST','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'ST-11','Nevinnomyssk','ST','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'ST-12','Pyatigorsk','ST','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'ST-13','Aleksandrovsky Area','ST','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'ST-14','Andropovsky Area','ST','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'ST-15','Apanasenkovsky Area','ST','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'ST-16','Arzgirsky Area','ST','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'ST-17','Blagodarnensky Area / incl. Blagodarny','ST','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'ST-18','Budennovsky Area','ST','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'ST-19','Georgievsky Area','ST','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'ST-20','Grachevsky Area','ST','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'ST-21','Izobilnensky Area / incl. Izobilny','ST','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'ST-22','Ipatovsky Area / incl. Ipatovo','ST','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'ST-23','Kirovsky Area / incl. Novopavlovsk','ST','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'ST-24','Kochubeevsky Area','ST','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'ST-25','Krasnogvardejsky Area','ST','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'ST-26','Kursky Area','ST','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'ST-27','Levokumsky Area','ST','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'ST-28','Mineralnye Vody City District','ST','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'ST-29','Neftekumsky Area / incl. Neftekumsk','ST','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'ST-30','Novoaleksandrovsky Area / incl. Novoaleksandrovsk','ST','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'ST-31','Novoselitsky Area','ST','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'ST-32','Petrovsky Area / incl. Svetlograd','ST','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'ST-33','Predgorny Area','ST','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'ST-34','Sovetsky Area  / incl. Zelenokumsk','ST','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'ST-35','Stepnovsky Area','ST','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'ST-36','Trunovsky Area','ST','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'ST-37','Turkmensky Area','ST','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'ST-38','Shpakovsky Area','ST','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'TA-01','Aviastroitelny District of Kazan','TA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'TA-02','Vakhitovsky District of Kazan','TA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'TA-03','Kirovsky District of Kazan','TA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'TA-04','Moskovsky District of Kazan','TA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'TA-05','Novo-Savinovsky District of Kazan','TA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'TA-06','Privolzhsky District of Kazan','TA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'TA-07','Sovetsky District of Kazan','TA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'TA-08','Aznakaevo','TA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'TA-09','Almet`evsk','TA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'TA-10','Bavly','TA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'TA-11','Bugult`ma','TA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'TA-12','Elabuga','TA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'TA-13','Zainsk','TA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'TA-14','Zelenodolsk','TA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'TA-15','Leninogorsk','TA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'TA-16','Naberezhny Chelny','TA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'TA-17','Nizhnekamsk','TA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'TA-18','Nurlat','TA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'TA-19','Chistopol','TA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'TA-20','Agryzsky Area / incl. Agryz','TA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'TA-21','Aznakayevsky Area / incl. Aznakaevo','TA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'TA-22','Aksubayevsky Area','TA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'TA-23','Aktanyshsky Area','TA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'TA-24','Alexeyevsky Area','TA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'TA-25','Alkeevsky Area','TA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'TA-26','Almetyevsk Area / incl. Almetyevsk','TA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'TA-27','Apastovsky Area','TA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'TA-28','Arsky Area','TA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'TA-29','Atninsky Area','TA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'TA-30','Bavlinsky Area / incl. Bavly','TA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'TA-31','Baltasinsky Area','TA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'TA-32','Bugul`minsky Area / incl. Bugulma','TA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'TA-33','Buinsky Area / incl. Buinsk','TA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'TA-34','Verhneuslonsky Area','TA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'TA-35','Vysokogorsky Area','TA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'TA-36','Drozhzhanovsky Area','TA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'TA-37','Elabuzhsky Area / incl. Elabuga','TA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'TA-38','Zainsky Area / incl. Zainsk','TA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'TA-39','Zelenodolsky Area / incl. Zelenodolsk','TA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'TA-40','Kaybitsky Area','TA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'TA-41','Kamsko-Ustinskiy Area','TA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'TA-42','Kukmorsky Area','TA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'TA-43','Laishevsky Area','TA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'TA-44','Leninogorsky Area / incl. Leninogorsk','TA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'TA-45','Mamadyshsky Area / incl. Mamadysh','TA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'TA-46','Mendeleevsky Area / incl. Mendeleevsk','TA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'TA-47','Menzelinsky Area / incl. Menzelinsk','TA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'TA-48','Musljumovsky Area','TA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'TA-49','Nizhnekamsky Area / incl. Nizhnekamsk','TA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'TA-50','Novosheshminsky Area','TA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'TA-51','Nurlatsky Area / incl. Nurlat','TA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'TA-52','Pestrechinsky Area','TA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'TA-53','Rybno-Slobodsky Area','TA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'TA-54','Sabinsky Area','TA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'TA-55','Sarmanovsky Area','TA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'TA-56','Spassky Area / incl. Bolgar','TA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'TA-57','Tetyushsky Area / incl. Tetyushi','TA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'TA-58','Tukayevsky Area','TA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'TA-59','Tyulyachinsky Area','TA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'TA-60','Cheremshansky Area','TA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'TA-61','Chistopolsky Area / incl. Chistopol','TA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'TA-62','Yutazinsky Area','TA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'TA-63','Buinsk','TA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'TA-64','Avtozavodskoj District of Naberezhnye Chelny','TA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'TA-65','Komsomolsky District of Naberezhnye Chelny','TA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'TA-66','Centralny District of Naberezhnye Chelny','TA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'TB-01','Leninsky District of Tambov','TB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'TB-02','Oktyabrsky District of Tambov','TB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'TB-03','Sovetsky District of Tambov','TB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'TB-04','Kirsanov','TB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'TB-05','Kotovsk','TB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'TB-06','Michurinsk','TB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'TB-07','Morshansk','TB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'TB-08','Rasskazovo','TB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'TB-09','Uvarovo','TB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'TB-10','Bondarsky Area','TB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'TB-11','Gavrilovsky Area','TB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'TB-12','Zherdevsky Area / incl. Zherdevka','TB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'TB-13','Znamensky Area','TB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'TB-14','Inzhavinsky Area','TB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'TB-15','Kirsanovsky Area','TB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'TB-16','Michurinsky Area','TB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'TB-17','Mordovsky Area','TB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'TB-18','Morshansky Area','TB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'TB-19','Muchkapsky Area','TB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'TB-20','Nikiforovsky Area','TB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'TB-21','Pervomajsky Area','TB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'TB-22','Petrovsky Area','TB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'TB-23','Pichayevsky Area','TB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'TB-24','Rasskazovsky Area','TB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'TB-25','Rzhaksinsky Area','TB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'TB-26','Sampursky Area','TB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'TB-27','Sosnovsky Area','TB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'TB-28','Staroyuryevsky Area','TB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'TB-29','Tambovsky Area','TB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'TB-30','Tokarevsky Area','TB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'TB-31','Uvarovsky Area','TB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'TB-32','Umetsky Area','TB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'TL-01','Zarechensky District of Tula','TL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'TL-02','Privokzal''ny District of Tula','TL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'TL-03','Proletarsky District of Tula','TL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'TL-04','Sovietsky District of Tula','TL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'TL-05','Centralny District of Tula','TL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'TL-08','Donskoy / incl. Severo-Zadonsk','TL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'TL-11','City District Novomoskovsk','TL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'TL-14','City Aleksin','TL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'TL-15','Arsenyevsky Area','TL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'TL-16','Belevsky Area / incl. Belev','TL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'TL-17','Bogoroditsky Area / incl. Bogoroditsk','TL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'TL-18','Venevsky Area / incl. Beheb','TL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'TL-19','Volovsky Area','TL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'TL-20','Dubenskii Area','TL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'TL-21','City Efremov','TL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'TL-22','Zaoksky Area','TL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'TL-23','Kamensky Area','TL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'TL-24','Kimovsky Area / incl. Kimovsk','TL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'TL-25','Kireevsky Area / incl. Kireevsk, Bolohovo, Lipki','TL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'TL-26','Kurkinsky Area','TL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'TL-27','Leninsky Area','TL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'TL-29','Odoevsky Area','TL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'TL-30','Plavsky Area / incl. Plavsk','TL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'TL-31','Suvorovsky Area / incl. Suvorov, Chekalin','TL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'TL-32','Teplo - Ogarevsky Area','TL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'TL-33','Uzlovsky Area / incl. Uzlovaya','TL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'TL-34','Chernsky Area','TL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'TL-35','Shchekinsky Area / incl. Shchekino, Sovetsk','TL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'TL-36','Yasnogorsky Area / incl. Yasnogorsk','TL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'TL-37','Novogurovsky','TL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'TL-38','Slavny','TL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'TV-01','Zavolzhsky District of Tver','TV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'TV-02','Moscowsky District of Tver','TV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'TV-03','Proletarsky District of Tver','TV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'TV-04','Centralny District of Tver','TV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'TV-07','Vyshny Volochek','TV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'TV-09','Kimry','TV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'TV-13','Rzhev','TV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'TV-14','Torzhok','TV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'TV-16','Andreapolsky Area / incl. Andreapol','TV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'TV-17','Bezhetsky Area / incl. Bezhetsk','TV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'TV-18','Belsky Area / incl. White','TV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'TV-19','Bologovsky Area / incl. Bologoye','TV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'TV-20','Vesyegonsky Area / incl. Vesegonsk','TV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'TV-21','Vyshnevolotsky Area','TV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'TV-22','Zharkovsky Area','TV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'TV-23','Zapadnodvinsky Area / incl. Zapadnaja Dvina','TV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'TV-24','Zubtsovsky Area / incl. Zubtsov','TV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'TV-25','Kalininsky Area','TV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'TV-26','Kalyazinsky Area / incl. Kalyazin ','TV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'TV-27','Kashinsky Area / incl. Kashin','TV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'TV-28','Kesovogorsky Area','TV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'TV-29','Kimrsky Area','TV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'TV-30','Konakovsky Area / incl. Konakovo','TV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'TV-31','Krasnokholmsky Area / incl. Krasnyj Holm','TV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'TV-32','Kuvshinovsky Area / incl. Kuvshinovo','TV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'TV-33','Lesnoj Area','TV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'TV-34','Likhoslavlsky Area / incl. Lihoslavl','TV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'TV-35','Maksatihinsky Area','TV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'TV-36','Molokovsky Area','TV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'TV-37','Nelidovsky Area / incl. Nelidovo','TV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'TV-38','Oleninsky Area','TV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'TV-39','Ostashkovsky Area / incl. Ostashkov','TV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'TV-40','Penovsky Area','TV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'TV-41','Rameshkovsky Area','TV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'TV-42','Rzhevsky Area','TV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'TV-43','Sandovsky Area','TV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'TV-44','Selizharovsky Area','TV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'TV-45','Sonkovsky Area','TV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'TV-46','Spirovsky Area','TV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'TV-47','Staritsky Area / incl. Staritsa','TV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'TV-48','Torzhoksky Area','TV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'TV-49','Toropetsky Area / incl. Toropets','TV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'TV-50','Udomlya Area / incl. Udomlya','TV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'TV-51','Firovsky Area','TV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'TV-52','Zato Solnechny','TV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'TV-53','Zato Ozerny','TV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'UD-01','Industrialny District of Izhevsk','UD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'UD-02','Leninsky District of Izhevsk','UD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'UD-03','Oktyabrsky District of Izhevsk','UD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'UD-04','Pervomajsky District of Izhevsk','UD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'UD-05','Ustinovsky District of Izhevsk','UD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'UD-06','Votkinsk','UD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'UD-07','Glazov','UD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'UD-08','Mozhga','UD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'UD-09','Sarapul','UD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'UD-10','Alnashsky Area','UD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'UD-11','Balezinsky Area','UD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'UD-12','Vavozhsky Area','UD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'UD-13','Votkinsky Area','UD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'UD-14','Glazovsky Area','UD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'UD-15','Grakhovsky Area','UD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'UD-16','Debessky Area','UD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'UD-17','Zavyalovsky Area','UD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'UD-18','Igrinsky Area','UD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'UD-19','Kambarsky Area / incl. Kambarka','UD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'UD-20','Kezsky Area','UD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'UD-21','Kiznersky Area','UD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'UD-22','Kiyasovsky Area','UD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'UD-23','Krasnogorsky Area','UD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'UD-24','Malopurginsky Area','UD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'UD-25','Mozhginsky Area','UD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'UD-26','Sarapulsky Area','UD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'UD-27','Seltinsky Area','UD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'UD-28','Syumsinsky Area','UD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'UD-29','Uvinsky Area','UD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'UD-30','Sharkansky Area','UD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'UD-31','Yukamensky Area','UD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'UD-32','Yakshur-Bodinskiy Area','UD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'UD-33','Yarsky Area','UD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'UD-34','Karakulinsky Area','UD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'UL-01','Zheleznodorozhny District of Ulyanovsk','UL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'UL-02','Zavolzhsky District of Ulyanovsk','UL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'UL-03','Zasviyazhsky District of Ulyanovsk','UL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'UL-04','Leninsky District of Ulyanovsk','UL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'UL-06','Dimitrovgrad','UL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'UL-07','Bazarnosyzgansky Area','UL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'UL-08','Baryshsky Area / incl. Barysh','UL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'UL-09','Veshkaymsky Area','UL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'UL-10','Inzensky Area / incl. Inza','UL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'UL-11','Karsunsky Area','UL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'UL-12','Kuzovatovsky Area','UL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'UL-13','Majnsky Area','UL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'UL-14','Melekessky Area','UL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'UL-15','Nikolaevsky Area','UL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'UL-16','Novomalyklinsky Area','UL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'UL-17','Novospassky Area','UL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'UL-18','Pavlovsky Area','UL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'UL-19','Radishchevsky Area','UL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'UL-20','Sengileevsky Area / incl. Sengiley','UL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'UL-21','Starokulatkinsky Area','UL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'UL-22','Staromajnsky Area','UL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'UL-23','Sursky Area','UL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'UL-24','Terengulsky Area','UL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'UL-25','Ulyanovsky Area','UL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'UL-26','Tsilninsky Area','UL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'UL-27','Cherdaklinsky Area','UL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'UL-28','Novoulyanovsk','UL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'VG-01','Voroshilovsky District of Volgograd','VG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'VG-02','Dzerzhinsky District of Volgograd','VG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'VG-03','Kirovsky District of Volgograd','VG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'VG-04','Krasnoarmeisky District of Volgograd','VG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'VG-05','Krasnooktyabrsky District of Volgograd','VG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'VG-06','Sovetsky District of Volgograd','VG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'VG-07','Traktorozavodsky District of Volgograd','VG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'VG-08','Centralny District of Volgograd','VG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'VG-09','Volzhsky','VG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'VG-10','Kamyshin','VG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'VG-11','City District City OF Mikhailovka','VG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'VG-12','Uryupinsk','VG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'VG-13','Frolovo','VG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'VG-14','Alexeyevsky Area','VG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'VG-15','Bykovsky Area','VG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'VG-16','Gorodishchensky Area','VG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'VG-17','Danilovsky Area','VG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'VG-18','Dubovsky Area / incl. Dubovka','VG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'VG-19','Elansky Area','VG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'VG-20','Zhirnovsky Area / incl. Zhirnovsk','VG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'VG-21','Ilovlinsky Area','VG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'VG-22','Kalachevsky Area / incl. Kalach-na-Donu','VG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'VG-23','Kamyshinsky Area / incl. Petrov Val','VG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'VG-24','Kikvidzensky Area','VG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'VG-25','Kletsky Area','VG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'VG-26','Kotelnikovsky Area / incl. Kotelnikovo','VG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'VG-27','Kotovsky Area / incl. Kotovo','VG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'VG-28','Kumylzhensky Area','VG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'VG-29','Leninsky Area / incl. Leninsk','VG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'VG-30','Mikhailovsky Area','VG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'VG-31','Nekhayevsky Area','VG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'VG-32','Nikolaevsky Area / incl. Nikolaevsk','VG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'VG-33','Novoanninsky Area / incl. Novoanninskiy','VG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'VG-34','Novonikolaevsky Area','VG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'VG-35','Oktyabrsky Area','VG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'VG-36','Olkhovsky Area','VG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'VG-37','Pallasovsky Area / incl. Pallasovka','VG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'VG-38','Rudnyansky Area','VG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'VG-39','Svetlojarsky Area','VG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'VG-40','Serafimovichsky Area / incl. Serafimovich','VG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'VG-41','Sredneakhtubinsky Area / incl. Krasnoslobodsk','VG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'VG-42','Staropoltavsky Area','VG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'VG-43','Surovikinsky Area / incl. Surovikino','VG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'VG-44','Uryupinsk Area','VG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'VG-45','Frolovsky Area','VG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'VG-46','Chernyshkovsky Area','VG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'VL-01','Leninsky District of Vladimir ','VL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'VL-02','Oktyabrsky District of Vladimir','VL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'VL-03','Frunzensky District of Vladimir','VL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'VL-06','Gus-Khrustalny','VL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'VL-07','Kovrov','VL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'VL-09','Murom','VL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'VL-12','Raduzhny','VL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'VL-13','Aleksandrovsky Area / incl. Alexandrov, Karabanovo, Strunino','VL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'VL-14','Vyaznikovsky Area /incl. of Vyazniki','VL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'VL-15','Gorokhovetsky Area / incl. Gorohovets','VL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'VL-16','Gus-Khrustalny Area','VL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'VL-17','Kameshkovsky Area / incl. Kameshkovo','VL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'VL-18','Kirzhachsky Area / incl. Kirzhach','VL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'VL-19','Kovrovsky Area','VL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'VL-20','Kolchuginsky Area / incl. Kolchugino','VL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'VL-21','Melenkovsky Area / incl. Melenki','VL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'VL-22','Muromsky Area','VL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'VL-23','Petushinsky Area / incl. Petushki, Kosterevo, Pokrov','VL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'VL-24','Selivanovsky Area','VL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'VL-25','Sobinsky Area / incl. Sobinka, Lakinsk','VL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'VL-26','Sudogodsky Area / incl. Sudogda','VL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'VL-27','Suzdalsky Area / incl. Suzdal','VL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'VL-28','Yurev-Polskiy Area / incl. Yurev-Polskiy','VL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'VO-01','Vologda','VO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'VO-02','Velikiy Ustyug','VO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'VO-03','Sokol','VO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'VO-04','Cherepovets','VO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'VO-05','Babayevsky Area / incl. Babayevo','VO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'VO-06','Babushkinsky Area','VO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'VO-07','Belozersky Area / incl. Belozersk','VO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'VO-08','Vashkinsky Area','VO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'VO-09','Velikoustyzhsky Area / incl. Veliky Ustyug, Krasavino','VO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'VO-10','Verhovazhsky Area','VO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'VO-11','Vozhegodsky Area','VO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'VO-12','Vologodsky Area','VO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'VO-13','Vytegorsky Area / incl. Vytegra','VO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'VO-14','Gryazovetsky Area / incl. Gryazovets','VO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'VO-15','Kadujsky Area','VO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'VO-16','Kirillovsky Area / incl. Kirillov','VO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'VO-17','Kichmengsko-Gorodetsky Area','VO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'VO-18','Mezhdurechensky Area','VO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'VO-19','Nikolsky Area / incl. Nikolsk','VO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'VO-20','Njuksensky Area','VO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'VO-21','Sokolsky Area / incl. Kadnikov','VO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'VO-22','Syamzhensky Area','VO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'VO-23','Tarnogsky Area','VO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'VO-24','Totemsky Area / incl. Tot`Ma','VO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'VO-25','Ust-Kubinskiy Area','VO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'VO-26','Ustyuzhensky Area / incl. Ustyuzhna','VO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'VO-27','Harovsky Area / incl. Harovsk','VO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'VO-28','Chagodoshchensky Area','VO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'VO-29','Cherepovetsky Area','VO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'VO-30','Sheksinsky Area','VO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'VR-01','Zheleznodorozhny District of Voronezh','VR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'VR-02','Kominternovsky District of Voronezh','VR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'VR-03','Levoberezhny District of Voronezh','VR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'VR-04','Leninsky District of Voronezh','VR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'VR-05','Sovetsky District of Voronezh','VR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'VR-06','Centralny District of Voronezh','VR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'VR-07','Borisoglebsk City District','VR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'VR-09','Novovoronezh','VR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'VR-13','Anninsky Area','VR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'VR-14','Bobrovsky Area / incl. Bobrov','VR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'VR-15','Bogucharsky Area / incl. Boguchar','VR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'VR-17','Buturlinovsky Area / incl. Buturlinovka','VR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'VR-18','Verhnemamonsky Area','VR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'VR-19','Verhnehavsky Area','VR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'VR-20','Vorobyevsky Area','VR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'VR-21','Gribanovsky Area','VR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'VR-22','Kalacheevsky Area / incl. Kalach','VR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'VR-23','Kamensky Area','VR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'VR-24','Kantemirovsky Area','VR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'VR-25','Kashirsky Area','VR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'VR-26','Liskinsky Area / incl. Liski','VR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'VR-27','Nizhnedevitsky Area','VR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'VR-28','Novousmansky Area','VR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'VR-29','Novokhopersky Area / incl. Novohopersk','VR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'VR-30','Olkhovatsky Area','VR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'VR-31','Ostrogozhsky Area / incl. Ostrogozhsk','VR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'VR-32','Pavlovsky Area / incl. Pavlovsk','VR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'VR-33','Paninsky Area','VR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'VR-34','Petropavlovsky Area','VR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'VR-35','Povorinsky Area / incl. Povorino','VR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'VR-36','Podgorensky Area','VR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'VR-37','Ramonsky Area','VR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'VR-38','Repyevsky Area','VR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'VR-39','Rossoshansky Area / incl. Rossosh','VR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'VR-40','Semiluksky Area / incl. Semiluki','VR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'VR-41','Talovsky Area','VR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'VR-42','Ternovsky Area','VR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'VR-43','Hokholsky Area','VR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'VR-44','Ertilsky Area / incl. Ertil','VR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'YR-01','Dzerzhinsky District of Yaroslavl','YR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'YR-02','Zavolzhsky District of Yaroslavl','YR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'YR-03','Kirovsky District of Yaroslavl','YR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'YR-04','Krasnoperekopsky District of Yaroslavl','YR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'YR-05','Leninsky District of Yaroslavl','YR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'YR-06','Frunzensky District of Yaroslavl','YR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'YR-07','Pereslavl-Zalessky','YR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'YR-08','Rostov','YR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'YR-09','Rybinsk','YR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'YR-10','Tutaev','YR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'YR-11','Uglich','YR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'YR-12','Bolsheselsky Area','YR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'YR-13','Borisoglebsky Area','YR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'YR-14','Breytovsky Area','YR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'YR-15','Gavrilov-Yamsky Area / incl. Gavrilov-Yam','YR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'YR-16','Danilovsky Area / incl. Danilov','YR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'YR-17','Lyubimsky Area / incl. Lyubim ','YR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'YR-18','Myshkinsky Area / incl. Myshkin','YR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'YR-19','Nekouzsky Area','YR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'YR-20','Nekrasovsky Area','YR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'YR-21','Pervomajsky Area','YR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'YR-22','Pereslavsky Area','YR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'YR-23','Poshekhonsky Area / incl. Poshekhon`e','YR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'YR-24','Rostovsky Area / incl. Rostov ','YR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'YR-25','Rybinsk Area','YR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'YR-26','Tutayevsky Area / incl. Tutaev','YR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'YR-27','Uglichsky Area / incl. Uglich ','YR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (54,'YR-28','Yaroslavsky Area','YR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (61,'AR-32','Franz-Josef Land','AR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (110,'HI,Hawaii','Hawaii','HI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (110,'HI,Honolulu','Honolulu','HI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (110,'HI,Kalawao','Kalawao','HI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (110,'HI,Kauai','Kauai','HI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (110,'HI,Maui','Maui','HI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (126,'KA-01','Baltijsky                                                  (Deleted)','KA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (126,'KA-02','Leningradsky District of Kaliningrad','KA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (126,'KA-03','Moskovsky District of Kaliningrad','KA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (126,'KA-04','Oktjabr`Sky                                              (Deleted)','KA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (126,'KA-05','Centralny District of Kaliningrad','KA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (126,'KA-06','Baltiysky Area / incl. Baltiysk, Primorsk','KA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (126,'KA-07','Pionersky City District','KA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (126,'KA-08','Svetlogorsk Area / incl. Svetlogorsk','KA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (126,'KA-09','Svetlovsky City District','KA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (126,'KA-10','Sovietsky City District','KA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (126,'KA-11','Bagrationovsky Area / incl. Bagrationovsk','KA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (126,'KA-12','Gvardeisky City District','KA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (126,'KA-13','Guryevsky City District','KA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (126,'KA-14','Gusevsky City District','KA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (126,'KA-15','Zelenogradsky Area / incl. Zelenogradsk','KA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (126,'KA-16','Krasnoznamensky Area / incl. Krasnoznamensk ','KA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (126,'KA-17','Nemansky Area / incl. Neman','KA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (126,'KA-18','Nesterovsky Area / incl. Nesterov','KA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (126,'KA-19','Ozyorsk City District','KA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (126,'KA-20','Polesye Area / incl. Polessk','KA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (126,'KA-21','Pravdinsky Area / incl. Pravdinsk','KA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (126,'KA-22','Slavsky Area / incl. Slavsk','KA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (126,'KA-24','Ladushkinsky City District','KA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (126,'KA-23','Chernyakhovsky Area / incl. Chernyakhovsk','KA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (126,'KA-25','Mamonovsky City District','KA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (126,'KA-26','Yantarny City District','KA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (151,'LO-24','Vyborgsky Area Including Vysotsk','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (170,'090','Chatham Islands','CAN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (170,'001','Mongonui','NTL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (170,'002','Whangaroa ','NTL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (170,'003','Hokianga ','NTL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (170,'004','Bay of Islands ','NTL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (170,'005','Whangarei ','NTL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (170,'006','Hobson ','NTL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (170,'007','Otamatea ','NTL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (170,'008','Rodney ','AUK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (170,'009','Great Barrier Island ','AUK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (170,'010','Waitemata ','AUK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (170,'011','Franklin','AUK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (170,'012','Raglan ','WKO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (170,'013','Waikato ','WKO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (170,'014','Waipa ','WKO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (170,'015','Otorohanga','WKO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (170,'016','Waitomo ','WKO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (170,'017','Taumaranui','MWT','');
INSERT INTO adif_enum_secondary_subdivision VALUES (170,'018','Coromandel ','WKO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (170,'019','Thames ','WKO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (170,'020','Hauraki Plains ','WKO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (170,'021','Ohinemuri','WKO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (170,'022','Piako ','WKO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (170,'023','Matamata','WKO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (170,'024','Tauranga ','BOP','');
INSERT INTO adif_enum_secondary_subdivision VALUES (170,'025','Rotorua ','BOP','');
INSERT INTO adif_enum_secondary_subdivision VALUES (170,'026','Taupo','WKO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (170,'027','Whakatane ','BOP','');
INSERT INTO adif_enum_secondary_subdivision VALUES (170,'028','Opotiki ','BOP','');
INSERT INTO adif_enum_secondary_subdivision VALUES (170,'029','Waiapu ','GIS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (170,'030','Waikohu','GIS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (170,'031','Cook ','GIS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (170,'032','Wairoa ','HKB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (170,'033','Hawke''s Bay ','HKB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (170,'034','Waipawa ','HKB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (170,'035','Patangata','HKB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (170,'036','Waipukurau ','HKB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (170,'037','Dannevirke ','MWT','');
INSERT INTO adif_enum_secondary_subdivision VALUES (170,'038','Woodville ','MWT','');
INSERT INTO adif_enum_secondary_subdivision VALUES (170,'039','Clifton ','TKI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (170,'040','Taranaki ','TKI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (170,'041','Inglewood','TKI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (170,'042','Stratford ','TKI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (170,'043','Egmont ','TKI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (170,'044','Eltham ','TKI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (170,'045','Waimate  West','TKI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (170,'046','Hawera ','TKI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (170,'047','Patea ','TKI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (170,'048','Waimarino ','MWT','');
INSERT INTO adif_enum_secondary_subdivision VALUES (170,'049','Waitotara ','MWT','');
INSERT INTO adif_enum_secondary_subdivision VALUES (170,'050','Wanganui ','MWT','');
INSERT INTO adif_enum_secondary_subdivision VALUES (170,'051','Rangitikei ','MWT','');
INSERT INTO adif_enum_secondary_subdivision VALUES (170,'052','Kiwitea ','MWT','');
INSERT INTO adif_enum_secondary_subdivision VALUES (170,'053','Pohangina ','MWT','');
INSERT INTO adif_enum_secondary_subdivision VALUES (170,'054','Oroua ','MWT','');
INSERT INTO adif_enum_secondary_subdivision VALUES (170,'055','Manawatu','MWT','');
INSERT INTO adif_enum_secondary_subdivision VALUES (170,'056','Kairanga ','MWT','');
INSERT INTO adif_enum_secondary_subdivision VALUES (170,'057','Horowhenua ','MWT','');
INSERT INTO adif_enum_secondary_subdivision VALUES (170,'058','Hutt ','WGN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (170,'059','Pahiatua','MWT','');
INSERT INTO adif_enum_secondary_subdivision VALUES (170,'060','Akitio ','MWT','');
INSERT INTO adif_enum_secondary_subdivision VALUES (170,'061','Eketahura ','MWT','');
INSERT INTO adif_enum_secondary_subdivision VALUES (170,'062','Masterton ','MWT','');
INSERT INTO adif_enum_secondary_subdivision VALUES (170,'063','Waiarapa South','WGN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (170,'064','Featherston','WGN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (170,'065','Marlborough ','MBH','');
INSERT INTO adif_enum_secondary_subdivision VALUES (170,'066','Awatere','MBH','');
INSERT INTO adif_enum_secondary_subdivision VALUES (170,'067','Kaikoura ','CAN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (170,'068','Golden Bay','TAS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (170,'069','Waimea ','TAS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (170,'070','Buller ','WTC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (170,'071','Inangahua ','WTC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (170,'072','Grey ','WTC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (170,'073','Westland ','WTC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (170,'074','Amuri ','CAN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (170,'075','Cheviot ','CAN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (170,'076','Waipara','CAN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (170,'077','Kowai ','CAN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (170,'078','Ashley ','CAN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (170,'079','Rangiora ','CAN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (170,'080','Eyre ','CAN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (170,'081','Oxford ','CAN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (170,'082','Tawera ','CAN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (170,'083','Malvern ','CAN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (170,'084','Paparua ','CAN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (170,'085','Waimari ','CAN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (170,'086','Heathcote ','CAN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (170,'087','Halswell ','CAN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (170,'088','Mount Herbert ','CAN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (170,'089','Akaroa ','CAN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (170,'091','Wairewa ','CAN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (170,'092','Ellesmere ','CAN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (170,'093','Ashburton ','CAN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (170,'094','Geraldine ','CAN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (170,'095','Levels ','CAN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (170,'096','Mackenzie ','CAN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (170,'097','Waimate','CAN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (170,'098','Waitaki ','CAN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (170,'099','Waihemo ','OTA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (170,'100','Waikouaiti ','OTA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (170,'101','Peninsula ','OTA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (170,'102','Tareri ','OTA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (170,'103','Bruce ','OTA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (170,'104','Clutha ','OTA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (170,'105','Tuapeka ','OTA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (170,'106','Maniototo ','OTA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (170,'107','Vincent ','OTA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (170,'108','Lake ','OTA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (170,'109','Southland ','OTA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (170,'110','Wallace ','STL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (170,'111','Fiord ','STL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (170,'112','Stewart Island ','STL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'01001','Alegría-Dulantzi','VI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'01002','Amurrio','VI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'01003','Aramaio','VI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'01004','Artziniega','VI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'01006','Armiñón','VI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'01008','Arratzua-Ubarrundia','VI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'01009','Asparrena','VI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'01010','Ayala/Aiara','VI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'01011','Baños de Ebro/Mañueta','VI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'01013','Barrundia','VI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'01014','Berantevilla','VI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'01016','Bernedo','VI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'01017','Campezo/Kanpezu','VI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'01018','Zigoitia','VI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'01019','Kripan','VI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'01020','Kuartango','VI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'01021','Elburgo/Burgelu','VI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'01022','Elciego','VI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'01023','Elvillar/Bilar','VI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'01027','Iruraiz-Gauna','VI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'01028','Labastida/Bastida','VI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'01030','Lagrán','VI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'01031','Laguardia','VI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'01032','Lanciego/Lantziego','VI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'01033','Lapuebla de Labarca','VI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'01034','Leza','VI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'01036','Laudio/Llodio','VI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'01037','Arraia-Maeztu','VI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'01039','Moreda de Álava/Moreda Araba','VI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'01041','Navaridas','VI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'01042','Okondo','VI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'01043','Oyón-Oion','VI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'01044','Peñacerrada-Urizaharra','VI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'01046','Ribera Alta','VI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'01047','Erriberabeitia','VI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'01049','Añana','VI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'01051','Agurain/Salvatierra','VI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'01052','Samaniego','VI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'01053','San Millán/Donemiliaga','VI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'01054','Urkabustaiz','VI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'01055','Valdegovía/Gaubea','VI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'01056','Harana/Valle de Arana','VI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'01057','Villabuena de Álava/Eskuernaga','VI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'01058','Legutio','VI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'01059','Vitoria-Gasteiz','VI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'01060','Yécora/Iekora','VI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'01061','Zalduondo','VI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'01062','Zambrana','VI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'01063','Zuia','VI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'01901','Iruña Oka/Iruña de Oca','VI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'01902','Lantarón','VI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'02001','Abengibre','AB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'02002','Alatoz','AB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'02003','Albacete','AB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'02004','Albatana','AB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'02005','Alborea','AB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'02006','Alcadozo','AB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'02007','Alcalá del Júcar','AB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'02008','Alcaraz','AB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'02009','Almansa','AB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'02010','Alpera','AB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'02011','Ayna','AB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'02012','Balazote','AB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'02013','Balsa de Ves','AB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'02014','Ballestero (El)','AB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'02015','Barrax','AB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'02016','Bienservida','AB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'02017','Bogarra','AB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'02018','Bonete','AB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'02019','Bonillo (El)','AB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'02020','Carcelén','AB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'02021','Casas de Juan Núñez','AB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'02022','Casas de Lázaro','AB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'02023','Casas de Ves','AB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'02024','Casas-Ibáñez','AB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'02025','Caudete','AB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'02026','Cenizate','AB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'02027','Corral-Rubio','AB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'02028','Cotillas','AB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'02029','Chinchilla de Monte-Aragón','AB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'02030','Elche de la Sierra','AB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'02031','Férez','AB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'02032','Fuensanta','AB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'02033','Fuente-Álamo','AB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'02034','Fuentealbilla','AB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'02035','Gineta (La)','AB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'02036','Golosalvo','AB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'02037','Hellín','AB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'02038','Herrera (La)','AB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'02039','Higueruela','AB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'02040','Hoya-Gonzalo','AB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'02041','Jorquera','AB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'02042','Letur','AB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'02043','Lezuza','AB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'02044','Liétor','AB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'02045','Madrigueras','AB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'02046','Mahora','AB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'02047','Masegoso','AB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'02048','Minaya','AB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'02049','Molinicos','AB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'02050','Montalvos','AB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'02051','Montealegre del Castillo','AB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'02052','Motilleja','AB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'02053','Munera','AB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'02054','Navas de Jorquera','AB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'02055','Nerpio','AB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'02056','Ontur','AB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'02057','Ossa de Montiel','AB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'02058','Paterna del Madera','AB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'02059','Peñascosa','AB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'02060','Peñas de San Pedro','AB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'02061','Pétrola','AB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'02062','Povedilla','AB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'02063','Pozohondo','AB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'02064','Pozo-Lorente','AB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'02065','Pozuelo','AB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'02066','Recueja (La)','AB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'02067','Riópar','AB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'02068','Robledo','AB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'02069','Roda (La)','AB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'02070','Salobre','AB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'02071','San Pedro','AB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'02072','Socovos','AB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'02073','Tarazona de la Mancha','AB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'02074','Tobarra','AB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'02075','Valdeganga','AB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'02076','Vianos','AB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'02077','Villa de Ves','AB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'02078','Villalgordo del Júcar','AB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'02079','Villamalea','AB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'02080','Villapalacios','AB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'02081','Villarrobledo','AB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'02082','Villatoya','AB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'02083','Villavaliente','AB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'02084','Villaverde de Guadalimar','AB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'02085','Viveros','AB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'02086','Yeste','AB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'02901','Pozo Cañada','AB','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'03001','Atzúbia, l''','A','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'03002','Agost','A','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'03003','Agres','A','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'03004','Aigües','A','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'03005','Albatera','A','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'03006','Alcalalí','A','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'03007','Alcocer de Planes','A','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'03008','Alcoleja','A','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'03009','Alcoy/Alcoi','A','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'03010','Alfafara','A','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'03011','Alfàs del Pi (l'')','A','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'03012','Algorfa','A','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'03013','Algueña','A','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'03014','Alicante/Alacant','A','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'03015','Almoradí','A','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'03016','Almudaina','A','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'03017','Alqueria d''Asnar (l'')','A','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'03018','Altea','A','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'03019','Aspe','A','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'03020','Balones','A','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'03021','Banyeres de Mariola','A','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'03022','Benasau','A','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'03023','Beneixama','A','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'03024','Benejúzar','A','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'03025','Benferri','A','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'03026','Beniarbeig','A','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'03027','Beniardá','A','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'03028','Beniarrés','A','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'03029','Benigembla','A','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'03030','Benidoleig','A','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'03031','Benidorm','A','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'03032','Benifallim','A','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'03033','Benifato','A','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'03034','Benijófar','A','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'03035','Benilloba','A','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'03036','Benillup','A','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'03037','Benimantell','A','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'03038','Benimarfull','A','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'03039','Benimassot','A','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'03040','Benimeli','A','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'03041','Benissa','A','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'03042','Benitachell/Poble Nou de Benitatxell (el)','A','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'03043','Biar','A','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'03044','Bigastro','A','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'03045','Bolulla','A','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'03046','Busot','A','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'03047','Calp','A','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'03048','Callosa d''en Sarrià','A','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'03049','Callosa de Segura','A','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'03050','Campello (el)','A','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'03051','Campo de Mirra/Camp de Mirra (el)','A','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'03052','Cañada','A','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'03053','Castalla','A','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'03054','Castell de Castells','A','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'03055','Catral','A','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'03056','Cocentaina','A','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'03057','Confrides','A','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'03058','Cox','A','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'03059','Crevillent','A','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'03060','Quatretondeta','A','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'03061','Daya Nueva','A','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'03062','Daya Vieja','A','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'03063','Dénia','A','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'03064','Dolores','A','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'03065','Elche/Elx','A','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'03066','Elda','A','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'03067','Facheca','A','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'03068','Famorca','A','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'03069','Finestrat','A','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'03070','Formentera del Segura','A','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'03071','Gata de Gorgos','A','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'03072','Gaianes','A','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'03073','Gorga','A','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'03074','Granja de Rocamora','A','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'03075','Castell de Guadalest (el)','A','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'03076','Guardamar del Segura','A','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'03077','Fondó de les Neus (el)/Hondón de las Nieves','A','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'03078','Hondón de los Frailes','A','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'03079','Ibi','A','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'03080','Jacarilla','A','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'03081','Xaló','A','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'03082','Jávea/Xàbia','A','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'03083','Jijona/Xixona','A','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'03084','Lorcha/Orxa (l'')','A','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'03085','Llíber','A','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'03086','Millena','A','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'03088','Monforte del Cid','A','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'03089','Monóvar/Monòver','A','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'03090','Mutxamel','A','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'03091','Murla','A','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'03092','Muro de Alcoy','A','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'03093','Novelda','A','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'03094','Nucia (la)','A','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'03095','Ondara','A','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'03096','Onil','A','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'03097','Orba','A','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'03098','Orxeta','A','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'03099','Orihuela','A','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'03100','Parcent','A','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'03101','Pedreguer','A','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'03102','Pego','A','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'03103','Penàguila','A','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'03104','Petrer','A','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'03105','Pinós (el)/Pinoso','A','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'03106','Planes','A','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'03107','Polop','A','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'03109','Rafal','A','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'03110','Ràfol d''Almúnia (El)','A','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'03111','Redován','A','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'03112','Relleu','A','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'03113','Rojales','A','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'03114','Romana (la)','A','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'03115','Sagra','A','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'03116','Salinas','A','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'03117','Sanet y Negrals','A','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'03118','San Fulgencio','A','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'03119','Sant Joan d''Alacant','A','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'03120','San Miguel de Salinas','A','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'03121','Santa Pola','A','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'03122','San Vicente del Raspeig/Sant Vicent del Raspeig','A','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'03123','Sax','A','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'03124','Sella','A','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'03125','Senija','A','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'03127','Tàrbena','A','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'03128','Teulada','A','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'03129','Tibi','A','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'03130','Tollos','A','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'03131','Tormos','A','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'03132','Torremanzanas/Torre de les Maçanes (la)','A','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'03133','Torrevieja','A','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'03134','Vall d''Alcalà (la)','A','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'03135','Vall d''Ebo (la)','A','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'03136','Vall de Gallinera','A','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'03137','Vall de Laguar (la)','A','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'03138','Verger (el)','A','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'03139','Villajoyosa/Vila Joiosa (la)','A','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'03140','Villena','A','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'03901','Poblets (els)','A','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'03902','Pilar de la Horadada','A','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'03903','Montesinos (Los)','A','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'03904','San Isidro','A','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'04001','Abla','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'04002','Abrucena','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'04003','Adra','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'04004','Albanchez','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'04005','Alboloduy','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'04006','Albox','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'04007','Alcolea','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'04008','Alcóntar','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'04009','Alcudia de Monteagud','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'04010','Alhabia','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'04011','Alhama de Almería','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'04012','Alicún','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'04013','Almería','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'04014','Almócita','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'04015','Alsodux','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'04016','Antas','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'04017','Arboleas','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'04018','Armuña de Almanzora','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'04019','Bacares','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'04020','Bayárcal','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'04021','Bayarque','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'04022','Bédar','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'04023','Beires','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'04024','Benahadux','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'04026','Benitagla','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'04027','Benizalón','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'04028','Bentarique','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'04029','Berja','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'04030','Canjáyar','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'04031','Cantoria','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'04032','Carboneras','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'04033','Castro de Filabres','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'04034','Cóbdar','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'04035','Cuevas del Almanzora','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'04036','Chercos','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'04037','Chirivel','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'04038','Dalías','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'04041','Enix','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'04043','Felix','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'04044','Fines','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'04045','Fiñana','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'04046','Fondón','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'04047','Gádor','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'04048','Gallardos (Los)','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'04049','Garrucha','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'04050','Gérgal','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'04051','Huécija','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'04052','Huércal de Almería','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'04053','Huércal-Overa','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'04054','Íllar','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'04055','Instinción','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'04056','Laroya','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'04057','Láujar de Andarax','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'04058','Líjar','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'04059','Lubrín','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'04060','Lucainena de las Torres','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'04061','Lúcar','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'04062','Macael','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'04063','María','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'04064','Mojácar','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'04065','Nacimiento','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'04066','Níjar','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'04067','Ohanes','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'04068','Olula de Castro','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'04069','Olula del Río','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'04070','Oria','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'04071','Padules','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'04072','Partaloa','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'04073','Paterna del Río','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'04074','Pechina','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'04075','Pulpí','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'04076','Purchena','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'04077','Rágol','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'04078','Rioja','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'04079','Roquetas de Mar','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'04080','Santa Cruz de Marchena','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'04081','Santa Fe de Mondújar','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'04082','Senés','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'04083','Serón','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'04084','Sierro','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'04085','Somontín','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'04086','Sorbas','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'04087','Suflí','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'04088','Tabernas','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'04089','Taberno','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'04090','Tahal','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'04091','Terque','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'04092','Tíjola','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'04093','Turre','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'04094','Turrillas','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'04095','Uleila del Campo','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'04096','Urrácal','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'04097','Velefique','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'04098','Vélez-Blanco','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'04099','Vélez-Rubio','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'04100','Vera','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'04101','Viator','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'04102','Vícar','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'04103','Zurgena','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'04901','Tres Villas (Las)','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'04902','Ejido (El)','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'04903','Mojonera (La)','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'04904','Balanegra','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05001','Adanero','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05002','Adrada (La)','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05005','Albornos','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05007','Aldeanueva de Santa Cruz','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05008','Aldeaseca','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05010','Aldehuela (La)','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05012','Amavida','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05013','Arenal (El)','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05014','Arenas de San Pedro','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05015','Arevalillo','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05016','Arévalo','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05017','Aveinte','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05018','Avellaneda','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05019','Ávila','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05021','Barco de Ávila (El)','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05022','Barraco (El)','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05023','Barromán','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05024','Becedas','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05025','Becedillas','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05026','Bercial de Zapardiel','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05027','Berlanas (Las)','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05029','Bernuy-Zapardiel','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05030','Berrocalejo de Aragona','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05033','Blascomillán','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05034','Blasconuño de Matacabras','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05035','Blascosancho','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05036','Bohodón (El)','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05037','Bohoyo','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05038','Bonilla de la Sierra','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05039','Brabos','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05040','Bularros','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05041','Burgohondo','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05042','Cabezas de Alambre','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05043','Cabezas del Pozo','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05044','Cabezas del Villar','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05045','Cabizuela','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05046','Canales','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05047','Candeleda','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05048','Cantiveros','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05049','Cardeñosa','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05051','Carrera (La)','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05052','Casas del Puerto','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05053','Casasola','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05054','Casavieja','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05055','Casillas','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05056','Castellanos de Zapardiel','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05057','Cebreros','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05058','Cepeda la Mora','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05059','Cillán','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05060','Cisla','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05061','Colilla (La)','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05062','Collado de Contreras','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05063','Collado del Mirón','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05064','Constanzana','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05065','Crespos','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05066','Cuevas del Valle','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05067','Chamartín','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05069','Donjimeno','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05070','Donvidas','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05072','Espinosa de los Caballeros','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05073','Flores de Ávila','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05074','Fontiveros','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05075','Fresnedilla','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05076','Fresno (El)','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05077','Fuente el Saúz','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05078','Fuentes de Año','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05079','Gallegos de Altamiros','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05080','Gallegos de Sobrinos','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05081','Garganta del Villar','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05082','Gavilanes','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05083','Gemuño','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05084','Gilbuena','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05085','Gil García','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05086','Gimialcón','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05087','Gotarrendura','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05088','Grandes y San Martín','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05089','Guisando','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05090','Gutierre-Muñoz','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05092','Hernansancho','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05093','Herradón de Pinares','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05094','Herreros de Suso','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05095','Higuera de las Dueñas','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05096','Hija de Dios (La)','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05097','Horcajada (La)','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05099','Horcajo de las Torres','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05100','Hornillo (El)','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05101','Hoyocasero','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05102','Hoyo de Pinares (El)','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05103','Hoyorredondo','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05104','Hoyos del Collado','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05105','Hoyos del Espino','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05106','Hoyos de Miguel Muñoz','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05107','Hurtumpascual','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05108','Junciana','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05109','Langa','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05110','Lanzahíta','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05112','Losar del Barco (El)','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05113','Llanos de Tormes (Los)','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05114','Madrigal de las Altas Torres','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05115','Maello','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05116','Malpartida de Corneja','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05117','Mamblas','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05118','Mancera de Arriba','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05119','Manjabálago y Ortigosa de Rioalmar','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05120','Marlín','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05121','Martiherrero','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05122','Martínez','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05123','Mediana de Voltoya','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05124','Medinilla','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05125','Mengamuñoz','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05126','Mesegar de Corneja','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05127','Mijares','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05128','Mingorría','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05129','Mirón (El)','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05130','Mironcillo','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05131','Mirueña de los Infanzones','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05132','Mombeltrán','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05133','Monsalupe','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05134','Moraleja de Matacabras','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05135','Muñana','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05136','Muñico','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05138','Muñogalindo','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05139','Muñogrande','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05140','Muñomer del Peco','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05141','Muñopepe','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05142','Muñosancho','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05143','Muñotello','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05144','Narrillos del Álamo','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05145','Narrillos del Rebollar','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05147','Narros del Castillo','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05148','Narros del Puerto','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05149','Narros de Saldueña','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05151','Navacepedilla de Corneja','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05152','Nava de Arévalo','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05153','Nava del Barco','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05154','Navadijos','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05155','Navaescurial','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05156','Navahondilla','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05157','Navalacruz','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05158','Navalmoral','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05159','Navalonguilla','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05160','Navalosa','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05161','Navalperal de Pinares','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05162','Navalperal de Tormes','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05163','Navaluenga','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05164','Navaquesera','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05165','Navarredonda de Gredos','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05166','Navarredondilla','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05167','Navarrevisca','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05168','Navas del Marqués (Las)','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05169','Navatalgordo','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05170','Navatejares','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05171','Neila de San Miguel','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05172','Niharra','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05173','Ojos-Albos','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05174','Orbita','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05175','Oso (El)','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05176','Padiernos','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05177','Pajares de Adaja','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05178','Palacios de Goda','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05179','Papatrigo','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05180','Parral (El)','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05181','Pascualcobo','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05182','Pedro Bernardo','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05183','Pedro-Rodríguez','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05184','Peguerinos','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05185','Peñalba de Ávila','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05186','Piedrahíta','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05187','Piedralaves','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05188','Poveda','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05189','Poyales del Hoyo','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05190','Pozanco','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05191','Pradosegar','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05192','Puerto Castilla','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05193','Rasueros','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05194','Riocabado','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05195','Riofrío','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05196','Rivilla de Barajas','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05197','Salobral','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05198','Salvadiós','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05199','San Bartolomé de Béjar','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05200','San Bartolomé de Corneja','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05201','San Bartolomé de Pinares','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05204','Sanchidrián','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05205','Sanchorreja','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05206','San Esteban de los Patos','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05207','San Esteban del Valle','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05208','San Esteban de Zapardiel','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05209','San García de Ingelmos','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05210','San Juan de la Encinilla','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05211','San Juan de la Nava','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05212','San Juan del Molinillo','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05213','San Juan del Olmo','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05214','San Lorenzo de Tormes','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05215','San Martín de la Vega del Alberche','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05216','San Martín del Pimpollar','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05217','San Miguel de Corneja','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05218','San Miguel de Serrezuela','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05219','San Pascual','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05220','San Pedro del Arroyo','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05221','Santa Cruz del Valle','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05222','Santa Cruz de Pinares','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05224','Santa María del Arroyo','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05225','Santa María del Berrocal','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05226','Santa María de los Caballeros','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05227','Santa María del Tiétar','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05228','Santiago del Collado','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05229','Santo Domingo de las Posadas','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05230','Santo Tomé de Zabarcos','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05231','San Vicente de Arévalo','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05232','Serrada (La)','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05233','Serranillos','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05234','Sigeres','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05235','Sinlabajos','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05236','Solana de Ávila','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05237','Solana de Rioalmar','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05238','Solosancho','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05239','Sotalbo','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05240','Sotillo de la Adrada','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05241','Tiemblo (El)','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05242','Tiñosillos','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05243','Tolbaños','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05244','Tormellas','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05245','Tornadizos de Ávila','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05246','Tórtoles','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05247','Torre (La)','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05249','Umbrías','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05251','Vadillo de la Sierra','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05252','Valdecasa','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05253','Vega de Santa María','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05254','Velayos','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05256','Villaflor','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05257','Villafranca de la Sierra','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05258','Villanueva de Gómez','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05259','Villanueva del Aceral','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05260','Villanueva del Campillo','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05261','Villar de Corneja','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05262','Villarejo del Valle','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05263','Villatoro','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05264','Viñegra de Moraña','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05265','Vita','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05266','Zapardiel de la Cañada','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05267','Zapardiel de la Ribera','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05901','San Juan de Gredos','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05902','Santa María del Cubillo','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05903','Diego del Carpio','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05904','Santiago del Tormes','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'05905','Villanueva de Ávila','AV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'06001','Acedera','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'06002','Aceuchal','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'06003','Ahillones','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'06004','Alange','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'06005','Albuera (La)','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'06006','Alburquerque','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'06007','Alconchel','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'06008','Alconera','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'06009','Aljucén','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'06010','Almendral','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'06011','Almendralejo','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'06012','Arroyo de San Serván','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'06013','Atalaya','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'06014','Azuaga','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'06015','Badajoz','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'06016','Barcarrota','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'06017','Baterno','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'06018','Benquerencia de la Serena','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'06019','Berlanga','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'06020','Bienvenida','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'06021','Bodonal de la Sierra','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'06022','Burguillos del Cerro','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'06023','Cabeza del Buey','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'06024','Cabeza la Vaca','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'06025','Calamonte','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'06026','Calera de León','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'06027','Calzadilla de los Barros','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'06028','Campanario','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'06029','Campillo de Llerena','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'06030','Capilla','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'06031','Carmonita','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'06032','Carrascalejo (El)','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'06033','Casas de Don Pedro','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'06034','Casas de Reina','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'06035','Castilblanco','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'06036','Castuera','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'06037','Codosera (La)','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'06038','Cordobilla de Lácara','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'06039','Coronada (La)','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'06040','Corte de Peleas','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'06041','Cristina','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'06042','Cheles','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'06043','Don Álvaro','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'06044','Don Benito','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'06045','Entrín Bajo','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'06046','Esparragalejo','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'06047','Esparragosa de la Serena','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'06048','Esparragosa de Lares','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'06049','Feria','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'06050','Fregenal de la Sierra','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'06051','Fuenlabrada de los Montes','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'06052','Fuente de Cantos','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'06053','Fuente del Arco','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'06054','Fuente del Maestre','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'06055','Fuentes de León','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'06056','Garbayuela','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'06057','Garlitos','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'06058','Garrovilla (La)','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'06059','Granja de Torrehermosa','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'06060','Guareña','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'06061','Haba (La)','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'06062','Helechosa de los Montes','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'06063','Herrera del Duque','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'06064','Higuera de la Serena','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'06065','Higuera de Llerena','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'06066','Higuera de Vargas','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'06067','Higuera la Real','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'06068','Hinojosa del Valle','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'06069','Hornachos','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'06070','Jerez de los Caballeros','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'06071','Lapa (La)','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'06072','Lobón','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'06073','Llera','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'06074','Llerena','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'06075','Magacela','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'06076','Maguilla','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'06077','Malcocinado','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'06078','Malpartida de la Serena','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'06079','Manchita','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'06080','Medellín','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'06081','Medina de las Torres','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'06082','Mengabril','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'06083','Mérida','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'06084','Mirandilla','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'06085','Monesterio','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'06086','Montemolín','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'06087','Monterrubio de la Serena','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'06088','Montijo','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'06089','Morera (La)','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'06090','Nava de Santiago (La)','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'06091','Navalvillar de Pela','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'06092','Nogales','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'06093','Oliva de la Frontera','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'06094','Oliva de Mérida','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'06095','Olivenza','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'06096','Orellana de la Sierra','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'06097','Orellana la Vieja','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'06098','Palomas','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'06099','Parra (La)','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'06100','Peñalsordo','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'06101','Peraleda del Zaucejo','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'06102','Puebla de Alcocer','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'06103','Puebla de la Calzada','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'06104','Puebla de la Reina','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'06105','Puebla del Maestre','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'06106','Puebla del Prior','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'06107','Puebla de Obando','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'06108','Puebla de Sancho Pérez','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'06109','Quintana de la Serena','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'06110','Reina','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'06111','Rena','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'06112','Retamal de Llerena','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'06113','Ribera del Fresno','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'06114','Risco','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'06115','Roca de la Sierra (La)','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'06116','Salvaleón','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'06117','Salvatierra de los Barros','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'06118','Sancti-Spíritus','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'06119','San Pedro de Mérida','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'06120','Santa Amalia','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'06121','Santa Marta','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'06122','Santos de Maimona (Los)','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'06123','San Vicente de Alcántara','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'06124','Segura de León','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'06125','Siruela','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'06126','Solana de los Barros','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'06127','Talarrubias','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'06128','Talavera la Real','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'06129','Táliga','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'06130','Tamurejo','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'06131','Torre de Miguel Sesmero','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'06132','Torremayor','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'06133','Torremejía','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'06134','Trasierra','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'06135','Trujillanos','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'06136','Usagre','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'06137','Valdecaballeros','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'06138','Valdetorres','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'06139','Valencia de las Torres','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'06140','Valencia del Mombuey','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'06141','Valencia del Ventoso','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'06142','Valverde de Burguillos','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'06143','Valverde de Leganés','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'06144','Valverde de Llerena','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'06145','Valverde de Mérida','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'06146','Valle de la Serena','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'06147','Valle de Matamoros','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'06148','Valle de Santa Ana','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'06149','Villafranca de los Barros','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'06150','Villagarcía de la Torre','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'06151','Villagonzalo','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'06152','Villalba de los Barros','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'06153','Villanueva de la Serena','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'06154','Villanueva del Fresno','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'06155','Villar del Rey','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'06156','Villar de Rena','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'06157','Villarta de los Montes','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'06158','Zafra','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'06159','Zahínos','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'06160','Zalamea de la Serena','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'06161','Zarza-Capilla','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'06162','Zarza (La)','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'06901','Valdelacalzada','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'06902','Pueblonuevo del Guadiana','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'06903','Guadiana del Caudillo','BA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08001','Abrera','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08002','Aguilar de Segarra','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08003','Alella','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08004','Alpens','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08005','Ametlla del Vallès (L'')','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08006','Arenys de Mar','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08007','Arenys de Munt','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08008','Argençola','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08009','Argentona','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08010','Artés','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08011','Avià','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08012','Avinyó','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08013','Avinyonet del Penedès','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08014','Aiguafreda','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08015','Badalona','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08016','Bagà','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08017','Balenyà','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08018','Balsareny','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08019','Barcelona','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08020','Begues','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08021','Bellprat','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08022','Berga','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08023','Bigues i Riells','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08024','Borredà','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08025','Bruc (El)','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08026','Brull (El)','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08027','Cabanyes (Les)','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08028','Cabrera d''Anoia','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08029','Cabrera de Mar','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08030','Cabrils','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08031','Calaf','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08032','Caldes d''Estrac','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08033','Caldes de Montbui','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08034','Calders','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08035','Calella','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08036','Calonge de Segarra','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08037','Calldetenes','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08038','Callús','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08039','Campins','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08040','Canet de Mar','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08041','Canovelles','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08042','Cànoves i Samalús','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08043','Canyelles','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08044','Capellades','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08045','Capolat','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08046','Cardedeu','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08047','Cardona','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08048','Carme','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08049','Casserres','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08050','Castellar del Riu','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08051','Castellar del Vallès','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08052','Castellar de n''Hug','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08053','Castellbell i el Vilar','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08054','Castellbisbal','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08055','Castellcir','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08056','Castelldefels','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08057','Castell de l''Areny','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08058','Castellet i la Gornal','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08059','Castellfollit del Boix','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08060','Castellfollit de Riubregós','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08061','Castellgalí','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08062','Castellnou de Bages','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08063','Castellolí','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08064','Castellterçol','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08065','Castellví de la Marca','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08066','Castellví de Rosanes','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08067','Centelles','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08068','Cervelló','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08069','Collbató','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08070','Collsuspina','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08071','Copons','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08072','Corbera de Llobregat','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08073','Cornellà de Llobregat','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08074','Cubelles','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08075','Dosrius','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08076','Esparreguera','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08077','Esplugues de Llobregat','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08078','Espunyola (L'')','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08079','Estany (L'')','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08080','Fígols','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08081','Fogars de Montclús','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08082','Fogars de la Selva','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08083','Folgueroles','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08084','Fonollosa','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08085','Font-rubí','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08086','Franqueses del Vallès (Les)','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08087','Gallifa','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08088','Garriga (La)','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08089','Gavà','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08090','Gaià','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08091','Gelida','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08092','Gironella','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08093','Gisclareny','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08094','Granada (La)','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08095','Granera','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08096','Granollers','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08097','Gualba','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08098','Sant Salvador de Guardiola','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08099','Guardiola de Berguedà','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08100','Gurb','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08101','Hospitalet de Llobregat (L'')','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08102','Igualada','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08103','Jorba','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08104','Llacuna (La)','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08105','Llagosta (La)','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08106','Llinars del Vallès','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08107','Lliçà d''Amunt','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08108','Lliçà de Vall','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08109','Lluçà','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08110','Malgrat de Mar','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08111','Malla','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08112','Manlleu','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08113','Manresa','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08114','Martorell','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08115','Martorelles','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08116','Masies de Roda (Les)','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08117','Masies de Voltregà (Les)','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08118','Masnou (El)','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08119','Masquefa','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08120','Matadepera','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08121','Mataró','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08122','Mediona','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08123','Molins de Rei','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08124','Mollet del Vallès','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08125','Montcada i Reixac','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08126','Montgat','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08127','Monistrol de Montserrat','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08128','Monistrol de Calders','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08129','Muntanyola','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08130','Montclar','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08131','Montesquiu','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08132','Montmajor','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08133','Montmaneu','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08134','Figaró-Montmany','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08135','Montmeló','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08136','Montornès del Vallès','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08137','Montseny','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08138','Moià','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08139','Mura','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08140','Navarcles','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08141','Navàs','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08142','Nou de Berguedà (La)','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08143','Òdena','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08144','Olvan','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08145','Olèrdola','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08146','Olesa de Bonesvalls','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08147','Olesa de Montserrat','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08148','Olivella','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08149','Olost','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08150','Orís','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08151','Oristà','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08152','Orpí','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08153','Òrrius','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08154','Pacs del Penedès','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08155','Palafolls','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08156','Palau-solità i Plegamans','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08157','Pallejà','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08158','Papiol (El)','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08159','Parets del Vallès','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08160','Perafita','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08161','Piera','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08162','Hostalets de Pierola (Els)','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08163','Pineda de Mar','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08164','Pla del Penedès (El)','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08165','Pobla de Claramunt (La)','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08166','Pobla de Lillet (La)','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08167','Polinyà','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08168','Pontons','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08169','Prat de Llobregat (El)','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08170','Prats de Rei (Els)','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08171','Prats de Lluçanès','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08172','Premià de Mar','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08174','Puigdàlber','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08175','Puig-reig','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08176','Pujalt','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08177','Quar (La)','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08178','Rajadell','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08179','Rellinars','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08180','Ripollet','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08181','Roca del Vallès (La)','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08182','Pont de Vilomara i Rocafort (El)','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08183','Roda de Ter','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08184','Rubí','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08185','Rubió','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08187','Sabadell','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08188','Sagàs','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08189','Sant Pere Sallavinera','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08190','Saldes','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08191','Sallent','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08192','Santpedor','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08193','Sant Iscle de Vallalta','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08194','Sant Adrià de Besòs','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08195','Sant Agustí de Lluçanès','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08196','Sant Andreu de la Barca','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08197','Sant Andreu de Llavaneres','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08198','Sant Antoni de Vilamajor','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08199','Sant Bartomeu del Grau','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08200','Sant Boi de Llobregat','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08201','Sant Boi de Lluçanès','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08202','Sant Celoni','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08203','Sant Cebrià de Vallalta','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08204','Sant Climent de Llobregat','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08205','Sant Cugat del Vallès','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08206','Sant Cugat Sesgarrigues','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08207','Sant Esteve de Palautordera','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08208','Sant Esteve Sesrovires','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08209','Sant Fost de Campsentelles','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08210','Sant Feliu de Codines','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08211','Sant Feliu de Llobregat','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08212','Sant Feliu Sasserra','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08213','Sant Fruitós de Bages','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08214','Vilassar de Dalt','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08215','Sant Hipòlit de Voltregà','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08216','Sant Jaume de Frontanyà','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08217','Sant Joan Despí','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08218','Sant Joan de Vilatorrada','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08219','Vilassar de Mar','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08220','Sant Julià de Vilatorta','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08221','Sant Just Desvern','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08222','Sant Llorenç d''Hortons','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08223','Sant Llorenç Savall','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08224','Sant Martí de Centelles','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08225','Sant Martí d''Albars','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08226','Sant Martí de Tous','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08227','Sant Martí Sarroca','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08228','Sant Martí Sesgueioles','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08229','Sant Mateu de Bages','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08230','Premià de Dalt','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08231','Sant Pere de Ribes','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08232','Sant Pere de Riudebitlles','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08233','Sant Pere de Torelló','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08234','Sant Pere de Vilamajor','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08235','Sant Pol de Mar','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08236','Sant Quintí de Mediona','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08237','Sant Quirze de Besora','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08238','Sant Quirze del Vallès','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08239','Sant Quirze Safaja','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08240','Sant Sadurní d''Anoia','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08241','Sant Sadurní d''Osormort','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08242','Marganell','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08243','Santa Cecília de Voltregà','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08244','Santa Coloma de Cervelló','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08245','Santa Coloma de Gramenet','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08246','Santa Eugènia de Berga','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08247','Santa Eulàlia de Riuprimer','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08248','Santa Eulàlia de Ronçana','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08249','Santa Fe del Penedès','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08250','Santa Margarida de Montbui','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08251','Santa Margarida i els Monjos','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08252','Barberà del Vallès','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08253','Santa Maria de Besora','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08254','Esquirol, L''','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08255','Santa Maria de Merlès','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08256','Santa Maria de Martorelles','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08257','Santa Maria de Miralles','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08258','Santa Maria d''Oló','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08259','Santa Maria de Palautordera','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08260','Santa Perpètua de Mogoda','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08261','Santa Susanna','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08262','Sant Vicenç de Castellet','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08263','Sant Vicenç dels Horts','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08264','Sant Vicenç de Montalt','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08265','Sant Vicenç de Torelló','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08266','Cerdanyola del Vallès','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08267','Sentmenat','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08268','Cercs','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08269','Seva','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08270','Sitges','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08271','Sobremunt','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08272','Sora','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08273','Subirats','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08274','Súria','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08275','Tavèrnoles','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08276','Tagamanent','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08277','Talamanca','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08278','Taradell','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08279','Terrassa','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08280','Tavertet','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08281','Teià','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08282','Tiana','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08283','Tona','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08284','Tordera','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08285','Torelló','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08286','Torre de Claramunt (La)','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08287','Torrelavit','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08288','Torrelles de Foix','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08289','Torrelles de Llobregat','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08290','Ullastrell','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08291','Vacarisses','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08292','Vallbona d''Anoia','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08293','Vallcebre','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08294','Vallgorguina','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08295','Vallirana','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08296','Vallromanes','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08297','Veciana','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08298','Vic','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08299','Vilada','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08300','Viladecavalls','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08301','Viladecans','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08302','Vilanova del Camí','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08303','Vilanova de Sau','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08304','Vilobí del Penedès','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08305','Vilafranca del Penedès','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08306','Vilalba Sasserra','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08307','Vilanova i la Geltrú','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08308','Viver i Serrateix','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08901','Rupit i Pruit','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08902','Vilanova del Vallès','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08903','Sant Julià de Cerdanyola','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08904','Badia del Vallès','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'08905','Palma de Cervelló (La)','B','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09001','Abajas','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09003','Adrada de Haza','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09006','Aguas Cándidas','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09007','Aguilar de Bureba','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09009','Albillos','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09010','Alcocero de Mola','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09011','Alfoz de Bricia','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09012','Alfoz de Santa Gadea','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09013','Altable','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09014','Altos (Los)','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09016','Ameyugo','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09017','Anguix','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09018','Aranda de Duero','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09019','Arandilla','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09020','Arauzo de Miel','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09021','Arauzo de Salce','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09022','Arauzo de Torre','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09023','Arcos','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09024','Arenillas de Riopisuerga','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09025','Arija','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09026','Arlanzón','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09027','Arraya de Oca','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09029','Atapuerca','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09030','Ausines (Los)','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09032','Avellanosa de Muñó','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09033','Bahabón de Esgueva','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09034','Balbases (Los)','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09035','Baños de Valdearados','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09036','Bañuelos de Bureba','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09037','Barbadillo de Herreros','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09038','Barbadillo del Mercado','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09039','Barbadillo del Pez','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09041','Barrio de Muñó','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09043','Barrios de Bureba (Los)','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09044','Barrios de Colina','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09045','Basconcillos del Tozo','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09046','Bascuñana','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09047','Belbimbre','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09048','Belorado','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09050','Berberana','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09051','Berlangas de Roa','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09052','Berzosa de Bureba','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09054','Bozoó','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09055','Brazacorta','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09056','Briviesca','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09057','Bugedo','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09058','Buniel','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09059','Burgos','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09060','Busto de Bureba','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09061','Cabañes de Esgueva','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09062','Cabezón de la Sierra','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09063','Cavia','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09064','Caleruega','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09065','Campillo de Aranda','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09066','Campolara','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09067','Canicosa de la Sierra','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09068','Cantabrana','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09070','Carazo','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09071','Carcedo de Bureba','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09072','Carcedo de Burgos','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09073','Cardeñadijo','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09074','Cardeñajimeno','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09075','Cardeñuela Riopico','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09076','Carrias','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09077','Cascajares de Bureba','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09078','Cascajares de la Sierra','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09079','Castellanos de Castro','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09082','Castildelgado','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09083','Castil de Peones','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09084','Castrillo de la Reina','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09085','Castrillo de la Vega','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09086','Castrillo del Val','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09088','Castrillo de Riopisuerga','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09090','Castrillo Mota de Judíos','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09091','Castrojeriz','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09093','Cayuela','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09094','Cebrecos','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09095','Celada del Camino','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09098','Cerezo de Río Tirón','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09100','Cerratón de Juarros','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09101','Ciadoncha','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09102','Cillaperlata','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09103','Cilleruelo de Abajo','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09104','Cilleruelo de Arriba','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09105','Ciruelos de Cervera','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09108','Cogollos','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09109','Condado de Treviño','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09110','Contreras','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09112','Coruña del Conde','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09113','Covarrubias','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09114','Cubillo del Campo','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09115','Cubo de Bureba','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09117','Cueva de Roa (La)','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09119','Cuevas de San Clemente','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09120','Encío','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09122','Espinosa de Cervera','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09123','Espinosa del Camino','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09124','Espinosa de los Monteros','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09125','Estépar','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09127','Fontioso','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09128','Frandovínez','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09129','Fresneda de la Sierra Tirón','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09130','Fresneña','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09131','Fresnillo de las Dueñas','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09132','Fresno de Río Tirón','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09133','Fresno de Rodilla','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09134','Frías','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09135','Fuentebureba','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09136','Fuentecén','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09137','Fuentelcésped','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09138','Fuentelisendo','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09139','Fuentemolinos','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09140','Fuentenebro','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09141','Fuentespina','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09143','Galbarros','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09144','Gallega (La)','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09148','Grijalba','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09149','Grisaleña','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09151','Gumiel de Izán','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09152','Gumiel de Mercado','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09154','Hacinas','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09155','Haza','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09159','Hontanas','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09160','Hontangas','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09162','Hontoria de la Cantera','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09163','Hontoria del Pinar','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09164','Hontoria de Valdearados','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09166','Hormazas (Las)','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09167','Hornillos del Camino','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09168','Horra (La)','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09169','Hortigüela','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09170','Hoyales de Roa','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09172','Huérmeces','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09173','Huerta de Arriba','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09174','Huerta de Rey','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09175','Humada','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09176','Hurones','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09177','Ibeas de Juarros','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09178','Ibrillos','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09179','Iglesiarrubia','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09180','Iglesias','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09181','Isar','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09182','Itero del Castillo','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09183','Jaramillo de la Fuente','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09184','Jaramillo Quemado','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09189','Junta de Traslaloma','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09190','Junta de Villalba de Losa','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09191','Jurisdicción de Lara','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09192','Jurisdicción de San Zadornil','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09194','Lerma','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09195','Llano de Bureba','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09196','Madrigal del Monte','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09197','Madrigalejo del Monte','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09198','Mahamud','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09199','Mambrilla de Castrejón','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09200','Mambrillas de Lara','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09201','Mamolar','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09202','Manciles','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09206','Mazuela','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09208','Mecerreyes','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09209','Medina de Pomar','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09211','Melgar de Fernamental','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09213','Merindad de Cuesta-Urria','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09214','Merindad de Montija','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09215','Merindad de Sotoscueva','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09216','Merindad de Valdeporres','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09217','Merindad de Valdivielso','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09218','Milagros','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09219','Miranda de Ebro','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09220','Miraveche','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09221','Modúbar de la Emparedada','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09223','Monasterio de la Sierra','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09224','Monasterio de Rodilla','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09225','Moncalvillo','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09226','Monterrubio de la Demanda','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09227','Montorio','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09228','Moradillo de Roa','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09229','Nava de Roa','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09230','Navas de Bureba','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09231','Nebreda','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09232','Neila','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09235','Olmedillo de Roa','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09236','Olmillos de Muñó','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09238','Oña','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09239','Oquillas','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09241','Orbaneja Riopico','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09242','Padilla de Abajo','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09243','Padilla de Arriba','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09244','Padrones de Bureba','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09246','Palacios de la Sierra','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09247','Palacios de Riopisuerga','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09248','Palazuelos de la Sierra','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09249','Palazuelos de Muñó','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09250','Pampliega','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09251','Pancorbo','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09253','Pardilla','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09255','Partido de la Sierra en Tobalina','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09256','Pedrosa de Duero','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09257','Pedrosa del Páramo','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09258','Pedrosa del Príncipe','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09259','Pedrosa de Río Úrbel','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09261','Peñaranda de Duero','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09262','Peral de Arlanza','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09265','Piérnigas','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09266','Pineda de la Sierra','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09267','Pineda Trasmonte','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09268','Pinilla de los Barruecos','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09269','Pinilla de los Moros','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09270','Pinilla Trasmonte','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09272','Poza de la Sal','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09273','Prádanos de Bureba','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09274','Pradoluengo','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09275','Presencio','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09276','Puebla de Arganzón (La)','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09277','Puentedura','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09279','Quemada','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09280','Quintanabureba','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09281','Quintana del Pidio','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09283','Quintanaélez','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09287','Quintanaortuño','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09288','Quintanapalla','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09289','Quintanar de la Sierra','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09292','Quintanavides','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09294','Quintanilla de la Mata','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09295','Quintanilla del Coco','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09297','Quintanillas (Las)','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09298','Quintanilla San García','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09301','Quintanilla Vivar','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09302','Rabanera del Pinar','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09303','Rábanos','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09304','Rabé de las Calzadas','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09306','Rebolledo de la Torre','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09307','Redecilla del Camino','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09308','Redecilla del Campo','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09309','Regumiel de la Sierra','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09310','Reinoso','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09311','Retuerta','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09312','Revilla y Ahedo (La)','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09314','Revilla del Campo','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09315','Revillarruz','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09316','Revilla Vallejera','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09317','Rezmondo','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09318','Riocavado de la Sierra','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09321','Roa','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09323','Rojas','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09325','Royuela de Río Franco','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09326','Rubena','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09327','Rublacedo de Abajo','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09328','Rucandio','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09329','Salas de Bureba','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09330','Salas de los Infantes','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09332','Saldaña de Burgos','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09334','Salinillas de Bureba','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09335','San Adrián de Juarros','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09337','San Juan del Monte','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09338','San Mamés de Burgos','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09339','San Martín de Rubiales','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09340','San Millán de Lara','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09343','Santa Cecilia','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09345','Santa Cruz de la Salceda','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09346','Santa Cruz del Valle Urbión','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09347','Santa Gadea del Cid','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09348','Santa Inés','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09350','Santa María del Campo','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09351','Santa María del Invierno','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09352','Santa María del Mercadillo','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09353','Santa María Rivarredonda','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09354','Santa Olalla de Bureba','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09355','Santibáñez de Esgueva','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09356','Santibáñez del Val','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09358','Santo Domingo de Silos','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09360','San Vicente del Valle','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09361','Sargentes de la Lora','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09362','Sarracín','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09363','Sasamón','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09365','Sequera de Haza (La)','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09366','Solarana','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09368','Sordillos','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09369','Sotillo de la Ribera','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09372','Sotragero','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09373','Sotresgudo','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09374','Susinos del Páramo','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09375','Tamarón','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09377','Tardajos','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09378','Tejada','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09380','Terradillos de Esgueva','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09381','Tinieblas de la Sierra','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09382','Tobar','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09384','Tordómar','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09386','Torrecilla del Monte','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09387','Torregalindo','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09388','Torrelara','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09389','Torrepadre','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09390','Torresandino','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09391','Tórtoles de Esgueva','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09392','Tosantos','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09394','Trespaderne','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09395','Tubilla del Agua','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09396','Tubilla del Lago','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09398','Úrbel del Castillo','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09400','Vadocondes','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09403','Valdeande','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09405','Valdezate','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09406','Valdorros','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09407','Valmala','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09408','Vallarta de Bureba','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09409','Valle de Manzanedo','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09410','Valle de Mena','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09411','Valle de Oca','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09412','Valle de Tobalina','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09413','Valle de Valdebezana','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09414','Valle de Valdelaguna','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09415','Valle de Valdelucio','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09416','Valle de Zamanzas','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09417','Vallejera','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09418','Valles de Palenzuela','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09419','Valluércanes','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09421','Vid y Barrios (La)','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09422','Vid de Bureba (La)','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09423','Vileña','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09424','Viloria de Rioja','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09425','Vilviestre del Pinar','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09427','Villadiego','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09428','Villaescusa de Roa','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09429','Villaescusa la Sombría','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09430','Villaespasa','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09431','Villafranca Montes de Oca','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09432','Villafruela','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09433','Villagalijo','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09434','Villagonzalo Pedernales','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09437','Villahoz','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09438','Villalba de Duero','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09439','Villalbilla de Burgos','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09440','Villalbilla de Gumiel','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09441','Villaldemiro','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09442','Villalmanzo','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09443','Villamayor de los Montes','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09444','Villamayor de Treviño','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09445','Villambistia','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09446','Villamedianilla','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09447','Villamiel de la Sierra','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09448','Villangómez','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09449','Villanueva de Argaño','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09450','Villanueva de Carazo','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09451','Villanueva de Gumiel','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09454','Villanueva de Teba','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09455','Villaquirán de la Puebla','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09456','Villaquirán de los Infantes','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09458','Villariezo','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09460','Villasandino','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09463','Villasur de Herreros','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09464','Villatuelda','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09466','Villaverde del Monte','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09467','Villaverde-Mogina','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09471','Villayerno Morquillas','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09472','Villazopeque','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09473','Villegas','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09476','Villoruebo','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09478','Vizcaínos','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09480','Zael','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09482','Zarzosa de Río Pisuerga','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09483','Zazuar','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09485','Zuñeda','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09901','Quintanilla del Agua y Tordueles','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09902','Valle de Santibáñez','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09903','Villarcayo de Merindad de Castilla la Vieja','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09904','Valle de las Navas','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09905','Valle de Sedano','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09906','Merindad de Río Ubierna','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09907','Alfoz de Quintanadueñas','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'09908','Valle de Losa','BU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10001','Abadía','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10002','Abertura','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10003','Acebo','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10004','Acehúche','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10005','Aceituna','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10006','Ahigal','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10007','Albalá','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10008','Alcántara','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10009','Alcollarín','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10010','Alcuéscar','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10011','Aldeacentenera','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10012','Aldea del Cano','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10013','Aldea del Obispo (La)','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10014','Aldeanueva de la Vera','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10015','Aldeanueva del Camino','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10016','Aldehuela de Jerte','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10017','Alía','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10018','Aliseda','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10019','Almaraz','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10020','Almoharín','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10021','Arroyo de la Luz','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10022','Arroyomolinos de la Vera','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10023','Arroyomolinos','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10024','Baños de Montemayor','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10025','Barrado','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10026','Belvís de Monroy','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10027','Benquerencia','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10028','Berrocalejo','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10029','Berzocana','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10030','Bohonal de Ibor','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10031','Botija','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10032','Brozas','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10033','Cabañas del Castillo','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10034','Cabezabellosa','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10035','Cabezuela del Valle','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10036','Cabrero','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10037','Cáceres','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10038','Cachorrilla','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10039','Cadalso','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10040','Calzadilla','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10041','Caminomorisco','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10042','Campillo de Deleitosa','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10043','Campo Lugar','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10044','Cañamero','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10045','Cañaveral','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10046','Carbajo','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10047','Carcaboso','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10048','Carrascalejo','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10049','Casar de Cáceres','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10050','Casar de Palomero','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10051','Casares de las Hurdes','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10052','Casas de Don Antonio','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10053','Casas de Don Gómez','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10054','Casas del Castañar','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10055','Casas del Monte','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10056','Casas de Millán','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10057','Casas de Miravete','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10058','Casatejada','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10059','Casillas de Coria','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10060','Castañar de Ibor','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10061','Ceclavín','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10062','Cedillo','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10063','Cerezo','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10064','Cilleros','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10065','Collado de la Vera','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10066','Conquista de la Sierra','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10067','Coria','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10068','Cuacos de Yuste','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10069','Cumbre (La)','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10070','Deleitosa','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10071','Descargamaría','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10072','Eljas','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10073','Escurial','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10075','Fresnedoso de Ibor','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10076','Galisteo','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10077','Garciaz','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10078','Garganta (La)','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10079','Garganta la Olla','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10080','Gargantilla','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10081','Gargüera','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10082','Garrovillas de Alconétar','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10083','Garvín','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10084','Gata','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10085','Gordo (El)','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10086','Granja (La)','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10087','Guadalupe','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10088','Guijo de Coria','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10089','Guijo de Galisteo','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10090','Guijo de Granadilla','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10091','Guijo de Santa Bárbara','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10092','Herguijuela','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10093','Hernán-Pérez','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10094','Herrera de Alcántara','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10095','Herreruela','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10096','Hervás','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10097','Higuera','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10098','Hinojal','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10099','Holguera','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10100','Hoyos','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10101','Huélaga','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10102','Ibahernando','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10103','Jaraicejo','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10104','Jaraíz de la Vera','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10105','Jarandilla de la Vera','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10106','Jarilla','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10107','Jerte','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10108','Ladrillar','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10109','Logrosán','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10110','Losar de la Vera','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10111','Madrigal de la Vera','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10112','Madrigalejo','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10113','Madroñera','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10114','Majadas','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10115','Malpartida de Cáceres','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10116','Malpartida de Plasencia','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10117','Marchagaz','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10118','Mata de Alcántara','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10119','Membrío','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10120','Mesas de Ibor','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10121','Miajadas','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10122','Millanes','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10123','Mirabel','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10124','Mohedas de Granadilla','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10125','Monroy','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10126','Montánchez','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10127','Montehermoso','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10128','Moraleja','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10129','Morcillo','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10130','Navaconcejo','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10131','Navalmoral de la Mata','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10132','Navalvillar de Ibor','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10133','Navas del Madroño','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10134','Navezuelas','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10135','Nuñomoral','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10136','Oliva de Plasencia','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10137','Palomero','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10138','Pasarón de la Vera','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10139','Pedroso de Acim','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10140','Peraleda de la Mata','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10141','Peraleda de San Román','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10142','Perales del Puerto','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10143','Pescueza','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10144','Pesga (La)','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10145','Piedras Albas','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10146','Pinofranqueado','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10147','Piornal','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10148','Plasencia','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10149','Plasenzuela','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10150','Portaje','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10151','Portezuelo','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10152','Pozuelo de Zarzón','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10153','Puerto de Santa Cruz','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10154','Rebollar','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10155','Riolobos','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10156','Robledillo de Gata','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10157','Robledillo de la Vera','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10158','Robledillo de Trujillo','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10159','Robledollano','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10160','Romangordo','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10161','Ruanes','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10162','Salorino','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10163','Salvatierra de Santiago','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10164','San Martín de Trevejo','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10165','Santa Ana','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10166','Santa Cruz de la Sierra','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10167','Santa Cruz de Paniagua','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10168','Santa Marta de Magasca','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10169','Santiago de Alcántara','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10170','Santiago del Campo','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10171','Santibáñez el Alto','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10172','Santibáñez el Bajo','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10173','Saucedilla','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10174','Segura de Toro','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10175','Serradilla','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10176','Serrejón','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10177','Sierra de Fuentes','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10178','Talaván','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10179','Talaveruela de la Vera','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10180','Talayuela','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10181','Tejeda de Tiétar','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10182','Toril','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10183','Tornavacas','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10184','Torno (El)','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10185','Torrecilla de los Ángeles','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10186','Torrecillas de la Tiesa','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10187','Torre de Don Miguel','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10188','Torre de Santa María','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10189','Torrejoncillo','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10190','Torrejón el Rubio','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10191','Torremenga','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10192','Torremocha','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10193','Torreorgaz','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10194','Torrequemada','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10195','Trujillo','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10196','Valdastillas','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10197','Valdecañas de Tajo','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10198','Valdefuentes','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10199','Valdehúncar','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10200','Valdelacasa de Tajo','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10201','Valdemorales','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10202','Valdeobispo','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10203','Valencia de Alcántara','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10204','Valverde de la Vera','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10205','Valverde del Fresno','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10206','Viandar de la Vera','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10207','Villa del Campo','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10208','Villa del Rey','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10209','Villamesías','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10210','Villamiel','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10211','Villanueva de la Sierra','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10212','Villanueva de la Vera','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10213','Villar del Pedroso','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10214','Villar de Plasencia','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10215','Villasbuenas de Gata','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10216','Zarza de Granadilla','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10217','Zarza de Montánchez','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10218','Zarza la Mayor','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10219','Zorita','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10901','Rosalejo','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10902','Vegaviana','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10903','Alagón del Río','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10904','Tiétar','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'10905','Pueblonuevo de Miramontes','CC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'11001','Alcalá de los Gazules','CA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'11002','Alcalá del Valle','CA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'11003','Algar','CA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'11004','Algeciras','CA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'11005','Algodonales','CA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'11006','Arcos de la Frontera','CA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'11007','Barbate','CA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'11008','Barrios (Los)','CA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'11009','Benaocaz','CA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'11010','Bornos','CA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'11011','Bosque (El)','CA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'11012','Cádiz','CA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'11013','Castellar de la Frontera','CA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'11014','Conil de la Frontera','CA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'11015','Chiclana de la Frontera','CA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'11016','Chipiona','CA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'11017','Espera','CA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'11018','Gastor (El)','CA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'11019','Grazalema','CA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'11020','Jerez de la Frontera','CA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'11021','Jimena de la Frontera','CA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'11022','Línea de la Concepción (La)','CA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'11023','Medina Sidonia','CA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'11024','Olvera','CA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'11025','Paterna de Rivera','CA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'11026','Prado del Rey','CA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'11027','Puerto de Santa María (El)','CA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'11028','Puerto Real','CA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'11029','Puerto Serrano','CA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'11030','Rota','CA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'11031','San Fernando','CA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'11032','Sanlúcar de Barrameda','CA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'11033','San Roque','CA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'11034','Setenil de las Bodegas','CA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'11035','Tarifa','CA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'11036','Torre Alháquime','CA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'11037','Trebujena','CA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'11038','Ubrique','CA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'11039','Vejer de la Frontera','CA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'11040','Villaluenga del Rosario','CA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'11041','Villamartín','CA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'11042','Zahara','CA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'11901','Benalup-Casas Viejas','CA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'11902','San José del Valle','CA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'11903','San Martín del Tesorillo','CA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'12001','Atzeneta del Maestrat','CS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'12002','Aín','CS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'12003','Albocàsser','CS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'12004','Alcalà de Xivert','CS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'12005','Alcora (l'')','CS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'12006','Alcudia de Veo','CS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'12007','Alfondeguilla','CS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'12008','Algimia de Almonacid','CS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'12009','Almassora','CS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'12010','Almedíjar','CS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'12011','Almenara','CS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'12012','Altura','CS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'12013','Arañuel','CS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'12014','Ares del Maestrat','CS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'12015','Argelita','CS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'12016','Artana','CS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'12017','Ayódar','CS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'12018','Azuébar','CS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'12020','Barracas','CS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'12021','Betxí','CS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'12022','Bejís','CS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'12024','Benafer','CS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'12025','Benafigos','CS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'12026','Benassal','CS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'12027','Benicarló','CS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'12028','Benicasim/Benicàssim','CS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'12029','Benlloc','CS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'12031','Borriol','CS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'12032','Burriana','CS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'12033','Cabanes','CS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'12034','Càlig','CS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'12036','Canet lo Roig','CS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'12037','Castell de Cabres','CS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'12038','Castellfort','CS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'12039','Castellnovo','CS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'12040','Castellón de la Plana/Castelló de la Plana','CS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'12041','Castillo de Villamalefa','CS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'12042','Catí','CS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'12043','Caudiel','CS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'12044','Cervera del Maestre','CS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'12045','Cinctorres','CS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'12046','Cirat','CS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'12048','Cortes de Arenoso','CS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'12049','Costur','CS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'12050','Coves de Vinromà (les)','CS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'12051','Culla','CS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'12052','Xert','CS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'12053','Chilches/Xilxes','CS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'12055','Chodos/Xodos','CS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'12056','Chóvar','CS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'12057','Eslida','CS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'12058','Espadilla','CS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'12059','Fanzara','CS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'12060','Figueroles','CS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'12061','Forcall','CS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'12063','Fuente la Reina','CS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'12064','Fuentes de Ayódar','CS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'12065','Gaibiel','CS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'12067','Geldo','CS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'12068','Herbés','CS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'12069','Higueras','CS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'12070','Jana (la)','CS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'12071','Jérica','CS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'12072','Llucena/Lucena del Cid','CS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'12073','Ludiente','CS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'12074','Llosa (la)','CS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'12075','Mata de Morella (la)','CS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'12076','Matet','CS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'12077','Moncofa','CS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'12078','Montán','CS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'12079','Montanejos','CS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'12080','Morella','CS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'12081','Navajas','CS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'12082','Nules','CS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'12083','Olocau del Rey','CS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'12084','Onda','CS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'12085','Oropesa del Mar/Orpesa','CS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'12087','Palanques','CS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'12088','Pavías','CS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'12089','Peñíscola','CS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'12090','Pina de Montalgrao','CS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'12091','Portell de Morella','CS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'12092','Puebla de Arenoso','CS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'12093','Pobla de Benifassà (la)','CS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'12094','Pobla Tornesa (la)','CS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'12095','Ribesalbes','CS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'12096','Rossell','CS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'12097','Sacañet','CS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'12098','Salzadella (la)','CS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'12099','Sant Jordi/San Jorge','CS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'12100','Sant Mateu','CS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'12101','San Rafael del Río','CS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'12102','Santa Magdalena de Pulpis','CS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'12103','Sarratella, la','CS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'12104','Segorbe','CS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'12105','Sierra Engarcerán','CS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'12106','Soneja','CS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'12107','Sot de Ferrer','CS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'12108','Sueras/Suera','CS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'12109','Tales','CS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'12110','Teresa','CS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'12111','Tírig','CS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'12112','Todolella','CS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'12113','Toga','CS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'12114','Torás','CS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'12115','Toro (El)','CS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'12116','Torralba del Pinar','CS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'12117','Torreblanca','CS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'12118','Torrechiva','CS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'12119','Torre d''En Besora (la)','CS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'12120','Torre Endoménech','CS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'12121','Traiguera','CS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'12122','Useras/Useres (les)','CS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'12123','Vallat','CS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'12124','Vall d''Alba','CS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'12125','Vall de Almonacid','CS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'12126','Vall d''Uixó (la)','CS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'12127','Vallibona','CS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'12128','Vilafamés','CS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'12129','Vilafranca/Villafranca del Cid','CS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'12130','Villahermosa del Río','CS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'12131','Villamalur','CS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'12132','Vilanova d''Alcolea','CS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'12133','Villanueva de Viver','CS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'12134','Vilar de Canes','CS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'12135','Vila-real','CS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'12136','Vilavella (la)','CS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'12137','Villores','CS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'12138','Vinaròs','CS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'12139','Vistabella del Maestrat','CS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'12140','Viver','CS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'12141','Zorita del Maestrazgo','CS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'12142','Zucaina','CS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'12901','Alqueries, les/Alquerías del Niño Perdido','CS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'12902','Sant Joan de Moró','CS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'13001','Abenójar','CR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'13002','Agudo','CR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'13003','Alamillo','CR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'13004','Albaladejo','CR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'13005','Alcázar de San Juan','CR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'13006','Alcoba','CR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'13007','Alcolea de Calatrava','CR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'13008','Alcubillas','CR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'13009','Aldea del Rey','CR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'13010','Alhambra','CR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'13011','Almadén','CR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'13012','Almadenejos','CR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'13013','Almagro','CR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'13014','Almedina','CR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'13015','Almodóvar del Campo','CR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'13016','Almuradiel','CR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'13017','Anchuras','CR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'13018','Arenas de San Juan','CR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'13019','Argamasilla de Alba','CR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'13020','Argamasilla de Calatrava','CR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'13021','Arroba de los Montes','CR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'13022','Ballesteros de Calatrava','CR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'13023','Bolaños de Calatrava','CR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'13024','Brazatortas','CR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'13025','Cabezarados','CR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'13026','Cabezarrubias del Puerto','CR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'13027','Calzada de Calatrava','CR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'13028','Campo de Criptana','CR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'13029','Cañada de Calatrava','CR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'13030','Caracuel de Calatrava','CR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'13031','Carrión de Calatrava','CR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'13032','Carrizosa','CR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'13033','Castellar de Santiago','CR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'13034','Ciudad Real','CR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'13035','Corral de Calatrava','CR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'13036','Cortijos (Los)','CR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'13037','Cózar','CR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'13038','Chillón','CR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'13039','Daimiel','CR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'13040','Fernán Caballero','CR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'13041','Fontanarejo','CR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'13042','Fuencaliente','CR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'13043','Fuenllana','CR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'13044','Fuente el Fresno','CR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'13045','Granátula de Calatrava','CR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'13046','Guadalmez','CR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'13047','Herencia','CR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'13048','Hinojosas de Calatrava','CR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'13049','Horcajo de los Montes','CR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'13050','Labores (Las)','CR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'13051','Luciana','CR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'13052','Malagón','CR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'13053','Manzanares','CR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'13054','Membrilla','CR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'13055','Mestanza','CR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'13056','Miguelturra','CR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'13057','Montiel','CR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'13058','Moral de Calatrava','CR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'13059','Navalpino','CR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'13060','Navas de Estena','CR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'13061','Pedro Muñoz','CR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'13062','Picón','CR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'13063','Piedrabuena','CR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'13064','Poblete','CR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'13065','Porzuna','CR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'13066','Pozuelo de Calatrava','CR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'13067','Pozuelos de Calatrava (Los)','CR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'13068','Puebla de Don Rodrigo','CR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'13069','Puebla del Príncipe','CR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'13070','Puerto Lápice','CR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'13071','Puertollano','CR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'13072','Retuerta del Bullaque','CR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'13073','Saceruela','CR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'13074','San Carlos del Valle','CR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'13075','San Lorenzo de Calatrava','CR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'13076','Santa Cruz de los Cáñamos','CR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'13077','Santa Cruz de Mudela','CR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'13078','Socuéllamos','CR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'13079','Solana (La)','CR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'13080','Solana del Pino','CR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'13081','Terrinches','CR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'13082','Tomelloso','CR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'13083','Torralba de Calatrava','CR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'13084','Torre de Juan Abad','CR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'13085','Torrenueva','CR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'13086','Valdemanco del Esteras','CR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'13087','Valdepeñas','CR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'13088','Valenzuela de Calatrava','CR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'13089','Villahermosa','CR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'13090','Villamanrique','CR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'13091','Villamayor de Calatrava','CR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'13092','Villanueva de la Fuente','CR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'13093','Villanueva de los Infantes','CR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'13094','Villanueva de San Carlos','CR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'13095','Villar del Pozo','CR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'13096','Villarrubia de los Ojos','CR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'13097','Villarta de San Juan','CR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'13098','Viso del Marqués','CR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'13901','Robledo (El)','CR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'13902','Ruidera','CR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'13903','Arenales de San Gregorio','CR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'13904','Llanos del Caudillo','CR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'14001','Adamuz','CO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'14002','Aguilar de la Frontera','CO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'14003','Alcaracejos','CO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'14004','Almedinilla','CO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'14005','Almodóvar del Río','CO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'14006','Añora','CO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'14007','Baena','CO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'14008','Belalcázar','CO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'14009','Belmez','CO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'14010','Benamejí','CO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'14011','Blázquez (Los)','CO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'14012','Bujalance','CO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'14013','Cabra','CO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'14014','Cañete de las Torres','CO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'14015','Carcabuey','CO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'14016','Cardeña','CO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'14017','Carlota (La)','CO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'14018','Carpio (El)','CO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'14019','Castro del Río','CO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'14020','Conquista','CO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'14021','Córdoba','CO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'14022','Doña Mencía','CO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'14023','Dos Torres','CO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'14024','Encinas Reales','CO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'14025','Espejo','CO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'14026','Espiel','CO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'14027','Fernán-Núñez','CO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'14028','Fuente la Lancha','CO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'14029','Fuente Obejuna','CO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'14030','Fuente Palmera','CO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'14031','Fuente-Tójar','CO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'14032','Granjuela (La)','CO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'14033','Guadalcázar','CO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'14034','Guijo (El)','CO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'14035','Hinojosa del Duque','CO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'14036','Hornachuelos','CO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'14037','Iznájar','CO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'14038','Lucena','CO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'14039','Luque','CO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'14040','Montalbán de Córdoba','CO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'14041','Montemayor','CO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'14042','Montilla','CO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'14043','Montoro','CO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'14044','Monturque','CO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'14045','Moriles','CO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'14046','Nueva Carteya','CO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'14047','Obejo','CO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'14048','Palenciana','CO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'14049','Palma del Río','CO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'14050','Pedro Abad','CO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'14051','Pedroche','CO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'14052','Peñarroya-Pueblonuevo','CO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'14053','Posadas','CO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'14054','Pozoblanco','CO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'14055','Priego de Córdoba','CO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'14056','Puente Genil','CO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'14057','Rambla (La)','CO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'14058','Rute','CO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'14059','San Sebastián de los Ballesteros','CO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'14060','Santaella','CO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'14061','Santa Eufemia','CO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'14062','Torrecampo','CO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'14063','Valenzuela','CO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'14064','Valsequillo','CO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'14065','Victoria (La)','CO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'14066','Villa del Río','CO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'14067','Villafranca de Córdoba','CO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'14068','Villaharta','CO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'14069','Villanueva de Córdoba','CO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'14070','Villanueva del Duque','CO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'14071','Villanueva del Rey','CO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'14072','Villaralto','CO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'14073','Villaviciosa de Córdoba','CO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'14074','Viso (El)','CO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'14075','Zuheros','CO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'14901','Fuente Carreteros','CO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'14902','Guijarrosa, La','CO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'15001','Abegondo','C','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'15002','Ames','C','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'15003','Aranga','C','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'15004','Ares','C','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'15005','Arteixo','C','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'15006','Arzúa','C','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'15007','Baña (A)','C','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'15008','Bergondo','C','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'15009','Betanzos','C','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'15010','Boimorto','C','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'15011','Boiro','C','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'15012','Boqueixón','C','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'15013','Brión','C','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'15014','Cabana de Bergantiños','C','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'15015','Cabanas','C','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'15016','Camariñas','C','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'15017','Cambre','C','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'15018','Capela (A)','C','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'15019','Carballo','C','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'15020','Carnota','C','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'15021','Carral','C','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'15022','Cedeira','C','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'15023','Cee','C','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'15024','Cerceda','C','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'15025','Cerdido','C','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'15027','Coirós','C','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'15028','Corcubión','C','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'15029','Coristanco','C','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'15030','Coruña (A)','C','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'15031','Culleredo','C','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'15032','Curtis','C','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'15033','Dodro','C','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'15034','Dumbría','C','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'15035','Fene','C','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'15036','Ferrol','C','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'15037','Fisterra','C','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'15038','Frades','C','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'15039','Irixoa','C','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'15040','Laxe','C','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'15041','Laracha (A)','C','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'15042','Lousame','C','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'15043','Malpica de Bergantiños','C','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'15044','Mañón','C','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'15045','Mazaricos','C','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'15046','Melide','C','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'15047','Mesía','C','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'15048','Miño','C','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'15049','Moeche','C','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'15050','Monfero','C','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'15051','Mugardos','C','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'15052','Muxía','C','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'15053','Muros','C','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'15054','Narón','C','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'15055','Neda','C','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'15056','Negreira','C','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'15057','Noia','C','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'15058','Oleiros','C','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'15059','Ordes','C','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'15060','Oroso','C','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'15061','Ortigueira','C','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'15062','Outes','C','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'15064','Paderne','C','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'15065','Padrón','C','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'15066','Pino (O)','C','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'15067','Pobra do Caramiñal (A)','C','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'15068','Ponteceso','C','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'15069','Pontedeume','C','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'15070','Pontes de García Rodríguez (As)','C','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'15071','Porto do Son','C','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'15072','Rianxo','C','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'15073','Ribeira','C','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'15074','Rois','C','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'15075','Sada','C','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'15076','San Sadurniño','C','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'15077','Santa Comba','C','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'15078','Santiago de Compostela','C','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'15079','Santiso','C','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'15080','Sobrado','C','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'15081','Somozas (As)','C','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'15082','Teo','C','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'15083','Toques','C','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'15084','Tordoia','C','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'15085','Touro','C','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'15086','Trazo','C','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'15087','Valdoviño','C','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'15088','Val do Dubra','C','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'15089','Vedra','C','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'15090','Vilasantar','C','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'15091','Vilarmaior','C','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'15092','Vimianzo','C','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'15093','Zas','C','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'15901','Cariño','C','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'15902','Oza-Cesuras','C','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16001','Abia de la Obispalía','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16002','Acebrón (El)','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16003','Alarcón','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16004','Albaladejo del Cuende','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16005','Albalate de las Nogueras','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16006','Albendea','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16007','Alberca de Záncara (La)','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16008','Alcalá de la Vega','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16009','Alcantud','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16010','Alcázar del Rey','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16011','Alcohujate','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16012','Alconchel de la Estrella','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16013','Algarra','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16014','Aliaguilla','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16015','Almarcha (La)','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16016','Almendros','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16017','Almodóvar del Pinar','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16018','Almonacid del Marquesado','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16019','Altarejos','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16020','Arandilla del Arroyo','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16022','Arcos de la Sierra','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16023','Chillarón de Cuenca','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16024','Arguisuelas','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16025','Arrancacepas','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16026','Atalaya del Cañavate','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16027','Barajas de Melo','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16029','Barchín del Hoyo','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16030','Bascuñana de San Pedro','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16031','Beamud','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16032','Belinchón','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16033','Belmonte','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16034','Belmontejo','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16035','Beteta','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16036','Boniches','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16038','Buciegas','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16039','Buenache de Alarcón','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16040','Buenache de la Sierra','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16041','Buendía','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16042','Campillo de Altobuey','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16043','Campillos-Paravientos','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16044','Campillos-Sierra','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16045','Canalejas del Arroyo','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16046','Cañada del Hoyo','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16047','Cañada Juncosa','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16048','Cañamares','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16049','Cañavate (El)','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16050','Cañaveras','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16051','Cañaveruelas','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16052','Cañete','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16053','Cañizares','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16055','Carboneras de Guadazaón','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16056','Cardenete','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16057','Carrascosa','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16058','Carrascosa de Haro','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16060','Casas de Benítez','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16061','Casas de Fernando Alonso','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16062','Casas de Garcimolina','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16063','Casas de Guijarro','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16064','Casas de Haro','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16065','Casas de los Pinos','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16066','Casasimarro','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16067','Castejón','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16068','Castillejo de Iniesta','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16070','Castillejo-Sierra','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16071','Castillo-Albaráñez','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16072','Castillo de Garcimuñoz','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16073','Cervera del Llano','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16074','Cierva (La)','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16078','Cuenca','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16079','Cueva del Hierro','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16081','Chumillas','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16082','Enguídanos','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16083','Fresneda de Altarejos','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16084','Fresneda de la Sierra','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16085','Frontera (La)','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16086','Fuente de Pedro Naharro','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16087','Fuentelespino de Haro','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16088','Fuentelespino de Moya','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16089','Fuentes','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16091','Fuertescusa','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16092','Gabaldón','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16093','Garaballa','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16094','Gascueña','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16095','Graja de Campalbo','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16096','Graja de Iniesta','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16097','Henarejos','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16098','Herrumblar (El)','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16099','Hinojosa (La)','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16100','Hinojosos (Los)','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16101','Hito (El)','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16102','Honrubia','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16103','Hontanaya','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16104','Hontecillas','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16106','Horcajo de Santiago','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16107','Huélamo','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16108','Huelves','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16109','Huérguina','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16110','Huerta de la Obispalía','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16111','Huerta del Marquesado','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16112','Huete','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16113','Iniesta','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16115','Laguna del Marquesado','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16116','Lagunaseca','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16117','Landete','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16118','Ledaña','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16119','Leganiel','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16121','Majadas (Las)','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16122','Mariana','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16123','Masegosa','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16124','Mesas (Las)','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16125','Minglanilla','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16126','Mira','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16128','Monreal del Llano','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16129','Montalbanejo','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16130','Montalbo','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16131','Monteagudo de las Salinas','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16132','Mota de Altarejos','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16133','Mota del Cuervo','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16134','Motilla del Palancar','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16135','Moya','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16137','Narboneta','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16139','Olivares de Júcar','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16140','Olmeda de la Cuesta','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16141','Olmeda del Rey','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16142','Olmedilla de Alarcón','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16143','Olmedilla de Eliz','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16145','Osa de la Vega','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16146','Pajarón','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16147','Pajaroncillo','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16148','Palomares del Campo','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16149','Palomera','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16150','Paracuellos','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16151','Paredes','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16152','Parra de las Vegas (La)','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16153','Pedernoso (El)','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16154','Pedroñeras (Las)','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16155','Peral (El)','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16156','Peraleja (La)','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16157','Pesquera (La)','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16158','Picazo (El)','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16159','Pinarejo','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16160','Pineda de Gigüela','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16161','Piqueras del Castillo','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16162','Portalrubio de Guadamejud','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16163','Portilla','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16165','Poyatos','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16166','Pozoamargo','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16167','Pozorrubio de Santiago','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16169','Pozuelo (El)','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16170','Priego','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16171','Provencio (El)','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16172','Puebla de Almenara','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16173','Valle de Altomira, El','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16174','Puebla del Salvador','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16175','Quintanar del Rey','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16176','Rada de Haro','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16177','Reíllo','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16181','Rozalén del Monte','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16185','Saceda-Trasierra','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16186','Saelices','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16187','Salinas del Manzano','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16188','Salmeroncillos','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16189','Salvacañete','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16190','San Clemente','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16191','San Lorenzo de la Parrilla','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16192','San Martín de Boniches','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16193','San Pedro Palmiches','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16194','Santa Cruz de Moya','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16195','Santa María del Campo Rus','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16196','Santa María de los Llanos','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16197','Santa María del Val','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16198','Sisante','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16199','Solera de Gabaldón','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16202','Talayuelas','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16203','Tarancón','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16204','Tébar','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16205','Tejadillos','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16206','Tinajas','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16209','Torralba','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16211','Torrejoncillo del Rey','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16212','Torrubia del Campo','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16213','Torrubia del Castillo','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16215','Tragacete','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16216','Tresjuncos','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16217','Tribaldos','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16218','Uclés','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16219','Uña','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16224','Valdemeca','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16225','Valdemorillo de la Sierra','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16227','Valdemoro-Sierra','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16228','Valdeolivas','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16231','Valhermoso de la Fuente','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16234','Valsalobre','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16236','Valverde de Júcar','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16237','Valverdejo','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16238','Vara de Rey','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16239','Vega del Codorno','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16240','Vellisca','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16242','Villaconejos de Trabaque','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16243','Villaescusa de Haro','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16244','Villagarcía del Llano','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16245','Villalba de la Sierra','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16246','Villalba del Rey','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16247','Villalgordo del Marquesado','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16248','Villalpardo','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16249','Villamayor de Santiago','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16250','Villanueva de Guadamejud','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16251','Villanueva de la Jara','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16253','Villar de Cañas','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16254','Villar de Domingo García','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16255','Villar de la Encina','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16258','Villar del Humo','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16259','Villar del Infantado','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16263','Villar de Olalla','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16264','Villarejo de Fuentes','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16265','Villarejo de la Peñuela','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16266','Villarejo-Periesteban','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16269','Villares del Saz','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16270','Villarrubio','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16271','Villarta','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16272','Villas de la Ventosa','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16273','Villaverde y Pasaconsol','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16274','Víllora','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16275','Vindel','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16276','Yémeda','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16277','Zafra de Záncara','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16278','Zafrilla','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16279','Zarza de Tajo','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16280','Zarzuela','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16901','Campos del Paraíso','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16902','Valdetórtola','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16903','Valeras (Las)','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16904','Fuentenava de Jábaga','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16905','Arcas','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16906','Valdecolmenas (Los)','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16908','Pozorrubielos de la Mancha','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16909','Sotorribas','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'16910','Villar y Velasco','CU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17001','Agullana','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17002','Aiguaviva','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17003','Albanyà','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17004','Albons','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17005','Far d''Empordà (El)','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17006','Alp','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17007','Amer','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17008','Anglès','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17009','Arbúcies','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17010','Argelaguer','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17011','Armentera (L'')','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17012','Avinyonet de Puigventós','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17013','Begur','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17014','Vajol (La)','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17015','Banyoles','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17016','Bàscara','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17018','Bellcaire d''Empordà','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17019','Besalú','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17020','Bescanó','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17021','Beuda','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17022','Bisbal d''Empordà (La)','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17023','Blanes','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17024','Bolvir','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17025','Bordils','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17026','Borrassà','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17027','Breda','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17028','Brunyola i Sant Martí Sapresa','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17029','Boadella i les Escaules','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17030','Cabanes','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17031','Cabanelles','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17032','Cadaqués','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17033','Caldes de Malavella','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17034','Calonge i Sant Antoni','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17035','Camós','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17036','Campdevànol','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17037','Campelles','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17038','Campllong','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17039','Camprodon','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17040','Canet d''Adri','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17041','Cantallops','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17042','Capmany','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17043','Queralbs','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17044','Cassà de la Selva','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17046','Castellfollit de la Roca','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17047','Castelló d''Empúries','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17048','Castell-Platja d''Aro','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17049','Celrà','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17050','Cervià de Ter','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17051','Cistella','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17052','Siurana','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17054','Colera','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17055','Colomers','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17056','Cornellà del Terri','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17057','Corçà','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17058','Crespià','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17060','Darnius','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17061','Das','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17062','Escala (L'')','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17063','Espinelves','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17064','Espolla','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17065','Esponellà','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17066','Figueres','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17067','Flaçà','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17068','Foixà','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17069','Fontanals de Cerdanya','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17070','Fontanilles','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17071','Fontcoberta','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17073','Fornells de la Selva','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17074','Fortià','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17075','Garrigàs','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17076','Garrigoles','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17077','Garriguella','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17078','Ger','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17079','Girona','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17080','Gombrèn','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17081','Gualta','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17082','Guils de Cerdanya','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17083','Hostalric','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17084','Isòvol','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17085','Jafre','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17086','Jonquera (La)','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17087','Juià','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17088','Lladó','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17089','Llagostera','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17090','Llambilles','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17091','Llanars','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17092','Llançà','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17093','Llers','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17094','Llívia','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17095','Lloret de Mar','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17096','Llosses (Les)','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17097','Madremanya','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17098','Maià de Montcal','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17099','Meranges','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17100','Masarac','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17101','Massanes','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17102','Maçanet de Cabrenys','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17103','Maçanet de la Selva','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17105','Mieres','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17106','Mollet de Peralada','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17107','Molló','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17109','Montagut i Oix','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17110','Mont-ras','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17111','Navata','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17112','Ogassa','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17114','Olot','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17115','Ordis','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17116','Osor','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17117','Palafrugell','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17118','Palamós','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17119','Palau de Santa Eulàlia','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17120','Palau-saverdera','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17121','Palau-sator','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17123','Palol de Revardit','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17124','Pals','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17125','Pardines','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17126','Parlavà','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17128','Pau','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17129','Pedret i Marzà','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17130','Pera (La)','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17132','Peralada','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17133','Planes d''Hostoles (Les)','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17134','Planoles','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17135','Pont de Molins','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17136','Pontós','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17137','Porqueres','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17138','Portbou','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17139','Preses (Les)','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17140','Port de la Selva (El)','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17141','Puigcerdà','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17142','Quart','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17143','Rabós','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17144','Regencós','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17145','Ribes de Freser','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17146','Riells i Viabrea','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17147','Ripoll','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17148','Riudarenes','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17149','Riudaura','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17150','Riudellots de la Selva','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17151','Riumors','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17152','Roses','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17153','Rupià','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17154','Sales de Llierca','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17155','Salt','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17157','Sant Andreu Salou','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17158','Sant Climent Sescebes','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17159','Sant Feliu de Buixalleu','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17160','Sant Feliu de Guíxols','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17161','Sant Feliu de Pallerols','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17162','Sant Ferriol','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17163','Sant Gregori','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17164','Sant Hilari Sacalm','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17165','Sant Jaume de Llierca','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17166','Sant Jordi Desvalls','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17167','Sant Joan de les Abadesses','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17168','Sant Joan de Mollet','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17169','Sant Julià de Ramis','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17170','Vallfogona de Ripollès','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17171','Sant Llorenç de la Muga','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17172','Sant Martí de Llémena','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17173','Sant Martí Vell','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17174','Sant Miquel de Campmajor','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17175','Sant Miquel de Fluvià','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17176','Sant Mori','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17177','Sant Pau de Segúries','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17178','Sant Pere Pescador','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17180','Santa Coloma de Farners','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17181','Santa Cristina d''Aro','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17182','Santa Llogaia d''Àlguema','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17183','Sant Aniol de Finestres','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17184','Santa Pau','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17185','Sant Joan les Fonts','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17186','Sarrià de Ter','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17187','Saus, Camallera i Llampaies','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17188','Selva de Mar (La)','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17189','Cellera de Ter (La)','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17190','Serinyà','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17191','Serra de Daró','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17192','Setcases','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17193','Sils','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17194','Susqueda','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17195','Tallada d''Empordà (La)','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17196','Terrades','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17197','Torrent','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17198','Torroella de Fluvià','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17199','Torroella de Montgrí','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17200','Tortellà','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17201','Toses','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17202','Tossa de Mar','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17203','Ultramort','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17204','Ullà','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17205','Ullastret','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17206','Urús','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17207','Vall d''en Bas (La)','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17208','Vall de Bianya (La)','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17209','Vall-llobrega','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17210','Ventalló','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17211','Verges','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17212','Vidrà','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17213','Vidreres','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17214','Vilabertran','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17215','Vilablareix','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17216','Viladasens','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17217','Viladamat','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17218','Vilademuls','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17220','Viladrau','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17221','Vilafant','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17222','Vilaür','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17223','Vilajuïga','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17224','Vilallonga de Ter','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17225','Vilamacolum','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17226','Vilamalla','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17227','Vilamaniscle','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17228','Vilanant','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17230','Vila-sacra','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17232','Vilopriu','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17233','Vilobí d''Onyar','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17234','Biure','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17901','Cruïlles, Monells i Sant Sadurní de l''Heura','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17902','Forallac','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'17903','Sant Julià del Llor i Bonmatí','GI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'18001','Agrón','GR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'18002','Alamedilla','GR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'18003','Albolote','GR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'18004','Albondón','GR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'18005','Albuñán','GR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'18006','Albuñol','GR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'18007','Albuñuelas','GR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'18010','Aldeire','GR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'18011','Alfacar','GR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'18012','Algarinejo','GR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'18013','Alhama de Granada','GR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'18014','Alhendín','GR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'18015','Alicún de Ortega','GR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'18016','Almegíjar','GR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'18017','Almuñécar','GR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'18018','Alquife','GR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'18020','Arenas del Rey','GR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'18021','Armilla','GR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'18022','Atarfe','GR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'18023','Baza','GR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'18024','Beas de Granada','GR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'18025','Beas de Guadix','GR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'18027','Benalúa','GR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'18028','Benalúa de las Villas','GR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'18029','Benamaurel','GR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'18030','Bérchules','GR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'18032','Bubión','GR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'18033','Busquístar','GR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'18034','Cacín','GR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'18035','Cádiar','GR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'18036','Cájar','GR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'18037','Calicasas','GR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'18038','Campotéjar','GR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'18039','Caniles','GR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'18040','Cáñar','GR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'18042','Capileira','GR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'18043','Carataunas','GR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'18044','Cástaras','GR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'18045','Castilléjar','GR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'18046','Castril','GR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'18047','Cenes de la Vega','GR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'18048','Cijuela','GR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'18049','Cogollos de Guadix','GR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'18050','Cogollos de la Vega','GR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'18051','Colomera','GR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'18053','Cortes de Baza','GR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'18054','Cortes y Graena','GR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'18056','Cúllar','GR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'18057','Cúllar Vega','GR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'18059','Chauchina','GR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'18061','Chimeneas','GR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'18062','Churriana de la Vega','GR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'18063','Darro','GR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'18064','Dehesas de Guadix','GR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'18065','Dehesas Viejas','GR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'18066','Deifontes','GR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'18067','Diezma','GR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'18068','Dílar','GR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'18069','Dólar','GR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'18070','Dúdar','GR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'18071','Dúrcal','GR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'18072','Escúzar','GR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'18074','Ferreira','GR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'18076','Fonelas','GR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'18077','Fornes','GR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'18078','Freila','GR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'18079','Fuente Vaqueros','GR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'18082','Galera','GR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'18083','Gobernador','GR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'18084','Gójar','GR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'18085','Gor','GR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'18086','Gorafe','GR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'18087','Granada','GR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'18088','Guadahortuna','GR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'18089','Guadix','GR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'18093','Gualchos','GR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'18094','Güéjar Sierra','GR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'18095','Güevéjar','GR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'18096','Huélago','GR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'18097','Huéneja','GR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'18098','Huéscar','GR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'18099','Huétor de Santillán','GR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'18100','Huétor Tájar','GR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'18101','Huétor Vega','GR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'18102','Íllora','GR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'18103','Ítrabo','GR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'18105','Iznalloz','GR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'18106','Játar','GR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'18107','Jayena','GR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'18108','Jérez del Marquesado','GR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'18109','Jete','GR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'18111','Jun','GR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'18112','Juviles','GR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'18114','Calahorra (La)','GR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'18115','Láchar','GR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'18116','Lanjarón','GR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'18117','Lanteira','GR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'18119','Lecrín','GR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'18120','Lentegí','GR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'18121','Lobras','GR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'18122','Loja','GR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'18123','Lugros','GR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'18124','Lújar','GR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'18126','Malahá (La)','GR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'18127','Maracena','GR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'18128','Marchal','GR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'18132','Moclín','GR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'18133','Molvízar','GR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'18134','Monachil','GR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'18135','Montefrío','GR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'18136','Montejícar','GR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'18137','Montillana','GR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'18138','Moraleda de Zafayona','GR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'18140','Motril','GR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'18141','Murtas','GR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'18143','Nigüelas','GR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'18144','Nívar','GR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'18145','Ogíjares','GR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'18146','Orce','GR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'18147','Órgiva','GR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'18148','Otívar','GR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'18149','Villa de Otura','GR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'18150','Padul','GR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'18151','Pampaneira','GR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'18152','Pedro Martínez','GR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'18153','Peligros','GR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'18154','Peza (La)','GR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'18157','Pinos Genil','GR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'18158','Pinos Puente','GR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'18159','Píñar','GR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'18161','Polícar','GR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'18162','Polopos','GR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'18163','Pórtugos','GR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'18164','Puebla de Don Fadrique','GR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'18165','Pulianas','GR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'18167','Purullena','GR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'18168','Quéntar','GR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'18170','Rubite','GR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'18171','Salar','GR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'18173','Salobreña','GR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'18174','Santa Cruz del Comercio','GR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'18175','Santa Fe','GR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'18176','Soportújar','GR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'18177','Sorvilán','GR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'18178','Torre-Cardela','GR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'18179','Torvizcón','GR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'18180','Trevélez','GR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'18181','Turón','GR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'18182','Ugíjar','GR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'18183','Válor','GR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'18184','Vélez de Benaudalla','GR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'18185','Ventas de Huelma','GR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'18187','Villanueva de las Torres','GR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'18188','Villanueva Mesía','GR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'18189','Víznar','GR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'18192','Zafarraya','GR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'18193','Zubia (La)','GR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'18194','Zújar','GR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'18901','Taha (La)','GR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'18902','Valle (El)','GR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'18903','Nevada','GR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'18904','Alpujarra de la Sierra','GR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'18905','Gabias (Las)','GR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'18906','Guájares, Los','GR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'18907','Valle del Zalabí','GR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'18908','Villamena','GR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'18909','Morelábor','GR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'18910','Pinar (El)','GR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'18911','Vegas del Genil','GR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'18912','Cuevas del Campo','GR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'18913','Zagra','GR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'18914','Valderrubio','GR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'18915','Domingo Pérez de Granada','GR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'18916','Torrenueva Costa','GR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19001','Abánades','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19002','Ablanque','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19003','Adobes','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19004','Alaminos','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19005','Alarilla','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19006','Albalate de Zorita','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19007','Albares','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19008','Albendiego','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19009','Alcocer','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19010','Alcolea de las Peñas','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19011','Alcolea del Pinar','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19013','Alcoroches','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19015','Aldeanueva de Guadalajara','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19016','Algar de Mesa','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19017','Algora','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19018','Alhóndiga','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19019','Alique','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19020','Almadrones','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19021','Almoguera','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19022','Almonacid de Zorita','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19023','Alocén','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19024','Alovera','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19027','Alustante','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19031','Angón','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19032','Anguita','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19033','Anquela del Ducado','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19034','Anquela del Pedregal','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19036','Aranzueque','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19037','Arbancón','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19038','Arbeteta','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19039','Argecilla','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19040','Armallones','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19041','Armuña de Tajuña','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19042','Arroyo de las Fraguas','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19043','Atanzón','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19044','Atienza','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19045','Auñón','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19046','Azuqueca de Henares','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19047','Baides','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19048','Baños de Tajo','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19049','Bañuelos','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19050','Barriopedro','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19051','Berninches','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19052','Bodera (La)','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19053','Brihuega','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19054','Budia','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19055','Bujalaro','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19057','Bustares','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19058','Cabanillas del Campo','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19059','Campillo de Dueñas','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19060','Campillo de Ranas','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19061','Campisábalos','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19064','Canredondo','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19065','Cantalojas','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19066','Cañizar','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19067','Cardoso de la Sierra (El)','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19070','Casa de Uceda','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19071','Casar (El)','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19073','Casas de San Galindo','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19074','Caspueñas','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19075','Castejón de Henares','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19076','Castellar de la Muela','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19078','Castilforte','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19079','Castilnuevo','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19080','Cendejas de Enmedio','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19081','Cendejas de la Torre','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19082','Centenera','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19086','Cifuentes','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19087','Cincovillas','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19088','Ciruelas','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19089','Ciruelos del Pinar','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19090','Cobeta','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19091','Cogollor','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19092','Cogolludo','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19095','Condemios de Abajo','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19096','Condemios de Arriba','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19097','Congostrina','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19098','Copernal','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19099','Corduente','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19102','Cubillo de Uceda (El)','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19103','Checa','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19104','Chequilla','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19105','Chiloeches','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19106','Chillarón del Rey','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19107','Driebes','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19108','Durón','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19109','Embid','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19110','Escamilla','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19111','Escariche','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19112','Escopete','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19113','Espinosa de Henares','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19114','Esplegares','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19115','Establés','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19116','Estriégana','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19117','Fontanar','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19118','Fuembellida','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19119','Fuencemillán','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19120','Fuentelahiguera de Albatages','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19121','Fuentelencina','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19122','Fuentelsaz','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19123','Fuentelviejo','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19124','Fuentenovilla','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19125','Gajanejos','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19126','Galápagos','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19127','Galve de Sorbe','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19129','Gascueña de Bornova','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19130','Guadalajara','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19132','Henche','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19133','Heras de Ayuso','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19134','Herrería','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19135','Hiendelaencina','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19136','Hijes','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19138','Hita','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19139','Hombrados','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19142','Hontoba','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19143','Horche','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19145','Hortezuela de Océn','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19146','Huerce (La)','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19147','Huérmeces del Cerro','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19148','Huertahernando','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19150','Hueva','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19151','Humanes','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19152','Illana','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19153','Iniéstola','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19154','Inviernas (Las)','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19155','Irueste','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19156','Jadraque','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19157','Jirueque','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19159','Ledanca','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19160','Loranca de Tajuña','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19161','Lupiana','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19162','Luzaga','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19163','Luzón','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19165','Majaelrayo','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19166','Málaga del Fresno','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19167','Malaguilla','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19168','Mandayona','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19169','Mantiel','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19170','Maranchón','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19171','Marchamalo','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19172','Masegoso de Tajuña','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19173','Matarrubia','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19174','Matillas','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19175','Mazarete','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19176','Mazuecos','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19177','Medranda','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19178','Megina','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19179','Membrillera','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19181','Miedes de Atienza','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19182','Mierla (La)','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19183','Milmarcos','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19184','Millana','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19185','Miñosa (La)','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19186','Mirabueno','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19187','Miralrío','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19188','Mochales','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19189','Mohernando','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19190','Molina de Aragón','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19191','Monasterio','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19192','Mondéjar','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19193','Montarrón','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19194','Moratilla de los Meleros','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19195','Morenilla','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19196','Muduex','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19197','Navas de Jadraque (Las)','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19198','Negredo','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19199','Ocentejo','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19200','Olivar (El)','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19201','Olmeda de Cobeta','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19202','Olmeda de Jadraque (La)','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19203','Ordial (El)','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19204','Orea','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19208','Pálmaces de Jadraque','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19209','Pardos','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19210','Paredes de Sigüenza','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19211','Pareja','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19212','Pastrana','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19213','Pedregal (El)','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19214','Peñalén','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19215','Peñalver','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19216','Peralejos de las Truchas','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19217','Peralveche','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19218','Pinilla de Jadraque','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19219','Pinilla de Molina','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19220','Pioz','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19221','Piqueras','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19222','Pobo de Dueñas (El)','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19223','Poveda de la Sierra','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19224','Pozo de Almoguera','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19225','Pozo de Guadalajara','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19226','Prádena de Atienza','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19227','Prados Redondos','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19228','Puebla de Beleña','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19229','Puebla de Valles','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19230','Quer','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19231','Rebollosa de Jadraque','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19232','Recuenco (El)','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19233','Renera','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19234','Retiendas','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19235','Riba de Saelices','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19237','Rillo de Gallo','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19238','Riofrío del Llano','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19239','Robledillo de Mohernando','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19240','Robledo de Corpes','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19241','Romanillos de Atienza','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19242','Romanones','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19243','Rueda de la Sierra','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19244','Sacecorbo','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19245','Sacedón','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19246','Saelices de la Sal','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19247','Salmerón','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19248','San Andrés del Congosto','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19249','San Andrés del Rey','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19250','Santiuste','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19251','Saúca','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19252','Sayatón','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19254','Selas','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19255','Setiles','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19256','Sienes','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19257','Sigüenza','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19258','Solanillos del Extremo','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19259','Somolinos','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19260','Sotillo (El)','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19261','Sotodosos','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19262','Tamajón','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19263','Taragudo','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19264','Taravilla','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19265','Tartanedo','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19266','Tendilla','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19267','Terzaga','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19268','Tierzo','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19269','Toba (La)','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19270','Tordelrábano','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19271','Tordellego','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19272','Tordesilos','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19274','Torija','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19277','Torrecuadrada de Molina','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19278','Torrecuadradilla','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19279','Torre del Burgo','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19280','Torrejón del Rey','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19281','Torremocha de Jadraque','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19282','Torremocha del Campo','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19283','Torremocha del Pinar','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19284','Torremochuela','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19285','Torrubia','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19286','Tórtola de Henares','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19287','Tortuera','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19288','Tortuero','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19289','Traíd','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19290','Trijueque','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19291','Trillo','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19293','Uceda','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19294','Ujados','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19296','Utande','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19297','Valdarachas','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19298','Valdearenas','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19299','Valdeavellano','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19300','Valdeaveruelo','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19301','Valdeconcha','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19302','Valdegrudas','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19303','Valdelcubo','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19304','Valdenuño Fernández','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19305','Valdepeñas de la Sierra','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19306','Valderrebollo','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19307','Valdesotos','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19308','Valfermoso de Tajuña','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19309','Valhermoso','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19310','Valtablado del Río','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19311','Valverde de los Arroyos','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19314','Viana de Jadraque','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19317','Villanueva de Alcorón','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19318','Villanueva de Argecilla','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19319','Villanueva de la Torre','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19321','Villares de Jadraque','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19322','Villaseca de Henares','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19323','Villaseca de Uceda','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19324','Villel de Mesa','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19325','Viñuelas','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19326','Yebes','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19327','Yebra','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19329','Yélamos de Abajo','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19330','Yélamos de Arriba','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19331','Yunquera de Henares','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19332','Yunta (La)','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19333','Zaorejas','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19334','Zarzuela de Jadraque','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19335','Zorita de los Canes','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'19901','Semillas','GU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'20001','Abaltzisketa','SS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'20002','Aduna','SS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'20003','Aizarnazabal','SS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'20004','Albiztur','SS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'20005','Alegia','SS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'20006','Alkiza','SS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'20007','Altzo','SS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'20008','Amezketa','SS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'20009','Andoain','SS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'20010','Anoeta','SS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'20011','Antzuola','SS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'20012','Arama','SS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'20013','Aretxabaleta','SS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'20014','Asteasu','SS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'20015','Ataun','SS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'20016','Aia','SS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'20017','Azkoitia','SS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'20018','Azpeitia','SS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'20019','Beasain','SS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'20020','Beizama','SS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'20021','Belauntza','SS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'20022','Berastegi','SS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'20023','Berrobi','SS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'20024','Bidania-Goiatz','SS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'20025','Zegama','SS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'20026','Zerain','SS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'20027','Zestoa','SS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'20028','Zizurkil','SS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'20029','Deba','SS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'20030','Eibar','SS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'20031','Elduain','SS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'20032','Elgoibar','SS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'20033','Elgeta','SS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'20034','Eskoriatza','SS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'20035','Ezkio-Itsaso','SS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'20036','Hondarribia','SS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'20037','Gaintza','SS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'20038','Gabiria','SS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'20039','Getaria','SS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'20040','Hernani','SS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'20041','Hernialde','SS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'20042','Ibarra','SS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'20043','Idiazabal','SS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'20044','Ikaztegieta','SS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'20045','Irun','SS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'20046','Irura','SS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'20047','Itsasondo','SS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'20048','Larraul','SS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'20049','Lazkao','SS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'20050','Leaburu','SS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'20051','Legazpi','SS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'20052','Legorreta','SS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'20053','Lezo','SS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'20054','Lizartza','SS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'20055','Arrasate/Mondragón','SS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'20056','Mutriku','SS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'20057','Mutiloa','SS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'20058','Olaberria','SS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'20059','Oñati','SS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'20060','Orexa','SS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'20061','Orio','SS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'20062','Ormaiztegi','SS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'20063','Oiartzun','SS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'20064','Pasaia','SS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'20065','Soraluze-Placencia de las Armas','SS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'20066','Errezil','SS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'20067','Errenteria','SS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'20068','Leintz-Gatzaga','SS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'20069','Donostia-San Sebastián','SS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'20070','Segura','SS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'20071','Tolosa','SS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'20072','Urnieta','SS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'20073','Usurbil','SS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'20074','Bergara','SS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'20075','Villabona','SS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'20076','Ordizia','SS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'20077','Urretxu','SS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'20078','Zaldibia','SS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'20079','Zarautz','SS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'20080','Zumarraga','SS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'20081','Zumaia','SS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'20901','Mendaro','SS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'20902','Lasarte-Oria','SS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'20903','Astigarraga','SS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'20904','Baliarrain','SS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'20905','Orendain','SS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'20906','Altzaga','SS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'20907','Gaztelu','SS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'21001','Alájar','H','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'21002','Aljaraque','H','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'21003','Almendro (El)','H','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'21004','Almonaster la Real','H','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'21005','Almonte','H','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'21006','Alosno','H','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'21007','Aracena','H','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'21008','Aroche','H','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'21009','Arroyomolinos de León','H','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'21010','Ayamonte','H','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'21011','Beas','H','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'21012','Berrocal','H','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'21013','Bollullos Par del Condado','H','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'21014','Bonares','H','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'21015','Cabezas Rubias','H','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'21016','Cala','H','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'21017','Calañas','H','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'21018','Campillo (El)','H','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'21019','Campofrío','H','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'21020','Cañaveral de León','H','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'21021','Cartaya','H','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'21022','Castaño del Robledo','H','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'21023','Cerro de Andévalo (El)','H','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'21024','Corteconcepción','H','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'21025','Cortegana','H','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'21026','Cortelazor','H','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'21027','Cumbres de Enmedio','H','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'21028','Cumbres de San Bartolomé','H','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'21029','Cumbres Mayores','H','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'21030','Chucena','H','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'21031','Encinasola','H','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'21032','Escacena del Campo','H','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'21033','Fuenteheridos','H','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'21034','Galaroza','H','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'21035','Gibraleón','H','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'21036','Granada de Río-Tinto (La)','H','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'21037','Granado (El)','H','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'21038','Higuera de la Sierra','H','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'21039','Hinojales','H','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'21040','Hinojos','H','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'21041','Huelva','H','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'21042','Isla Cristina','H','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'21043','Jabugo','H','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'21044','Lepe','H','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'21045','Linares de la Sierra','H','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'21046','Lucena del Puerto','H','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'21047','Manzanilla','H','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'21048','Marines (Los)','H','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'21049','Minas de Riotinto','H','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'21050','Moguer','H','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'21051','Nava (La)','H','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'21052','Nerva','H','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'21053','Niebla','H','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'21054','Palma del Condado (La)','H','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'21055','Palos de la Frontera','H','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'21056','Paterna del Campo','H','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'21057','Paymogo','H','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'21058','Puebla de Guzmán','H','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'21059','Puerto Moral','H','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'21060','Punta Umbría','H','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'21061','Rociana del Condado','H','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'21062','Rosal de la Frontera','H','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'21063','San Bartolomé de la Torre','H','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'21064','San Juan del Puerto','H','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'21065','Sanlúcar de Guadiana','H','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'21066','San Silvestre de Guzmán','H','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'21067','Santa Ana la Real','H','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'21068','Santa Bárbara de Casa','H','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'21069','Santa Olalla del Cala','H','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'21070','Trigueros','H','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'21071','Valdelarco','H','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'21072','Valverde del Camino','H','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'21073','Villablanca','H','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'21074','Villalba del Alcor','H','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'21075','Villanueva de las Cruces','H','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'21076','Villanueva de los Castillejos','H','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'21077','Villarrasa','H','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'21078','Zalamea la Real','H','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'21079','Zufre','H','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'21902','Zarza-Perrunal, La','H','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22001','Abiego','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22002','Abizanda','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22003','Adahuesca','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22004','Agüero','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22006','Aisa','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22007','Albalate de Cinca','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22008','Albalatillo','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22009','Albelda','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22011','Albero Alto','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22012','Albero Bajo','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22013','Alberuela de Tubo','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22014','Alcalá de Gurrea','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22015','Alcalá del Obispo','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22016','Alcampell','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22017','Alcolea de Cinca','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22018','Alcubierre','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22019','Alerre','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22020','Alfántega','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22021','Almudévar','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22022','Almunia de San Juan','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22023','Almuniente','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22024','Alquézar','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22025','Altorricón','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22027','Angüés','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22028','Ansó','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22029','Antillón','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22032','Aragüés del Puerto','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22035','Arén','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22036','Argavieso','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22037','Arguis','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22039','Ayerbe','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22040','Azanuy-Alins','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22041','Azara','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22042','Azlor','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22043','Baélls','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22044','Bailo','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22045','Baldellou','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22046','Ballobar','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22047','Banastás','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22048','Barbastro','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22049','Barbués','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22050','Barbuñales','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22051','Bárcabo','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22052','Belver de Cinca','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22053','Benabarre','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22054','Benasque','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22055','Berbegal','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22057','Bielsa','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22058','Bierge','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22059','Biescas','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22060','Binaced','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22061','Binéfar','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22062','Bisaurri','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22063','Biscarrués','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22064','Blecua y Torres','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22066','Boltaña','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22067','Bonansa','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22068','Borau','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22069','Broto','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22072','Caldearenas','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22074','Campo','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22075','Camporrélls','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22076','Canal de Berdún','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22077','Candasnos','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22078','Canfranc','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22079','Capdesaso','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22080','Capella','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22081','Casbas de Huesca','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22082','Castejón del Puente','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22083','Castejón de Monegros','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22084','Castejón de Sos','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22085','Castelflorite','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22086','Castiello de Jaca','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22087','Castigaleu','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22088','Castillazuelo','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22089','Castillonroy','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22090','Colungo','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22094','Chalamera','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22095','Chía','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22096','Chimillas','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22099','Esplús','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22102','Estada','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22103','Estadilla','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22105','Estopiñán del Castillo','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22106','Fago','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22107','Fanlo','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22109','Fiscal','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22110','Fonz','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22111','Foradada del Toscar','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22112','Fraga','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22113','Fueva (La)','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22114','Gistaín','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22115','Grado (El)','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22116','Grañén','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22117','Graus','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22119','Gurrea de Gállego','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22122','Hoz de Jaca','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22124','Huerto','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22125','Huesca','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22126','Ibieca','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22127','Igriés','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22128','Ilche','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22129','Isábena','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22130','Jaca','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22131','Jasa','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22133','Labuerda','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22135','Laluenga','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22136','Lalueza','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22137','Lanaja','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22139','Laperdiguera','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22141','Lascellas-Ponzano','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22142','Lascuarre','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22143','Laspaúles','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22144','Laspuña','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22149','Loarre','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22150','Loporzano','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22151','Loscorrales','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22155','Monesma y Cajigar','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22156','Monflorite-Lascasas','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22157','Montanuy','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22158','Monzón','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22160','Naval','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22162','Novales','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22163','Nueno','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22164','Olvena','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22165','Ontiñena','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22167','Osso de Cinca','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22168','Palo','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22170','Panticosa','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22172','Peñalba','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22173','Peñas de Riglos (Las)','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22174','Peralta de Alcofea','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22175','Peralta de Calasanz','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22176','Peraltilla','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22177','Perarrúa','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22178','Pertusa','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22181','Piracés','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22182','Plan','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22184','Poleñino','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22186','Pozán de Vero','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22187','Puebla de Castro (La)','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22188','Puente de Montañana','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22189','Puértolas','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22190','Pueyo de Araguás (El)','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22193','Pueyo de Santa Cruz','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22195','Quicena','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22197','Robres','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22199','Sabiñánigo','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22200','Sahún','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22201','Salas Altas','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22202','Salas Bajas','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22203','Salillas','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22204','Sallent de Gállego','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22205','San Esteban de Litera','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22206','Sangarrén','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22207','San Juan de Plan','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22208','Santa Cilia','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22209','Santa Cruz de la Serós','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22212','Santaliestra y San Quílez','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22213','Sariñena','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22214','Secastilla','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22215','Seira','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22217','Sena','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22218','Senés de Alcubierre','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22220','Sesa','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22221','Sesué','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22222','Siétamo','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22223','Sopeira','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22225','Tamarite de Litera','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22226','Tardienta','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22227','Tella-Sin','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22228','Tierz','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22229','Tolva','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22230','Torla-Ordesa','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22232','Torralba de Aragón','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22233','Torre la Ribera','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22234','Torrente de Cinca','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22235','Torres de Alcanadre','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22236','Torres de Barbués','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22239','Tramaced','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22242','Valfarta','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22243','Valle de Bardají','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22244','Valle de Lierp','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22245','Velilla de Cinca','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22246','Beranuy','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22247','Viacamp y Litera','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22248','Vicién','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22249','Villanova','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22250','Villanúa','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22251','Villanueva de Sigena','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22252','Yebra de Basa','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22253','Yésero','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22254','Zaidín','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22901','Valle de Hecho','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22902','Puente la Reina de Jaca','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22903','San Miguel del Cinca','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22904','Sotonera (La)','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22905','Lupiñén-Ortilla','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22906','Santa María de Dulcis','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22907','Aínsa-Sobrarbe','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22908','Hoz y Costean','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'22909','Vencillón','HU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'23001','Albanchez de Mágina','J','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'23002','Alcalá la Real','J','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'23003','Alcaudete','J','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'23004','Aldeaquemada','J','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'23005','Andújar','J','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'23006','Arjona','J','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'23007','Arjonilla','J','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'23008','Arquillos','J','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'23009','Baeza','J','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'23010','Bailén','J','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'23011','Baños de la Encina','J','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'23012','Beas de Segura','J','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'23014','Begíjar','J','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'23015','Bélmez de la Moraleda','J','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'23016','Benatae','J','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'23017','Cabra del Santo Cristo','J','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'23018','Cambil','J','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'23019','Campillo de Arenas','J','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'23020','Canena','J','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'23021','Carboneros','J','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'23024','Carolina (La)','J','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'23025','Castellar','J','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'23026','Castillo de Locubín','J','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'23027','Cazalilla','J','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'23028','Cazorla','J','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'23029','Chiclana de Segura','J','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'23030','Chilluévar','J','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'23031','Escañuela','J','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'23032','Espeluy','J','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'23033','Frailes','J','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'23034','Fuensanta de Martos','J','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'23035','Fuerte del Rey','J','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'23037','Génave','J','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'23038','Guardia de Jaén (La)','J','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'23039','Guarromán','J','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'23040','Lahiguera','J','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'23041','Higuera de Calatrava','J','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'23042','Hinojares','J','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'23043','Hornos','J','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'23044','Huelma','J','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'23045','Huesa','J','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'23046','Ibros','J','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'23047','Iruela (La)','J','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'23048','Iznatoraf','J','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'23049','Jabalquinto','J','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'23050','Jaén','J','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'23051','Jamilena','J','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'23052','Jimena','J','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'23053','Jódar','J','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'23054','Larva','J','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'23055','Linares','J','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'23056','Lopera','J','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'23057','Lupión','J','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'23058','Mancha Real','J','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'23059','Marmolejo','J','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'23060','Martos','J','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'23061','Mengíbar','J','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'23062','Montizón','J','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'23063','Navas de San Juan','J','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'23064','Noalejo','J','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'23065','Orcera','J','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'23066','Peal de Becerro','J','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'23067','Pegalajar','J','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'23069','Porcuna','J','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'23070','Pozo Alcón','J','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'23071','Puente de Génave','J','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'23072','Puerta de Segura (La)','J','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'23073','Quesada','J','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'23074','Rus','J','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'23075','Sabiote','J','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'23076','Santa Elena','J','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'23077','Santiago de Calatrava','J','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'23079','Santisteban del Puerto','J','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'23080','Santo Tomé','J','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'23081','Segura de la Sierra','J','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'23082','Siles','J','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'23084','Sorihuela del Guadalimar','J','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'23085','Torreblascopedro','J','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'23086','Torredelcampo','J','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'23087','Torredonjimeno','J','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'23088','Torreperogil','J','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'23090','Torres','J','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'23091','Torres de Albánchez','J','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'23092','Úbeda','J','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'23093','Valdepeñas de Jaén','J','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'23094','Vilches','J','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'23095','Villacarrillo','J','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'23096','Villanueva de la Reina','J','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'23097','Villanueva del Arzobispo','J','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'23098','Villardompardo','J','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'23099','Villares (Los)','J','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'23101','Villarrodrigo','J','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'23901','Cárcheles','J','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'23902','Bedmar y Garcíez','J','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'23903','Villatorres','J','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'23904','Santiago-Pontones','J','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'23905','Arroyo del Ojanco','J','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24001','Acebedo','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24002','Algadefe','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24003','Alija del Infantado','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24004','Almanza','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24005','Antigua (La)','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24006','Ardón','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24007','Arganza','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24008','Astorga','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24009','Balboa','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24010','Bañeza (La)','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24011','Barjas','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24012','Barrios de Luna (Los)','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24014','Bembibre','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24015','Benavides','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24016','Benuza','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24017','Bercianos del Páramo','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24018','Bercianos del Real Camino','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24019','Berlanga del Bierzo','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24020','Boca de Huérgano','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24021','Boñar','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24022','Borrenes','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24023','Brazuelo','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24024','Burgo Ranero (El)','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24025','Burón','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24026','Bustillo del Páramo','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24027','Cabañas Raras','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24028','Cabreros del Río','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24029','Cabrillanes','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24030','Cacabelos','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24031','Calzada del Coto','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24032','Campazas','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24033','Campo de Villavidel','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24034','Camponaraya','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24036','Candín','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24037','Cármenes','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24038','Carracedelo','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24039','Carrizo','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24040','Carrocera','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24041','Carucedo','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24042','Castilfalé','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24043','Castrillo de Cabrera','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24044','Castrillo de la Valduerna','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24046','Castrocalbón','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24047','Castrocontrigo','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24049','Castropodame','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24050','Castrotierra de Valmadrigal','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24051','Cea','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24052','Cebanico','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24053','Cebrones del Río','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24054','Cimanes de la Vega','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24055','Cimanes del Tejar','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24056','Cistierna','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24057','Congosto','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24058','Corbillos de los Oteros','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24059','Corullón','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24060','Crémenes','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24061','Cuadros','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24062','Cubillas de los Oteros','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24063','Cubillas de Rueda','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24064','Cubillos del Sil','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24065','Chozas de Abajo','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24066','Destriana','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24067','Encinedo','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24068','Ercina (La)','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24069','Escobar de Campos','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24070','Fabero','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24071','Folgoso de la Ribera','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24073','Fresno de la Vega','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24074','Fuentes de Carbajal','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24076','Garrafe de Torío','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24077','Gordaliza del Pino','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24078','Gordoncillo','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24079','Gradefes','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24080','Grajal de Campos','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24081','Gusendos de los Oteros','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24082','Hospital de Órbigo','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24083','Igüeña','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24084','Izagre','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24086','Joarilla de las Matas','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24087','Laguna Dalga','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24088','Laguna de Negrillos','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24089','León','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24090','Lucillo','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24091','Luyego','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24092','Llamas de la Ribera','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24093','Magaz de Cepeda','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24094','Mansilla de las Mulas','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24095','Mansilla Mayor','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24096','Maraña','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24097','Matadeón de los Oteros','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24098','Matallana de Torío','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24099','Matanza','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24100','Molinaseca','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24101','Murias de Paredes','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24102','Noceda del Bierzo','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24103','Oencia','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24104','Omañas (Las)','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24105','Onzonilla','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24106','Oseja de Sajambre','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24107','Pajares de los Oteros','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24108','Palacios de la Valduerna','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24109','Palacios del Sil','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24110','Páramo del Sil','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24112','Peranzanes','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24113','Pobladura de Pelayo García','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24114','Pola de Gordón (La)','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24115','Ponferrada','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24116','Posada de Valdeón','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24117','Pozuelo del Páramo','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24118','Prado de la Guzpeña','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24119','Priaranza del Bierzo','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24120','Prioro','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24121','Puebla de Lillo','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24122','Puente de Domingo Flórez','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24123','Quintana del Castillo','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24124','Quintana del Marco','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24125','Quintana y Congosto','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24127','Regueras de Arriba','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24129','Reyero','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24130','Riaño','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24131','Riego de la Vega','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24132','Riello','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24133','Rioseco de Tapia','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24134','Robla (La)','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24136','Roperuelos del Páramo','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24137','Sabero','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24139','Sahagún','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24141','San Adrián del Valle','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24142','San Andrés del Rabanedo','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24143','Sancedo','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24144','San Cristóbal de la Polantera','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24145','San Emiliano','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24146','San Esteban de Nogales','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24148','San Justo de la Vega','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24149','San Millán de los Caballeros','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24150','San Pedro Bercianos','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24151','Santa Colomba de Curueño','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24152','Santa Colomba de Somoza','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24153','Santa Cristina de Valmadrigal','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24154','Santa Elena de Jamuz','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24155','Santa María de la Isla','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24156','Santa María del Monte de Cea','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24157','Santa María del Páramo','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24158','Santa María de Ordás','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24159','Santa Marina del Rey','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24160','Santas Martas','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24161','Santiago Millas','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24162','Santovenia de la Valdoncina','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24163','Sariegos','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24164','Sena de Luna','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24165','Sobrado','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24166','Soto de la Vega','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24167','Soto y Amío','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24168','Toral de los Guzmanes','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24169','Toreno','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24170','Torre del Bierzo','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24171','Trabadelo','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24172','Truchas','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24173','Turcia','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24174','Urdiales del Páramo','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24175','Valdefresno','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24176','Valdefuentes del Páramo','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24177','Valdelugueros','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24178','Valdemora','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24179','Valdepiélago','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24180','Valdepolo','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24181','Valderas','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24182','Valderrey','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24183','Valderrueda','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24184','Valdesamario','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24185','Val de San Lorenzo','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24187','Valdevimbre','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24188','Valencia de Don Juan','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24189','Valverde de la Virgen','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24190','Valverde-Enrique','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24191','Vallecillo','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24193','Vecilla (La)','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24194','Vegacervera','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24196','Vega de Espinareda','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24197','Vega de Infanzones','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24198','Vega de Valcarce','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24199','Vegaquemada','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24201','Vegas del Condado','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24202','Villablino','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24203','Villabraz','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24205','Villadangos del Páramo','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24206','Toral de los Vados','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24207','Villademor de la Vega','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24209','Villafranca del Bierzo','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24210','Villagatón','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24211','Villamandos','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24212','Villamañán','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24213','Villamartín de Don Sancho','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24214','Villamejil','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24215','Villamol','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24216','Villamontán de la Valduerna','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24217','Villamoratiel de las Matas','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24218','Villanueva de las Manzanas','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24219','Villaobispo de Otero','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24221','Villaquejida','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24222','Villaquilambre','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24223','Villarejo de Órbigo','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24224','Villares de Órbigo','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24225','Villasabariego','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24226','Villaselán','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24227','Villaturiel','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24228','Villazala','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24229','Villazanzo de Valderaduey','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24230','Zotes del Páramo','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24901','Villamanín','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'24902','Villaornate y Castro','LE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25001','Abella de la Conca','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25002','Àger','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25003','Agramunt','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25004','Alamús (Els)','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25005','Alàs i Cerc','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25006','Albagés (L'')','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25007','Albatàrrec','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25008','Albesa','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25009','Albi (L'')','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25010','Alcanó','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25011','Alcarràs','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25012','Alcoletge','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25013','Alfarràs','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25014','Alfés','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25015','Algerri','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25016','Alguaire','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25017','Alins','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25019','Almacelles','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25020','Almatret','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25021','Almenar','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25022','Alòs de Balaguer','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25023','Alpicat','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25024','Alt Àneu','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25025','Naut Aran','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25027','Anglesola','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25029','Arbeca','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25030','Pont de Bar (El)','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25031','Arres','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25032','Arsèguel','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25033','Artesa de Lleida','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25034','Artesa de Segre','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25035','Sentiu de Sió (La)','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25036','Aspa','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25037','Avellanes i Santa Linya (Les)','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25038','Aitona','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25039','Baix Pallars','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25040','Balaguer','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25041','Barbens','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25042','Baronia de Rialb (La)','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25043','Vall de Boí (La)','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25044','Bassella','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25045','Bausen','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25046','Belianes','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25047','Bellcaire d''Urgell','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25048','Bell-lloc d''Urgell','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25049','Bellmunt d''Urgell','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25050','Bellpuig','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25051','Bellver de Cerdanya','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25052','Bellvís','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25053','Benavent de Segrià','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25055','Biosca','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25056','Bovera','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25057','Bòrdes (Es)','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25058','Borges Blanques (Les)','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25059','Bossòst','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25060','Cabanabona','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25061','Cabó','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25062','Camarasa','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25063','Canejan','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25064','Castellar de la Ribera','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25067','Castelldans','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25068','Castellnou de Seana','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25069','Castelló de Farfanya','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25070','Castellserà','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25071','Cava','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25072','Cervera','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25073','Cervià de les Garrigues','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25074','Ciutadilla','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25075','Clariana de Cardener','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25076','Cogul (El)','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25077','Coll de Nargó','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25078','Corbins','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25079','Cubells','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25081','Espluga Calba (L'')','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25082','Espot','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25085','Estaràs','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25086','Esterri d''Àneu','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25087','Esterri de Cardós','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25088','Estamariu','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25089','Farrera','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25092','Floresta (La)','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25093','Fondarella','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25094','Foradada','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25096','Fuliola (La)','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25097','Fulleda','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25098','Gavet de la Conca','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25099','Golmés','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25100','Gósol','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25101','Granadella (La)','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25102','Granja d''Escarp (La)','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25103','Granyanella','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25104','Granyena de Segarra','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25105','Granyena de les Garrigues','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25109','Guimerà','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25110','Guissona','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25111','Guixers','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25112','Ivars de Noguera','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25113','Ivars d''Urgell','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25114','Ivorra','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25115','Isona i Conca Dellà','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25118','Juncosa','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25119','Juneda','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25120','Lleida','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25121','Les','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25122','Linyola','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25123','Lladorre','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25124','Lladurs','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25125','Llardecans','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25126','Llavorsí','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25127','Lles de Cerdanya','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25128','Llimiana','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25129','Llobera','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25130','Maldà','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25131','Massalcoreig','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25132','Massoteres','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25133','Maials','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25134','Menàrguens','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25135','Miralcamp','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25136','Molsosa (La)','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25137','Mollerussa','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25138','Montgai','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25139','Montellà i Martinet','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25140','Montferrer i Castellbò','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25141','Montoliu de Segarra','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25142','Montoliu de Lleida','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25143','Montornès de Segarra','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25145','Nalec','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25146','Navès','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25148','Odèn','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25149','Oliana','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25150','Oliola','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25151','Olius','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25152','Oluges (Les)','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25153','Omellons (Els)','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25154','Omells de na Gaia (Els)','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25155','Organyà','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25156','Os de Balaguer','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25157','Ossó de Sió','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25158','Palau d''Anglesola (El)','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25161','Conca de Dalt','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25163','Coma i la Pedra (La)','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25164','Penelles','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25165','Peramola','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25166','Pinell de Solsonès','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25167','Pinós','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25168','Poal (El)','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25169','Pobla de Cérvoles (La)','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25170','Bellaguarda','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25171','Pobla de Segur (La)','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25172','Ponts','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25173','Pont de Suert (El)','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25174','Portella (La)','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25175','Prats i Sansor','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25176','Preixana','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25177','Preixens','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25179','Prullans','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25180','Puiggròs','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25181','Puigverd d''Agramunt','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25182','Puigverd de Lleida','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25183','Rialp','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25185','Ribera d''Urgellet','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25186','Riner','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25189','Rosselló','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25190','Salàs de Pallars','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25191','Sanaüja','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25192','Sant Guim de Freixenet','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25193','Sant Llorenç de Morunys','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25194','Sant Ramon','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25196','Sant Esteve de la Sarga','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25197','Sant Guim de la Plana','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25200','Sarroca de Lleida','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25201','Sarroca de Bellera','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25202','Senterada','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25203','Seu d''Urgell (La)','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25204','Seròs','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25205','Sidamon','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25206','Soleràs (El)','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25207','Solsona','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25208','Soriguera','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25209','Sort','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25210','Soses','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25211','Sudanell','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25212','Sunyer','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25215','Talarn','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25216','Talavera','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25217','Tàrrega','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25218','Tarrés','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25219','Tarroja de Segarra','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25220','Térmens','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25221','Tírvia','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25222','Tiurana','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25223','Torà','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25224','Torms (Els)','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25225','Tornabous','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25226','Torrebesses','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25227','Torre de Cabdella (La)','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25228','Torrefarrera','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25230','Torregrossa','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25231','Torrelameu','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25232','Torres de Segre','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25233','Torre-serona','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25234','Tremp','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25238','Vallbona de les Monges','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25239','Valls de Valira (Les)','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25240','Vallfogona de Balaguer','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25242','Verdú','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25243','Vielha e Mijaran','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25244','Vilagrassa','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25245','Vilaller','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25247','Vilamòs','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25248','Vilanova de Bellpuig','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25249','Vilanova de l''Aguda','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25250','Vilanova de Meià','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25251','Vilanova de Segrià','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25252','Vila-sana','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25253','Vilosell (El)','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25254','Vilanova de la Barca','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25255','Vinaixa','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25901','Vall de Cardós','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25902','Sant Martí de Riucorb','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25903','Guingueta d''Àneu (La)','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25904','Castell de Mur','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25905','Ribera d''Ondara','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25906','Valls d''Aguilar (Les)','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25907','Torrefeta i Florejacs','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25908','Fígols i Alinyà','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25909','Vansa i Fórnols (La)','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25910','Josa i Tuixén','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25911','Plans de Sió (Els)','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25912','Gimenells i el Pla de la Font','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'25913','Riu de Cerdanya','L','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'26001','Ábalos','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'26002','Agoncillo','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'26003','Aguilar del Río Alhama','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'26004','Ajamil','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'26005','Albelda de Iregua','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'26006','Alberite','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'26007','Alcanadre','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'26008','Aldeanueva de Ebro','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'26009','Alesanco','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'26010','Alesón','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'26011','Alfaro','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'26012','Almarza de Cameros','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'26013','Anguciana','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'26014','Anguiano','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'26015','Arenzana de Abajo','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'26016','Arenzana de Arriba','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'26017','Arnedillo','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'26018','Arnedo','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'26019','Arrúbal','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'26020','Ausejo','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'26021','Autol','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'26022','Azofra','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'26023','Badarán','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'26024','Bañares','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'26025','Baños de Rioja','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'26026','Baños de Río Tobía','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'26027','Berceo','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'26028','Bergasa','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'26029','Bergasillas Bajera','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'26030','Bezares','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'26031','Bobadilla','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'26032','Brieva de Cameros','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'26033','Briñas','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'26034','Briones','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'26035','Cabezón de Cameros','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'26036','Calahorra','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'26037','Camprovín','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'26038','Canales de la Sierra','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'26039','Canillas de Río Tuerto','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'26040','Cañas','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'26041','Cárdenas','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'26042','Casalarreina','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'26043','Castañares de Rioja','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'26044','Castroviejo','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'26045','Cellorigo','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'26046','Cenicero','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'26047','Cervera del Río Alhama','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'26048','Cidamón','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'26049','Cihuri','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'26050','Cirueña','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'26051','Clavijo','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'26052','Cordovín','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'26053','Corera','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'26054','Cornago','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'26055','Corporales','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'26056','Cuzcurrita de Río Tirón','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'26057','Daroca de Rioja','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'26058','Enciso','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'26059','Entrena','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'26060','Estollo','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'26061','Ezcaray','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'26062','Foncea','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'26063','Fonzaleche','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'26064','Fuenmayor','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'26065','Galbárruli','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'26066','Galilea','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'26067','Gallinero de Cameros','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'26068','Gimileo','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'26069','Grañón','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'26070','Grávalos','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'26071','Haro','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'26072','Herce','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'26073','Herramélluri','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'26074','Hervías','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'26075','Hormilla','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'26076','Hormilleja','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'26077','Hornillos de Cameros','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'26078','Hornos de Moncalvillo','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'26079','Huércanos','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'26080','Igea','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'26081','Jalón de Cameros','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'26082','Laguna de Cameros','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'26083','Lagunilla del Jubera','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'26084','Lardero','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'26086','Ledesma de la Cogolla','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'26087','Leiva','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'26088','Leza de Río Leza','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'26089','Logroño','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'26091','Lumbreras','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'26092','Manjarrés','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'26093','Mansilla de la Sierra','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'26094','Manzanares de Rioja','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'26095','Matute','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'26096','Medrano','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'26098','Munilla','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'26099','Murillo de Río Leza','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'26100','Muro de Aguas','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'26101','Muro en Cameros','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'26102','Nájera','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'26103','Nalda','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'26104','Navajún','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'26105','Navarrete','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'26106','Nestares','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'26107','Nieva de Cameros','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'26108','Ocón','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'26109','Ochánduri','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'26110','Ojacastro','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'26111','Ollauri','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'26112','Ortigosa de Cameros','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'26113','Pazuengos','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'26114','Pedroso','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'26115','Pinillos','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'26117','Pradejón','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'26118','Pradillo','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'26119','Préjano','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'26120','Quel','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'26121','Rabanera','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'26122','Rasillo de Cameros (El)','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'26123','Redal (El)','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'26124','Ribafrecha','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'26125','Rincón de Soto','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'26126','Robres del Castillo','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'26127','Rodezno','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'26128','Sajazarra','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'26129','San Asensio','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'26130','San Millán de la Cogolla','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'26131','San Millán de Yécora','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'26132','San Román de Cameros','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'26134','Santa Coloma','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'26135','Santa Engracia del Jubera','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'26136','Santa Eulalia Bajera','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'26138','Santo Domingo de la Calzada','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'26139','San Torcuato','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'26140','Santurde de Rioja','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'26141','Santurdejo','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'26142','San Vicente de la Sonsierra','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'26143','Sojuela','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'26144','Sorzano','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'26145','Sotés','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'26146','Soto en Cameros','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'26147','Terroba','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'26148','Tirgo','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'26149','Tobía','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'26150','Tormantos','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'26151','Torrecilla en Cameros','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'26152','Torrecilla sobre Alesanco','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'26153','Torre en Cameros','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'26154','Torremontalbo','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'26155','Treviana','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'26157','Tricio','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'26158','Tudelilla','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'26160','Uruñuela','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'26161','Valdemadera','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'26162','Valgañón','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'26163','Ventosa','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'26164','Ventrosa','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'26165','Viguera','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'26166','Villalba de Rioja','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'26167','Villalobar de Rioja','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'26168','Villamediana de Iregua','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'26169','Villanueva de Cameros','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'26170','Villar de Arnedo (El)','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'26171','Villar de Torre','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'26172','Villarejo','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'26173','Villarroya','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'26174','Villarta-Quintana','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'26175','Villavelayo','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'26176','Villaverde de Rioja','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'26177','Villoslada de Cameros','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'26178','Viniegra de Abajo','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'26179','Viniegra de Arriba','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'26180','Zarratón','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'26181','Zarzosa','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'26183','Zorraquín','LO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'27001','Abadín','LU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'27002','Alfoz','LU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'27003','Antas de Ulla','LU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'27004','Baleira','LU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'27005','Barreiros','LU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'27006','Becerreá','LU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'27007','Begonte','LU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'27008','Bóveda','LU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'27009','Carballedo','LU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'27010','Castro de Rei','LU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'27011','Castroverde','LU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'27012','Cervantes','LU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'27013','Cervo','LU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'27014','Corgo (O)','LU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'27015','Cospeito','LU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'27016','Chantada','LU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'27017','Folgoso do Courel','LU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'27018','Fonsagrada (A)','LU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'27019','Foz','LU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'27020','Friol','LU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'27021','Xermade','LU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'27022','Guitiriz','LU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'27023','Guntín','LU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'27024','Incio (O)','LU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'27025','Xove','LU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'27026','Láncara','LU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'27027','Lourenzá','LU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'27028','Lugo','LU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'27029','Meira','LU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'27030','Mondoñedo','LU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'27031','Monforte de Lemos','LU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'27032','Monterroso','LU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'27033','Muras','LU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'27034','Navia de Suarna','LU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'27035','Negueira de Muñiz','LU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'27037','Nogais (As)','LU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'27038','Ourol','LU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'27039','Outeiro de Rei','LU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'27040','Palas de Rei','LU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'27041','Pantón','LU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'27042','Paradela','LU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'27043','Páramo (O)','LU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'27044','Pastoriza (A)','LU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'27045','Pedrafita do Cebreiro','LU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'27046','Pol','LU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'27047','Pobra do Brollón (A)','LU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'27048','Pontenova (A)','LU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'27049','Portomarín','LU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'27050','Quiroga','LU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'27051','Ribadeo','LU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'27052','Ribas de Sil','LU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'27053','Ribeira de Piquín','LU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'27054','Riotorto','LU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'27055','Samos','LU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'27056','Rábade','LU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'27057','Sarria','LU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'27058','Saviñao (O)','LU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'27059','Sober','LU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'27060','Taboada','LU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'27061','Trabada','LU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'27062','Triacastela','LU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'27063','Valadouro (O)','LU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'27064','Vicedo (O)','LU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'27065','Vilalba','LU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'27066','Viveiro','LU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'27901','Baralla','LU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'27902','Burela','LU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'28001','Acebeda (La)','M','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'28002','Ajalvir','M','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'28003','Alameda del Valle','M','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'28004','Álamo (El)','M','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'28005','Alcalá de Henares','M','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'28006','Alcobendas','M','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'28007','Alcorcón','M','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'28008','Aldea del Fresno','M','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'28009','Algete','M','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'28010','Alpedrete','M','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'28011','Ambite','M','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'28012','Anchuelo','M','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'28013','Aranjuez','M','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'28014','Arganda del Rey','M','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'28015','Arroyomolinos','M','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'28016','Atazar (El)','M','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'28017','Batres','M','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'28018','Becerril de la Sierra','M','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'28019','Belmonte de Tajo','M','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'28020','Berzosa del Lozoya','M','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'28021','Berrueco (El)','M','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'28022','Boadilla del Monte','M','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'28023','Boalo (El)','M','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'28024','Braojos','M','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'28025','Brea de Tajo','M','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'28026','Brunete','M','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'28027','Buitrago del Lozoya','M','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'28028','Bustarviejo','M','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'28029','Cabanillas de la Sierra','M','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'28030','Cabrera (La)','M','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'28031','Cadalso de los Vidrios','M','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'28032','Camarma de Esteruelas','M','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'28033','Campo Real','M','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'28034','Canencia','M','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'28035','Carabaña','M','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'28036','Casarrubuelos','M','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'28037','Cenicientos','M','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'28038','Cercedilla','M','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'28039','Cervera de Buitrago','M','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'28040','Ciempozuelos','M','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'28041','Cobeña','M','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'28042','Colmenar del Arroyo','M','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'28043','Colmenar de Oreja','M','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'28044','Colmenarejo','M','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'28045','Colmenar Viejo','M','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'28046','Collado Mediano','M','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'28047','Collado Villalba','M','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'28048','Corpa','M','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'28049','Coslada','M','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'28050','Cubas de la Sagra','M','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'28051','Chapinería','M','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'28052','Chinchón','M','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'28053','Daganzo de Arriba','M','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'28054','Escorial (El)','M','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'28055','Estremera','M','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'28056','Fresnedillas de la Oliva','M','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'28057','Fresno de Torote','M','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'28058','Fuenlabrada','M','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'28059','Fuente el Saz de Jarama','M','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'28060','Fuentidueña de Tajo','M','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'28061','Galapagar','M','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'28062','Garganta de los Montes','M','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'28063','Gargantilla del Lozoya y Pinilla de Buitrago','M','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'28064','Gascones','M','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'28065','Getafe','M','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'28066','Griñón','M','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'28067','Guadalix de la Sierra','M','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'28068','Guadarrama','M','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'28069','Hiruela (La)','M','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'28070','Horcajo de la Sierra-Aoslos','M','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'28071','Horcajuelo de la Sierra','M','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'28072','Hoyo de Manzanares','M','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'28073','Humanes de Madrid','M','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'28074','Leganés','M','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'28075','Loeches','M','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'28076','Lozoya','M','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'28078','Madarcos','M','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'28079','Madrid','M','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'28080','Majadahonda','M','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'28082','Manzanares el Real','M','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'28083','Meco','M','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'28084','Mejorada del Campo','M','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'28085','Miraflores de la Sierra','M','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'28086','Molar (El)','M','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'28087','Molinos (Los)','M','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'28088','Montejo de la Sierra','M','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'28089','Moraleja de Enmedio','M','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'28090','Moralzarzal','M','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'28091','Morata de Tajuña','M','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'28092','Móstoles','M','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'28093','Navacerrada','M','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'28094','Navalafuente','M','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'28095','Navalagamella','M','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'28096','Navalcarnero','M','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'28097','Navarredonda y San Mamés','M','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'28099','Navas del Rey','M','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'28100','Nuevo Baztán','M','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'28101','Olmeda de las Fuentes','M','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'28102','Orusco de Tajuña','M','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'28104','Paracuellos de Jarama','M','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'28106','Parla','M','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'28107','Patones','M','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'28108','Pedrezuela','M','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'28109','Pelayos de la Presa','M','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'28110','Perales de Tajuña','M','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'28111','Pezuela de las Torres','M','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'28112','Pinilla del Valle','M','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'28113','Pinto','M','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'28114','Piñuécar-Gandullas','M','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'28115','Pozuelo de Alarcón','M','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'28116','Pozuelo del Rey','M','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'28117','Prádena del Rincón','M','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'28118','Puebla de la Sierra','M','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'28119','Quijorna','M','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'28120','Rascafría','M','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'28121','Redueña','M','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'28122','Ribatejada','M','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'28123','Rivas-Vaciamadrid','M','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'28124','Robledillo de la Jara','M','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'28125','Robledo de Chavela','M','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'28126','Robregordo','M','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'28127','Rozas de Madrid (Las)','M','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'28128','Rozas de Puerto Real','M','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'28129','San Agustín del Guadalix','M','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'28130','San Fernando de Henares','M','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'28131','San Lorenzo de El Escorial','M','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'28132','San Martín de la Vega','M','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'28133','San Martín de Valdeiglesias','M','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'28134','San Sebastián de los Reyes','M','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'28135','Santa María de la Alameda','M','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'28136','Santorcaz','M','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'28137','Santos de la Humosa (Los)','M','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'28138','Serna del Monte (La)','M','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'28140','Serranillos del Valle','M','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'28141','Sevilla la Nueva','M','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'28143','Somosierra','M','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'28144','Soto del Real','M','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'28145','Talamanca de Jarama','M','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'28146','Tielmes','M','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'28147','Titulcia','M','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'28148','Torrejón de Ardoz','M','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'28149','Torrejón de la Calzada','M','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'28150','Torrejón de Velasco','M','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'28151','Torrelaguna','M','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'28152','Torrelodones','M','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'28153','Torremocha de Jarama','M','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'28154','Torres de la Alameda','M','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'28155','Valdaracete','M','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'28156','Valdeavero','M','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'28157','Valdelaguna','M','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'28158','Valdemanco','M','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'28159','Valdemaqueda','M','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'28160','Valdemorillo','M','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'28161','Valdemoro','M','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'28162','Valdeolmos-Alalpardo','M','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'28163','Valdepiélagos','M','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'28164','Valdetorres de Jarama','M','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'28165','Valdilecha','M','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'28166','Valverde de Alcalá','M','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'28167','Velilla de San Antonio','M','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'28168','Vellón (El)','M','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'28169','Venturada','M','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'28170','Villaconejos','M','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'28171','Villa del Prado','M','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'28172','Villalbilla','M','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'28173','Villamanrique de Tajo','M','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'28174','Villamanta','M','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'28175','Villamantilla','M','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'28176','Villanueva de la Cañada','M','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'28177','Villanueva del Pardillo','M','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'28178','Villanueva de Perales','M','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'28179','Villar del Olmo','M','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'28180','Villarejo de Salvanés','M','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'28181','Villaviciosa de Odón','M','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'28182','Villavieja del Lozoya','M','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'28183','Zarzalejo','M','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'28901','Lozoyuela-Navas-Sieteiglesias','M','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'28902','Puentes Viejas','M','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'28903','Tres Cantos','M','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'29001','Alameda','MA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'29002','Alcaucín','MA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'29003','Alfarnate','MA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'29004','Alfarnatejo','MA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'29005','Algarrobo','MA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'29006','Algatocín','MA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'29007','Alhaurín de la Torre','MA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'29008','Alhaurín el Grande','MA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'29009','Almáchar','MA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'29010','Almargen','MA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'29011','Almogía','MA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'29012','Álora','MA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'29013','Alozaina','MA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'29014','Alpandeire','MA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'29015','Antequera','MA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'29016','Árchez','MA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'29017','Archidona','MA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'29018','Ardales','MA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'29019','Arenas','MA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'29020','Arriate','MA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'29021','Atajate','MA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'29022','Benadalid','MA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'29023','Benahavís','MA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'29024','Benalauría','MA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'29025','Benalmádena','MA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'29026','Benamargosa','MA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'29027','Benamocarra','MA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'29028','Benaoján','MA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'29029','Benarrabá','MA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'29030','Borge (El)','MA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'29031','Burgo (El)','MA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'29032','Campillos','MA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'29033','Canillas de Aceituno','MA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'29034','Canillas de Albaida','MA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'29035','Cañete la Real','MA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'29036','Carratraca','MA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'29037','Cartajima','MA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'29038','Cártama','MA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'29039','Casabermeja','MA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'29040','Casarabonela','MA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'29041','Casares','MA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'29042','Coín','MA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'29043','Colmenar','MA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'29044','Comares','MA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'29045','Cómpeta','MA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'29046','Cortes de la Frontera','MA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'29047','Cuevas Bajas','MA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'29048','Cuevas del Becerro','MA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'29049','Cuevas de San Marcos','MA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'29050','Cútar','MA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'29051','Estepona','MA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'29052','Faraján','MA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'29053','Frigiliana','MA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'29054','Fuengirola','MA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'29055','Fuente de Piedra','MA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'29056','Gaucín','MA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'29057','Genalguacil','MA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'29058','Guaro','MA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'29059','Humilladero','MA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'29060','Igualeja','MA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'29061','Istán','MA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'29062','Iznate','MA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'29063','Jimera de Líbar','MA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'29064','Jubrique','MA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'29065','Júzcar','MA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'29066','Macharaviaya','MA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'29067','Málaga','MA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'29068','Manilva','MA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'29069','Marbella','MA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'29070','Mijas','MA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'29071','Moclinejo','MA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'29072','Mollina','MA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'29073','Monda','MA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'29074','Montejaque','MA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'29075','Nerja','MA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'29076','Ojén','MA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'29077','Parauta','MA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'29079','Periana','MA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'29080','Pizarra','MA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'29081','Pujerra','MA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'29082','Rincón de la Victoria','MA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'29083','Riogordo','MA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'29084','Ronda','MA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'29085','Salares','MA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'29086','Sayalonga','MA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'29087','Sedella','MA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'29088','Sierra de Yeguas','MA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'29089','Teba','MA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'29090','Tolox','MA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'29091','Torrox','MA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'29092','Totalán','MA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'29093','Valle de Abdalajís','MA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'29094','Vélez-Málaga','MA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'29095','Villanueva de Algaidas','MA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'29096','Villanueva del Rosario','MA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'29097','Villanueva del Trabuco','MA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'29098','Villanueva de Tapia','MA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'29099','Viñuela','MA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'29100','Yunquera','MA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'29901','Torremolinos','MA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'29902','Villanueva de la Concepción','MA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'29903','Montecorto','MA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'29904','Serrato','MA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'30001','Abanilla','MU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'30002','Abarán','MU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'30003','Águilas','MU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'30004','Albudeite','MU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'30005','Alcantarilla','MU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'30006','Aledo','MU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'30007','Alguazas','MU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'30008','Alhama de Murcia','MU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'30009','Archena','MU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'30010','Beniel','MU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'30011','Blanca','MU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'30012','Bullas','MU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'30013','Calasparra','MU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'30014','Campos del Río','MU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'30015','Caravaca de la Cruz','MU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'30016','Cartagena','MU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'30017','Cehegín','MU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'30018','Ceutí','MU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'30019','Cieza','MU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'30020','Fortuna','MU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'30021','Fuente Álamo de Murcia','MU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'30022','Jumilla','MU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'30023','Librilla','MU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'30024','Lorca','MU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'30025','Lorquí','MU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'30026','Mazarrón','MU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'30027','Molina de Segura','MU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'30028','Moratalla','MU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'30029','Mula','MU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'30030','Murcia','MU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'30031','Ojós','MU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'30032','Pliego','MU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'30033','Puerto Lumbreras','MU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'30034','Ricote','MU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'30035','San Javier','MU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'30036','San Pedro del Pinatar','MU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'30037','Torre-Pacheco','MU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'30038','Torres de Cotillas (Las)','MU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'30039','Totana','MU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'30040','Ulea','MU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'30041','Unión (La)','MU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'30042','Villanueva del Río Segura','MU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'30043','Yecla','MU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'30901','Santomera','MU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'30902','Alcázares (Los)','MU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31001','Abáigar','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31002','Abárzuza/Abartzuza','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31003','Abaurregaina/Abaurrea Alta','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31004','Abaurrepea/Abaurrea Baja','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31005','Aberin','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31006','Ablitas','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31007','Adiós','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31008','Aguilar de Codés','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31009','Aibar/Oibar','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31010','Altsasu/Alsasua','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31011','Allín/Allin','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31012','Allo','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31013','Améscoa Baja','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31014','Ancín','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31015','Andosilla','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31016','Ansoáin/Antsoain','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31017','Anue','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31018','Añorbe','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31019','Aoiz/Agoitz','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31020','Araitz','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31021','Aranarache/Aranaratxe','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31022','Arantza','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31023','Aranguren','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31024','Arano','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31025','Arakil','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31026','Aras','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31027','Arbizu','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31028','Arce/Artzi','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31029','Arcos (Los)','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31030','Arellano','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31031','Areso','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31032','Arguedas','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31033','Aria','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31034','Aribe','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31035','Armañanzas','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31036','Arróniz','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31037','Arruazu','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31038','Artajona','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31039','Artazu','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31040','Atez/Atetz','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31041','Ayegui/Aiegi','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31042','Azagra','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31043','Azuelo','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31044','Bakaiku','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31045','Barásoain','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31046','Barbarin','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31047','Bargota','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31048','Barillas','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31049','Basaburua','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31050','Baztan','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31051','Beire','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31052','Belascoáin','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31053','Berbinzana','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31054','Bertizarana','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31055','Betelu','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31056','Biurrun-Olcoz','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31057','Buñuel','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31058','Auritz/Burguete','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31059','Burgui/Burgi','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31060','Burlada/Burlata','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31061','Busto (El)','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31062','Cabanillas','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31063','Cabredo','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31064','Cadreita','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31065','Caparroso','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31066','Cárcar','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31067','Carcastillo','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31068','Cascante','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31069','Cáseda','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31070','Castejón','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31071','Castillonuevo','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31072','Cintruénigo','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31073','Ziordia','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31074','Cirauqui/Zirauki','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31075','Ciriza/Ziritza','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31076','Cizur','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31077','Corella','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31078','Cortes','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31079','Desojo','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31080','Dicastillo','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31081','Donamaria','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31082','Etxalar','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31083','Echarri/Etxarri','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31084','Etxarri Aranatz','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31085','Etxauri','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31086','Valle de Egüés/Eguesibar','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31087','Elgorriaga','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31088','Noáin (Valle de Elorz)/Noain (Elortzibar)','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31089','Enériz/Eneritz','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31090','Eratsun','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31091','Ergoiena','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31092','Erro','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31093','Ezcároz/Ezkaroze','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31094','Eslava','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31095','Esparza de Salazar/Espartza Zaraitzu','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31096','Espronceda','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31097','Estella-Lizarra','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31098','Esteribar','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31099','Etayo','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31100','Eulate','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31101','Ezcabarte','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31102','Ezkurra','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31103','Ezprogui','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31104','Falces','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31105','Fitero','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31106','Fontellas','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31107','Funes','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31108','Fustiñana','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31109','Galar','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31110','Gallipienzo/Galipentzu','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31111','Gallués/Galoze','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31112','Garaioa','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31113','Garde','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31114','Garínoain','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31115','Garralda','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31116','Genevilla','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31117','Goizueta','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31118','Goñi','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31119','Güesa/Gorza','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31120','Guesálaz/Gesalatz','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31121','Guirguillano','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31122','Huarte/Uharte','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31123','Uharte Arakil','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31124','Ibargoiti','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31125','Igúzquiza','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31126','Imotz','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31127','Irañeta','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31128','Isaba/Izaba','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31129','Ituren','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31130','Iturmendi','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31131','Iza/Itza','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31132','Izagaondoa','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31133','Izalzu/Itzaltzu','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31134','Jaurrieta','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31135','Javier','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31136','Juslapeña','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31137','Beintza-Labaien','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31138','Lakuntza','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31139','Lana','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31140','Lantz','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31141','Lapoblación','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31142','Larraga','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31143','Larraona','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31144','Larraun','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31145','Lazagurría','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31146','Leache/Leatxe','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31147','Legarda','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31148','Legaria','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31149','Leitza','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31150','Leoz/Leotz','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31151','Lerga','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31152','Lerín','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31153','Lesaka','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31154','Lezaun','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31155','Liédena','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31156','Lizoáin-Arriasgoiti','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31157','Lodosa','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31158','Lónguida/Longida','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31159','Lumbier','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31160','Luquin','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31161','Mañeru','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31162','Marañón','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31163','Marcilla','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31164','Mélida','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31165','Mendavia','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31166','Mendaza','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31167','Mendigorría','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31168','Metauten','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31169','Milagro','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31170','Mirafuentes','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31171','Miranda de Arga','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31172','Monreal/Elo','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31173','Monteagudo','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31174','Morentin','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31175','Mues','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31176','Murchante','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31177','Murieta','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31178','Murillo el Cuende','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31179','Murillo el Fruto','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31180','Muruzábal','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31181','Navascués/Nabaskoze','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31182','Nazar','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31183','Obanos','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31184','Oco','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31185','Ochagavía/Otsagabia','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31186','Odieta','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31187','Oiz','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31188','Olaibar','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31189','Olazti/Olazagutía','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31190','Olejua','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31191','Olite/Erriberri','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31192','Olóriz/Oloritz','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31193','Cendea de Olza/Oltza Zendea','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31194','Valle de Ollo/Ollaran','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31195','Orbaizeta','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31196','Orbara','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31197','Orísoain','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31198','Oronz/Orontze','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31199','Oroz-Betelu/Orotz-Betelu','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31200','Oteiza','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31201','Pamplona/Iruña','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31202','Peralta/Azkoien','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31203','Petilla de Aragón','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31204','Piedramillera','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31205','Pitillas','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31206','Puente la Reina/Gares','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31207','Pueyo','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31208','Ribaforada','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31209','Romanzado','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31210','Roncal/Erronkari','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31211','Orreaga/Roncesvalles','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31212','Sada','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31213','Saldías','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31214','Salinas de Oro/Jaitz','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31215','San Adrián','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31216','Sangüesa/Zangoza','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31217','San Martín de Unx','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31219','Sansol','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31220','Santacara','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31221','Doneztebe/Santesteban','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31222','Sarriés/Sartze','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31223','Sartaguda','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31224','Sesma','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31225','Sorlada','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31226','Sunbilla','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31227','Tafalla','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31228','Tiebas-Muruarte de Reta','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31229','Tirapu','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31230','Torralba del Río','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31231','Torres del Río','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31232','Tudela','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31233','Tulebras','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31234','Ucar','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31235','Ujué/Uxue','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31236','Ultzama','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31237','Unciti','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31238','Unzué/Untzue','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31239','Urdazubi/Urdax','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31240','Urdiain','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31241','Urraul Alto','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31242','Urraul Bajo','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31243','Urroz-Villa','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31244','Urroz','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31245','Urzainqui/Urzainki','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31246','Uterga','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31247','Uztárroz/Uztarroze','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31248','Luzaide/Valcarlos','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31249','Valtierra','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31250','Bera','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31251','Viana','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31252','Vidángoz/Bidankoze','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31253','Bidaurreta','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31254','Villafranca','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31255','Villamayor de Monjardín','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31256','Hiriberri/Villanueva de Aezkoa','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31257','Villatuerta','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31258','Villava/Atarrabia','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31259','Igantzi','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31260','Valle de Yerri/Deierri','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31261','Yesa','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31262','Zabalza/Zabaltza','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31263','Zubieta','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31264','Zugarramurdi','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31265','Zúñiga','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31901','Barañáin/Barañain','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31902','Berrioplano/Berriobeiti','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31903','Berriozar','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31904','Irurtzun','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31905','Beriáin','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31906','Orkoien','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31907','Zizur Mayor/Zizur Nagusia','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'31908','Lekunberri','','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'32001','Allariz','OU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'32002','Amoeiro','OU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'32003','Arnoia (A)','OU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'32004','Avión','OU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'32005','Baltar','OU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'32006','Bande','OU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'32007','Baños de Molgas','OU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'32008','Barbadás','OU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'32009','Barco de Valdeorras (O)','OU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'32010','Beade','OU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'32011','Beariz','OU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'32012','Blancos (Os)','OU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'32013','Boborás','OU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'32014','Bola (A)','OU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'32015','Bolo (O)','OU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'32016','Calvos de Randín','OU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'32017','Carballeda de Valdeorras','OU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'32018','Carballeda de Avia','OU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'32019','Carballiño (O)','OU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'32020','Cartelle','OU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'32021','Castrelo do Val','OU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'32022','Castrelo de Miño','OU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'32023','Castro Caldelas','OU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'32024','Celanova','OU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'32025','Cenlle','OU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'32026','Coles','OU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'32027','Cortegada','OU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'32028','Cualedro','OU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'32029','Chandrexa de Queixa','OU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'32030','Entrimo','OU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'32031','Esgos','OU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'32032','Xinzo de Limia','OU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'32033','Gomesende','OU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'32034','Gudiña (A)','OU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'32035','Irixo (O)','OU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'32036','Xunqueira de Ambía','OU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'32037','Xunqueira de Espadanedo','OU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'32038','Larouco','OU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'32039','Laza','OU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'32040','Leiro','OU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'32041','Lobeira','OU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'32042','Lobios','OU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'32043','Maceda','OU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'32044','Manzaneda','OU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'32045','Maside','OU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'32046','Melón','OU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'32047','Merca (A)','OU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'32048','Mezquita (A)','OU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'32049','Montederramo','OU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'32050','Monterrei','OU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'32051','Muíños','OU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'32052','Nogueira de Ramuín','OU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'32053','Oímbra','OU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'32054','Ourense','OU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'32055','Paderne de Allariz','OU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'32056','Padrenda','OU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'32057','Parada de Sil','OU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'32058','Pereiro de Aguiar (O)','OU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'32059','Peroxa (A)','OU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'32060','Petín','OU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'32061','Piñor','OU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'32062','Porqueira','OU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'32063','Pobra de Trives (A)','OU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'32064','Pontedeva','OU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'32065','Punxín','OU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'32066','Quintela de Leirado','OU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'32067','Rairiz de Veiga','OU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'32068','Ramirás','OU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'32069','Ribadavia','OU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'32070','San Xoán de Río','OU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'32071','Riós','OU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'32072','Rúa (A)','OU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'32073','Rubiá','OU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'32074','San Amaro','OU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'32075','San Cibrao das Viñas','OU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'32076','San Cristovo de Cea','OU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'32077','Sandiás','OU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'32078','Sarreaus','OU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'32079','Taboadela','OU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'32080','Teixeira (A)','OU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'32081','Toén','OU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'32082','Trasmiras','OU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'32083','Veiga (A)','OU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'32084','Verea','OU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'32085','Verín','OU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'32086','Viana do Bolo','OU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'32087','Vilamarín','OU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'32088','Vilamartín de Valdeorras','OU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'32089','Vilar de Barrio','OU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'32090','Vilar de Santos','OU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'32091','Vilardevós','OU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'32092','Vilariño de Conso','OU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'33001','Allande','O','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'33002','Aller','O','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'33003','Amieva','O','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'33004','Avilés','O','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'33005','Belmonte de Miranda','O','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'33006','Bimenes','O','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'33007','Boal','O','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'33008','Cabrales','O','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'33009','Cabranes','O','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'33010','Candamo','O','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'33011','Cangas del Narcea','O','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'33012','Cangas de Onís','O','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'33013','Caravia','O','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'33014','Carreño','O','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'33015','Caso','O','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'33016','Castrillón','O','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'33017','Castropol','O','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'33018','Coaña','O','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'33019','Colunga','O','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'33020','Corvera de Asturias','O','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'33021','Cudillero','O','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'33022','Degaña','O','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'33023','Franco (El)','O','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'33024','Gijón','O','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'33025','Gozón','O','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'33026','Grado','O','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'33027','Grandas de Salime','O','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'33028','Ibias','O','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'33029','Illano','O','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'33030','Illas','O','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'33031','Langreo','O','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'33032','Laviana','O','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'33033','Lena','O','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'33034','Valdés','O','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'33035','Llanera','O','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'33036','Llanes','O','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'33037','Mieres','O','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'33038','Morcín','O','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'33039','Muros de Nalón','O','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'33040','Nava','O','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'33041','Navia','O','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'33042','Noreña','O','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'33043','Onís','O','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'33044','Oviedo','O','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'33045','Parres','O','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'33046','Peñamellera Alta','O','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'33047','Peñamellera Baja','O','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'33048','Pesoz','O','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'33049','Piloña','O','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'33050','Ponga','O','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'33051','Pravia','O','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'33052','Proaza','O','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'33053','Quirós','O','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'33054','Regueras (Las)','O','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'33055','Ribadedeva','O','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'33056','Ribadesella','O','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'33057','Ribera de Arriba','O','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'33058','Riosa','O','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'33059','Salas','O','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'33060','San Martín del Rey Aurelio','O','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'33061','San Martín de Oscos','O','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'33062','Santa Eulalia de Oscos','O','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'33063','San Tirso de Abres','O','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'33064','Santo Adriano','O','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'33065','Sariego','O','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'33066','Siero','O','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'33067','Sobrescobio','O','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'33068','Somiedo','O','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'33069','Soto del Barco','O','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'33070','Tapia de Casariego','O','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'33071','Taramundi','O','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'33072','Teverga','O','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'33073','Tineo','O','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'33074','Vegadeo','O','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'33075','Villanueva de Oscos','O','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'33076','Villaviciosa','O','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'33077','Villayón','O','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'33078','Yernes y Tameza','O','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'34001','Abarca de Campos','P','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'34003','Abia de las Torres','P','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'34004','Aguilar de Campoo','P','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'34005','Alar del Rey','P','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'34006','Alba de Cerrato','P','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'34009','Amayuelas de Arriba','P','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'34010','Ampudia','P','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'34011','Amusco','P','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'34012','Antigüedad','P','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'34015','Arconada','P','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'34017','Astudillo','P','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'34018','Autilla del Pino','P','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'34019','Autillo de Campos','P','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'34020','Ayuela','P','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'34022','Baltanás','P','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'34023','Venta de Baños','P','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'34024','Baquerín de Campos','P','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'34025','Bárcena de Campos','P','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'34027','Barruelo de Santullán','P','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'34028','Báscones de Ojeda','P','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'34029','Becerril de Campos','P','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'34031','Belmonte de Campos','P','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'34032','Berzosilla','P','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'34033','Boada de Campos','P','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'34034','Boadilla del Camino','P','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'34035','Boadilla de Rioseco','P','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'34036','Brañosera','P','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'34037','Buenavista de Valdavia','P','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'34038','Bustillo de la Vega','P','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'34039','Bustillo del Páramo de Carrión','P','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'34041','Calahorra de Boedo','P','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'34042','Calzada de los Molinos','P','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'34045','Capillas','P','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'34046','Cardeñosa de Volpejera','P','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'34047','Carrión de los Condes','P','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'34048','Castil de Vela','P','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'34049','Castrejón de la Peña','P','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'34050','Castrillo de Don Juan','P','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'34051','Castrillo de Onielo','P','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'34052','Castrillo de Villavega','P','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'34053','Castromocho','P','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'34055','Cervatos de la Cueza','P','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'34056','Cervera de Pisuerga','P','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'34057','Cevico de la Torre','P','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'34058','Cevico Navero','P','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'34059','Cisneros','P','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'34060','Cobos de Cerrato','P','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'34061','Collazos de Boedo','P','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'34062','Congosto de Valdavia','P','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'34063','Cordovilla la Real','P','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'34066','Cubillas de Cerrato','P','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'34067','Dehesa de Montejo','P','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'34068','Dehesa de Romanos','P','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'34069','Dueñas','P','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'34070','Espinosa de Cerrato','P','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'34071','Espinosa de Villagonzalo','P','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'34072','Frechilla','P','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'34073','Fresno del Río','P','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'34074','Frómista','P','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'34076','Fuentes de Nava','P','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'34077','Fuentes de Valdepero','P','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'34079','Grijota','P','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'34080','Guardo','P','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'34081','Guaza de Campos','P','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'34082','Hérmedes de Cerrato','P','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'34083','Herrera de Pisuerga','P','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'34084','Herrera de Valdecañas','P','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'34086','Hontoria de Cerrato','P','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'34087','Hornillos de Cerrato','P','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'34088','Husillos','P','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'34089','Itero de la Vega','P','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'34091','Lagartos','P','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'34092','Lantadilla','P','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'34093','Vid de Ojeda (La)','P','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'34094','Ledigos','P','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'34096','Lomas','P','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'34098','Magaz de Pisuerga','P','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'34099','Manquillos','P','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'34100','Mantinos','P','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'34101','Marcilla de Campos','P','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'34102','Mazariegos','P','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'34103','Mazuecos de Valdeginate','P','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'34104','Melgar de Yuso','P','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'34106','Meneses de Campos','P','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'34107','Micieces de Ojeda','P','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'34108','Monzón de Campos','P','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'34109','Moratinos','P','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'34110','Mudá','P','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'34112','Nogal de las Huertas','P','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'34113','Olea de Boedo','P','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'34114','Olmos de Ojeda','P','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'34116','Osornillo','P','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'34120','Palencia','P','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'34121','Palenzuela','P','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'34122','Páramo de Boedo','P','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'34123','Paredes de Nava','P','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'34124','Payo de Ojeda','P','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'34125','Pedraza de Campos','P','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'34126','Pedrosa de la Vega','P','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'34127','Perales','P','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'34129','Pino del Río','P','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'34130','Piña de Campos','P','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'34131','Población de Arroyo','P','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'34132','Población de Campos','P','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'34133','Población de Cerrato','P','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'34134','Polentinos','P','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'34135','Pomar de Valdivia','P','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'34136','Poza de la Vega','P','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'34137','Pozo de Urama','P','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'34139','Prádanos de Ojeda','P','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'34140','Puebla de Valdavia (La)','P','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'34141','Quintana del Puente','P','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'34143','Quintanilla de Onsoña','P','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'34146','Reinoso de Cerrato','P','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'34147','Renedo de la Vega','P','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'34149','Requena de Campos','P','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'34151','Respenda de la Peña','P','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'34152','Revenga de Campos','P','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'34154','Revilla de Collazos','P','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'34155','Ribas de Campos','P','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'34156','Riberos de la Cueza','P','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'34157','Saldaña','P','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'34158','Salinas de Pisuerga','P','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'34159','San Cebrián de Campos','P','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'34160','San Cebrián de Mudá','P','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'34161','San Cristóbal de Boedo','P','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'34163','San Mamés de Campos','P','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'34165','San Román de la Cuba','P','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'34167','Santa Cecilia del Alcor','P','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'34168','Santa Cruz de Boedo','P','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'34169','Santervás de la Vega','P','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'34170','Santibáñez de Ecla','P','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'34171','Santibáñez de la Peña','P','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'34174','Santoyo','P','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'34175','Serna (La)','P','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'34176','Sotobañado y Priorato','P','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'34177','Soto de Cerrato','P','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'34178','Tabanera de Cerrato','P','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'34179','Tabanera de Valdavia','P','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'34180','Támara de Campos','P','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'34181','Tariego de Cerrato','P','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'34182','Torquemada','P','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'34184','Torremormojón','P','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'34185','Triollo','P','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'34186','Valbuena de Pisuerga','P','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'34189','Valdeolmillos','P','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'34190','Valderrábano','P','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'34192','Valde-Ucieza','P','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'34196','Valle de Cerrato','P','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'34199','Velilla del Río Carrión','P','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'34201','Vertavillo','P','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'34202','Villabasta de Valdavia','P','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'34204','Villacidaler','P','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'34205','Villaconancio','P','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'34206','Villada','P','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'34208','Villaeles de Valdavia','P','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'34210','Villahán','P','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'34211','Villaherreros','P','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'34213','Villalaco','P','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'34214','Villalba de Guardo','P','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'34215','Villalcázar de Sirga','P','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'34216','Villalcón','P','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'34217','Villalobón','P','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'34218','Villaluenga de la Vega','P','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'34220','Villamartín de Campos','P','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'34221','Villamediana','P','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'34222','Villameriel','P','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'34223','Villamoronta','P','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'34224','Villamuera de la Cueza','P','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'34225','Villamuriel de Cerrato','P','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'34227','Villanueva del Rebollar','P','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'34228','Villanuño de Valdavia','P','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'34229','Villaprovedo','P','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'34230','Villarmentero de Campos','P','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'34231','Villarrabé','P','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'34232','Villarramiel','P','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'34233','Villasarracino','P','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'34234','Villasila de Valdavia','P','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'34236','Villaturde','P','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'34237','Villaumbrales','P','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'34238','Villaviudas','P','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'34240','Villerías de Campos','P','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'34241','Villodre','P','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'34242','Villodrigo','P','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'34243','Villoldo','P','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'34245','Villota del Páramo','P','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'34246','Villovieco','P','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'34901','Osorno la Mayor','P','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'34902','Valle del Retortillo','P','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'34903','Loma de Ucieza','P','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'34904','Pernía (La)','P','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'36001','Arbo','PO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'36002','Barro','PO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'36003','Baiona','PO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'36004','Bueu','PO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'36005','Caldas de Reis','PO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'36006','Cambados','PO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'36007','Campo Lameiro','PO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'36008','Cangas','PO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'36009','Cañiza (A)','PO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'36010','Catoira','PO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'36013','Covelo','PO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'36014','Crecente','PO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'36015','Cuntis','PO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'36016','Dozón','PO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'36017','Estrada (A)','PO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'36018','Forcarei','PO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'36019','Fornelos de Montes','PO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'36020','Agolada','PO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'36021','Gondomar','PO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'36022','Grove (O)','PO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'36023','Guarda (A)','PO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'36024','Lalín','PO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'36025','Lama (A)','PO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'36026','Marín','PO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'36027','Meaño','PO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'36028','Meis','PO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'36029','Moaña','PO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'36030','Mondariz','PO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'36031','Mondariz-Balneario','PO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'36032','Moraña','PO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'36033','Mos','PO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'36034','Neves (As)','PO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'36035','Nigrán','PO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'36036','Oia','PO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'36037','Pazos de Borbén','PO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'36038','Pontevedra','PO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'36039','Porriño (O)','PO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'36040','Portas','PO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'36041','Poio','PO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'36042','Ponteareas','PO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'36043','Ponte Caldelas','PO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'36044','Pontecesures','PO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'36045','Redondela','PO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'36046','Ribadumia','PO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'36047','Rodeiro','PO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'36048','Rosal (O)','PO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'36049','Salceda de Caselas','PO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'36050','Salvaterra de Miño','PO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'36051','Sanxenxo','PO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'36052','Silleda','PO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'36053','Soutomaior','PO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'36054','Tomiño','PO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'36055','Tui','PO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'36056','Valga','PO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'36057','Vigo','PO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'36058','Vilaboa','PO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'36059','Vila de Cruces','PO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'36060','Vilagarcía de Arousa','PO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'36061','Vilanova de Arousa','PO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'36901','Illa de Arousa (A)','PO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'36902','Cerdedo-Cotobade','PO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37001','Abusejo','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37002','Agallas','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37003','Ahigal de los Aceiteros','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37004','Ahigal de Villarino','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37005','Alameda de Gardón (La)','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37006','Alamedilla (La)','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37007','Alaraz','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37008','Alba de Tormes','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37009','Alba de Yeltes','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37010','Alberca (La)','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37011','Alberguería de Argañán (La)','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37012','Alconada','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37013','Aldeacipreste','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37014','Aldeadávila de la Ribera','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37015','Aldea del Obispo','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37016','Aldealengua','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37017','Aldeanueva de Figueroa','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37018','Aldeanueva de la Sierra','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37019','Aldearrodrigo','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37020','Aldearrubia','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37021','Aldeaseca de Alba','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37022','Aldeaseca de la Frontera','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37023','Aldeatejada','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37024','Aldeavieja de Tormes','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37025','Aldehuela de la Bóveda','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37026','Aldehuela de Yeltes','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37027','Almenara de Tormes','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37028','Almendra','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37029','Anaya de Alba','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37030','Añover de Tormes','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37031','Arabayona de Mógica','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37032','Arapiles','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37033','Arcediano','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37034','Arco (El)','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37035','Armenteros','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37036','San Miguel del Robledo','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37037','Atalaya (La)','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37038','Babilafuente','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37039','Bañobárez','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37040','Barbadillo','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37041','Barbalos','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37042','Barceo','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37044','Barruecopardo','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37045','Bastida (La)','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37046','Béjar','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37047','Beleña','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37049','Bermellar','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37050','Berrocal de Huebra','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37051','Berrocal de Salvatierra','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37052','Boada','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37054','Bodón (El)','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37055','Bogajo','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37056','Bouza (La)','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37057','Bóveda del Río Almar','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37058','Brincones','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37059','Buenamadre','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37060','Buenavista','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37061','Cabaco (El)','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37062','Cabezabellosa de la Calzada','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37063','Cabeza de Béjar (La)','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37065','Cabeza del Caballo','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37067','Cabrerizos','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37068','Cabrillas','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37069','Calvarrasa de Abajo','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37070','Calvarrasa de Arriba','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37071','Calzada de Béjar (La)','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37072','Calzada de Don Diego','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37073','Calzada de Valdunciel','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37074','Campillo de Azaba','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37077','Campo de Peñaranda (El)','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37078','Candelario','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37079','Canillas de Abajo','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37080','Cantagallo','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37081','Cantalapiedra','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37082','Cantalpino','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37083','Cantaracillo','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37085','Carbajosa de la Sagrada','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37086','Carpio de Azaba','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37087','Carrascal de Barregas','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37088','Carrascal del Obispo','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37089','Casafranca','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37090','Casas del Conde (Las)','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37091','Casillas de Flores','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37092','Castellanos de Moriscos','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37096','Castillejo de Martín Viejo','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37097','Castraz','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37098','Cepeda','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37099','Cereceda de la Sierra','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37100','Cerezal de Peñahorcada','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37101','Cerralbo','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37102','Cerro (El)','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37103','Cespedosa de Tormes','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37104','Cilleros de la Bastida','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37106','Cipérez','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37107','Ciudad Rodrigo','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37108','Coca de Alba','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37109','Colmenar de Montemayor','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37110','Cordovilla','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37112','Cristóbal','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37113','Cubo de Don Sancho (El)','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37114','Chagarcía Medianero','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37115','Dios le Guarde','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37116','Doñinos de Ledesma','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37117','Doñinos de Salamanca','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37118','Éjeme','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37119','Encina (La)','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37120','Encina de San Silvestre','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37121','Encinas de Abajo','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37122','Encinas de Arriba','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37123','Encinasola de los Comendadores','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37124','Endrinal','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37125','Escurial de la Sierra','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37126','Espadaña','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37127','Espeja','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37128','Espino de la Orbada','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37129','Florida de Liébana','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37130','Forfoleda','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37131','Frades de la Sierra','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37132','Fregeneda (La)','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37133','Fresnedoso','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37134','Fresno Alhándiga','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37135','Fuente de San Esteban (La)','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37136','Fuenteguinaldo','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37137','Fuenteliante','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37138','Fuenterroble de Salvatierra','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37139','Fuentes de Béjar','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37140','Fuentes de Oñoro','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37141','Gajates','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37142','Galindo y Perahuy','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37143','Galinduste','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37144','Galisancho','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37145','Gallegos de Argañán','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37146','Gallegos de Solmirón','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37147','Garcibuey','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37148','Garcihernández','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37149','Garcirrey','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37150','Gejuelo del Barro','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37151','Golpejas','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37152','Gomecello','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37154','Guadramiro','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37155','Guijo de Ávila','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37156','Guijuelo','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37157','Herguijuela de Ciudad Rodrigo','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37158','Herguijuela de la Sierra','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37159','Herguijuela del Campo','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37160','Hinojosa de Duero','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37161','Horcajo de Montemayor','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37162','Horcajo Medianero','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37163','Hoya (La)','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37164','Huerta','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37165','Iruelos','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37166','Ituero de Azaba','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37167','Juzbado','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37168','Lagunilla','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37169','Larrodrigo','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37170','Ledesma','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37171','Ledrada','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37172','Linares de Riofrío','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37173','Lumbrales','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37174','Macotera','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37175','Machacón','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37176','Madroñal','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37177','Maíllo (El)','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37178','Malpartida','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37179','Mancera de Abajo','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37180','Manzano (El)','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37181','Martiago','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37182','Martinamor','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37183','Martín de Yeltes','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37184','Masueco','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37185','Castellanos de Villiquera','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37186','Mata de Ledesma (La)','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37187','Matilla de los Caños del Río','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37188','Maya (La)','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37189','Membribe de la Sierra','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37190','Mieza','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37191','Milano (El)','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37192','Miranda de Azán','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37193','Miranda del Castañar','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37194','Mogarraz','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37195','Molinillo','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37196','Monforte de la Sierra','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37197','Monleón','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37198','Monleras','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37199','Monsagro','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37200','Montejo','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37201','Montemayor del Río','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37202','Monterrubio de Armuña','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37203','Monterrubio de la Sierra','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37204','Morasverdes','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37205','Morille','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37206','Moríñigo','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37207','Moriscos','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37208','Moronta','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37209','Mozárbez','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37211','Narros de Matalayegua','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37212','Navacarros','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37213','Nava de Béjar','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37214','Nava de Francia','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37215','Nava de Sotrobal','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37216','Navales','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37217','Navalmoral de Béjar','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37218','Navamorales','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37219','Navarredonda de la Rinconada','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37221','Navasfrías','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37222','Negrilla de Palencia','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37223','Olmedo de Camaces','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37224','Orbada (La)','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37225','Pajares de la Laguna','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37226','Palacios del Arzobispo','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37228','Palaciosrubios','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37229','Palencia de Negrilla','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37230','Parada de Arriba','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37231','Parada de Rubiales','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37232','Paradinas de San Juan','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37233','Pastores','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37234','Payo (El)','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37235','Pedraza de Alba','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37236','Pedrosillo de Alba','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37237','Pedrosillo de los Aires','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37238','Pedrosillo el Ralo','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37239','Pedroso de la Armuña (El)','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37240','Pelabravo','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37241','Pelarrodríguez','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37242','Pelayos','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37243','Peña (La)','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37244','Peñacaballera','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37245','Peñaparda','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37246','Peñaranda de Bracamonte','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37247','Peñarandilla','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37248','Peralejos de Abajo','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37249','Peralejos de Arriba','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37250','Pereña de la Ribera','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37251','Peromingo','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37252','Pinedas','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37253','Pino de Tormes (El)','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37254','Pitiegua','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37255','Pizarral','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37256','Poveda de las Cintas','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37257','Pozos de Hinojo','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37258','Puebla de Azaba','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37259','Puebla de San Medel','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37260','Puebla de Yeltes','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37261','Puente del Congosto','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37262','Puertas','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37263','Puerto de Béjar','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37264','Puerto Seguro','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37265','Rágama','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37266','Redonda (La)','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37267','Retortillo','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37268','Rinconada de la Sierra (La)','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37269','Robleda','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37270','Robliza de Cojos','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37271','Rollán','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37272','Saelices el Chico','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37273','Sagrada (La)','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37274','Salamanca','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37275','Saldeana','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37276','Salmoral','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37277','Salvatierra de Tormes','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37278','San Cristóbal de la Cuesta','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37279','Sancti-Spíritus','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37280','Sanchón de la Ribera','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37281','Sanchón de la Sagrada','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37282','Sanchotello','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37283','Sando','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37284','San Esteban de la Sierra','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37285','San Felices de los Gallegos','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37286','San Martín del Castañar','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37287','San Miguel de Valero','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37288','San Morales','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37289','San Muñoz','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37290','San Pedro del Valle','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37291','San Pedro de Rozados','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37292','San Pelayo de Guareña','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37293','Santa María de Sando','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37294','Santa Marta de Tormes','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37296','Santiago de la Puebla','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37297','Santibáñez de Béjar','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37298','Santibáñez de la Sierra','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37299','Santiz','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37300','Santos (Los)','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37301','Sardón de los Frailes','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37302','Saucelle','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37303','Sahugo (El)','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37304','Sepulcro-Hilario','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37305','Sequeros','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37306','Serradilla del Arroyo','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37307','Serradilla del Llano','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37309','Sierpe (La)','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37310','Sieteiglesias de Tormes','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37311','Sobradillo','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37312','Sorihuela','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37313','Sotoserrano','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37314','Tabera de Abajo','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37315','Tala (La)','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37316','Tamames','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37317','Tarazona de Guareña','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37318','Tardáguila','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37319','Tejado (El)','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37320','Tejeda y Segoyuela','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37321','Tenebrón','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37322','Terradillos','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37323','Topas','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37324','Tordillos','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37325','Tornadizo (El)','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37327','Torresmenudas','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37328','Trabanca','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37329','Tremedal de Tormes','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37330','Valdecarros','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37331','Valdefuentes de Sangusín','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37332','Valdehijaderos','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37333','Valdelacasa','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37334','Valdelageve','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37335','Valdelosa','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37336','Valdemierque','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37337','Valderrodrigo','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37338','Valdunciel','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37339','Valero','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37340','Valsalabroso','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37341','Valverde de Valdelacasa','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37342','Valverdón','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37343','Vallejera de Riofrío','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37344','Vecinos','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37345','Vega de Tirados','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37346','Veguillas (Las)','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37347','Vellés (La)','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37348','Ventosa del Río Almar','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37349','Vídola (La)','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37350','Vilvestre','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37351','Villaflores','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37352','Villagonzalo de Tormes','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37353','Villalba de los Llanos','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37354','Villamayor','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37355','Villanueva del Conde','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37356','Villar de Argañán','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37357','Villar de Ciervo','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37358','Villar de Gallimazo','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37359','Villar de la Yegua','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37360','Villar de Peralonso','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37361','Villar de Samaniego','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37362','Villares de la Reina','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37363','Villares de Yeltes','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37364','Villarino de los Aires','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37365','Villarmayor','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37366','Villarmuerto','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37367','Villasbuenas','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37368','Villasdardo','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37369','Villaseco de los Gamitos','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37370','Villaseco de los Reyes','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37371','Villasrubias','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37372','Villaverde de Guareña','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37373','Villavieja de Yeltes','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37374','Villoria','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37375','Villoruela','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37376','Vitigudino','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37377','Yecla de Yeltes','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37378','Zamarra','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37379','Zamayón','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37380','Zarapicos','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37381','Zarza de Pumareda (La)','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'37382','Zorita de la Frontera','SA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'39001','Alfoz de Lloredo','S','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'39002','Ampuero','S','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'39003','Anievas','S','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'39004','Arenas de Iguña','S','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'39005','Argoños','S','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'39006','Arnuero','S','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'39007','Arredondo','S','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'39008','Astillero (El)','S','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'39009','Bárcena de Cicero','S','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'39010','Bárcena de Pie de Concha','S','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'39011','Bareyo','S','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'39012','Cabezón de la Sal','S','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'39013','Cabezón de Liébana','S','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'39014','Cabuérniga','S','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'39015','Camaleño','S','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'39016','Camargo','S','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'39017','Campoo de Yuso','S','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'39018','Cartes','S','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'39019','Castañeda','S','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'39020','Castro-Urdiales','S','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'39021','Cieza','S','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'39022','Cillorigo de Liébana','S','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'39023','Colindres','S','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'39024','Comillas','S','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'39025','Corrales de Buelna (Los)','S','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'39026','Corvera de Toranzo','S','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'39027','Campoo de Enmedio','S','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'39028','Entrambasaguas','S','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'39029','Escalante','S','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'39030','Guriezo','S','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'39031','Hazas de Cesto','S','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'39032','Hermandad de Campoo de Suso','S','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'39033','Herrerías','S','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'39034','Lamasón','S','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'39035','Laredo','S','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'39036','Liendo','S','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'39037','Liérganes','S','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'39038','Limpias','S','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'39039','Luena','S','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'39040','Marina de Cudeyo','S','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'39041','Mazcuerras','S','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'39042','Medio Cudeyo','S','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'39043','Meruelo','S','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'39044','Miengo','S','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'39045','Miera','S','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'39046','Molledo','S','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'39047','Noja','S','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'39048','Penagos','S','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'39049','Peñarrubia','S','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'39050','Pesaguero','S','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'39051','Pesquera','S','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'39052','Piélagos','S','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'39053','Polaciones','S','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'39054','Polanco','S','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'39055','Potes','S','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'39056','Puente Viesgo','S','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'39057','Ramales de la Victoria','S','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'39058','Rasines','S','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'39059','Reinosa','S','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'39060','Reocín','S','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'39061','Ribamontán al Mar','S','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'39062','Ribamontán al Monte','S','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'39063','Rionansa','S','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'39064','Riotuerto','S','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'39065','Rozas de Valdearroyo (Las)','S','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'39066','Ruente','S','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'39067','Ruesga','S','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'39068','Ruiloba','S','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'39069','San Felices de Buelna','S','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'39070','San Miguel de Aguayo','S','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'39071','San Pedro del Romeral','S','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'39072','San Roque de Riomiera','S','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'39073','Santa Cruz de Bezana','S','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'39074','Santa María de Cayón','S','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'39075','Santander','S','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'39076','Santillana del Mar','S','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'39077','Santiurde de Reinosa','S','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'39078','Santiurde de Toranzo','S','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'39079','Santoña','S','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'39080','San Vicente de la Barquera','S','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'39081','Saro','S','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'39082','Selaya','S','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'39083','Soba','S','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'39084','Solórzano','S','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'39085','Suances','S','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'39086','Tojos (Los)','S','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'39087','Torrelavega','S','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'39088','Tresviso','S','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'39089','Tudanca','S','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'39090','Udías','S','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'39091','Valdáliga','S','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'39092','Valdeolea','S','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'39093','Valdeprado del Río','S','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'39094','Valderredible','S','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'39095','Val de San Vicente','S','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'39096','Vega de Liébana','S','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'39097','Vega de Pas','S','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'39098','Villacarriedo','S','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'39099','Villaescusa','S','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'39100','Villafufre','S','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'39101','Valle de Villaverde','S','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'39102','Voto','S','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40001','Abades','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40002','Adrada de Pirón','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40003','Adrados','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40004','Aguilafuente','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40005','Alconada de Maderuelo','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40006','Aldealcorvo','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40007','Aldealengua de Pedraza','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40008','Aldealengua de Santa María','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40009','Aldeanueva de la Serrezuela','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40010','Aldeanueva del Codonal','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40012','Aldea Real','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40013','Aldeasoña','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40014','Aldehorno','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40015','Aldehuela del Codonal','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40016','Aldeonte','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40017','Anaya','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40018','Añe','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40019','Arahuetes','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40020','Arcones','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40021','Arevalillo de Cega','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40022','Armuña','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40024','Ayllón','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40025','Barbolla','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40026','Basardilla','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40028','Bercial','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40029','Bercimuel','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40030','Bernardos','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40031','Bernuy de Porreros','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40032','Boceguillas','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40033','Brieva','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40034','Caballar','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40035','Cabañas de Polendos','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40036','Cabezuela','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40037','Calabazas de Fuentidueña','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40039','Campo de San Pedro','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40040','Cantalejo','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40041','Cantimpalos','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40043','Carbonero el Mayor','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40044','Carrascal del Río','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40045','Casla','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40046','Castillejo de Mesleón','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40047','Castro de Fuentidueña','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40048','Castrojimeno','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40049','Castroserna de Abajo','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40051','Castroserracín','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40052','Cedillo de la Torre','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40053','Cerezo de Abajo','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40054','Cerezo de Arriba','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40055','Cilleruelo de San Mamés','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40056','Cobos de Fuentidueña','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40057','Coca','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40058','Codorniz','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40059','Collado Hermoso','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40060','Condado de Castilnovo','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40061','Corral de Ayllón','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40062','Cubillo','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40063','Cuéllar','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40065','Chañe','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40068','Domingo García','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40069','Donhierro','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40070','Duruelo','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40071','Encinas','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40072','Encinillas','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40073','Escalona del Prado','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40074','Escarabajosa de Cabezas','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40075','Escobar de Polendos','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40076','Espinar (El)','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40077','Espirdo','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40078','Fresneda de Cuéllar','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40079','Fresno de Cantespino','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40080','Fresno de la Fuente','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40081','Frumales','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40082','Fuente de Santa Cruz','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40083','Fuente el Olmo de Fuentidueña','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40084','Fuente el Olmo de Íscar','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40086','Fuentepelayo','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40087','Fuentepiñel','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40088','Fuenterrebollo','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40089','Fuentesaúco de Fuentidueña','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40091','Fuentesoto','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40092','Fuentidueña','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40093','Gallegos','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40094','Garcillán','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40095','Gomezserracín','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40097','Grajera','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40099','Honrubia de la Cuesta','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40100','Hontalbilla','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40101','Hontanares de Eresma','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40103','Huertos (Los)','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40104','Ituero y Lama','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40105','Juarros de Riomoros','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40106','Juarros de Voltoya','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40107','Labajos','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40108','Laguna de Contreras','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40109','Languilla','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40110','Lastras de Cuéllar','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40111','Lastras del Pozo','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40112','Lastrilla (La)','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40113','Losa (La)','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40115','Maderuelo','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40118','Marazuela','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40119','Martín Miguel','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40120','Martín Muñoz de la Dehesa','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40121','Martín Muñoz de las Posadas','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40122','Marugán','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40123','Matabuena','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40124','Mata de Cuéllar','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40125','Matilla (La)','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40126','Melque de Cercos','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40127','Membibre de la Hoz','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40128','Migueláñez','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40129','Montejo de Arévalo','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40130','Montejo de la Vega de la Serrezuela','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40131','Monterrubio','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40132','Moral de Hornuez','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40134','Mozoncillo','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40135','Muñopedro','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40136','Muñoveros','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40138','Nava de la Asunción','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40139','Navafría','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40140','Navalilla','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40141','Navalmanzano','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40142','Navares de Ayuso','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40143','Navares de Enmedio','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40144','Navares de las Cuevas','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40145','Navas de Oro','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40146','Navas de San Antonio','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40148','Nieva','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40149','Olombrada','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40150','Orejana','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40151','Ortigosa de Pestaño','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40152','Otero de Herreros','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40154','Pajarejos','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40155','Palazuelos de Eresma','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40156','Pedraza','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40157','Pelayos del Arroyo','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40158','Perosillo','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40159','Pinarejos','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40160','Pinarnegrillo','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40161','Carabias','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40162','Prádena','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40163','Puebla de Pedraza','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40164','Rapariegos','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40165','Rebollo','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40166','Remondo','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40168','Riaguas de San Bartolomé','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40170','Riaza','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40171','Ribota','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40172','Riofrío de Riaza','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40173','Roda de Eresma','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40174','Sacramenia','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40176','Samboal','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40177','San Cristóbal de Cuéllar','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40178','San Cristóbal de la Vega','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40179','Sanchonuño','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40180','Sangarcía','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40181','Real Sitio de San Ildefonso','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40182','San Martín y Mudrián','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40183','San Miguel de Bernuy','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40184','San Pedro de Gaíllos','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40185','Santa María la Real de Nieva','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40186','Santa Marta del Cerro','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40188','Santiuste de Pedraza','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40189','Santiuste de San Juan Bautista','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40190','Santo Domingo de Pirón','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40191','Santo Tomé del Puerto','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40192','Sauquillo de Cabezas','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40193','Sebúlcor','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40194','Segovia','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40195','Sepúlveda','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40196','Sequera de Fresno','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40198','Sotillo','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40199','Sotosalbos','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40200','Tabanera la Luenga','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40201','Tolocirio','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40202','Torreadrada','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40203','Torrecaballeros','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40204','Torrecilla del Pinar','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40205','Torreiglesias','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40206','Torre Val de San Pedro','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40207','Trescasas','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40208','Turégano','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40210','Urueñas','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40211','Valdeprados','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40212','Valdevacas de Montejo','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40213','Valdevacas y Guijar','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40214','Valseca','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40215','Valtiendas','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40216','Valverde del Majano','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40218','Valle de Tabladillo','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40219','Vallelado','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40220','Valleruela de Pedraza','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40221','Valleruela de Sepúlveda','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40222','Veganzones','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40223','Vegas de Matute','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40224','Ventosilla y Tejadilla','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40225','Villacastín','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40228','Villaverde de Íscar','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40229','Villaverde de Montejo','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40230','Villeguillo','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40231','Yanguas de Eresma','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40233','Zarzuela del Monte','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40234','Zarzuela del Pinar','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40901','Ortigosa del Monte','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40902','Cozuelos de Fuentidueña','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40903','Marazoleja','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40904','Navas de Riofrío','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40905','Cuevas de Provanco','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'40906','San Cristóbal de Segovia','SG','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'41001','Aguadulce','SE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'41002','Alanís','SE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'41003','Albaida del Aljarafe','SE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'41004','Alcalá de Guadaíra','SE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'41005','Alcalá del Río','SE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'41006','Alcolea del Río','SE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'41007','Algaba (La)','SE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'41008','Algámitas','SE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'41009','Almadén de la Plata','SE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'41010','Almensilla','SE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'41011','Arahal','SE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'41012','Aznalcázar','SE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'41013','Aznalcóllar','SE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'41014','Badolatosa','SE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'41015','Benacazón','SE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'41016','Bollullos de la Mitación','SE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'41017','Bormujos','SE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'41018','Brenes','SE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'41019','Burguillos','SE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'41020','Cabezas de San Juan (Las)','SE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'41021','Camas','SE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'41022','Campana (La)','SE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'41023','Cantillana','SE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'41024','Carmona','SE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'41025','Carrión de los Céspedes','SE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'41026','Casariche','SE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'41027','Castilblanco de los Arroyos','SE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'41028','Castilleja de Guzmán','SE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'41029','Castilleja de la Cuesta','SE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'41030','Castilleja del Campo','SE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'41031','Castillo de las Guardas (El)','SE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'41032','Cazalla de la Sierra','SE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'41033','Constantina','SE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'41034','Coria del Río','SE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'41035','Coripe','SE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'41036','Coronil (El)','SE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'41037','Corrales (Los)','SE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'41038','Dos Hermanas','SE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'41039','Écija','SE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'41040','Espartinas','SE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'41041','Estepa','SE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'41042','Fuentes de Andalucía','SE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'41043','Garrobo (El)','SE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'41044','Gelves','SE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'41045','Gerena','SE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'41046','Gilena','SE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'41047','Gines','SE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'41048','Guadalcanal','SE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'41049','Guillena','SE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'41050','Herrera','SE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'41051','Huévar del Aljarafe','SE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'41052','Lantejuela','SE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'41053','Lebrija','SE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'41054','Lora de Estepa','SE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'41055','Lora del Río','SE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'41056','Luisiana (La)','SE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'41057','Madroño (El)','SE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'41058','Mairena del Alcor','SE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'41059','Mairena del Aljarafe','SE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'41060','Marchena','SE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'41061','Marinaleda','SE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'41062','Martín de la Jara','SE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'41063','Molares (Los)','SE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'41064','Montellano','SE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'41065','Morón de la Frontera','SE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'41066','Navas de la Concepción (Las)','SE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'41067','Olivares','SE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'41068','Osuna','SE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'41069','Palacios y Villafranca (Los)','SE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'41070','Palomares del Río','SE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'41071','Paradas','SE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'41072','Pedrera','SE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'41073','Pedroso (El)','SE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'41074','Peñaflor','SE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'41075','Pilas','SE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'41076','Pruna','SE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'41077','Puebla de Cazalla (La)','SE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'41078','Puebla de los Infantes (La)','SE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'41079','Puebla del Río (La)','SE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'41080','Real de la Jara (El)','SE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'41081','Rinconada (La)','SE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'41082','Roda de Andalucía (La)','SE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'41083','Ronquillo (El)','SE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'41084','Rubio (El)','SE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'41085','Salteras','SE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'41086','San Juan de Aznalfarache','SE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'41087','Sanlúcar la Mayor','SE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'41088','San Nicolás del Puerto','SE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'41089','Santiponce','SE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'41090','Saucejo (El)','SE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'41091','Sevilla','SE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'41092','Tocina','SE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'41093','Tomares','SE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'41094','Umbrete','SE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'41095','Utrera','SE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'41096','Valencina de la Concepción','SE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'41097','Villamanrique de la Condesa','SE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'41098','Villanueva del Ariscal','SE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'41099','Villanueva del Río y Minas','SE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'41100','Villanueva de San Juan','SE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'41101','Villaverde del Río','SE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'41102','Viso del Alcor (El)','SE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'41901','Cañada Rosal','SE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'41902','Isla Mayor','SE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'41903','Cuervo de Sevilla (El)','SE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'41904','Palmar de Troya, El','SE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'42001','Abejar','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'42003','Adradas','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'42004','Ágreda','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'42006','Alconaba','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'42007','Alcubilla de Avellaneda','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'42008','Alcubilla de las Peñas','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'42009','Aldealafuente','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'42010','Aldealices','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'42011','Aldealpozo','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'42012','Aldealseñor','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'42013','Aldehuela de Periáñez','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'42014','Aldehuelas (Las)','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'42015','Alentisque','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'42016','Aliud','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'42017','Almajano','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'42018','Almaluez','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'42019','Almarza','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'42020','Almazán','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'42021','Almazul','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'42022','Almenar de Soria','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'42023','Alpanseque','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'42024','Arancón','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'42025','Arcos de Jalón','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'42026','Arenillas','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'42027','Arévalo de la Sierra','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'42028','Ausejo de la Sierra','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'42029','Baraona','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'42030','Barca','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'42031','Barcones','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'42032','Bayubas de Abajo','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'42033','Bayubas de Arriba','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'42034','Beratón','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'42035','Berlanga de Duero','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'42036','Blacos','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'42037','Bliecos','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'42038','Borjabad','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'42039','Borobia','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'42041','Buberos','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'42042','Buitrago','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'42043','Burgo de Osma-Ciudad de Osma','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'42044','Cabrejas del Campo','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'42045','Cabrejas del Pinar','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'42046','Calatañazor','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'42048','Caltojar','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'42049','Candilichera','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'42050','Cañamaque','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'42051','Carabantes','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'42052','Caracena','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'42053','Carrascosa de Abajo','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'42054','Carrascosa de la Sierra','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'42055','Casarejos','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'42056','Castilfrío de la Sierra','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'42057','Castilruiz','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'42058','Castillejo de Robledo','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'42059','Centenera de Andaluz','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'42060','Cerbón','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'42061','Cidones','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'42062','Cigudosa','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'42063','Cihuela','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'42064','Ciria','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'42065','Cirujales del Río','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'42068','Coscurita','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'42069','Covaleda','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'42070','Cubilla','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'42071','Cubo de la Solana','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'42073','Cueva de Ágreda','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'42075','Dévanos','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'42076','Deza','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'42078','Duruelo de la Sierra','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'42079','Escobosa de Almazán','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'42080','Espeja de San Marcelino','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'42081','Espejón','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'42082','Estepa de San Juan','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'42083','Frechilla de Almazán','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'42084','Fresno de Caracena','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'42085','Fuentearmegil','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'42086','Fuentecambrón','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'42087','Fuentecantos','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'42088','Fuentelmonge','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'42089','Fuentelsaz de Soria','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'42090','Fuentepinilla','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'42092','Fuentes de Magaña','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'42093','Fuentestrún','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'42094','Garray','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'42095','Golmayo','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'42096','Gómara','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'42097','Gormaz','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'42098','Herrera de Soria','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'42100','Hinojosa del Campo','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'42103','Langa de Duero','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'42105','Liceras','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'42106','Losilla (La)','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'42107','Magaña','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'42108','Maján','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'42110','Matalebreras','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'42111','Matamala de Almazán','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'42113','Medinaceli','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'42115','Miño de Medinaceli','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'42116','Miño de San Esteban','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'42117','Molinos de Duero','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'42118','Momblona','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'42119','Monteagudo de las Vicarías','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'42120','Montejo de Tiermes','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'42121','Montenegro de Cameros','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'42123','Morón de Almazán','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'42124','Muriel de la Fuente','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'42125','Muriel Viejo','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'42127','Nafría de Ucero','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'42128','Narros','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'42129','Navaleno','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'42130','Nepas','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'42131','Nolay','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'42132','Noviercas','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'42134','Ólvega','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'42135','Oncala','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'42139','Pinilla del Campo','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'42140','Portillo de Soria','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'42141','Póveda de Soria (La)','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'42142','Pozalmuro','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'42144','Quintana Redonda','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'42145','Quintanas de Gormaz','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'42148','Quiñonería','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'42149','Rábanos (Los)','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'42151','Rebollar','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'42152','Recuerda','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'42153','Rello','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'42154','Renieblas','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'42155','Retortillo de Soria','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'42156','Reznos','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'42157','Riba de Escalote (La)','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'42158','Rioseco de Soria','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'42159','Rollamienta','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'42160','Royo (El)','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'42161','Salduero','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'42162','San Esteban de Gormaz','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'42163','San Felices','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'42164','San Leonardo de Yagüe','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'42165','San Pedro Manrique','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'42166','Santa Cruz de Yanguas','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'42167','Santa María de Huerta','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'42168','Santa María de las Hoyas','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'42171','Serón de Nágima','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'42172','Soliedra','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'42173','Soria','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'42174','Sotillo del Rincón','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'42175','Suellacabras','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'42176','Tajahuerce','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'42177','Tajueco','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'42178','Talveila','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'42181','Tardelcuende','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'42182','Taroda','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'42183','Tejado','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'42184','Torlengua','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'42185','Torreblacos','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'42187','Torrubia de Soria','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'42188','Trévago','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'42189','Ucero','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'42190','Vadillo','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'42191','Valdeavellano de Tera','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'42192','Valdegeña','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'42193','Valdelagua del Cerro','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'42194','Valdemaluque','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'42195','Valdenebro','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'42196','Valdeprado','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'42197','Valderrodilla','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'42198','Valtajeros','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'42200','Velamazán','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'42201','Velilla de la Sierra','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'42202','Velilla de los Ajos','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'42204','Viana de Duero','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'42205','Villaciervos','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'42206','Villanueva de Gormaz','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'42207','Villar del Ala','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'42208','Villar del Campo','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'42209','Villar del Río','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'42211','Villares de Soria (Los)','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'42212','Villasayas','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'42213','Villaseca de Arciel','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'42215','Vinuesa','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'42216','Vizmanos','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'42217','Vozmediano','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'42218','Yanguas','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'42219','Yelo','SO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'43001','Aiguamúrcia','T','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'43002','Albinyana','T','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'43003','Albiol (L'')','T','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'43004','Alcanar','T','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'43005','Alcover','T','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'43006','Aldover','T','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'43007','Aleixar (L'')','T','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'43008','Alfara de Carles','T','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'43009','Alforja','T','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'43010','Alió','T','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'43011','Almoster','T','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'43012','Altafulla','T','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'43013','Ametlla de Mar (L'')','T','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'43014','Amposta','T','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'43015','Arbolí','T','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'43016','Arboç (L'')','T','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'43017','Argentera (L'')','T','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'43018','Arnes','T','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'43019','Ascó','T','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'43020','Banyeres del Penedès','T','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'43021','Barberà de la Conca','T','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'43022','Batea','T','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'43023','Bellmunt del Priorat','T','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'43024','Bellvei','T','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'43025','Benifallet','T','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'43026','Benissanet','T','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'43027','Bisbal de Falset (La)','T','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'43028','Bisbal del Penedès (La)','T','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'43029','Blancafort','T','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'43030','Bonastre','T','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'43031','Borges del Camp (Les)','T','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'43032','Bot','T','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'43033','Botarell','T','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'43034','Bràfim','T','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'43035','Cabacés','T','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'43036','Cabra del Camp','T','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'43037','Calafell','T','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'43038','Cambrils','T','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'43039','Capafonts','T','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'43040','Capçanes','T','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'43041','Caseres','T','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'43042','Castellvell del Camp','T','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'43043','Catllar (El)','T','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'43044','Sénia (La)','T','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'43045','Colldejou','T','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'43046','Conesa','T','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'43047','Constantí','T','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'43048','Corbera d''Ebre','T','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'43049','Cornudella de Montsant','T','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'43050','Creixell','T','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'43051','Cunit','T','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'43052','Xerta','T','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'43053','Duesaigües','T','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'43054','Espluga de Francolí (L'')','T','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'43055','Falset','T','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'43056','Fatarella (La)','T','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'43057','Febró (La)','T','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'43058','Figuera (La)','T','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'43059','Figuerola del Camp','T','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'43060','Flix','T','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'43061','Forès','T','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'43062','Freginals','T','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'43063','Galera (La)','T','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'43064','Gandesa','T','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'43065','Garcia','T','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'43066','Garidells (Els)','T','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'43067','Ginestar','T','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'43068','Godall','T','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'43069','Gratallops','T','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'43070','Guiamets (Els)','T','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'43071','Horta de Sant Joan','T','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'43072','Lloar (El)','T','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'43073','Llorac','T','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'43074','Llorenç del Penedès','T','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'43075','Margalef','T','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'43076','Marçà','T','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'43077','Mas de Barberans','T','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'43078','Masdenverge','T','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'43079','Masllorenç','T','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'43080','Masó (La)','T','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'43081','Maspujols','T','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'43082','Masroig (El)','T','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'43083','Milà (El)','T','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'43084','Miravet','T','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'43085','Molar (El)','T','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'43086','Montblanc','T','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'43088','Montbrió del Camp','T','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'43089','Montferri','T','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'43090','Montmell (El)','T','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'43091','Mont-ral','T','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'43092','Mont-roig del Camp','T','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'43093','Móra d''Ebre','T','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'43094','Móra la Nova','T','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'43095','Morell (El)','T','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'43096','Morera de Montsant (La)','T','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'43097','Nou de Gaià (La)','T','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'43098','Nulles','T','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'43099','Palma d''Ebre (La)','T','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'43100','Pallaresos (Els)','T','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'43101','Passanant i Belltall','T','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'43102','Paüls','T','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'43103','Perafort','T','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'43104','Perelló (El)','T','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'43105','Piles (Les)','T','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'43106','Pinell de Brai (El)','T','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'43107','Pira','T','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'43108','Pla de Santa Maria (El)','T','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'43109','Pobla de Mafumet (La)','T','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'43110','Pobla de Massaluca (La)','T','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'43111','Pobla de Montornès (La)','T','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'43112','Poboleda','T','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'43113','Pont d''Armentera (El)','T','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'43114','Porrera','T','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'43115','Pradell de la Teixeta','T','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'43116','Prades','T','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'43117','Prat de Comte','T','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'43118','Pratdip','T','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'43119','Puigpelat','T','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'43120','Querol','T','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'43121','Rasquera','T','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'43122','Renau','T','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'43123','Reus','T','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'43124','Riba (La)','T','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'43125','Riba-roja d''Ebre','T','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'43126','Riera de Gaià (La)','T','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'43127','Riudecanyes','T','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'43128','Riudecols','T','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'43129','Riudoms','T','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'43130','Rocafort de Queralt','T','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'43131','Roda de Berà','T','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'43132','Rodonyà','T','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'43133','Roquetes','T','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'43134','Rourell (El)','T','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'43135','Salomó','T','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'43136','Sant Carles de la Ràpita','T','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'43137','Sant Jaume dels Domenys','T','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'43138','Santa Bàrbara','T','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'43139','Santa Coloma de Queralt','T','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'43140','Santa Oliva','T','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'43141','Pontils','T','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'43142','Sarral','T','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'43143','Savallà del Comtat','T','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'43144','Secuita (La)','T','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'43145','Selva del Camp (La)','T','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'43146','Senan','T','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'43147','Solivella','T','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'43148','Tarragona','T','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'43149','Tivenys','T','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'43150','Tivissa','T','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'43151','Torre de Fontaubella (La)','T','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'43152','Torre de l''Espanyol (La)','T','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'43153','Torredembarra','T','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'43154','Torroja del Priorat','T','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'43155','Tortosa','T','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'43156','Ulldecona','T','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'43157','Ulldemolins','T','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'43158','Vallclara','T','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'43159','Vallfogona de Riucorb','T','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'43160','Vallmoll','T','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'43161','Valls','T','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'43162','Vandellòs i l''Hospitalet de l''Infant','T','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'43163','Vendrell (El)','T','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'43164','Vespella de Gaià','T','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'43165','Vilabella','T','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'43166','Vilallonga del Camp','T','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'43167','Vilanova d''Escornalbou','T','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'43168','Vilanova de Prades','T','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'43169','Vilaplana','T','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'43170','Vila-rodona','T','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'43171','Vila-seca','T','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'43172','Vilaverd','T','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'43173','Vilella Alta (La)','T','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'43174','Vilella Baixa (La)','T','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'43175','Vilalba dels Arcs','T','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'43176','Vimbodí i Poblet','T','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'43177','Vinebre','T','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'43178','Vinyols i els Arcs','T','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'43901','Deltebre','T','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'43902','Sant Jaume d''Enveja','T','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'43903','Camarles','T','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'43904','Aldea (L'')','T','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'43905','Salou','T','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'43906','Ampolla (L'')','T','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'43907','Canonja, La','T','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44001','Ababuj','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44002','Abejuela','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44003','Aguatón','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44004','Aguaviva','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44005','Aguilar del Alfambra','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44006','Alacón','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44007','Alba','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44008','Albalate del Arzobispo','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44009','Albarracín','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44010','Albentosa','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44011','Alcaine','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44012','Alcalá de la Selva','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44013','Alcañiz','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44014','Alcorisa','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44016','Alfambra','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44017','Aliaga','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44018','Almohaja','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44019','Alobras','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44020','Alpeñés','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44021','Allepuz','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44022','Alloza','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44023','Allueva','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44024','Anadón','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44025','Andorra','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44026','Arcos de las Salinas','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44027','Arens de Lledó','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44028','Argente','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44029','Ariño','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44031','Azaila','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44032','Bádenas','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44033','Báguena','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44034','Bañón','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44035','Barrachina','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44036','Bea','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44037','Beceite','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44038','Belmonte de San José','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44039','Bello','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44040','Berge','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44041','Bezas','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44042','Blancas','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44043','Blesa','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44044','Bordón','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44045','Bronchales','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44046','Bueña','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44047','Burbáguena','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44048','Cabra de Mora','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44049','Calaceite','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44050','Calamocha','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44051','Calanda','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44052','Calomarde','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44053','Camañas','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44054','Camarena de la Sierra','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44055','Camarillas','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44056','Caminreal','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44059','Cantavieja','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44060','Cañada de Benatanduz','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44061','Cañada de Verich (La)','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44062','Cañada Vellida','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44063','Cañizar del Olivar','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44064','Cascante del Río','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44065','Castejón de Tornos','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44066','Castel de Cabra','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44067','Castelnou','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44068','Castelserás','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44070','Castellar (El)','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44071','Castellote','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44074','Cedrillas','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44075','Celadas','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44076','Cella','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44077','Cerollera (La)','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44080','Codoñera (La)','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44082','Corbalán','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44084','Cortes de Aragón','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44085','Cosa','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44086','Cretas','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44087','Crivillén','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44088','Cuba (La)','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44089','Cubla','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44090','Cucalón','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44092','Cuervo (El)','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44093','Cuevas de Almudén','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44094','Cuevas Labradas','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44096','Ejulve','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44097','Escorihuela','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44099','Escucha','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44100','Estercuel','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44101','Ferreruela de Huerva','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44102','Fonfría','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44103','Formiche Alto','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44105','Fórnoles','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44106','Fortanete','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44107','Foz-Calanda','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44108','Fresneda (La)','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44109','Frías de Albarracín','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44110','Fuenferrada','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44111','Fuentes Calientes','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44112','Fuentes Claras','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44113','Fuentes de Rubielos','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44114','Fuentespalda','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44115','Galve','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44116','Gargallo','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44117','Gea de Albarracín','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44118','Ginebrosa (La)','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44119','Griegos','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44120','Guadalaviar','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44121','Gúdar','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44122','Híjar','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44123','Hinojosa de Jarque','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44124','Hoz de la Vieja (La)','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44125','Huesa del Común','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44126','Iglesuela del Cid (La)','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44127','Jabaloyas','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44128','Jarque de la Val','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44129','Jatiel','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44130','Jorcas','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44131','Josa','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44132','Lagueruela','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44133','Lanzuela','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44135','Libros','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44136','Lidón','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44137','Linares de Mora','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44138','Loscos','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44141','Lledó','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44142','Maicas','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44143','Manzanera','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44144','Martín del Río','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44145','Mas de las Matas','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44146','Mata de los Olmos (La)','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44147','Mazaleón','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44148','Mezquita de Jarque','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44149','Mirambel','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44150','Miravete de la Sierra','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44151','Molinos','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44152','Monforte de Moyuela','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44153','Monreal del Campo','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44154','Monroyo','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44155','Montalbán','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44156','Monteagudo del Castillo','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44157','Monterde de Albarracín','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44158','Mora de Rubielos','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44159','Moscardón','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44160','Mosqueruela','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44161','Muniesa','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44163','Noguera de Albarracín','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44164','Nogueras','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44165','Nogueruelas','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44167','Obón','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44168','Odón','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44169','Ojos Negros','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44171','Olba','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44172','Oliete','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44173','Olmos (Los)','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44174','Orihuela del Tremedal','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44175','Orrios','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44176','Palomar de Arroyos','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44177','Pancrudo','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44178','Parras de Castellote (Las)','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44179','Peñarroya de Tastavins','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44180','Peracense','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44181','Peralejos','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44182','Perales del Alfambra','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44183','Pitarque','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44184','Plou','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44185','Pobo (El)','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44187','Portellada (La)','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44189','Pozondón','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44190','Pozuel del Campo','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44191','Puebla de Híjar (La)','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44192','Puebla de Valverde (La)','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44193','Puertomingalvo','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44194','Ráfales','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44195','Rillo','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44196','Riodeva','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44197','Ródenas','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44198','Royuela','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44199','Rubiales','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44200','Rubielos de la Cérida','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44201','Rubielos de Mora','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44203','Salcedillo','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44204','Saldón','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44205','Samper de Calanda','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44206','San Agustín','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44207','San Martín del Río','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44208','Santa Cruz de Nogueras','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44209','Santa Eulalia','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44210','Sarrión','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44211','Segura de los Baños','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44212','Seno','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44213','Singra','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44215','Terriente','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44216','Teruel','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44217','Toril y Masegoso','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44218','Tormón','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44219','Tornos','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44220','Torralba de los Sisones','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44221','Torrecilla de Alcañiz','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44222','Torrecilla del Rebollar','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44223','Torre de Arcas','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44224','Torre de las Arcas','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44225','Torre del Compte','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44226','Torrelacárcel','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44227','Torre los Negros','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44228','Torremocha de Jiloca','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44229','Torres de Albarracín','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44230','Torrevelilla','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44231','Torrijas','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44232','Torrijo del Campo','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44234','Tramacastiel','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44235','Tramacastilla','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44236','Tronchón','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44237','Urrea de Gaén','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44238','Utrillas','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44239','Valacloche','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44240','Valbona','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44241','Valdealgorfa','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44243','Valdecuenca','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44244','Valdelinares','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44245','Valdeltormo','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44246','Valderrobres','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44247','Valjunquera','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44249','Vallecillo (El)','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44250','Veguillas de la Sierra','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44251','Villafranca del Campo','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44252','Villahermosa del Campo','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44256','Villanueva del Rebollar de la Sierra','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44257','Villar del Cobo','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44258','Villar del Salz','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44260','Villarluengo','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44261','Villarquemado','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44262','Villarroya de los Pinares','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44263','Villastar','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44264','Villel','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44265','Vinaceite','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44266','Visiedo','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44267','Vivel del Río Martín','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'44268','Zoma (La)','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45001','Ajofrín','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45002','Alameda de la Sagra','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45003','Albarreal de Tajo','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45004','Alcabón','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45005','Alcañizo','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45006','Alcaudete de la Jara','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45007','Alcolea de Tajo','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45008','Aldea en Cabo','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45009','Aldeanueva de Barbarroya','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45010','Aldeanueva de San Bartolomé','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45011','Almendral de la Cañada','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45012','Almonacid de Toledo','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45013','Almorox','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45014','Añover de Tajo','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45015','Arcicóllar','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45016','Argés','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45017','Azután','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45018','Barcience','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45019','Bargas','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45020','Belvís de la Jara','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45021','Borox','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45022','Buenaventura','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45023','Burguillos de Toledo','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45024','Burujón','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45025','Cabañas de la Sagra','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45026','Cabañas de Yepes','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45027','Cabezamesada','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45028','Calera y Chozas','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45029','Caleruela','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45030','Calzada de Oropesa','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45031','Camarena','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45032','Camarenilla','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45033','Campillo de la Jara (El)','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45034','Camuñas','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45035','Cardiel de los Montes','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45036','Carmena','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45037','Carpio de Tajo (El)','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45038','Carranque','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45039','Carriches','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45040','Casar de Escalona (El)','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45041','Casarrubios del Monte','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45042','Casasbuenas','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45043','Castillo de Bayuela','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45045','Cazalegas','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45046','Cebolla','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45047','Cedillo del Condado','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45048','Cerralbos (Los)','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45049','Cervera de los Montes','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45050','Ciruelos','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45051','Cobeja','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45052','Cobisa','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45053','Consuegra','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45054','Corral de Almaguer','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45055','Cuerva','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45056','Chozas de Canales','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45057','Chueca','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45058','Domingo Pérez','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45059','Dosbarrios','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45060','Erustes','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45061','Escalona','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45062','Escalonilla','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45063','Espinoso del Rey','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45064','Esquivias','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45065','Estrella (La)','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45066','Fuensalida','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45067','Gálvez','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45068','Garciotum','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45069','Gerindote','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45070','Guadamur','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45071','Guardia (La)','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45072','Herencias (Las)','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45073','Herreruela de Oropesa','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45074','Hinojosa de San Vicente','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45075','Hontanar','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45076','Hormigos','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45077','Huecas','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45078','Huerta de Valdecarábanos','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45079','Iglesuela del Tiétar, La','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45080','Illán de Vacas','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45081','Illescas','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45082','Lagartera','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45083','Layos','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45084','Lillo','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45085','Lominchar','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45086','Lucillos','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45087','Madridejos','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45088','Magán','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45089','Malpica de Tajo','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45090','Manzaneque','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45091','Maqueda','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45092','Marjaliza','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45093','Marrupe','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45094','Mascaraque','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45095','Mata (La)','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45096','Mazarambroz','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45097','Mejorada','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45098','Menasalbas','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45099','Méntrida','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45100','Mesegar de Tajo','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45101','Miguel Esteban','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45102','Mocejón','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45103','Mohedas de la Jara','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45104','Montearagón','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45105','Montesclaros','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45106','Mora','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45107','Nambroca','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45108','Nava de Ricomalillo (La)','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45109','Navahermosa','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45110','Navalcán','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45111','Navalmoralejo','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45112','Navalmorales (Los)','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45113','Navalucillos (Los)','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45114','Navamorcuende','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45115','Noblejas','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45116','Noez','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45117','Nombela','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45118','Novés','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45119','Numancia de la Sagra','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45120','Nuño Gómez','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45121','Ocaña','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45122','Olías del Rey','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45123','Ontígola','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45124','Orgaz','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45125','Oropesa','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45126','Otero','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45127','Palomeque','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45128','Pantoja','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45129','Paredes de Escalona','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45130','Parrillas','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45131','Pelahustán','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45132','Pepino','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45133','Polán','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45134','Portillo de Toledo','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45135','Puebla de Almoradiel (La)','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45136','Puebla de Montalbán (La)','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45137','Pueblanueva (La)','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45138','Puente del Arzobispo (El)','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45139','Puerto de San Vicente','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45140','Pulgar','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45141','Quero','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45142','Quintanar de la Orden','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45143','Quismondo','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45144','Real de San Vicente (El)','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45145','Recas','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45146','Retamoso de la Jara','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45147','Rielves','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45148','Robledo del Mazo','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45149','Romeral (El)','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45150','San Bartolomé de las Abiertas','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45151','San Martín de Montalbán','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45152','San Martín de Pusa','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45153','San Pablo de los Montes','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45154','San Román de los Montes','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45155','Santa Ana de Pusa','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45156','Santa Cruz de la Zarza','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45157','Santa Cruz del Retamar','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45158','Santa Olalla','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45159','Sartajada','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45160','Segurilla','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45161','Seseña','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45162','Sevilleja de la Jara','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45163','Sonseca','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45164','Sotillo de las Palomas','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45165','Talavera de la Reina','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45166','Tembleque','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45167','Toboso (El)','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45168','Toledo','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45169','Torralba de Oropesa','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45170','Torrecilla de la Jara','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45171','Torre de Esteban Hambrán (La)','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45172','Torrico','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45173','Torrijos','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45174','Totanés','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45175','Turleque','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45176','Ugena','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45177','Urda','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45179','Valdeverdeja','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45180','Valmojado','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45181','Velada','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45182','Ventas con Peña Aguilera (Las)','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45183','Ventas de Retamosa (Las)','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45184','Ventas de San Julián (Las)','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45185','Villacañas','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45186','Villa de Don Fadrique (La)','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45187','Villafranca de los Caballeros','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45188','Villaluenga de la Sagra','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45189','Villamiel de Toledo','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45190','Villaminaya','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45191','Villamuelas','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45192','Villanueva de Alcardete','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45193','Villanueva de Bogas','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45194','Villarejo de Montalbán','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45195','Villarrubia de Santiago','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45196','Villaseca de la Sagra','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45197','Villasequilla','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45198','Villatobas','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45199','Viso de San Juan (El)','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45200','Yébenes (Los)','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45201','Yeles','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45202','Yepes','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45203','Yuncler','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45204','Yunclillos','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45205','Yuncos','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'45901','Santo Domingo-Caudilla','TO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46001','Ademuz','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46002','Ador','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46003','Atzeneta d''Albaida','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46004','Agullent','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46005','Alaquàs','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46006','Albaida','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46007','Albal','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46008','Albalat de la Ribera','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46009','Albalat dels Sorells','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46010','Albalat dels Tarongers','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46011','Alberic','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46012','Alborache','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46013','Alboraia/Alboraya','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46014','Albuixech','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46015','Alcàsser','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46016','Alcàntera de Xúquer','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46017','Alzira','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46018','Alcublas','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46019','Alcúdia (l'')','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46020','Alcúdia de Crespins (l'')','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46021','Aldaia','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46022','Alfafar','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46023','Alfauir','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46024','Alfara de la Baronia','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46025','Alfara del Patriarca','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46026','Alfarp','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46027','Alfarrasí','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46028','Algar de Palancia','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46029','Algemesí','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46030','Algímia d’Alfara','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46031','Alginet','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46032','Almàssera','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46033','Almiserà','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46034','Almoines','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46035','Almussafes','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46036','Alpuente','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46037','Alqueria de la Comtessa (l'')','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46038','Andilla','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46039','Anna','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46040','Antella','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46041','Aras de los Olmos','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46042','Aielo de Malferit','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46043','Aielo de Rugat','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46044','Ayora','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46045','Barxeta','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46046','Barx','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46047','Bèlgida','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46048','Bellreguard','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46049','Bellús','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46050','Benagéber','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46051','Benaguasil','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46052','Benavites','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46053','Beneixida','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46054','Benetússer','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46055','Beniarjó','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46056','Beniatjar','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46057','Benicolet','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46058','Benifairó de les Valls','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46059','Benifairó de la Valldigna','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46060','Benifaió','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46061','Beniflá','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46062','Benigànim','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46063','Benimodo','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46064','Benimuslem','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46065','Beniparrell','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46066','Benirredrà','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46067','Benissanó','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46068','Benissoda','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46069','Benissuera','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46070','Bétera','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46071','Bicorp','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46072','Bocairent','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46073','Bolbaite','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46074','Bonrepòs i Mirambell','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46075','Bufali','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46076','Bugarra','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46077','Buñol','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46078','Burjassot','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46079','Calles','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46080','Camporrobles','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46081','Canals','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46082','Canet d''En Berenguer','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46083','Carcaixent','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46084','Càrcer','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46085','Carlet','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46086','Carrícola','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46087','Casas Altas','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46088','Casas Bajas','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46089','Casinos','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46090','Castelló de Rugat','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46091','Castellonet de la Conquesta','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46092','Castielfabib','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46093','Catadau','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46094','Catarroja','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46095','Caudete de las Fuentes','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46096','Cerdà','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46097','Cofrentes','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46098','Corbera','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46099','Cortes de Pallás','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46100','Cotes','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46101','Quart de les Valls','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46102','Quart de Poblet','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46103','Quartell','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46104','Quatretonda','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46105','Cullera','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46106','Chelva','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46107','Chella','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46108','Chera','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46109','Cheste','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46110','Xirivella','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46111','Chiva','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46112','Chulilla','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46113','Daimús','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46114','Domeño','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46115','Dos Aguas','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46116','Eliana (l'')','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46117','Emperador','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46118','Enguera','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46119','Ènova (l'')','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46120','Estivella','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46121','Estubeny','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46122','Faura','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46123','Favara','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46124','Fontanars dels Alforins','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46125','Fortaleny','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46126','Foios','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46127','Font d''En Carròs (la)','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46128','Font de la Figuera (la)','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46129','Fuenterrobles','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46130','Gavarda','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46131','Gandia','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46132','Genovés, el','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46133','Gestalgar','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46134','Gilet','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46135','Godella','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46136','Godelleta','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46137','Granja de la Costera (la)','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46138','Guadasequies/Guadasséquies','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46139','Guadassuar','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46140','Guardamar de la Safor','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46141','Higueruelas','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46142','Jalance','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46143','Xeraco','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46144','Jarafuel','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46145','Xàtiva','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46146','Xeresa','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46147','Llíria','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46148','Loriguilla','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46149','Losa del Obispo','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46150','Llutxent','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46151','Llocnou d''En Fenollet','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46152','Llocnou de la Corona','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46153','Llocnou de Sant Jeroni','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46154','Llanera de Ranes','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46155','Llaurí','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46156','Llombai','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46157','Llosa de Ranes (la)','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46158','Macastre','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46159','Manises','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46160','Manuel','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46161','Marines','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46162','Massalavés','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46163','Massalfassar','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46164','Massamagrell','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46165','Massanassa','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46166','Meliana','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46167','Millares','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46168','Miramar','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46169','Mislata','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46170','Mogente/Moixent','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46171','Moncada','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46172','Montserrat','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46173','Montaverner','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46174','Montesa','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46175','Montitxelvo/Montichelvo','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46176','Montroi/Montroy','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46177','Museros','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46178','Nàquera/Náquera','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46179','Navarrés','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46180','Novelé/Novetlè','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46181','Oliva','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46182','Olocau','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46183','Olleria (l'')','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46184','Ontinyent','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46185','Otos','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46186','Paiporta','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46187','Palma de Gandía','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46188','Palmera','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46189','Palomar (el)','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46190','Paterna','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46191','Pedralba','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46192','Petrés','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46193','Picanya','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46194','Picassent','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46195','Piles','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46196','Pinet','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46197','Polinyà de Xúquer','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46198','Potries','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46199','Pobla de Farnals (la)','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46200','Pobla del Duc (la)','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46201','Puebla de San Miguel','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46202','Pobla de Vallbona (la)','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46203','Pobla Llarga (la)','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46204','Puig de Santa María, El','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46205','Puçol','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46206','Quesa','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46207','Rafelbunyol','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46208','Rafelcofer','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46209','Rafelguaraf','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46210','Ráfol de Salem','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46211','Real de Gandia, el','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46212','Real','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46213','Requena','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46214','Riba-roja de Túria','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46215','Riola','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46216','Rocafort','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46217','Rotglà i Corberà','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46218','Rótova','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46219','Rugat','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46220','Sagunto/Sagunt','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46221','Salem','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46222','Sant Joanet','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46223','Sedaví','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46224','Segart','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46225','Sellent','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46226','Sempere','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46227','Senyera','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46228','Serra','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46229','Siete Aguas','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46230','Silla','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46231','Simat de la Valldigna','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46232','Sinarcas','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46233','Sollana','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46234','Sot de Chera','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46235','Sueca','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46236','Sumacàrcer','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46237','Tavernes Blanques','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46238','Tavernes de la Valldigna','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46239','Teresa de Cofrentes','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46240','Terrateig','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46241','Titaguas','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46242','Torrebaja','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46243','Torrella','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46244','Torrent','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46245','Torres Torres','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46246','Tous','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46247','Tuéjar','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46248','Turís','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46249','Utiel','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46250','València','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46251','Vallada','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46252','Vallanca','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46253','Vallés','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46254','Venta del Moro','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46255','Vilallonga/Villalonga','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46256','Vilamarxant','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46257','Villanueva de Castellón','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46258','Villar del Arzobispo','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46259','Villargordo del Cabriel','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46260','Vinalesa','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46261','Yátova','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46262','Yesa (La)','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46263','Zarra','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46902','Gátova','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46903','San Antonio de Benagéber','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'46904','Benicull de Xúquer','V','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47001','Adalia','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47002','Aguasal','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47003','Aguilar de Campos','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47004','Alaejos','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47005','Alcazarén','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47006','Aldea de San Miguel','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47007','Aldeamayor de San Martín','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47008','Almenara de Adaja','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47009','Amusquillo','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47010','Arroyo de la Encomienda','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47011','Ataquines','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47012','Bahabón','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47013','Barcial de la Loma','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47014','Barruelo del Valle','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47015','Becilla de Valderaduey','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47016','Benafarces','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47017','Bercero','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47018','Berceruelo','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47019','Berrueces','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47020','Bobadilla del Campo','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47021','Bocigas','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47022','Bocos de Duero','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47023','Boecillo','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47024','Bolaños de Campos','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47025','Brahojos de Medina','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47026','Bustillo de Chaves','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47027','Cabezón de Pisuerga','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47028','Cabezón de Valderaduey','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47029','Cabreros del Monte','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47030','Campaspero','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47031','Campillo (El)','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47032','Camporredondo','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47033','Canalejas de Peñafiel','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47034','Canillas de Esgueva','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47035','Carpio','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47036','Casasola de Arión','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47037','Castrejón de Trabancos','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47038','Castrillo de Duero','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47039','Castrillo-Tejeriego','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47040','Castrobol','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47041','Castrodeza','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47042','Castromembibre','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47043','Castromonte','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47044','Castronuevo de Esgueva','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47045','Castronuño','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47046','Castroponce','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47047','Castroverde de Cerrato','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47048','Ceinos de Campos','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47049','Cervillego de la Cruz','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47050','Cigales','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47051','Ciguñuela','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47052','Cistérniga','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47053','Cogeces de Íscar','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47054','Cogeces del Monte','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47055','Corcos','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47056','Corrales de Duero','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47057','Cubillas de Santa Marta','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47058','Cuenca de Campos','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47059','Curiel de Duero','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47060','Encinas de Esgueva','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47061','Esguevillas de Esgueva','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47062','Fombellida','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47063','Fompedraza','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47064','Fontihoyuelo','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47065','Fresno el Viejo','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47066','Fuensaldaña','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47067','Fuente el Sol','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47068','Fuente-Olmedo','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47069','Gallegos de Hornija','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47070','Gatón de Campos','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47071','Geria','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47073','Herrín de Campos','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47074','Hornillos de Eresma','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47075','Íscar','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47076','Laguna de Duero','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47077','Langayo','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47078','Lomoviejo','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47079','Llano de Olmedo','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47080','Manzanillo','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47081','Marzales','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47082','Matapozuelos','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47083','Matilla de los Caños','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47084','Mayorga','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47085','Medina del Campo','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47086','Medina de Rioseco','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47087','Megeces','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47088','Melgar de Abajo','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47089','Melgar de Arriba','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47090','Mojados','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47091','Monasterio de Vega','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47092','Montealegre de Campos','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47093','Montemayor de Pililla','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47094','Moral de la Reina','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47095','Moraleja de las Panaderas','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47096','Morales de Campos','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47097','Mota del Marqués','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47098','Mucientes','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47099','Mudarra (La)','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47100','Muriel','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47101','Nava del Rey','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47102','Nueva Villa de las Torres','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47103','Olivares de Duero','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47104','Olmedo','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47105','Olmos de Esgueva','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47106','Olmos de Peñafiel','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47109','Palazuelo de Vedija','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47110','Parrilla (La)','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47111','Pedraja de Portillo (La)','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47112','Pedrajas de San Esteban','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47113','Pedrosa del Rey','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47114','Peñafiel','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47115','Peñaflor de Hornija','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47116','Pesquera de Duero','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47117','Piña de Esgueva','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47118','Piñel de Abajo','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47119','Piñel de Arriba','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47121','Pollos','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47122','Portillo','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47123','Pozal de Gallinas','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47124','Pozaldez','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47125','Pozuelo de la Orden','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47126','Puras','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47127','Quintanilla de Arriba','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47128','Quintanilla del Molar','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47129','Quintanilla de Onésimo','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47130','Quintanilla de Trigueros','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47131','Rábano','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47132','Ramiro','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47133','Renedo de Esgueva','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47134','Roales de Campos','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47135','Robladillo','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47137','Roturas','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47138','Rubí de Bracamonte','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47139','Rueda','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47140','Saelices de Mayorga','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47141','Salvador de Zapardiel','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47142','San Cebrián de Mazote','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47143','San Llorente','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47144','San Martín de Valvení','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47145','San Miguel del Arroyo','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47146','San Miguel del Pino','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47147','San Pablo de la Moraleja','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47148','San Pedro de Latarce','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47149','San Pelayo','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47150','San Román de Hornija','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47151','San Salvador','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47152','Santa Eufemia del Arroyo','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47153','Santervás de Campos','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47154','Santibáñez de Valcorba','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47155','Santovenia de Pisuerga','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47156','San Vicente del Palacio','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47157','Sardón de Duero','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47158','Seca (La)','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47159','Serrada','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47160','Siete Iglesias de Trabancos','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47161','Simancas','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47162','Tamariz de Campos','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47163','Tiedra','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47164','Tordehumos','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47165','Tordesillas','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47166','Torrecilla de la Abadesa','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47167','Torrecilla de la Orden','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47168','Torrecilla de la Torre','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47169','Torre de Esgueva','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47170','Torre de Peñafiel','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47171','Torrelobatón','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47172','Torrescárcela','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47173','Traspinedo','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47174','Trigueros del Valle','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47175','Tudela de Duero','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47176','Unión de Campos (La)','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47177','Urones de Castroponce','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47178','Urueña','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47179','Valbuena de Duero','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47180','Valdearcos de la Vega','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47181','Valdenebro de los Valles','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47182','Valdestillas','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47183','Valdunquillo','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47184','Valoria la Buena','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47185','Valverde de Campos','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47186','Valladolid','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47187','Vega de Ruiponce','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47188','Vega de Valdetronco','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47189','Velascálvaro','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47190','Velilla','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47191','Velliza','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47192','Ventosa de la Cuesta','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47193','Viana de Cega','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47194','Viloria','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47195','Villabáñez','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47196','Villabaruz de Campos','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47197','Villabrágima','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47198','Villacarralón','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47199','Villacid de Campos','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47200','Villaco','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47203','Villafrades de Campos','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47204','Villafranca de Duero','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47205','Villafrechós','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47206','Villafuerte','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47207','Villagarcía de Campos','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47208','Villagómez la Nueva','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47209','Villalán de Campos','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47210','Villalar de los Comuneros','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47211','Villalba de la Loma','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47212','Villalba de los Alcores','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47213','Villalbarba','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47214','Villalón de Campos','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47215','Villamuriel de Campos','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47216','Villán de Tordesillas','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47217','Villanubla','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47218','Villanueva de Duero','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47219','Villanueva de la Condesa','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47220','Villanueva de los Caballeros','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47221','Villanueva de los Infantes','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47222','Villanueva de San Mancio','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47223','Villardefrades','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47224','Villarmentero de Esgueva','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47225','Villasexmir','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47226','Villavaquerín','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47227','Villavellid','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47228','Villaverde de Medina','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47229','Villavicencio de los Caballeros','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47230','Wamba','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47231','Zaratán','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'47232','Zarza (La)','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'48001','Abadiño','BI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'48002','Abanto y Ciérvana-Abanto Zierbena','BI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'48003','Amorebieta-Etxano','BI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'48004','Amoroto','BI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'48005','Arakaldo','BI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'48006','Arantzazu','BI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'48007','Munitibar-Arbatzegi Gerrikaitz','BI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'48008','Artzentales','BI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'48009','Arrankudiaga','BI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'48010','Arrieta','BI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'48011','Arrigorriaga','BI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'48012','Bakio','BI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'48013','Barakaldo','BI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'48014','Barrika','BI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'48015','Basauri','BI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'48016','Berango','BI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'48017','Bermeo','BI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'48018','Berriatua','BI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'48019','Berriz','BI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'48020','Bilbao','BI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'48021','Busturia','BI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'48022','Karrantza Harana/Valle de Carranza','BI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'48023','Artea','BI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'48024','Zeanuri','BI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'48025','Zeberio','BI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'48026','Dima','BI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'48027','Durango','BI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'48028','Ea','BI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'48029','Etxebarri','BI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'48030','Etxebarria','BI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'48031','Elantxobe','BI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'48032','Elorrio','BI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'48033','Ereño','BI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'48034','Ermua','BI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'48035','Fruiz','BI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'48036','Galdakao','BI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'48037','Galdames','BI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'48038','Gamiz-Fika','BI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'48039','Garai','BI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'48040','Gatika','BI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'48041','Gautegiz Arteaga','BI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'48042','Gordexola','BI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'48043','Gorliz','BI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'48044','Getxo','BI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'48045','Güeñes','BI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'48046','Gernika-Lumo','BI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'48047','Gizaburuaga','BI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'48048','Ibarrangelu','BI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'48049','Ispaster','BI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'48050','Izurtza','BI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'48051','Lanestosa','BI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'48052','Larrabetzu','BI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'48053','Laukiz','BI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'48054','Leioa','BI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'48055','Lemoa','BI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'48056','Lemoiz','BI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'48057','Lekeitio','BI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'48058','Mallabia','BI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'48059','Mañaria','BI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'48060','Markina-Xemein','BI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'48061','Maruri-Jatabe','BI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'48062','Mendata','BI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'48063','Mendexa','BI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'48064','Meñaka','BI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'48065','Ugao-Miraballes','BI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'48066','Morga','BI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'48067','Muxika','BI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'48068','Mundaka','BI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'48069','Mungia','BI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'48070','Aulesti','BI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'48071','Muskiz','BI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'48072','Otxandio','BI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'48073','Ondarroa','BI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'48074','Urduña-Orduña','BI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'48075','Orozko','BI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'48076','Sukarrieta','BI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'48077','Plentzia','BI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'48078','Portugalete','BI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'48079','Errigoiti','BI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'48080','Valle de Trápaga-Trapagaran','BI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'48081','Lezama','BI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'48082','Santurtzi','BI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'48083','Ortuella','BI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'48084','Sestao','BI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'48085','Sopela','BI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'48086','Sopuerta','BI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'48087','Trucios-Turtzioz','BI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'48088','Ubide','BI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'48089','Urduliz','BI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'48090','Balmaseda','BI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'48091','Atxondo','BI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'48092','Bedia','BI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'48093','Areatza','BI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'48094','Igorre','BI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'48095','Zaldibar','BI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'48096','Zalla','BI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'48097','Zaratamo','BI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'48901','Derio','BI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'48902','Erandio','BI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'48903','Loiu','BI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'48904','Sondika','BI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'48905','Zamudio','BI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'48906','Forua','BI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'48907','Kortezubi','BI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'48908','Murueta','BI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'48909','Nabarniz','BI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'48910','Iurreta','BI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'48911','Ajangiz','BI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'48912','Alonsotegi','BI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'48913','Zierbena','BI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'48914','Arratzu','BI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'48915','Ziortza-Bolibar','BI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49002','Abezames','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49003','Alcañices','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49004','Alcubilla de Nogales','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49005','Alfaraz de Sayago','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49006','Algodre','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49007','Almaraz de Duero','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49008','Almeida de Sayago','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49009','Andavías','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49010','Arcenillas','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49011','Arcos de la Polvorosa','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49012','Argañín','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49013','Argujillo','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49014','Arquillinos','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49015','Arrabalde','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49016','Aspariegos','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49017','Asturianos','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49018','Ayoó de Vidriales','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49019','Barcial del Barco','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49020','Belver de los Montes','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49021','Benavente','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49022','Benegiles','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49023','Bermillo de Sayago','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49024','Bóveda de Toro (La)','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49025','Bretó','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49026','Bretocino','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49027','Brime de Sog','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49028','Brime de Urz','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49029','Burganes de Valverde','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49030','Bustillo del Oro','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49031','Cabañas de Sayago','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49032','Calzadilla de Tera','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49033','Camarzana de Tera','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49034','Cañizal','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49035','Cañizo','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49036','Carbajales de Alba','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49037','Carbellino','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49038','Casaseca de Campeán','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49039','Casaseca de las Chanas','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49040','Castrillo de la Guareña','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49041','Castrogonzalo','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49042','Castronuevo','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49043','Castroverde de Campos','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49044','Cazurra','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49046','Cerecinos de Campos','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49047','Cerecinos del Carrizal','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49048','Cernadilla','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49050','Cobreros','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49052','Coomonte','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49053','Coreses','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49054','Corrales del Vino','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49055','Cotanes del Monte','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49056','Cubillos','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49057','Cubo de Benavente','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49058','Cubo de Tierra del Vino (El)','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49059','Cuelgamures','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49061','Entrala','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49062','Espadañedo','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49063','Faramontanos de Tábara','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49064','Fariza','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49065','Fermoselle','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49066','Ferreras de Abajo','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49067','Ferreras de Arriba','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49068','Ferreruela','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49069','Figueruela de Arriba','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49071','Fonfría','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49075','Fresno de la Polvorosa','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49076','Fresno de la Ribera','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49077','Fresno de Sayago','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49078','Friera de Valverde','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49079','Fuente Encalada','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49080','Fuentelapeña','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49081','Fuentesaúco','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49082','Fuentes de Ropel','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49083','Fuentesecas','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49084','Fuentespreadas','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49085','Galende','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49086','Gallegos del Pan','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49087','Gallegos del Río','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49088','Gamones','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49090','Gema','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49091','Granja de Moreruela','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49092','Granucillo','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49093','Guarrate','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49094','Hermisende','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49095','Hiniesta (La)','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49096','Jambrina','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49097','Justel','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49098','Losacino','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49099','Losacio','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49100','Lubián','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49101','Luelmo','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49102','Maderal (El)','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49103','Madridanos','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49104','Mahide','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49105','Maire de Castroponce','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49107','Malva','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49108','Manganeses de la Lampreana','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49109','Manganeses de la Polvorosa','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49110','Manzanal de Arriba','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49111','Manzanal del Barco','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49112','Manzanal de los Infantes','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49113','Matilla de Arzón','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49114','Matilla la Seca','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49115','Mayalde','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49116','Melgar de Tera','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49117','Micereces de Tera','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49118','Milles de la Polvorosa','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49119','Molacillos','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49120','Molezuelas de la Carballeda','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49121','Mombuey','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49122','Monfarracinos','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49123','Montamarta','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49124','Moral de Sayago','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49125','Moraleja del Vino','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49126','Moraleja de Sayago','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49127','Morales del Vino','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49128','Morales de Rey','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49129','Morales de Toro','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49130','Morales de Valverde','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49131','Moralina','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49132','Moreruela de los Infanzones','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49133','Moreruela de Tábara','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49134','Muelas de los Caballeros','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49135','Muelas del Pan','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49136','Muga de Sayago','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49137','Navianos de Valverde','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49138','Olmillos de Castro','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49139','Otero de Bodas','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49141','Pajares de la Lampreana','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49142','Palacios del Pan','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49143','Palacios de Sanabria','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49145','Pedralba de la Pradería','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49146','Pego (El)','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49147','Peleagonzalo','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49148','Peleas de Abajo','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49149','Peñausende','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49150','Peque','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49151','Perdigón (El)','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49152','Pereruela','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49153','Perilla de Castro','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49154','Pías','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49155','Piedrahita de Castro','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49156','Pinilla de Toro','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49157','Pino del Oro','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49158','Piñero (El)','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49159','Pobladura del Valle','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49160','Pobladura de Valderaduey','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49162','Porto','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49163','Pozoantiguo','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49164','Pozuelo de Tábara','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49165','Prado','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49166','Puebla de Sanabria','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49167','Pueblica de Valverde','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49168','Quintanilla del Monte','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49169','Quintanilla del Olmo','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49170','Quintanilla de Urz','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49171','Quiruelas de Vidriales','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49172','Rabanales','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49173','Rábano de Aliste','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49174','Requejo','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49175','Revellinos','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49176','Riofrío de Aliste','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49177','Rionegro del Puente','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49178','Roales','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49179','Robleda-Cervantes','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49180','Roelos de Sayago','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49181','Rosinos de la Requejada','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49183','Salce','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49184','Samir de los Caños','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49185','San Agustín del Pozo','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49186','San Cebrián de Castro','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49187','San Cristóbal de Entreviñas','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49188','San Esteban del Molar','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49189','San Justo','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49190','San Martín de Valderaduey','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49191','San Miguel de la Ribera','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49192','San Miguel del Valle','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49193','San Pedro de Ceque','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49194','San Pedro de la Nave-Almendra','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49197','Santa Clara de Avedillo','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49199','Santa Colomba de las Monjas','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49200','Santa Cristina de la Polvorosa','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49201','Santa Croya de Tera','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49202','Santa Eufemia del Barco','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49203','Santa María de la Vega','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49204','Santa María de Valverde','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49205','Santibáñez de Tera','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49206','Santibáñez de Vidriales','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49207','Santovenia','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49208','San Vicente de la Cabeza','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49209','San Vitero','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49210','Sanzoles','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49214','Tábara','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49216','Tapioles','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49219','Toro','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49220','Torre del Valle (La)','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49221','Torregamones','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49222','Torres del Carrizal','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49223','Trabazos','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49224','Trefacio','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49225','Uña de Quintana','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49226','Vadillo de la Guareña','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49227','Valcabado','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49228','Valdefinjas','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49229','Valdescorriel','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49230','Vallesa de la Guareña','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49231','Vega de Tera','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49232','Vega de Villalobos','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49233','Vegalatrave','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49234','Venialbo','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49235','Vezdemarbán','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49236','Vidayanes','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49237','Videmala','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49238','Villabrázaro','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49239','Villabuena del Puente','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49240','Villadepera','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49241','Villaescusa','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49242','Villafáfila','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49243','Villaferrueña','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49244','Villageriz','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49245','Villalazán','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49246','Villalba de la Lampreana','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49247','Villalcampo','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49248','Villalobos','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49249','Villalonso','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49250','Villalpando','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49251','Villalube','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49252','Villamayor de Campos','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49255','Villamor de los Escuderos','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49256','Villanázar','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49257','Villanueva de Azoague','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49258','Villanueva de Campeán','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49259','Villanueva de las Peras','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49260','Villanueva del Campo','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49261','Villaralbo','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49262','Villardeciervos','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49263','Villar de Fallaves','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49264','Villar del Buey','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49265','Villardiegua de la Ribera','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49266','Villárdiga','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49267','Villardondiego','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49268','Villarrín de Campos','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49269','Villaseco del Pan','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49270','Villavendimio','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49271','Villaveza del Agua','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49272','Villaveza de Valverde','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49273','Viñas','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'49275','Zamora','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50001','Abanto','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50002','Acered','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50003','Agón','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50004','Aguarón','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50005','Aguilón','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50006','Ainzón','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50007','Aladrén','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50008','Alagón','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50009','Alarba','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50010','Alberite de San Juan','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50011','Albeta','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50012','Alborge','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50013','Alcalá de Ebro','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50014','Alcalá de Moncayo','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50015','Alconchel de Ariza','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50016','Aldehuela de Liestos','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50017','Alfajarín','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50018','Alfamén','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50019','Alforque','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50020','Alhama de Aragón','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50021','Almochuel','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50022','Almolda (La)','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50023','Almonacid de la Cuba','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50024','Almonacid de la Sierra','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50025','Almunia de Doña Godina (La)','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50026','Alpartir','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50027','Ambel','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50028','Anento','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50029','Aniñón','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50030','Añón de Moncayo','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50031','Aranda de Moncayo','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50032','Arándiga','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50033','Ardisa','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50034','Ariza','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50035','Artieda','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50036','Asín','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50037','Atea','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50038','Ateca','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50039','Azuara','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50040','Badules','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50041','Bagüés','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50042','Balconchán','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50043','Bárboles','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50044','Bardallur','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50045','Belchite','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50046','Belmonte de Gracián','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50047','Berdejo','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50048','Berrueco','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50050','Bijuesca','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50051','Biota','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50052','Bisimbre','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50053','Boquiñeni','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50054','Bordalba','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50055','Borja','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50056','Botorrita','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50057','Brea de Aragón','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50058','Bubierca','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50059','Bujaraloz','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50060','Bulbuente','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50061','Bureta','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50062','Burgo de Ebro (El)','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50063','Buste (El)','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50064','Cabañas de Ebro','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50065','Cabolafuente','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50066','Cadrete','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50067','Calatayud','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50068','Calatorao','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50069','Calcena','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50070','Calmarza','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50071','Campillo de Aragón','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50072','Carenas','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50073','Cariñena','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50074','Caspe','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50075','Castejón de Alarba','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50076','Castejón de las Armas','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50077','Castejón de Valdejasa','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50078','Castiliscar','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50079','Cervera de la Cañada','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50080','Cerveruela','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50081','Cetina','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50082','Cimballa','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50083','Cinco Olivas','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50084','Clarés de Ribota','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50085','Codo','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50086','Codos','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50087','Contamina','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50088','Cosuenda','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50089','Cuarte de Huerva','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50090','Cubel','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50091','Cuerlas (Las)','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50092','Chiprana','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50093','Chodes','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50094','Daroca','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50095','Ejea de los Caballeros','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50096','Embid de Ariza','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50098','Encinacorba','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50099','Épila','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50100','Erla','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50101','Escatrón','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50102','Fabara','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50104','Farlete','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50105','Fayón','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50106','Fayos (Los)','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50107','Figueruelas','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50108','Fombuena','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50109','Frago (El)','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50110','Frasno (El)','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50111','Fréscano','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50113','Fuendejalón','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50114','Fuendetodos','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50115','Fuentes de Ebro','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50116','Fuentes de Jiloca','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50117','Gallocanta','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50118','Gallur','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50119','Gelsa','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50120','Godojos','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50121','Gotor','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50122','Grisel','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50123','Grisén','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50124','Herrera de los Navarros','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50125','Ibdes','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50126','Illueca','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50128','Isuerre','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50129','Jaraba','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50130','Jarque de Moncayo','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50131','Jaulín','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50132','Joyosa (La)','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50133','Lagata','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50134','Langa del Castillo','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50135','Layana','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50136','Lécera','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50137','Leciñena','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50138','Lechón','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50139','Letux','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50140','Litago','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50141','Lituénigo','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50142','Lobera de Onsella','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50143','Longares','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50144','Longás','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50146','Lucena de Jalón','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50147','Luceni','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50148','Luesia','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50149','Luesma','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50150','Lumpiaque','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50151','Luna','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50152','Maella','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50153','Magallón','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50154','Mainar','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50155','Malanquilla','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50156','Maleján','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50157','Malón','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50159','Maluenda','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50160','Mallén','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50161','Manchones','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50162','Mara','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50163','María de Huerva','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50164','Mediana de Aragón','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50165','Mequinenza','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50166','Mesones de Isuela','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50167','Mezalocha','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50168','Mianos','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50169','Miedes de Aragón','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50170','Monegrillo','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50171','Moneva','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50172','Monreal de Ariza','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50173','Monterde','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50174','Montón','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50175','Morata de Jalón','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50176','Morata de Jiloca','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50177','Morés','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50178','Moros','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50179','Moyuela','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50180','Mozota','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50181','Muel','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50182','Muela (La)','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50183','Munébrega','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50184','Murero','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50185','Murillo de Gállego','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50186','Navardún','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50187','Nigüella','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50188','Nombrevilla','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50189','Nonaspe','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50190','Novallas','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50191','Novillas','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50192','Nuévalos','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50193','Nuez de Ebro','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50194','Olvés','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50195','Orcajo','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50196','Orera','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50197','Orés','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50198','Oseja','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50199','Osera de Ebro','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50200','Paniza','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50201','Paracuellos de Jiloca','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50202','Paracuellos de la Ribera','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50203','Pastriz','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50204','Pedrola','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50205','Pedrosas (Las)','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50206','Perdiguera','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50207','Piedratajada','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50208','Pina de Ebro','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50209','Pinseque','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50210','Pintanos (Los)','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50211','Plasencia de Jalón','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50212','Pleitas','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50213','Plenas','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50214','Pomer','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50215','Pozuel de Ariza','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50216','Pozuelo de Aragón','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50217','Pradilla de Ebro','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50218','Puebla de Albortón','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50219','Puebla de Alfindén (La)','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50220','Puendeluna','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50221','Purujosa','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50222','Quinto','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50223','Remolinos','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50224','Retascón','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50225','Ricla','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50227','Romanos','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50228','Rueda de Jalón','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50229','Ruesca','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50230','Sádaba','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50231','Salillas de Jalón','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50232','Salvatierra de Esca','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50233','Samper del Salz','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50234','San Martín de la Virgen de Moncayo','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50235','San Mateo de Gállego','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50236','Santa Cruz de Grío','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50237','Santa Cruz de Moncayo','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50238','Santa Eulalia de Gállego','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50239','Santed','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50240','Sástago','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50241','Sabiñán','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50242','Sediles','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50243','Sestrica','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50244','Sierra de Luna','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50245','Sigüés','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50246','Sisamón','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50247','Sobradiel','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50248','Sos del Rey Católico','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50249','Tabuenca','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50250','Talamantes','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50251','Tarazona','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50252','Tauste','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50253','Terrer','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50254','Tierga','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50255','Tobed','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50256','Torralba de los Frailes','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50257','Torralba de Ribota','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50258','Torralbilla','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50259','Torrehermosa','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50260','Torrelapaja','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50261','Torrellas','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50262','Torres de Berrellén','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50263','Torrijo de la Cañada','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50264','Tosos','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50265','Trasmoz','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50266','Trasobares','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50267','Uncastillo','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50268','Undués de Lerda','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50269','Urrea de Jalón','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50270','Urriés','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50271','Used','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50272','Utebo','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50273','Valdehorna','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50274','Val de San Martín','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50275','Valmadrid','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50276','Valpalmas','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50277','Valtorres','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50278','Velilla de Ebro','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50279','Velilla de Jiloca','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50280','Vera de Moncayo','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50281','Vierlas','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50282','Vilueña (La)','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50283','Villadoz','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50284','Villafeliche','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50285','Villafranca de Ebro','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50286','Villalba de Perejil','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50287','Villalengua','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50288','Villanueva de Gállego','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50289','Villanueva de Jiloca','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50290','Villanueva de Huerva','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50291','Villar de los Navarros','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50292','Villarreal de Huerva','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50293','Villarroya de la Sierra','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50294','Villarroya del Campo','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50295','Vistabella','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50296','Zaida (La)','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50297','Zaragoza','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50298','Zuera','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50901','Biel','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50902','Marracos','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (281,'50903','Villamayor de Gállego','Z','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'CH-01','Cherkasy, Prydniprovs''kyi district','CH','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'CH-02','Cherkasy, Sosnivs''kyi distric','CH','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'CH-03','Vatutine','CH','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'CH-04','Horodyshche district','CH','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'CH-05','Drabiv district','CH','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'CH-06','Zhashkiv district','CH','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'CH-07','Zvenyhorodka district','CH','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'CH-08','Zolotonosha','CH','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'CH-09','Zolotonosha district','CH','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'CH-10','Kamianka district','CH','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'CH-11','Kaniv','CH','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'CH-12','Kaniv district','CH','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'CH-13','Katerynopil'' district','CH','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'CH-14','Korsun''-Shevchenkivs''kyi district','CH','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'CH-15','Lysianka district','CH','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'CH-16','Man''kivka district','CH','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'CH-17','Monastyryshche district','CH','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'CH-18','Smila','CH','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'CH-19','Smila district','CH','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'CH-20','Tal''ne district','CH','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'CH-21','Uman''','CH','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'CH-22','Uman'' district','CH','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'CH-23','Khrystynivka district','CH','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'CH-24','Cherkasy district','CH','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'CH-25','Chyhyryn district','CH','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'CH-26','Chornobai district','CH','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'CH-27','Shpola district','CH','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'CN-01','Chernivtsi, Shevchenkovs''kiy district','CN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'CN-02','Chernivtsi, Pershotravnevyi distric','CN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'CN-03','Chernivtsi, Sadhors''kyi distric','CN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'CN-04','Vyzhnytsia district','CN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'CN-05','Hertsa district','CN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'CN-06','Hlyboka district','CN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'CN-07','Zastavna district','CN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'CN-08','Kel''mentsi district','CN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'CN-09','Kitsman'' district','CN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'CN-10','istrovs''k','CN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'CN-11','Novoselytsia district','CN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'CN-12','Pytyla district','CN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'CN-13','Sokyriany district','CN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'CN-14','Storozhynets'' district','CN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'CN-15','Hotyn district','CN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'CR-01','Chernihiv, Desnians''kyi district','CR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'CR-02','Chernihiv, Novozavods''kyi distric','CR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'CR-03','Bakhmach district','CR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'CR-04','Bobrovyts''a district','CR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'CR-05','Borzna district','CR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'CR-06','Varva district','CR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'CR-07','Horodnia district','CR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'CR-08','Ichnia district','CR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'CR-09','Kozelets'' district','CR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'CR-10','Korop district','CR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'CR-11','Koriukivka district','CR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'CR-12','Kulykivka district','CR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'CR-13','Mena district','CR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'CR-14','Nizhyn','CR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'CR-15','Nizhyn district','CR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'CR-16','NovhorOD-Sivers''ky district','CR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'CR-17','Nosivka district','CR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'CR-18','Pryluky','CR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'CR-19','Pryluky districkt','CR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'CR-20','Ripky district','CR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'CR-21','Semenivka district','CR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'CR-22','Sosnytsia district','CR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'CR-23','Sribne district','CR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'CR-24','Talalaivka district','CR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'CR-25','Chernihiv district','CR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'CR-26','Shchors district','CR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'CR-27','Slavutych city','CR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'DN-01','Dnipropetrovs''k, Amur-Nyzhn''odniprovs''kyi district','DN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'DN-02','Dnipropetrovs''k, Babushkins''kyi distric','DN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'DN-03','Dnipropetrovs''k, Zhovtnevyi distric','DN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'DN-04','Dnipropetrovs''k, Idustrial''nyi distric','DN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'DN-05','Dnipropetrovs''k, Kirovs''kyi distric','DN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'DN-06','Dnipropetrovs''k, Chervonohvardijs''kyi distric','DN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'DN-07','Dnipropetrovs''k, Lenins''kyi distric','DN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'DN-08','Dnipropetrovs''k, Samars''kyi distric','DN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'DN-09','Apostolove district','DN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'DN-10','Vasyl''kivka district','DN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'DN-11','Verkhn''odniprovs''k district','DN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'DN-12','Vil''nohirs''k district','DN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'DN-13','Dnipridzerzhyns''k , Bahlejs''kyi district','DN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'DN-14','Dnipropetrovs''k, district','DN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'DN-15','Zhovti Vody district','DN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'DN-16','Kryvyi Rih, Dzerzhyns''kyi district','DN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'DN-17','Kryvyi Rih district','DN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'DN-18','Krynychky district','DN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'DN-19','Mahdalynivka district','DN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'DN-20','Marhanets''','DN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'DN-21','Mezhova district','DN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'DN-22','Nikopol''','DN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'DN-23','Nikopol'' district','DN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'DN-24','Novomoskovs''k','DN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'DN-25','Novomoskovs''k district','DN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'DN-26','Ordzhonikidze','DN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'DN-27','Pavlohrad','DN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'DN-28','Pavlohrad district','DN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'DN-29','Pershotravens''k','DN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'DN-30','Petrykivka district','DN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'DN-31','Petropavlivka district','DN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'DN-32','Pokrovs''ke district','DN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'DN-33','Piatykhatky district','DN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'DN-34','Synel''nykove','DN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'DN-35','Synel''nykove district','DN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'DN-36','Solone district','DN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'DN-37','Sofiivka district','DN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'DN-38','Ternivka','DN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'DN-39','Ternivka district','DN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'DN-40','Tomakivka district','DN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'DN-41','Tsarychanka district','DN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'DN-42','Shyroke district','DN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'DN-43','Iurivka district','DN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'DN-44','Dnipridzerzhyns''k, Zavodskyi district','DN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'DN-45','Dnipridzerzhyns''k, Dniprovs''kyi district','DN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'DN-46','Kryvyi Rih, Dolhintsevs''kyi district','DN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'DN-47','Kryvyi Rih, Zhovtnevyi district','DN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'DN-48','Kryvyi Rih, Inhulets''kyi district','DN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'DN-49','Kryvyi Rih, Saksahans''kyi district','DN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'DN-50','Kryvyi Rih, Ternivs''kyi district','DN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'DN-51','Kryvyi Rih, Tsentral''no-mis''kyi district','DN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'DO-01','Donets''k, Budjonivs''kyi district','DO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'DO-02','Donets''k, Voroshylovs''kyi distric','DO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'DO-03','Donets''k, Kalinins''kyi distric','DO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'DO-04','Donets''k, Kyivs''kyi distric','DO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'DO-05','Donets''k, Kirovs''kyi distric','DO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'DO-06','Donets''k, Kuybyshevs''kyi distric','DO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'DO-07','Donets''k, Lenins''kyi distric','DO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'DO-08','Donets''k, Petrovs''kyi distric','DO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'DO-09','Donets''k, Proletars''kyi distric','DO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'DO-10','Avdiivka','DO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'DO-11','Oleksandrivka district','DO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'DO-12','Amvrosiivka district','DO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'DO-13','Artemove','DO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'DO-14','Artemivs''k district','DO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'DO-15','Velyka Novosilka district','DO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'DO-16','Volnovakha district','DO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'DO-17','Volodars''k district','DO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'DO-18','Horlivka, Tsentral''no-mis''kyi district','DO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'DO-19','Debal''tseve','DO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'DO-20','Dzerzhyns''k','DO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'DO-21','Dymytrov','DO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'DO-22','Dobropillia','DO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'DO-23','Dobropillia district','DO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'DO-24','Dokuchaievs''k','DO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'DO-25','Druzhkivka','DO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'DO-26','Ienakieve','DO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'DO-27','Zhdanivk','DO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'DO-28','Kirovs''ke','DO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'DO-29','Kostiantynivka','DO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'DO-30','Kostiantynivka district','DO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'DO-31','Kramators''k','DO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'DO-32','Krasny Lyman','DO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'DO-33','Krasny Lyman district','DO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'DO-34','Krasnoarmiis''k','DO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'DO-35','Krasoarmiis''k district','DO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'DO-36','Makiyivka, Tsentral''no-mis''kyi district','DO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'DO-37','Manhush district','DO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'DO-38','Maryinka district','DO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'DO-39','Mariupol'', Zhovtnevyi district','DO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'DO-40','Novoazovs''k district','DO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'DO-41','Novohrodivka','DO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'DO-42','Selydove','DO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'DO-43','Slovians''k','DO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'DO-44','Slovians''k district','DO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'DO-45','Snizhne','DO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'DO-46','Starobesheve district','DO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'DO-47','Tel''manove district','DO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'DO-48','Torez','DO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'DO-49','Vuhledar district','DO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'DO-50','Khartsyz''k','DO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'DO-51','Shakhtars''k','DO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'DO-52','Shakhtars''k district','DO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'DO-53','Iasynuvata','DO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'DO-54','Iasynuvata district','DO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'DO-55','Horlivka , Kalinins''kyi district','DO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'DO-56','Horlivka, Nikitovs''kyi district','DO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'DO-57','Makiivka','DO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'DO-58','Makiivka district','DO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'DO-59','Mariupol'', Il''ichovskyi district','DO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'DO-60','Mariupol'', Ordzhonikidzevs''kyi district','DO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'DO-61','Mariupol'', Prymors''kyi district','DO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'DO-62','Makiivka, Chervonogvardiis''kyi district','DO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'DO-63','akiivka, Kirovs''kyi district','DO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'HA-01','Kharkiv, Dzerzhyns''kyi district','HA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'HA-02','Kharkiv, Kyivs''kyi distric','HA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'HA-03','Kharkiv, Kominternivs''kyi distric','HA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'HA-04','Kharkiv, Lenins''kyi distric','HA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'HA-05','Kharkiv, Moscows''kyi distric','HA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'HA-06','Kharkiv, Zhovtnevyi distric','HA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'HA-07','Kharkiv, Ordzhonikidzevs''kyi district','HA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'HA-08','Kharkiv, Phrunzens''kyi distric','HA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'HA-09','Kharkiv, Chervonozavods''kyi distric','HA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'HA-10','Balakliia distric','HA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'HA-11','Barvinkove district','HA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'HA-12','Blyzniuky district','HA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'HA-13','Bohodukhiv district','HA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'HA-14','Borova district','HA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'HA-15','Valky district','HA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'HA-16','Velyky Burluk district','HA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'HA-17','Vovchans''k district','HA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'HA-18','Dvorichna district','HA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'HA-19','Derhachi district','HA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'HA-20','Zachepylivka district','HA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'HA-21','Zmiyiv district','HA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'HA-22','Zolochiv district','HA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'HA-23','Izium','HA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'HA-24','Izium district','HA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'HA-25','Kehychivka district','HA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'HA-26','Kolomak district','HA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'HA-27','Krasnohrad district','HA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'HA-28','Krasnokuts''k district','HA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'HA-29','Kupians''k','HA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'HA-30','Kupians''k district','HA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'HA-31','Lozova','HA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'HA-32','Lozova district','HA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'HA-33','Liubotyn','HA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'HA-34','Nova Vodolaha district','HA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'HA-35','Pervomais''kyi','HA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'HA-36','Pervomais''kyi district','HA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'HA-37','Pechenihy district','HA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'HA-38','Sakhnovshchyna district','HA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'HA-39','Kharkiv district','HA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'HA-40','Chuhuyiv','HA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'HA-41','Chuhuyiv district','HA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'HA-42','Shevchenkove district','HA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'HE-01','Kherson, Dniprovs''kyi district','HE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'HE-02','Kherson, Komsomol''s''kyi distric','HE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'HE-03','Kherson, Suvorovs''kyi district','HE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'HE-04','Beryslav district','HE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'HE-05','Bilozerka district','HE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'HE-06','Velyka Lepetykha district','HE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'HE-07','Velyka Oleksandrivka district','HE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'HE-08','Verkhniy Rohachyk district','HE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'HE-09','Vysokopillia district','HE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'HE-10','Heniches''k district','HE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'HE-11','Hola Prystan'' district','HE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'HE-12','Hornostayivka district','HE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'HE-13','Ivanivka district','HE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'HE-14','Kalanchak district','HE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'HE-15','Kakhovka','HE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'HE-16','Kakhovka district','HE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'HE-17','Nyzhni Sirohozy district','HE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'HE-18','Nova Kakhovka','HE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'HE-19','Novovorontsovka district','HE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'HE-20','Novotroyits''ke district','HE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'HE-21','Skadovs''k district','HE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'HE-22','Tsiurupyns''k district','HE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'HE-23','Chaplynka district','HE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'HM-01','Khmel''nyts''kyi','HM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'HM-02','Bilohiria district','HM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'HM-03','Vin''kivtsi district','HM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'HM-04','Volochys''k district','HM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'HM-05','Horodok district','HM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'HM-06','Derazhnia district','HM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'HM-07','Dunayivtsi district','HM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'HM-08','Iziaslav district','HM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'HM-09','Kamianets''-Podil''s''kyi','HM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'HM-10','Kamianets''-Podil''s''kyi district','HM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'HM-11','Krasyliv district','HM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'HM-12','Letychiv district','HM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'HM-13','Netishyn','HM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'HM-14','Nova Ushytsia district','HM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'HM-15','Polonne district','HM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'HM-16','Slavuta','HM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'HM-17','Slavuta district','HM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'HM-18','Starokostiantyniv','HM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'HM-19','Starokostiantyniv district','HM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'HM-20','Stara Syniava district','HM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'HM-21','Teofipol'' district','HM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'HM-22','Khmel''nyts''kyi district','HM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'HM-23','Chemerivtsi district','HM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'HM-24','Shepetivka','HM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'HM-25','Shepetivka district','HM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'HM-26','Iarmolyntsi district','HM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'IF-01','Ivano-Frankivs''k','IF','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'IF-02','Bohorodchany district','IF','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'IF-03','Bolekhi','IF','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'IF-05','Verkhovyna district','IF','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'IF-06','Halych district','IF','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'IF-07','Horodenka district','IF','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'IF-08','Dolyna district','IF','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'IF-09','Kalush','IF','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'IF-10','Lalysh district','IF','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'IF-11','Kolomyia','IF','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'IF-12','Kolomyia district','IF','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'IF-13','Kosiv district','IF','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'IF-14','Nadvirna district','IF','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'IF-15','Rohatyn district','IF','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'IF-16','Rozhniativ district','IF','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'IF-17','Sniatyn district','IF','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'IF-18','Tysmenytsia district','IF','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'IF-19','Tlumach district','IF','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'IF-20','Iaremcha','IF','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'KI-01','Kirovohrad, Kirovs''kyi district','KI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'KI-02','Kirovohrad, Lenins''kyi distric','KI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'KI-03','Oleksandriia','KI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'KI-04','Oleksandriia district','KI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'KI-05','Oleksandrivka district','KI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'KI-06','Bobrynets'' district','KI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'KI-07','Haivoron district','KI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'KI-08','Holovanivs''k district','KI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'KI-09','Dobrovelychkivka district','KI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'KI-10','Dolyns''ka district','KI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'KI-11','Znamianka','KI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'KI-12','Znamianka district','KI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'KI-13','Kirovohrad district','KI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'KI-14','Kompaniyivka district','KI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'KI-15','Mala Vyska district','KI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'KI-16','Novhorodka district','KI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'KI-17','Novoarkhanhel''s''k district','KI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'KI-18','Novomyrhorod district','KI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'KI-19','Novoukrayinka district','KI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'KI-20','Vil''shanka district','KI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'KI-21','Onufriyivka district','KI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'KI-22','Petrove district','KI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'KI-23','Svitlovods''k','KI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'KI-24','Svitlovods''k district','KI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'KI-25','Ul''ianovka district','KI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'KI-26','Ustyninka district','KI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'KO-01','Bila Tserkva','KO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'KO-02','Baryshivka district','KO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'KO-03','Bila Tserkva district','KO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'KO-04','Berzan'' district','KO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'KO-05','Bohuslav district','KO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'KO-06','Boryspil''','KO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'KO-07','Boryspil'' district','KO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'KO-08','Borodianka district','KO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'KO-09','Brovary','KO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'KO-10','Brovary district','KO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'KO-11','Vasyl''kiv','KO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'KO-12','Vasyl''kiv district','KO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'KO-13','Vyshhorod district','KO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'KO-14','Volodarka district','KO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'KO-15','Zhurivka district','KO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'KO-16','Ivankiv district','KO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'KO-17','Irpin''','KO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'KO-18','Kaharlyk district','KO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'KO-19','KyiVO-Sviatoshyns''ky district','KO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'KO-20','Makariv district','KO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'KO-21','Myronivka district','KO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'KO-22','Obukhiv district','KO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'KO-23','Pereiaslav-Khmel''nyts''kyi','KO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'KO-24','Pereiaslav-Khmel''nyts''kyi district','KO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'KO-25','Polis''ke district','KO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'KO-26','Chernobyls''kyi district (city Pripyiat'')','KO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'KO-27','Rokytne district','KO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'KO-28','Rzhyshchiv','KO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'KO-29','Skvyra district','KO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'KO-30','Stavyshche district','KO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'KO-31','Tarashcha district','KO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'KO-32','Tetiyiv district','KO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'KO-33','Fastiv','KO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'KO-34','Fastiv district','KO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'KO-35','Iahotyn district','KO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'KR-01','Simferopol'', Zaliznychnyi district','KR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'KR-02','Simferopol'', Kyivs''kyi distric','KR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'KR-03','Simferopol'', Tsentral''nyi distric','KR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'KR-04','Alushta','KR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'KR-05','Armians''k','KR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'KR-06','Bakhchysarai district','KR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'KR-07','Bilohirs''k district','KR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'KR-08','Dzhankoi','KR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'KR-09','Dzhankoi district','KR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'KR-10','Ievpatoriia','KR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'KR-11','Kerch','KR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'KR-12','Kirovs''ke district','KR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'KR-13','Krasnohvardiis''ke district','KR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'KR-14','Krasnoperekops''k','KR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'KR-15','Krasnoperekops''k district','KR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'KR-16','Lenine district','KR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'KR-17','Nyzhniohirs''kyi district','KR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'KR-18','Pervomais''ke district','KR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'KR-19','Rozdol''ne district','KR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'KR-20','Saky','KR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'KR-21','Saky district','KR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'KR-22','Simferopol'' district','KR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'KR-23','Soviets''kyi district','KR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'KR-24','Sudak','KR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'KR-25','Feodosiia','KR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'KR-26','Chornomors''ke district','KR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'KR-27','Yalta','KR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'KV-01','Kyiv, Holosiivs''kyi distric','KV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'KV-02','Kyiv, Darnyts''kyi distric','KV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'KV-03','Kyiv. Desnians''kyi distric','KV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'KV-04','Kyiv, Dniprovs''kyi distric','KV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'KV-05','Kyiv, Obolons''kyi district','KV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'KV-06','Kyiv, Pechers''kyi distric','KV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'KV-07','Kyiv, Podols''kyi distric','KV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'KV-08','Kyiv, Sviatoshyns''kyi distric','KV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'KV-09','Kyiv, Solomens''kyi distric','KV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'KV-10','Kyiv, Shevchenkivs''kyi distric','KV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'LU-01','Luhans''k, Artemivs''kyi district','LU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'LU-02','Luhans''k, Zhovtnevyi distric','LU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'LU-03','Luhans''k, Kamenobrods''kyi distric','LU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'LU-04','Luhans''k, Lenins''kyi distric','LU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'LU-05','Alchevs''k','LU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'LU-06','Antratsyt','LU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'LU-07','Antratsyt district','LU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'LU-08','Bilovods''k district','LU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'LU-09','Brianka','LU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'LU-10','Bilikurakyne district','LU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'LU-11','Kirovs''k','LU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'LU-12','Krasnodon','LU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'LU-13','Krasnodon district','LU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'LU-14','Krasny Luch','LU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'LU-15','Kreminna district','LU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'LU-16','Lysychans''k','LU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'LU-17','Lutuhyne district','LU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'LU-18','Markivka district','LU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'LU-19','Milove district','LU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'LU-20','Novoajdar district','LU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'LU-21','Novopskov district','LU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'LU-22','Pervomais''k','LU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'LU-23','Pereval''s''k district','LU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'LU-24','Popasna district','LU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'LU-25','Roven''ky','LU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'LU-26','Rubizhne','LU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'LU-27','Svatove district','LU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'LU-28','Sverdlovs''k','LU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'LU-29','Sverdlovs''k district','LU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'LU-30','Sieverodonets''k','LU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'LU-31','Slovianoserbs''k district','LU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'LU-32','Stanychno-Luhans''ke district','LU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'LU-33','Starobil''s''k district','LU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'LU-34','Stakhanov','LU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'LU-35','Troyits''ke district','LU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'LV-01','L''viv, Halyts''kyi district','LV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'LV-02','L''viv, Zaliznychnyi distric','LV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'LV-03','L''viv, Lychakivs''kyi distric','LV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'LV-04','L''viv, Sykhivs''kyi distric','LV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'LV-05','L''viv, Frankivs''kyi distric','LV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'LV-06','L''viv, Shevchenkivs''kyi distric','LV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'LV-07','Boryslav','LV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'LV-08','Brody district','LV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'LV-09','Bus''k district','LV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'LV-10','Horodok dostrict','LV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'LV-11','Drohobych','LV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'LV-12','Drohobych district','LV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'LV-13','Zhydachiv district','LV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'LV-14','Zhovkva district','LV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'LV-15','Zolochiv district','LV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'LV-16','Kamianka-Buz''ka district','LV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'LV-17','Mykolayiv district','LV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'LV-18','Mostys''ka district','LV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'LV-19','Peremyshliany district','LV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'LV-20','Pustomyty district','LV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'LV-21','Radekhiv district','LV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'LV-22','Sambir','LV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'LV-23','Sambir district','LV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'LV-24','Skole district','LV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'LV-25','Sokal'' district','LV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'LV-26','Staryi Sambir district','LV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'LV-27','Stryi','LV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'LV-28','Stryi district','LV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'LV-29','Truskavets''','LV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'LV-30','Turka district','LV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'LV-31','Chervonohrad','LV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'LV-32','Iavoriv district','LV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'LV-33','Morshyn','LV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'NI-01','Mykolayiv, Zavods''kyi district','NI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'NI-02','v, Korabel''nyi district','NI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'NI-03','v, Lenins''kyi district','NI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'NI-04','Mykolayiv, Tsentral''nyi distric','NI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'NI-05','Arbuzynka district','NI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'NI-06','Bashtanka district','NI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'NI-07','Berezanka district','NI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'NI-08','Bereznehuvate district','NI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'NI-09','Brats''ke district','NI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'NI-10','Veselynove district','NI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'NI-11','Voznesens''k','NI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'NI-12','Voznesens''k district','NI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'NI-13','Vradiyivka district','NI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'NI-14','Domanivka district','NI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'NI-15','Ielanets'' district','NI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'NI-16','Zhovtnevy district','NI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'NI-17','Kazanka district','NI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'NI-18','Kryve Ozero district','NI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'NI-19','Mykolayiv district','NI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'NI-20','Novyi Buh district','NI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'NI-21','Nova Odesa district','NI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'NI-22','Ochakiv','NI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'NI-23','Ochakiv district','NI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'NI-24','Pervomais''k','NI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'NI-25','Pervomais''k district','NI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'NI-26','Snihurivka district','NI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'NI-27','Iuzhnoukrayins''k','NI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'OD-01','Odesa','OD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'OD-02','Odesa, Illichivs''kyi distric','OD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'OD-03','Odesa, Kyivs''kyi distric','OD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'OD-04','Odesa, Lenins''kyi distric','OD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'OD-05','Odesa, Malinovs''kyi distric','OD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'OD-06','Odesa, Prymors''kyi distric','OD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'OD-07','Odesa, Suvorovs''kyi distric','OD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'OD-08','Odesa, Tsentral''nyi distric','OD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'OD-09','Anan''yiv district','OD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'OD-10','Artsyz district','OD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'OD-11','Balta district','OD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'OD-12','Berezivka district','OD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'OD-13','BilhorOD-Dnistrovs''kyi','OD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'OD-14','BilhorOD-Dnistrovs''kyi district','OD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'OD-15','Biliayivka district','OD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'OD-16','Bolhrad district','OD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'OD-17','Velyka Mykhailivka district','OD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'OD-18','Ivanivka district','OD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'OD-19','Izmayil','OD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'OD-20','Izmayil district','OD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'OD-21','Illichivs''k','OD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'OD-22','Kiliia district','OD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'OD-23','Kodyma district','OD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'OD-24','Kominternivs''ke district','OD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'OD-25','Kotovs''k','OD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'OD-26','Kotovs''k district','OD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'OD-27','Krasni Okny district','OD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'OD-28','Liubashivka district','OD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'OD-29','Mykolayivka district','OD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'OD-30','Ovidiopol'' district','OD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'OD-31','Reni district','OD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'OD-32','Rozdil''na district','OD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'OD-33','Savran'' district','OD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'OD-34','Sarata district','OD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'OD-35','Tarutyne district','OD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'OD-36','Tatarbunary district','OD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'OD-37','Teplodar','OD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'OD-38','Frunzivka district','OD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'OD-39','Shyriaieve district','OD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'OD-40','Iuzhne','OD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'PO-01','Poltava, Kyivs''kyi district','PO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'PO-02','Poltava, Lenins''kyi distric','PO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'PO-03','Poltava, Zhovtnevyi distric','PO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'PO-04','Velyka Bahachka district','PO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'PO-05','Hadiach district','PO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'PO-06','Hlobyne district','PO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'PO-07','Hrebinka district','PO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'PO-08','Dykan''ka district','PO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'PO-09','Zin''kiv district','PO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'PO-10','Karlivka district','PO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'PO-11','Kobeliaky district','PO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'PO-12','Kozel''shchyna district','PO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'PO-13','Komsomol''s''k','PO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'PO-14','Kotel''va district','PO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'PO-15','Kremenchuk, Avtozavods''ky distric','PO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'PO-16','Kremenchuk, Kryukovs''ky distric','PO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'PO-17','Kremenchuk district','PO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'PO-18','Lokhvytsia district','PO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'PO-19','Lubny','PO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'PO-20','Lubny district','PO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'PO-21','Mashivka district','PO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'PO-22','Myrhorod','PO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'PO-23','Myrhorod district','PO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'PO-24','Novi Sanzhary district','PO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'PO-25','Orzhytsia district','PO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'PO-26','Pyriatyn district','PO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'PO-27','Poltava district','PO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'PO-28','Reshetylivka district','PO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'PO-29','Semenivka district','PO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'PO-30','Khorol district','PO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'PO-31','Chornukhy district','PO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'PO-32','Chutove district','PO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'PO-33','Shyshaky district','PO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'RI-01','Rivne','RI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'RI-02','Berezne district','RI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'RI-03','Volodymyrets'' district','RI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'RI-04','Hoshcha district','RI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'RI-05','Demydivka district','RI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'RI-06','Dubno','RI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'RI-07','Dubno district','RI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'RI-08','Dubrovytsia district','RI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'RI-09','Zarichne district','RI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'RI-10','Zdolbuniv district','RI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'RI-11','Korets'' district','RI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'RI-12','Kostopil'' district','RI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'RI-13','Kuznetsovs''k','RI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'RI-14','Mlyniv district','RI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'RI-15','Ostroh','RI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'RI-16','Ostroh district','RI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'RI-17','Radyvyliv district','RI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'RI-18','Rivne district','RI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'RI-19','Rokytne district','RI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'RI-20','Sarny district','RI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'SL-01','Sevastopol'', Gagarins''kyi district','SL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'SL-02','Sevastopol'', Lenins''kyi distric','SL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'SL-03','Sevastopol'', Nakhimovs''kyi distric','SL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'SL-04','Sevastopol''','SL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'SU-01','Sumy, Zarichnyi district','SU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'SU-02','Sumy, Kovpakivs''kyi distric','SU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'SU-03','Okhtyrka','SU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'SU-04','Okhtyrka district','SU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'SU-05','Bilopillia district','SU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'SU-06','Buryn'' district','SU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'SU-07','Velyka Pysarivka district','SU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'SU-08','Hlukhiv','SU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'SU-09','Hlukhiv district','SU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'SU-10','Konotop','SU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'SU-11','Konotop district','SU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'SU-12','Krasnopillia district','SU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'SU-13','Krolevets'' district','SU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'SU-14','Lebedyn','SU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'SU-15','Lebedyn district','SU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'SU-16','Lypova Dolyna district','SU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'SU-17','Nedryhailiv district','SU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'SU-18','Putyvl'' district','SU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'SU-19','Romny','SU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'SU-20','Romny district','SU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'SU-21','Seredyna-Buda district','SU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'SU-22','Sumy district','SU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'SU-23','Trostianets'' district','SU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'SU-24','Shostka','SU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'SU-25','Shostka district','SU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'SU-26','Iampil'' district','SU','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'TE-01','Ternopil''','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'TE-02','Berezany district','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'TE-03','Borshchiv district','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'TE-04','Buchach district','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'TE-05','Husiatyn district','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'TE-06','Zalishchyky district','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'TE-07','Zbarazh district','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'TE-08','Zboriv district','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'TE-09','Kozova district','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'TE-10','Kremenets'' district','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'TE-11','Lanivtsi district','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'TE-12','Monastyrys''k district','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'TE-13','Pidvolochys''k district','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'TE-14','Pidhajtsi district','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'TE-15','Terebovlia district','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'TE-16','Ternopil'' district','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'TE-17','Chortkiv district','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'TE-18','Shums''k district','TE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'VI-01','Vinnytsia, Zamostians''kyi district','VI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'VI-02','Vinnytsia, Lenins''kyi distric','VI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'VI-03','Vinnytsia, Starohorods''kyi distric','VI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'VI-04','Bar district','VI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'VI-05','Bershad'' district','VI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'VI-06','Vinnytsia district','VI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'VI-07','Haisyn district','VI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'VI-08','Zhmerynka','VI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'VI-09','Zhmerynka district','VI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'VI-10','Illintsi district','VI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'VI-11','Kalynivka district','VI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'VI-12','Koziatyn','VI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'VI-13','Koziatyn district','VI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'VI-14','Kryzhopil'' district','VI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'VI-15','Ladyzhyn district','VI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'VI-16','Lypovets'' district','VI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'VI-17','Lityn district','VI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'VI-18','Mohyliv-Podil''s''kyi','VI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'VI-19','Mohyliv-Podil''s''kyi district','VI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'VI-20','Murovani Kurylivtsi district','VI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'VI-21','Nemyriv district','VI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'VI-22','Orativ district','VI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'VI-23','Pishchanka district','VI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'VI-24','Pohrebyshche district','VI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'VI-25','Teplyk district','VI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'VI-26','Tyvriv district','VI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'VI-27','Tomashpil'' district','VI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'VI-28','Trostianets'' district','VI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'VI-29','Tul''chyn district','VI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'VI-30','Khmil''nyk','VI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'VI-31','Khmil''nyk district','VI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'VI-32','Chernivtsi district','VI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'VI-33','Chechel''nyk district','VI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'VI-34','Sharhorod district','VI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'VI-35','Iampil'' district','VI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'VO-01','Luts''k','VO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'VO-02','Volodymyr-Volyns''kyi','VO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'VO-03','Volodymyr-Volyns''kyi district','VO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'VO-04','Horokhiv district','VO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'VO-05','Ivanychi district','VO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'VO-06','Kamin''-Kashyrs''kyi district','VO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'VO-07','Kivertsi district','VO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'VO-08','Kovel''','VO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'VO-09','Kovel'' district','VO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'VO-10','Lokachi district','VO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'VO-11','Luts''k district','VO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'VO-12','Liubeshiv district','VO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'VO-13','Liuboml'' district','VO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'VO-14','Manevychi district','VO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'VO-15','Novovolyns''k','VO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'VO-16','Ratne district','VO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'VO-17','Rozhyshche district','VO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'VO-18','Stara Vyzhivka district','VO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'VO-19','Turiis''k district','VO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'VO-20','Shats''k district','VO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'ZA-01','Uzhhorod','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'ZA-02','Berehove','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'ZA-03','Berehove district','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'ZA-04','Velykyi-Bereznyi district','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'ZA-05','Vynohradiv district','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'ZA-06','Volovets'' district','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'ZA-07','Irshava district','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'ZA-08','Mizhhiria district','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'ZA-09','Mukacheve','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'ZA-10','Mukacheve district','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'ZA-11','Perechyn district','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'ZA-12','Rakhiv district','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'ZA-13','Svaliava district','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'ZA-14','Tiachiv district','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'ZA-15','Uzhhorod','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'ZA-16','hust','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'ZA-17','Khust district','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'ZA-18','Chop','ZA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'ZH-01','Zhytomyr, Bohuns''kyi district','ZH','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'ZH-02','Zhytomyr, Korolevs''kyi district','ZH','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'ZH-03','Andrushivka district','ZH','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'ZH-04','Baranivka district','ZH','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'ZH-05','Berdychiv','ZH','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'ZH-06','Berdychiv distict','ZH','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'ZH-07','Brusyliv district','ZH','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'ZH-08','Volodars''k-Volyns''kyi district','ZH','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'ZH-09','Romanovs''ky district (Up 09.07.2003)','ZH','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'ZH-10','Iemil''chyne district','ZH','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'ZH-11','Zhytomyr district','ZH','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'ZH-12','Korosten''','ZH','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'ZH-13','Korosten'' district','ZH','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'ZH-14','Korostyshiv district','ZH','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'ZH-15','Luhyny district','ZH','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'ZH-16','Liubar district','ZH','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'ZH-17','Malyn district','ZH','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'ZH-18','Narodychi district','ZH','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'ZH-19','Novohrad-Volyns''kyi','ZH','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'ZH-20','Novohrad-Volyns''kyi district','ZH','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'ZH-21','Ovruch district','ZH','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'ZH-22','Olevs''k district','ZH','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'ZH-23','Popil''nia district','ZH','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'ZH-24','Radomyshl'' district','ZH','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'ZH-25','Ruzhyn district','ZH','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'ZH-26','Chervonoarmiis''k district','ZH','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'ZH-27','Cherniakhiv district','ZH','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'ZH-28','Chudniv district','ZH','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'ZP-01','Zaporizhzhia, Zhovtnevyi district','ZP','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'ZP-02','Zaporizhzhia, Zavods''kyi distic','ZP','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'ZP-03','Zaporizhzhia, Komunars''kyi distric','ZP','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'ZP-04','Zaporizhzhia, Lenins''kyi distric','ZP','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'ZP-05','Zaporizhzhia, Ordzhonikidzevs''kyi distric','ZP','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'ZP-06','Zaporizhzhia, Khortyts''kyi distric','ZP','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'ZP-07','Zaporizhzhia, Shevchenkivs''kyi distric','ZP','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'ZP-08','Iakymivka district','ZP','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'ZP-09','Berdians''k','ZP','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'ZP-10','Berdians''k district','ZP','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'ZP-11','Vasylivka district','ZP','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'ZP-12','Velyka Bilozerka district','ZP','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'ZP-13','Vesele district','ZP','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'ZP-14','Vil''nians''k district','ZP','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'ZP-15','Huliaipole district','ZP','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'ZP-16','Zaporizhzhia district','ZP','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'ZP-17','Kamianka-Dniprovs''ka district','ZP','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'ZP-18','Kuibysheve district','ZP','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'ZP-19','Melitopol''','ZP','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'ZP-20','Melitopol'' district','ZP','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'ZP-21','Mykhailivka district','ZP','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'ZP-22','Novomykolayivka district','ZP','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'ZP-23','Orikhiv district','ZP','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'ZP-24','Polohy district','ZP','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'ZP-25','Pryazovs''ke district','ZP','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'ZP-26','Prymors''k district','ZP','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'ZP-27','Rozivka district','ZP','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'ZP-28','Tokmak','ZP','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'ZP-29','Tokmak district','ZP','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'ZP-30','Chernihivka district','ZP','');
INSERT INTO adif_enum_secondary_subdivision VALUES (288,'ZP-31','Enerhodar','ZP','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'CA,Alameda','Alameda','CA','EB');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'CA,Contra Costa','Contra Costa','CA','EB');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'CA,Napa','Napa','CA','EB');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'CA,Solano','Solano','CA','EB');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MA,Barnstable','Barnstable','MA','EMA');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MA,Bristol','Bristol','MA','EMA');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MA,Dukes','Dukes','MA','EMA');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MA,Essex','Essex','MA','EMA');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MA,Middlesex','Middlesex','MA','EMA');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MA,Nantucket','Nantucket','MA','EMA');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MA,Norfolk','Norfolk','MA','EMA');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MA,Plymouth','Plymouth','MA','EMA');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MA,Suffolk','Suffolk','MA','EMA');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NY,Albany','Albany','NY','ENY');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NY,Columbia','Columbia','NY','ENY');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NY,Dutchess','Dutchess','NY','ENY');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NY,Greene','Greene','NY','ENY');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NY,Orange','Orange','NY','ENY');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NY,Putnam','Putnam','NY','ENY');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NY,Rensselaer','Rensselaer','NY','ENY');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NY,Rockland','Rockland','NY','ENY');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NY,Saratoga','Saratoga','NY','ENY');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NY,Schenectady','Schenectady','NY','ENY');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NY,Sullivan','Sullivan','NY','ENY');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NY,Ulster','Ulster','NY','ENY');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NY,Warren','Warren','NY','ENY');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NY,Washington','Washington','NY','ENY');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NY,Westchester','Westchester','NY','ENY');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'PA,Adams','Adams','PA','EPA');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'PA,Berks','Berks','PA','EPA');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'PA,Bradford','Bradford','PA','EPA');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'PA,Bucks','Bucks','PA','EPA');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'PA,Carbon','Carbon','PA','EPA');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'PA,Chester','Chester','PA','EPA');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'PA,Columbia','Columbia','PA','EPA');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'PA,Cumberland','Cumberland','PA','EPA');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'PA,Dauphin','Dauphin','PA','EPA');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'PA,Delaware','Delaware','PA','EPA');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'PA,Juniata','Juniata','PA','EPA');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'PA,Lackawanna','Lackawanna','PA','EPA');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'PA,Lancaster','Lancaster','PA','EPA');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'PA,Lebanon','Lebanon','PA','EPA');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'PA,Lehigh','Lehigh','PA','EPA');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'PA,Luzerne','Luzerne','PA','EPA');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'PA,Lycoming','Lycoming','PA','EPA');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'PA,Monroe','Monroe','PA','EPA');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'PA,Montgomery','Montgomery','PA','EPA');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'PA,Montour','Montour','PA','EPA');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'PA,Northampton','Northampton','PA','EPA');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'PA,Northumberland','Northumberland','PA','EPA');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'PA,Perry','Perry','PA','EPA');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'PA,Philadelphia','Philadelphia','PA','EPA');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'PA,Pike','Pike','PA','EPA');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'PA,Schuylkill','Schuylkill','PA','EPA');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'PA,Snyder','Snyder','PA','EPA');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'PA,Sullivan','Sullivan','PA','EPA');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'PA,Susquehanna','Susquehanna','PA','EPA');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'PA,Tioga','Tioga','PA','EPA');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'PA,Union','Union','PA','EPA');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'PA,Wayne','Wayne','PA','EPA');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'PA,Wyoming','Wyoming','PA','EPA');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'PA,York','York','PA','EPA');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'WA,Adams','Adams','WA','EWA');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'WA,Asotin','Asotin','WA','EWA');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'WA,Benton','Benton','WA','EWA');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'WA,Chelan','Chelan','WA','EWA');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'WA,Columbia','Columbia','WA','EWA');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'WA,Douglas','Douglas','WA','EWA');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'WA,Ferry','Ferry','WA','EWA');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'WA,Franklin','Franklin','WA','EWA');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'WA,Garfield','Garfield','WA','EWA');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'WA,Grant','Grant','WA','EWA');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'WA,Kittitas','Kittitas','WA','EWA');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'WA,Klickitat','Klickitat','WA','EWA');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'WA,Lincoln','Lincoln','WA','EWA');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'WA,Okanogan','Okanogan','WA','EWA');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'WA,Pend Oreille','Pend Oreille','WA','EWA');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'WA,Spokane','Spokane','WA','EWA');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'WA,Stevens','Stevens','WA','EWA');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'WA,Walla Walla','Walla Walla','WA','EWA');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'WA,Whitman','Whitman','WA','EWA');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'WA,Yakima','Yakima','WA','EWA');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'CA,Los Angeles','Los Angeles','CA','LAX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'FL,Alachua','Alachua','FL','NFL');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'FL,Baker','Baker','FL','NFL');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'FL,Bay','Bay','FL','NFL');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'FL,Bradford','Bradford','FL','NFL');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'FL,Calhoun','Calhoun','FL','NFL');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'FL,Citrus','Citrus','FL','NFL');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'FL,Clay','Clay','FL','NFL');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'FL,Columbia','Columbia','FL','NFL');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'FL,Dixie','Dixie','FL','NFL');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'FL,Duval','Duval','FL','NFL');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'FL,Escambia','Escambia','FL','NFL');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'FL,Flagler','Flagler','FL','NFL');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'FL,Franklin','Franklin','FL','NFL');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'FL,Gadsden','Gadsden','FL','NFL');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'FL,Gilchrist','Gilchrist','FL','NFL');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'FL,Gulf','Gulf','FL','NFL');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'FL,Hamilton','Hamilton','FL','NFL');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'FL,Hernando','Hernando','FL','NFL');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'FL,Holmes','Holmes','FL','NFL');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'FL,Jackson','Jackson','FL','NFL');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'FL,Jefferson','Jefferson','FL','NFL');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'FL,Lafayette','Lafayette','FL','NFL');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'FL,Lake','Lake','FL','NFL');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'FL,Leon','Leon','FL','NFL');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'FL,Levy','Levy','FL','NFL');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'FL,Liberty','Liberty','FL','NFL');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'FL,Madison','Madison','FL','NFL');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'FL,Marion','Marion','FL','NFL');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'FL,Nassau','Nassau','FL','NFL');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'FL,Okaloosa','Okaloosa','FL','NFL');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'FL,Orange','Orange','FL','NFL');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'FL,Putnam','Putnam','FL','NFL');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'FL,Santa Rosa','Santa Rosa','FL','NFL');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'FL,Seminole','Seminole','FL','NFL');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'FL,St. Johns','St. Johns','FL','NFL');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'FL,Sumter','Sumter','FL','NFL');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'FL,Suwannee','Suwannee','FL','NFL');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'FL,Taylor','Taylor','FL','NFL');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'FL,Union','Union','FL','NFL');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'FL,Volusia','Volusia','FL','NFL');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'FL,Wakulla','Wakulla','FL','NFL');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'FL,Walton','Walton','FL','NFL');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'FL,Washington','Washington','FL','NFL');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NY,Bronx','Bronx','NY','NLI');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NY,Kings','Kings','NY','NLI');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NY,Nassau','Nassau','NY','NLI');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NY,New York','New York','NY','NLI');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NY,Queens','Queens','NY','NLI');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NY,Richmond','Richmond','NY','NLI');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NY,Suffolk','Suffolk','NY','NLI');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NJ,Bergen','Bergen','NJ','NNJ');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NJ,Essex','Essex','NJ','NNJ');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NJ,Hudson','Hudson','NJ','NNJ');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NJ,Hunterdon','Hunterdon','NJ','NNJ');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NJ,Middlesex','Middlesex','NJ','NNJ');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NJ,Monmouth','Monmouth','NJ','NNJ');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NJ,Morris','Morris','NJ','NNJ');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NJ,Passaic','Passaic','NJ','NNJ');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NJ,Somerset','Somerset','NJ','NNJ');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NJ,Sussex','Sussex','NJ','NNJ');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NJ,Union','Union','NJ','NNJ');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NJ,Warren','Warren','NJ','NNJ');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NY,Clinton','Clinton','NY','NNY');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NY,Essex','Essex','NY','NNY');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NY,Franklin','Franklin','NY','NNY');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NY,Fulton','Fulton','NY','NNY');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NY,Hamilton','Hamilton','NY','NNY');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NY,Jefferson','Jefferson','NY','NNY');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NY,Lewis','Lewis','NY','NNY');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NY,Montgomery','Montgomery','NY','NNY');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NY,Schoharie','Schoharie','NY','NNY');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NY,St. Lawrence','St. Lawrence','NY','NNY');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Anderson','Anderson','TX','NTX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Archer','Archer','TX','NTX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Baylor','Baylor','TX','NTX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Bell','Bell','TX','NTX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Bosque','Bosque','TX','NTX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Bowie','Bowie','TX','NTX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Brown','Brown','TX','NTX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Camp','Camp','TX','NTX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Cass','Cass','TX','NTX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Cherokee','Cherokee','TX','NTX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Clay','Clay','TX','NTX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Collin','Collin','TX','NTX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Comanche','Comanche','TX','NTX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Cooke','Cooke','TX','NTX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Coryell','Coryell','TX','NTX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Dallas','Dallas','TX','NTX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Delta','Delta','TX','NTX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Denton','Denton','TX','NTX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Eastland','Eastland','TX','NTX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Ellis','Ellis','TX','NTX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Erath','Erath','TX','NTX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Falls','Falls','TX','NTX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Fannin','Fannin','TX','NTX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Franklin','Franklin','TX','NTX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Freestone','Freestone','TX','NTX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Grayson','Grayson','TX','NTX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Gregg','Gregg','TX','NTX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Hamilton','Hamilton','TX','NTX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Harrison','Harrison','TX','NTX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Henderson','Henderson','TX','NTX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Hill','Hill','TX','NTX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Hood','Hood','TX','NTX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Hopkins','Hopkins','TX','NTX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Hunt','Hunt','TX','NTX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Jack','Jack','TX','NTX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Johnson','Johnson','TX','NTX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Kaufman','Kaufman','TX','NTX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Lamar','Lamar','TX','NTX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Lampasas','Lampasas','TX','NTX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Limestone','Limestone','TX','NTX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Marion','Marion','TX','NTX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,McLennan','McLennan','TX','NTX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Mills','Mills','TX','NTX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Montague','Montague','TX','NTX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Morris','Morris','TX','NTX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Nacogdoches','Nacogdoches','TX','NTX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Navarro','Navarro','TX','NTX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Palo Pinto','Palo Pinto','TX','NTX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Panola','Panola','TX','NTX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Parker','Parker','TX','NTX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Rains','Rains','TX','NTX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Red River','Red River','TX','NTX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Rockwall','Rockwall','TX','NTX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Rusk','Rusk','TX','NTX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Shelby','Shelby','TX','NTX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Smith','Smith','TX','NTX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Somervell','Somervell','TX','NTX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Stephens','Stephens','TX','NTX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Tarrant','Tarrant','TX','NTX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Throckmorton','Throckmorton','TX','NTX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Titus','Titus','TX','NTX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Upshur','Upshur','TX','NTX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Van Zandt','Van Zandt','TX','NTX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Wichita','Wichita','TX','NTX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Wilbarger','Wilbarger','TX','NTX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Wise','Wise','TX','NTX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Wood','Wood','TX','NTX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Young','Young','TX','NTX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'CA,Inyo','Inyo','CA','ORG');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'CA,Orange','Orange','CA','ORG');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'CA,Riverside','Riverside','CA','ORG');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'CA,San Bernardino','San Bernardino','CA','ORG');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'CA,San Luis Obispo','San Luis Obispo','CA','SB');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'CA,Santa Barbara','Santa Barbara','CA','SB');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'CA,Ventura','Ventura','CA','SB');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'CA,Monterey','Monterey','CA','SCV');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'CA,San Benito','San Benito','CA','SCV');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'CA,San Mateo','San Mateo','CA','SCV');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'CA,Santa Clara','Santa Clara','CA','SCV');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'CA,Santa Cruz','Santa Cruz','CA','SCV');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'CA,Imperial','Imperial','CA','SDG');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'CA,San Diego','San Diego','CA','SDG');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'CA,Del Norte','Del Norte','CA','SF');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'CA,Humboldt','Humboldt','CA','SF');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'CA,Lake','Lake','CA','SF');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'CA,Marin','Marin','CA','SF');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'CA,Mendocino','Mendocino','CA','SF');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'CA,San Francisco','San Francisco','CA','SF');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'CA,Sonoma','Sonoma','CA','SF');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'FL,Brevard','Brevard','FL','SFL');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'FL,Broward','Broward','FL','SFL');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'FL,Collier','Collier','FL','SFL');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'FL,Miami-Dade','Miami-Dade','FL','SFL');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'FL,Glades','Glades','FL','SFL');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'FL,Hendry','Hendry','FL','SFL');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'FL,Indian River','Indian River','FL','SFL');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'FL,Lee','Lee','FL','SFL');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'FL,Martin','Martin','FL','SFL');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'FL,Monroe','Monroe','FL','SFL');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'FL,Okeechobee','Okeechobee','FL','SFL');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'FL,Osceola','Osceola','FL','SFL');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'FL,Palm Beach','Palm Beach','FL','SFL');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'FL,St. Lucie','St. Lucie','FL','SFL');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'CA,Calaveras','Calaveras','CA','SJV');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'CA,Fresno','Fresno','CA','SJV');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'CA,Kern','Kern','CA','SJV');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'CA,Kings','Kings','CA','SJV');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'CA,Madera','Madera','CA','SJV');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'CA,Mariposa','Mariposa','CA','SJV');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'CA,Merced','Merced','CA','SJV');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'CA,Mono','Mono','CA','SJV');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'CA,San Joaquin','San Joaquin','CA','SJV');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'CA,Stanislaus','Stanislaus','CA','SJV');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'CA,Tulare','Tulare','CA','SJV');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'CA,Tuolumne','Tuolumne','CA','SJV');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NJ,Atlantic','Atlantic','NJ','SNJ');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NJ,Burlington','Burlington','NJ','SNJ');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NJ,Camden','Camden','NJ','SNJ');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NJ,Cape May','Cape May','NJ','SNJ');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NJ,Cumberland','Cumberland','NJ','SNJ');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NJ,Gloucester','Gloucester','NJ','SNJ');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NJ,Mercer','Mercer','NJ','SNJ');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NJ,Ocean','Ocean','NJ','SNJ');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NJ,Salem','Salem','NJ','SNJ');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Angelina','Angelina','TX','STX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Aransas','Aransas','TX','STX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Atascosa','Atascosa','TX','STX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Austin','Austin','TX','STX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Bandera','Bandera','TX','STX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Bastrop','Bastrop','TX','STX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Bee','Bee','TX','STX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Bexar','Bexar','TX','STX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Blanco','Blanco','TX','STX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Brazoria','Brazoria','TX','STX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Brazos','Brazos','TX','STX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Brooks','Brooks','TX','STX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Burleson','Burleson','TX','STX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Burnet','Burnet','TX','STX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Caldwell','Caldwell','TX','STX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Calhoun','Calhoun','TX','STX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Cameron','Cameron','TX','STX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Chambers','Chambers','TX','STX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Colorado','Colorado','TX','STX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Comal','Comal','TX','STX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Concho','Concho','TX','STX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,DeWitt','DeWitt','TX','STX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Dimmit','Dimmit','TX','STX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Duval','Duval','TX','STX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Edwards','Edwards','TX','STX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Fayette','Fayette','TX','STX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Fort Bend','Fort Bend','TX','STX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Frio','Frio','TX','STX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Galveston','Galveston','TX','STX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Gillespie','Gillespie','TX','STX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Goliad','Goliad','TX','STX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Gonzales','Gonzales','TX','STX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Grimes','Grimes','TX','STX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Guadalupe','Guadalupe','TX','STX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Hardin','Hardin','TX','STX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Harris','Harris','TX','STX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Hays','Hays','TX','STX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Hidalgo','Hidalgo','TX','STX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Houston','Houston','TX','STX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Jackson','Jackson','TX','STX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Jasper','Jasper','TX','STX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Jefferson','Jefferson','TX','STX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Jim Hogg','Jim Hogg','TX','STX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Jim Wells','Jim Wells','TX','STX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Karnes','Karnes','TX','STX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Kendall','Kendall','TX','STX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Kenedy','Kenedy','TX','STX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Kerr','Kerr','TX','STX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Kimble','Kimble','TX','STX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Kinney','Kinney','TX','STX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Kleberg','Kleberg','TX','STX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,La Salle','La Salle','TX','STX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Lavaca','Lavaca','TX','STX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Lee','Lee','TX','STX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Leon','Leon','TX','STX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Liberty','Liberty','TX','STX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Live Oak','Live Oak','TX','STX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Llano','Llano','TX','STX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Madison','Madison','TX','STX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Mason','Mason','TX','STX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Matagorda','Matagorda','TX','STX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Maverick','Maverick','TX','STX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,McCulloch','McCulloch','TX','STX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,McMullen','McMullen','TX','STX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Medina','Medina','TX','STX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Menard','Menard','TX','STX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Milam','Milam','TX','STX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Montgomery','Montgomery','TX','STX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Newton','Newton','TX','STX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Nueces','Nueces','TX','STX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Orange','Orange','TX','STX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Polk','Polk','TX','STX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Real','Real','TX','STX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Refugio','Refugio','TX','STX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Robertson','Robertson','TX','STX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Sabine','Sabine','TX','STX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,San Augustine','San Augustine','TX','STX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,San Jacinto','San Jacinto','TX','STX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,San Patricio','San Patricio','TX','STX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,San Saba','San Saba','TX','STX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Starr','Starr','TX','STX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Travis','Travis','TX','STX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Trinity','Trinity','TX','STX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Tyler','Tyler','TX','STX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Uvalde','Uvalde','TX','STX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Val Verde','Val Verde','TX','STX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Victoria','Victoria','TX','STX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Walker','Walker','TX','STX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Waller','Waller','TX','STX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Washington','Washington','TX','STX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Webb','Webb','TX','STX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Wharton','Wharton','TX','STX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Willacy','Willacy','TX','STX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Williamson','Williamson','TX','STX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Wilson','Wilson','TX','STX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Zapata','Zapata','TX','STX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Zavala','Zavala','TX','STX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'CA,Alpine','Alpine','CA','SV');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'CA,Amador','Amador','CA','SV');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'CA,Butte','Butte','CA','SV');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'CA,Colusa','Colusa','CA','SV');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'CA,El Dorado','El Dorado','CA','SV');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'CA,Glenn','Glenn','CA','SV');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'CA,Lassen','Lassen','CA','SV');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'CA,Modoc','Modoc','CA','SV');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'CA,Nevada','Nevada','CA','SV');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'CA,Placer','Placer','CA','SV');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'CA,Plumas','Plumas','CA','SV');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'CA,Sacramento','Sacramento','CA','SV');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'CA,Shasta','Shasta','CA','SV');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'CA,Sierra','Sierra','CA','SV');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'CA,Siskiyou','Siskiyou','CA','SV');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'CA,Sutter','Sutter','CA','SV');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'CA,Tehama','Tehama','CA','SV');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'CA,Trinity','Trinity','CA','SV');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'CA,Yolo','Yolo','CA','SV');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'CA,Yuba','Yuba','CA','SV');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'FL,Charlotte','Charlotte','FL','WCF');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'FL,DeSoto','DeSoto','FL','WCF');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'FL,Hardee','Hardee','FL','WCF');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'FL,Highlands','Highlands','FL','WCF');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'FL,Hillsborough','Hillsborough','FL','WCF');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'FL,Manatee','Manatee','FL','WCF');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'FL,Pasco','Pasco','FL','WCF');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'FL,Pinellas','Pinellas','FL','WCF');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'FL,Polk','Polk','FL','WCF');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'FL,Sarasota','Sarasota','FL','WCF');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MA,Berkshire','Berkshire','MA','WMA');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MA,Franklin','Franklin','MA','WMA');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MA,Hampden','Hampden','MA','WMA');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MA,Hampshire','Hampshire','MA','WMA');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MA,Worcester','Worcester','MA','WMA');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NY,Allegany','Allegany','NY','WNY');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NY,Broome','Broome','NY','WNY');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NY,Cattaraugus','Cattaraugus','NY','WNY');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NY,Cayuga','Cayuga','NY','WNY');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NY,Chautauqua','Chautauqua','NY','WNY');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NY,Chemung','Chemung','NY','WNY');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NY,Chenango','Chenango','NY','WNY');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NY,Cortland','Cortland','NY','WNY');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NY,Delaware','Delaware','NY','WNY');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NY,Erie','Erie','NY','WNY');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NY,Genesee','Genesee','NY','WNY');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NY,Herkimer','Herkimer','NY','WNY');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NY,Livingston','Livingston','NY','WNY');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NY,Madison','Madison','NY','WNY');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NY,Monroe','Monroe','NY','WNY');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NY,Niagara','Niagara','NY','WNY');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NY,Oneida','Oneida','NY','WNY');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NY,Onondaga','Onondaga','NY','WNY');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NY,Ontario','Ontario','NY','WNY');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NY,Orleans','Orleans','NY','WNY');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NY,Oswego','Oswego','NY','WNY');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NY,Otsego','Otsego','NY','WNY');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NY,Schuyler','Schuyler','NY','WNY');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NY,Seneca','Seneca','NY','WNY');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NY,Steuben','Steuben','NY','WNY');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NY,Tioga','Tioga','NY','WNY');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NY,Tompkins','Tompkins','NY','WNY');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NY,Wayne','Wayne','NY','WNY');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NY,Wyoming','Wyoming','NY','WNY');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NY,Yates','Yates','NY','WNY');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'PA,Allegheny','Allegheny','PA','WPA');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'PA,Armstrong','Armstrong','PA','WPA');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'PA,Beaver','Beaver','PA','WPA');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'PA,Bedford','Bedford','PA','WPA');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'PA,Blair','Blair','PA','WPA');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'PA,Butler','Butler','PA','WPA');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'PA,Cambria','Cambria','PA','WPA');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'PA,Cameron','Cameron','PA','WPA');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'PA,Centre','Centre','PA','WPA');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'PA,Clarion','Clarion','PA','WPA');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'PA,Clearfield','Clearfield','PA','WPA');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'PA,Clinton','Clinton','PA','WPA');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'PA,Crawford','Crawford','PA','WPA');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'PA,Elk','Elk','PA','WPA');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'PA,Erie','Erie','PA','WPA');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'PA,Fayette','Fayette','PA','WPA');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'PA,Forest','Forest','PA','WPA');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'PA,Franklin','Franklin','PA','WPA');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'PA,Fulton','Fulton','PA','WPA');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'PA,Greene','Greene','PA','WPA');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'PA,Huntingdon','Huntingdon','PA','WPA');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'PA,Indiana','Indiana','PA','WPA');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'PA,Jefferson','Jefferson','PA','WPA');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'PA,Lawrence','Lawrence','PA','WPA');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'PA,McKean','McKean','PA','WPA');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'PA,Mercer','Mercer','PA','WPA');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'PA,Mifflin','Mifflin','PA','WPA');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'PA,Potter','Potter','PA','WPA');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'PA,Somerset','Somerset','PA','WPA');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'PA,Venango','Venango','PA','WPA');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'PA,Warren','Warren','PA','WPA');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'PA,Washington','Washington','PA','WPA');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'PA,Westmoreland','Westmoreland','PA','WPA');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Andrews','Andrews','TX','WTX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Armstrong','Armstrong','TX','WTX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Bailey','Bailey','TX','WTX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Borden','Borden','TX','WTX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Brewster','Brewster','TX','WTX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Briscoe','Briscoe','TX','WTX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Callahan','Callahan','TX','WTX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Carson','Carson','TX','WTX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Castro','Castro','TX','WTX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Childress','Childress','TX','WTX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Cochran','Cochran','TX','WTX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Coke','Coke','TX','WTX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Coleman','Coleman','TX','WTX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Collingsworth','Collingsworth','TX','WTX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Cottle','Cottle','TX','WTX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Crane','Crane','TX','WTX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Crockett','Crockett','TX','WTX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Crosby','Crosby','TX','WTX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Culberson','Culberson','TX','WTX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Dallam','Dallam','TX','WTX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Dawson','Dawson','TX','WTX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Deaf Smith','Deaf Smith','TX','WTX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Dickens','Dickens','TX','WTX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Donley','Donley','TX','WTX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Ector','Ector','TX','WTX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,El Paso','El Paso','TX','WTX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Fisher','Fisher','TX','WTX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Floyd','Floyd','TX','WTX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Foard','Foard','TX','WTX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Gaines','Gaines','TX','WTX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Garza','Garza','TX','WTX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Glasscock','Glasscock','TX','WTX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Gray','Gray','TX','WTX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Hale','Hale','TX','WTX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Hall','Hall','TX','WTX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Hansford','Hansford','TX','WTX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Hardeman','Hardeman','TX','WTX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Hartley','Hartley','TX','WTX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Haskell','Haskell','TX','WTX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Hemphill','Hemphill','TX','WTX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Hockley','Hockley','TX','WTX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Howard','Howard','TX','WTX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Hudspeth','Hudspeth','TX','WTX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Hutchinson','Hutchinson','TX','WTX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Irion','Irion','TX','WTX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Jeff Davis','Jeff Davis','TX','WTX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Jones','Jones','TX','WTX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Kent','Kent','TX','WTX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,King','King','TX','WTX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Knox','Knox','TX','WTX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Lamb','Lamb','TX','WTX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Lipscomb','Lipscomb','TX','WTX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Loving','Loving','TX','WTX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Lubbock','Lubbock','TX','WTX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Lynn','Lynn','TX','WTX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Martin','Martin','TX','WTX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Midland','Midland','TX','WTX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Mitchell','Mitchell','TX','WTX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Moore','Moore','TX','WTX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Motley','Motley','TX','WTX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Nolan','Nolan','TX','WTX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Ochiltree','Ochiltree','TX','WTX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Oldham','Oldham','TX','WTX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Parmer','Parmer','TX','WTX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Pecos','Pecos','TX','WTX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Potter','Potter','TX','WTX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Presidio','Presidio','TX','WTX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Randall','Randall','TX','WTX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Reagan','Reagan','TX','WTX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Reeves','Reeves','TX','WTX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Roberts','Roberts','TX','WTX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Runnels','Runnels','TX','WTX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Schleicher','Schleicher','TX','WTX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Scurry','Scurry','TX','WTX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Shackelford','Shackelford','TX','WTX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Sherman','Sherman','TX','WTX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Sterling','Sterling','TX','WTX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Stonewall','Stonewall','TX','WTX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Sutton','Sutton','TX','WTX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Swisher','Swisher','TX','WTX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Taylor','Taylor','TX','WTX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Terrell','Terrell','TX','WTX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Terry','Terry','TX','WTX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Tom Green','Tom Green','TX','WTX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Upton','Upton','TX','WTX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Ward','Ward','TX','WTX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Wheeler','Wheeler','TX','WTX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Winkler','Winkler','TX','WTX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TX,Yoakum','Yoakum','TX','WTX');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'WA,Clallam','Clallam','WA','WWA');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'WA,Clark','Clark','WA','WWA');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'WA,Cowlitz','Cowlitz','WA','WWA');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'WA,Grays Harbor','Grays Harbor','WA','WWA');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'WA,Island','Island','WA','WWA');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'WA,Jefferson','Jefferson','WA','WWA');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'WA,King','King','WA','WWA');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'WA,Kitsap','Kitsap','WA','WWA');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'WA,Lewis','Lewis','WA','WWA');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'WA,Mason','Mason','WA','WWA');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'WA,Pacific','Pacific','WA','WWA');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'WA,Pierce','Pierce','WA','WWA');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'WA,San Juan','San Juan','WA','WWA');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'WA,Skagit','Skagit','WA','WWA');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'WA,Skamania','Skamania','WA','WWA');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'WA,Snohomish','Snohomish','WA','WWA');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'WA,Thurston','Thurston','WA','WWA');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'WA,Wahkiakum','Wahkiakum','WA','WWA');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'WA,Whatcom','Whatcom','WA','WWA');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'AL,Autauga','Autauga','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'AL,Baldwin','Baldwin','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'AL,Barbour','Barbour','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'AL,Bibb','Bibb','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'AL,Blount','Blount','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'AL,Bullock','Bullock','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'AL,Butler','Butler','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'AL,Calhoun','Calhoun','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'AL,Chambers','Chambers','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'AL,Cherokee','Cherokee','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'AL,Chilton','Chilton','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'AL,Choctaw','Choctaw','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'AL,Clarke','Clarke','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'AL,Clay','Clay','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'AL,Cleburne','Cleburne','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'AL,Coffee','Coffee','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'AL,Colbert','Colbert','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'AL,Conecuh','Conecuh','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'AL,Coosa','Coosa','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'AL,Covington','Covington','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'AL,Crenshaw','Crenshaw','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'AL,Cullman','Cullman','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'AL,Dale','Dale','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'AL,Dallas','Dallas','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'AL,DeKalb','DeKalb','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'AL,Elmore','Elmore','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'AL,Escambia','Escambia','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'AL,Etowah','Etowah','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'AL,Fayette','Fayette','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'AL,Franklin','Franklin','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'AL,Geneva','Geneva','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'AL,Greene','Greene','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'AL,Hale','Hale','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'AL,Henry','Henry','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'AL,Houston','Houston','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'AL,Jackson','Jackson','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'AL,Jefferson','Jefferson','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'AL,Lamar','Lamar','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'AL,Lauderdale','Lauderdale','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'AL,Lawrence','Lawrence','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'AL,Lee','Lee','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'AL,Limestone','Limestone','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'AL,Lowndes','Lowndes','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'AL,Macon','Macon','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'AL,Madison','Madison','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'AL,Marengo','Marengo','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'AL,Marion','Marion','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'AL,Marshall','Marshall','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'AL,Mobile','Mobile','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'AL,Monroe','Monroe','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'AL,Montgomery','Montgomery','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'AL,Morgan','Morgan','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'AL,Perry','Perry','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'AL,Pickens','Pickens','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'AL,Pike','Pike','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'AL,Randolph','Randolph','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'AL,Russell','Russell','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'AL,Shelby','Shelby','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'AL,St. Clair','St. Clair','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'AL,Sumter','Sumter','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'AL,Talladega','Talladega','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'AL,Tallapoosa','Tallapoosa','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'AL,Tuscaloosa','Tuscaloosa','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'AL,Walker','Walker','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'AL,Washington','Washington','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'AL,Wilcox','Wilcox','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'AL,Winston','Winston','AL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'AR,Arkansas','Arkansas','AR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'AR,Ashley','Ashley','AR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'AR,Baxter','Baxter','AR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'AR,Benton','Benton','AR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'AR,Boone','Boone','AR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'AR,Bradley','Bradley','AR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'AR,Calhoun','Calhoun','AR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'AR,Carroll','Carroll','AR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'AR,Chicot','Chicot','AR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'AR,Clark','Clark','AR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'AR,Clay','Clay','AR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'AR,Cleburne','Cleburne','AR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'AR,Cleveland','Cleveland','AR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'AR,Columbia','Columbia','AR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'AR,Conway','Conway','AR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'AR,Craighead','Craighead','AR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'AR,Crawford','Crawford','AR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'AR,Crittenden','Crittenden','AR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'AR,Cross','Cross','AR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'AR,Dallas','Dallas','AR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'AR,Desha','Desha','AR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'AR,Drew','Drew','AR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'AR,Faulkner','Faulkner','AR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'AR,Franklin','Franklin','AR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'AR,Fulton','Fulton','AR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'AR,Garland','Garland','AR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'AR,Grant','Grant','AR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'AR,Greene','Greene','AR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'AR,Hempstead','Hempstead','AR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'AR,Hot Spring','Hot Spring','AR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'AR,Howard','Howard','AR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'AR,Independence','Independence','AR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'AR,Izard','Izard','AR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'AR,Jackson','Jackson','AR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'AR,Jefferson','Jefferson','AR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'AR,Johnson','Johnson','AR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'AR,Lafayette','Lafayette','AR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'AR,Lawrence','Lawrence','AR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'AR,Lee','Lee','AR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'AR,Lincoln','Lincoln','AR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'AR,Little River','Little River','AR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'AR,Logan','Logan','AR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'AR,Lonoke','Lonoke','AR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'AR,Madison','Madison','AR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'AR,Marion','Marion','AR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'AR,Miller','Miller','AR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'AR,Mississippi','Mississippi','AR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'AR,Monroe','Monroe','AR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'AR,Montgomery','Montgomery','AR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'AR,Nevada','Nevada','AR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'AR,Newton','Newton','AR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'AR,Ouachita','Ouachita','AR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'AR,Perry','Perry','AR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'AR,Phillips','Phillips','AR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'AR,Pike','Pike','AR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'AR,Poinsett','Poinsett','AR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'AR,Polk','Polk','AR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'AR,Pope','Pope','AR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'AR,Prairie','Prairie','AR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'AR,Pulaski','Pulaski','AR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'AR,Randolph','Randolph','AR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'AR,Saline','Saline','AR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'AR,Scott','Scott','AR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'AR,Searcy','Searcy','AR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'AR,Sebastian','Sebastian','AR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'AR,Sevier','Sevier','AR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'AR,Sharp','Sharp','AR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'AR,St. Francis','St. Francis','AR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'AR,Stone','Stone','AR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'AR,Union','Union','AR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'AR,Van Buren','Van Buren','AR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'AR,Washington','Washington','AR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'AR,White','White','AR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'AR,Woodruff','Woodruff','AR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'AR,Yell','Yell','AR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'AZ,La Paz','La Paz','AZ','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'AZ,Apache','Apache','AZ','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'AZ,Cochise','Cochise','AZ','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'AZ,Coconino','Coconino','AZ','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'AZ,Gila','Gila','AZ','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'AZ,Graham','Graham','AZ','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'AZ,Greenlee','Greenlee','AZ','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'AZ,Maricopa','Maricopa','AZ','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'AZ,Mohave','Mohave','AZ','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'AZ,Navajo','Navajo','AZ','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'AZ,Pima','Pima','AZ','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'AZ,Pinal','Pinal','AZ','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'AZ,Santa Cruz','Santa Cruz','AZ','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'AZ,Yavapai','Yavapai','AZ','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'AZ,Yuma','Yuma','AZ','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'CO,Adams','Adams','CO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'CO,Alamosa','Alamosa','CO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'CO,Arapahoe','Arapahoe','CO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'CO,Archuleta','Archuleta','CO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'CO,Baca','Baca','CO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'CO,Bent','Bent','CO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'CO,Boulder','Boulder','CO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'CO,Broomfield','Broomfield','CO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'CO,Chaffee','Chaffee','CO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'CO,Cheyenne','Cheyenne','CO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'CO,Clear Creek','Clear Creek','CO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'CO,Conejos','Conejos','CO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'CO,Costilla','Costilla','CO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'CO,Crowley','Crowley','CO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'CO,Custer','Custer','CO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'CO,Delta','Delta','CO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'CO,Denver','Denver','CO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'CO,Dolores','Dolores','CO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'CO,Douglas','Douglas','CO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'CO,Eagle','Eagle','CO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'CO,El Paso','El Paso','CO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'CO,Elbert','Elbert','CO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'CO,Fremont','Fremont','CO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'CO,Garfield','Garfield','CO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'CO,Gilpin','Gilpin','CO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'CO,Grand','Grand','CO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'CO,Gunnison','Gunnison','CO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'CO,Hinsdale','Hinsdale','CO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'CO,Huerfano','Huerfano','CO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'CO,Jackson','Jackson','CO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'CO,Jefferson','Jefferson','CO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'CO,Kiowa','Kiowa','CO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'CO,Kit Carson','Kit Carson','CO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'CO,La Plata','La Plata','CO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'CO,Lake','Lake','CO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'CO,Larimer','Larimer','CO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'CO,Las Animas','Las Animas','CO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'CO,Lincoln','Lincoln','CO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'CO,Logan','Logan','CO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'CO,Mesa','Mesa','CO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'CO,Mineral','Mineral','CO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'CO,Moffat','Moffat','CO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'CO,Montezuma','Montezuma','CO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'CO,Montrose','Montrose','CO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'CO,Morgan','Morgan','CO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'CO,Otero','Otero','CO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'CO,Ouray','Ouray','CO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'CO,Park','Park','CO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'CO,Phillips','Phillips','CO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'CO,Pitkin','Pitkin','CO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'CO,Prowers','Prowers','CO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'CO,Pueblo','Pueblo','CO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'CO,Rio Blanco','Rio Blanco','CO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'CO,Rio Grande','Rio Grande','CO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'CO,Routt','Routt','CO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'CO,Saguache','Saguache','CO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'CO,San Juan','San Juan','CO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'CO,San Miguel','San Miguel','CO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'CO,Sedgwick','Sedgwick','CO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'CO,Summit','Summit','CO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'CO,Teller','Teller','CO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'CO,Washington','Washington','CO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'CO,Weld','Weld','CO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'CO,Yuma','Yuma','CO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'CT,Fairfield','Fairfield','CT','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'CT,Hartford','Hartford','CT','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'CT,Litchfield','Litchfield','CT','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'CT,Middlesex','Middlesex','CT','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'CT,New Haven','New Haven','CT','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'CT,New London','New London','CT','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'CT,Tolland','Tolland','CT','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'CT,Windham','Windham','CT','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'DC,Montgomery','Montgomery','DC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'DC,Prince George''s','Prince George''s','DC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'DE,Kent','Kent','DE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'DE,New Castle','New Castle','DE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'DE,Sussex','Sussex','DE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'GA,Appling','Appling','GA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'GA,Atkinson','Atkinson','GA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'GA,Bacon','Bacon','GA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'GA,Baker','Baker','GA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'GA,Baldwin','Baldwin','GA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'GA,Banks','Banks','GA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'GA,Barrow','Barrow','GA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'GA,Bartow','Bartow','GA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'GA,Ben Hill','Ben Hill','GA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'GA,Berrien','Berrien','GA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'GA,Bibb','Bibb','GA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'GA,Bleckley','Bleckley','GA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'GA,Brantley','Brantley','GA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'GA,Brooks','Brooks','GA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'GA,Bryan','Bryan','GA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'GA,Bulloch','Bulloch','GA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'GA,Burke','Burke','GA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'GA,Butts','Butts','GA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'GA,Calhoun','Calhoun','GA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'GA,Camden','Camden','GA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'GA,Candler','Candler','GA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'GA,Carroll','Carroll','GA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'GA,Catoosa','Catoosa','GA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'GA,Charlton','Charlton','GA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'GA,Chatham','Chatham','GA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'GA,Chattahoochee','Chattahoochee','GA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'GA,Chattooga','Chattooga','GA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'GA,Cherokee','Cherokee','GA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'GA,Clarke','Clarke','GA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'GA,Clay','Clay','GA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'GA,Clayton','Clayton','GA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'GA,Clinch','Clinch','GA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'GA,Cobb','Cobb','GA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'GA,Coffee','Coffee','GA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'GA,Colquitt','Colquitt','GA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'GA,Columbia','Columbia','GA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'GA,Cook','Cook','GA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'GA,Coweta','Coweta','GA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'GA,Crawford','Crawford','GA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'GA,Crisp','Crisp','GA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'GA,Dade','Dade','GA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'GA,Dawson','Dawson','GA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'GA,Decatur','Decatur','GA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'GA,DeKalb','DeKalb','GA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'GA,Dodge','Dodge','GA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'GA,Dooly','Dooly','GA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'GA,Dougherty','Dougherty','GA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'GA,Douglas','Douglas','GA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'GA,Early','Early','GA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'GA,Echols','Echols','GA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'GA,Effingham','Effingham','GA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'GA,Elbert','Elbert','GA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'GA,Emanuel','Emanuel','GA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'GA,Evans','Evans','GA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'GA,Fannin','Fannin','GA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'GA,Fayette','Fayette','GA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'GA,Floyd','Floyd','GA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'GA,Forsyth','Forsyth','GA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'GA,Franklin','Franklin','GA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'GA,Fulton','Fulton','GA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'GA,Gilmer','Gilmer','GA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'GA,Glascock','Glascock','GA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'GA,Glynn','Glynn','GA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'GA,Gordon','Gordon','GA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'GA,Grady','Grady','GA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'GA,Greene','Greene','GA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'GA,Gwinnett','Gwinnett','GA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'GA,Habersham','Habersham','GA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'GA,Hall','Hall','GA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'GA,Hancock','Hancock','GA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'GA,Haralson','Haralson','GA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'GA,Harris','Harris','GA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'GA,Hart','Hart','GA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'GA,Heard','Heard','GA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'GA,Henry','Henry','GA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'GA,Houston','Houston','GA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'GA,Irwin','Irwin','GA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'GA,Jackson','Jackson','GA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'GA,Jasper','Jasper','GA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'GA,Jeff Davis','Jeff Davis','GA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'GA,Jefferson','Jefferson','GA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'GA,Jenkins','Jenkins','GA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'GA,Johnson','Johnson','GA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'GA,Jones','Jones','GA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'GA,Lamar','Lamar','GA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'GA,Lanier','Lanier','GA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'GA,Laurens','Laurens','GA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'GA,Lee','Lee','GA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'GA,Liberty','Liberty','GA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'GA,Lincoln','Lincoln','GA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'GA,Long','Long','GA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'GA,Lowndes','Lowndes','GA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'GA,Lumpkin','Lumpkin','GA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'GA,Macon','Macon','GA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'GA,Madison','Madison','GA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'GA,Marion','Marion','GA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'GA,McDuffie','McDuffie','GA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'GA,McIntosh','McIntosh','GA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'GA,Meriwether','Meriwether','GA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'GA,Miller','Miller','GA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'GA,Mitchell','Mitchell','GA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'GA,Monroe','Monroe','GA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'GA,Montgomery','Montgomery','GA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'GA,Morgan','Morgan','GA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'GA,Murray','Murray','GA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'GA,Muscogee','Muscogee','GA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'GA,Newton','Newton','GA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'GA,Oconee','Oconee','GA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'GA,Oglethorpe','Oglethorpe','GA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'GA,Paulding','Paulding','GA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'GA,Peach','Peach','GA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'GA,Pickens','Pickens','GA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'GA,Pierce','Pierce','GA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'GA,Pike','Pike','GA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'GA,Polk','Polk','GA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'GA,Pulaski','Pulaski','GA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'GA,Putnam','Putnam','GA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'GA,Quitman','Quitman','GA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'GA,Rabun','Rabun','GA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'GA,Randolph','Randolph','GA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'GA,Richmond','Richmond','GA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'GA,Rockdale','Rockdale','GA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'GA,Schley','Schley','GA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'GA,Screven','Screven','GA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'GA,Seminole','Seminole','GA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'GA,Spalding','Spalding','GA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'GA,Stephens','Stephens','GA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'GA,Stewart','Stewart','GA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'GA,Sumter','Sumter','GA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'GA,Talbot','Talbot','GA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'GA,Taliaferro','Taliaferro','GA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'GA,Tattnall','Tattnall','GA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'GA,Taylor','Taylor','GA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'GA,Telfair','Telfair','GA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'GA,Terrell','Terrell','GA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'GA,Thomas','Thomas','GA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'GA,Tift','Tift','GA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'GA,Toombs','Toombs','GA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'GA,Towns','Towns','GA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'GA,Treutlen','Treutlen','GA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'GA,Troup','Troup','GA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'GA,Turner','Turner','GA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'GA,Twiggs','Twiggs','GA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'GA,Union','Union','GA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'GA,Upson','Upson','GA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'GA,Walker','Walker','GA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'GA,Walton','Walton','GA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'GA,Ware','Ware','GA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'GA,Warren','Warren','GA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'GA,Washington','Washington','GA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'GA,Wayne','Wayne','GA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'GA,Webster','Webster','GA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'GA,Wheeler','Wheeler','GA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'GA,White','White','GA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'GA,Whitfield','Whitfield','GA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'GA,Wilcox','Wilcox','GA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'GA,Wilkes','Wilkes','GA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'GA,Wilkinson','Wilkinson','GA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'GA,Worth','Worth','GA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IA,Adair','Adair','IA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IA,Adams','Adams','IA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IA,Allamakee','Allamakee','IA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IA,Appanoose','Appanoose','IA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IA,Audubon','Audubon','IA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IA,Benton','Benton','IA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IA,Black Hawk','Black Hawk','IA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IA,Boone','Boone','IA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IA,Bremer','Bremer','IA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IA,Buchanan','Buchanan','IA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IA,Buena Vista','Buena Vista','IA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IA,Butler','Butler','IA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IA,Calhoun','Calhoun','IA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IA,Carroll','Carroll','IA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IA,Cass','Cass','IA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IA,Cedar','Cedar','IA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IA,Cerro Gordo','Cerro Gordo','IA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IA,Cherokee','Cherokee','IA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IA,Chickasaw','Chickasaw','IA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IA,Clarke','Clarke','IA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IA,Clay','Clay','IA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IA,Clayton','Clayton','IA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IA,Clinton','Clinton','IA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IA,Crawford','Crawford','IA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IA,Dallas','Dallas','IA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IA,Davis','Davis','IA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IA,Decatur','Decatur','IA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IA,Delaware','Delaware','IA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IA,Des Moines','Des Moines','IA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IA,Dickinson','Dickinson','IA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IA,Dubuque','Dubuque','IA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IA,Emmet','Emmet','IA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IA,Fayette','Fayette','IA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IA,Floyd','Floyd','IA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IA,Franklin','Franklin','IA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IA,Fremont','Fremont','IA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IA,Greene','Greene','IA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IA,Grundy','Grundy','IA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IA,Guthrie','Guthrie','IA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IA,Hamilton','Hamilton','IA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IA,Hancock','Hancock','IA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IA,Hardin','Hardin','IA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IA,Harrison','Harrison','IA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IA,Henry','Henry','IA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IA,Howard','Howard','IA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IA,Humboldt','Humboldt','IA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IA,Ida','Ida','IA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IA,Iowa','Iowa','IA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IA,Jackson','Jackson','IA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IA,Jasper','Jasper','IA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IA,Jefferson','Jefferson','IA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IA,Johnson','Johnson','IA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IA,Jones','Jones','IA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IA,Keokuk','Keokuk','IA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IA,Kossuth','Kossuth','IA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IA,Lee','Lee','IA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IA,Linn','Linn','IA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IA,Louisa','Louisa','IA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IA,Lucas','Lucas','IA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IA,Lyon','Lyon','IA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IA,Madison','Madison','IA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IA,Mahaska','Mahaska','IA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IA,Marion','Marion','IA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IA,Marshall','Marshall','IA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IA,Mills','Mills','IA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IA,Mitchell','Mitchell','IA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IA,Monona','Monona','IA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IA,Monroe','Monroe','IA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IA,Montgomery','Montgomery','IA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IA,Muscatine','Muscatine','IA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IA,O''Brien','O''Brien','IA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IA,Osceola','Osceola','IA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IA,Page','Page','IA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IA,Palo Alto','Palo Alto','IA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IA,Plymouth','Plymouth','IA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IA,Pocahontas','Pocahontas','IA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IA,Polk','Polk','IA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IA,Pottawattamie','Pottawattamie','IA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IA,Poweshiek','Poweshiek','IA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IA,Ringgold','Ringgold','IA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IA,Sac','Sac','IA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IA,Scott','Scott','IA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IA,Shelby','Shelby','IA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IA,Sioux','Sioux','IA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IA,Story','Story','IA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IA,Tama','Tama','IA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IA,Taylor','Taylor','IA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IA,Union','Union','IA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IA,Van Buren','Van Buren','IA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IA,Wapello','Wapello','IA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IA,Warren','Warren','IA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IA,Washington','Washington','IA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IA,Wayne','Wayne','IA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IA,Webster','Webster','IA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IA,Winnebago','Winnebago','IA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IA,Winneshiek','Winneshiek','IA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IA,Woodbury','Woodbury','IA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IA,Worth','Worth','IA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IA,Wright','Wright','IA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'ID,Ada','Ada','ID','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'ID,Adams','Adams','ID','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'ID,Bannock','Bannock','ID','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'ID,Bear Lake','Bear Lake','ID','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'ID,Benewah','Benewah','ID','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'ID,Bingham','Bingham','ID','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'ID,Blaine','Blaine','ID','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'ID,Boise','Boise','ID','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'ID,Bonner','Bonner','ID','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'ID,Bonneville','Bonneville','ID','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'ID,Boundary','Boundary','ID','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'ID,Butte','Butte','ID','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'ID,Camas','Camas','ID','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'ID,Canyon','Canyon','ID','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'ID,Caribou','Caribou','ID','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'ID,Cassia','Cassia','ID','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'ID,Clark','Clark','ID','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'ID,Clearwater','Clearwater','ID','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'ID,Custer','Custer','ID','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'ID,Elmore','Elmore','ID','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'ID,Franklin','Franklin','ID','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'ID,Fremont','Fremont','ID','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'ID,Gem','Gem','ID','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'ID,Gooding','Gooding','ID','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'ID,Idaho','Idaho','ID','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'ID,Jefferson','Jefferson','ID','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'ID,Jerome','Jerome','ID','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'ID,Kootenai','Kootenai','ID','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'ID,Latah','Latah','ID','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'ID,Lemhi','Lemhi','ID','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'ID,Lewis','Lewis','ID','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'ID,Lincoln','Lincoln','ID','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'ID,Madison','Madison','ID','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'ID,Minidoka','Minidoka','ID','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'ID,Nez Perce','Nez Perce','ID','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'ID,Oneida','Oneida','ID','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'ID,Owyhee','Owyhee','ID','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'ID,Payette','Payette','ID','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'ID,Power','Power','ID','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'ID,Shoshone','Shoshone','ID','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'ID,Teton','Teton','ID','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'ID,Twin Falls','Twin Falls','ID','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'ID,Valley','Valley','ID','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'ID,Washington','Washington','ID','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IL,Adams','Adams','IL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IL,Alexander','Alexander','IL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IL,Bond','Bond','IL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IL,Boone','Boone','IL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IL,Brown','Brown','IL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IL,Bureau','Bureau','IL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IL,Calhoun','Calhoun','IL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IL,Carroll','Carroll','IL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IL,Cass','Cass','IL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IL,Champaign','Champaign','IL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IL,Christian','Christian','IL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IL,Clark','Clark','IL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IL,Clay','Clay','IL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IL,Clinton','Clinton','IL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IL,Coles','Coles','IL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IL,Cook','Cook','IL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IL,Crawford','Crawford','IL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IL,Cumberland','Cumberland','IL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IL,DeWitt','DeWitt','IL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IL,DeKalb','DeKalb','IL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IL,Douglas','Douglas','IL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IL,DuPage','DuPage','IL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IL,Edgar','Edgar','IL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IL,Edwards','Edwards','IL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IL,Effingham','Effingham','IL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IL,Fayette','Fayette','IL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IL,Ford','Ford','IL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IL,Franklin','Franklin','IL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IL,Fulton','Fulton','IL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IL,Gallatin','Gallatin','IL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IL,Greene','Greene','IL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IL,Grundy','Grundy','IL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IL,Hamilton','Hamilton','IL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IL,Hancock','Hancock','IL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IL,Hardin','Hardin','IL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IL,Henderson','Henderson','IL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IL,Henry','Henry','IL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IL,Iroquois','Iroquois','IL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IL,Jackson','Jackson','IL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IL,Jasper','Jasper','IL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IL,Jefferson','Jefferson','IL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IL,Jersey','Jersey','IL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IL,Jo Daviess','Jo Daviess','IL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IL,Johnson','Johnson','IL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IL,Kane','Kane','IL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IL,Kankakee','Kankakee','IL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IL,Kendall','Kendall','IL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IL,Knox','Knox','IL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IL,La Salle','La Salle','IL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IL,Lake','Lake','IL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IL,Lawrence','Lawrence','IL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IL,Lee','Lee','IL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IL,Livingston','Livingston','IL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IL,Logan','Logan','IL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IL,Macon','Macon','IL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IL,Macoupin','Macoupin','IL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IL,Madison','Madison','IL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IL,Marion','Marion','IL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IL,Marshall','Marshall','IL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IL,Mason','Mason','IL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IL,Massac','Massac','IL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IL,McDonough','McDonough','IL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IL,McHenry','McHenry','IL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IL,McLean','McLean','IL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IL,Menard','Menard','IL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IL,Mercer','Mercer','IL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IL,Monroe','Monroe','IL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IL,Montgomery','Montgomery','IL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IL,Morgan','Morgan','IL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IL,Moultrie','Moultrie','IL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IL,Ogle','Ogle','IL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IL,Peoria','Peoria','IL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IL,Perry','Perry','IL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IL,Piatt','Piatt','IL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IL,Pike','Pike','IL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IL,Pope','Pope','IL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IL,Pulaski','Pulaski','IL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IL,Putnam','Putnam','IL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IL,Randolph','Randolph','IL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IL,Richland','Richland','IL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IL,Rock Island','Rock Island','IL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IL,Saline','Saline','IL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IL,Sangamon','Sangamon','IL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IL,Schuyler','Schuyler','IL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IL,Scott','Scott','IL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IL,Shelby','Shelby','IL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IL,St. Clair','St. Clair','IL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IL,Stark','Stark','IL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IL,Stephenson','Stephenson','IL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IL,Tazewell','Tazewell','IL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IL,Union','Union','IL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IL,Vermilion','Vermilion','IL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IL,Wabash','Wabash','IL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IL,Warren','Warren','IL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IL,Washington','Washington','IL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IL,Wayne','Wayne','IL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IL,White','White','IL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IL,Whiteside','Whiteside','IL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IL,Will','Will','IL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IL,Williamson','Williamson','IL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IL,Winnebago','Winnebago','IL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IL,Woodford','Woodford','IL','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IN,Adams','Adams','IN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IN,Allen','Allen','IN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IN,Bartholomew','Bartholomew','IN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IN,Benton','Benton','IN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IN,Blackford','Blackford','IN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IN,Boone','Boone','IN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IN,Brown','Brown','IN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IN,Carroll','Carroll','IN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IN,Cass','Cass','IN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IN,Clark','Clark','IN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IN,Clay','Clay','IN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IN,Clinton','Clinton','IN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IN,Crawford','Crawford','IN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IN,Daviess','Daviess','IN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IN,DeKalb','DeKalb','IN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IN,Dearborn','Dearborn','IN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IN,Decatur','Decatur','IN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IN,Delaware','Delaware','IN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IN,Dubois','Dubois','IN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IN,Elkhart','Elkhart','IN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IN,Fayette','Fayette','IN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IN,Floyd','Floyd','IN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IN,Fountain','Fountain','IN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IN,Franklin','Franklin','IN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IN,Fulton','Fulton','IN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IN,Gibson','Gibson','IN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IN,Grant','Grant','IN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IN,Greene','Greene','IN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IN,Hamilton','Hamilton','IN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IN,Hancock','Hancock','IN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IN,Harrison','Harrison','IN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IN,Hendricks','Hendricks','IN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IN,Henry','Henry','IN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IN,Howard','Howard','IN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IN,Huntington','Huntington','IN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IN,Jackson','Jackson','IN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IN,Jasper','Jasper','IN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IN,Jay','Jay','IN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IN,Jefferson','Jefferson','IN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IN,Jennings','Jennings','IN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IN,Johnson','Johnson','IN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IN,Knox','Knox','IN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IN,Kosciusko','Kosciusko','IN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IN,La Porte','La Porte','IN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IN,Lagrange','Lagrange','IN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IN,Lake','Lake','IN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IN,Lawrence','Lawrence','IN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IN,Madison','Madison','IN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IN,Marion','Marion','IN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IN,Marshall','Marshall','IN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IN,Martin','Martin','IN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IN,Miami','Miami','IN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IN,Monroe','Monroe','IN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IN,Montgomery','Montgomery','IN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IN,Morgan','Morgan','IN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IN,Newton','Newton','IN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IN,Noble','Noble','IN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IN,Ohio','Ohio','IN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IN,Orange','Orange','IN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IN,Owen','Owen','IN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IN,Parke','Parke','IN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IN,Perry','Perry','IN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IN,Pike','Pike','IN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IN,Porter','Porter','IN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IN,Posey','Posey','IN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IN,Pulaski','Pulaski','IN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IN,Putnam','Putnam','IN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IN,Randolph','Randolph','IN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IN,Ripley','Ripley','IN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IN,Rush','Rush','IN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IN,Scott','Scott','IN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IN,Shelby','Shelby','IN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IN,Spencer','Spencer','IN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IN,St. Joseph','St. Joseph','IN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IN,Starke','Starke','IN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IN,Steuben','Steuben','IN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IN,Sullivan','Sullivan','IN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IN,Switzerland','Switzerland','IN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IN,Tippecanoe','Tippecanoe','IN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IN,Tipton','Tipton','IN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IN,Union','Union','IN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IN,Vanderburgh','Vanderburgh','IN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IN,Vermillion','Vermillion','IN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IN,Vigo','Vigo','IN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IN,Wabash','Wabash','IN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IN,Warren','Warren','IN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IN,Warrick','Warrick','IN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IN,Washington','Washington','IN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IN,Wayne','Wayne','IN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IN,Wells','Wells','IN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IN,White','White','IN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'IN,Whitley','Whitley','IN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KS,Allen','Allen','KS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KS,Anderson','Anderson','KS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KS,Atchison','Atchison','KS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KS,Barber','Barber','KS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KS,Barton','Barton','KS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KS,Bourbon','Bourbon','KS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KS,Brown','Brown','KS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KS,Butler','Butler','KS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KS,Chase','Chase','KS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KS,Chautauqua','Chautauqua','KS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KS,Cherokee','Cherokee','KS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KS,Cheyenne','Cheyenne','KS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KS,Clark','Clark','KS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KS,Clay','Clay','KS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KS,Cloud','Cloud','KS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KS,Coffey','Coffey','KS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KS,Comanche','Comanche','KS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KS,Cowley','Cowley','KS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KS,Crawford','Crawford','KS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KS,Decatur','Decatur','KS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KS,Dickinson','Dickinson','KS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KS,Doniphan','Doniphan','KS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KS,Douglas','Douglas','KS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KS,Edwards','Edwards','KS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KS,Elk','Elk','KS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KS,Ellis','Ellis','KS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KS,Ellsworth','Ellsworth','KS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KS,Finney','Finney','KS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KS,Ford','Ford','KS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KS,Franklin','Franklin','KS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KS,Geary','Geary','KS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KS,Gove','Gove','KS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KS,Graham','Graham','KS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KS,Grant','Grant','KS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KS,Gray','Gray','KS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KS,Greeley','Greeley','KS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KS,Greenwood','Greenwood','KS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KS,Hamilton','Hamilton','KS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KS,Harper','Harper','KS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KS,Harvey','Harvey','KS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KS,Haskell','Haskell','KS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KS,Hodgeman','Hodgeman','KS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KS,Jackson','Jackson','KS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KS,Jefferson','Jefferson','KS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KS,Jewell','Jewell','KS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KS,Johnson','Johnson','KS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KS,Kearny','Kearny','KS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KS,Kingman','Kingman','KS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KS,Kiowa','Kiowa','KS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KS,Labette','Labette','KS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KS,Lane','Lane','KS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KS,Leavenworth','Leavenworth','KS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KS,Lincoln','Lincoln','KS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KS,Linn','Linn','KS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KS,Logan','Logan','KS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KS,Lyon','Lyon','KS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KS,Marion','Marion','KS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KS,Marshall','Marshall','KS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KS,McPherson','McPherson','KS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KS,Meade','Meade','KS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KS,Miami','Miami','KS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KS,Mitchell','Mitchell','KS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KS,Montgomery','Montgomery','KS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KS,Morris','Morris','KS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KS,Morton','Morton','KS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KS,Nemaha','Nemaha','KS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KS,Neosho','Neosho','KS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KS,Ness','Ness','KS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KS,Norton','Norton','KS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KS,Osage','Osage','KS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KS,Osborne','Osborne','KS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KS,Ottawa','Ottawa','KS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KS,Pawnee','Pawnee','KS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KS,Phillips','Phillips','KS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KS,Pottawatomie','Pottawatomie','KS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KS,Pratt','Pratt','KS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KS,Rawlins','Rawlins','KS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KS,Reno','Reno','KS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KS,Republic','Republic','KS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KS,Rice','Rice','KS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KS,Riley','Riley','KS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KS,Rooks','Rooks','KS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KS,Rush','Rush','KS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KS,Russell','Russell','KS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KS,Saline','Saline','KS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KS,Scott','Scott','KS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KS,Sedgwick','Sedgwick','KS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KS,Seward','Seward','KS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KS,Shawnee','Shawnee','KS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KS,Sheridan','Sheridan','KS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KS,Sherman','Sherman','KS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KS,Smith','Smith','KS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KS,Stafford','Stafford','KS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KS,Stanton','Stanton','KS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KS,Stevens','Stevens','KS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KS,Sumner','Sumner','KS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KS,Thomas','Thomas','KS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KS,Trego','Trego','KS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KS,Wabaunsee','Wabaunsee','KS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KS,Wallace','Wallace','KS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KS,Washington','Washington','KS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KS,Wichita','Wichita','KS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KS,Wilson','Wilson','KS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KS,Woodson','Woodson','KS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KS,Wyandotte','Wyandotte','KS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KY,Adair','Adair','KY','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KY,Allen','Allen','KY','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KY,Anderson','Anderson','KY','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KY,Ballard','Ballard','KY','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KY,Barren','Barren','KY','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KY,Bath','Bath','KY','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KY,Bell','Bell','KY','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KY,Boone','Boone','KY','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KY,Bourbon','Bourbon','KY','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KY,Boyd','Boyd','KY','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KY,Boyle','Boyle','KY','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KY,Bracken','Bracken','KY','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KY,Breathitt','Breathitt','KY','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KY,Breckinridge','Breckinridge','KY','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KY,Bullitt','Bullitt','KY','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KY,Butler','Butler','KY','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KY,Caldwell','Caldwell','KY','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KY,Calloway','Calloway','KY','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KY,Campbell','Campbell','KY','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KY,Carlisle','Carlisle','KY','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KY,Carroll','Carroll','KY','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KY,Carter','Carter','KY','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KY,Casey','Casey','KY','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KY,Christian','Christian','KY','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KY,Clark','Clark','KY','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KY,Clay','Clay','KY','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KY,Clinton','Clinton','KY','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KY,Crittenden','Crittenden','KY','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KY,Cumberland','Cumberland','KY','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KY,Daviess','Daviess','KY','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KY,Edmonson','Edmonson','KY','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KY,Elliott','Elliott','KY','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KY,Estill','Estill','KY','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KY,Fayette','Fayette','KY','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KY,Fleming','Fleming','KY','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KY,Floyd','Floyd','KY','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KY,Franklin','Franklin','KY','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KY,Fulton','Fulton','KY','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KY,Gallatin','Gallatin','KY','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KY,Garrard','Garrard','KY','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KY,Grant','Grant','KY','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KY,Graves','Graves','KY','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KY,Grayson','Grayson','KY','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KY,Green','Green','KY','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KY,Greenup','Greenup','KY','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KY,Hancock','Hancock','KY','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KY,Hardin','Hardin','KY','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KY,Harlan','Harlan','KY','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KY,Harrison','Harrison','KY','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KY,Hart','Hart','KY','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KY,Henderson','Henderson','KY','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KY,Henry','Henry','KY','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KY,Hickman','Hickman','KY','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KY,Hopkins','Hopkins','KY','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KY,Jackson','Jackson','KY','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KY,Jefferson','Jefferson','KY','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KY,Jessamine','Jessamine','KY','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KY,Johnson','Johnson','KY','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KY,Kenton','Kenton','KY','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KY,Knott','Knott','KY','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KY,Knox','Knox','KY','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KY,Larue','Larue','KY','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KY,Laurel','Laurel','KY','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KY,Lawrence','Lawrence','KY','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KY,Lee','Lee','KY','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KY,Leslie','Leslie','KY','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KY,Letcher','Letcher','KY','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KY,Lewis','Lewis','KY','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KY,Lincoln','Lincoln','KY','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KY,Livingston','Livingston','KY','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KY,Logan','Logan','KY','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KY,Lyon','Lyon','KY','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KY,Madison','Madison','KY','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KY,Magoffin','Magoffin','KY','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KY,Marion','Marion','KY','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KY,Marshall','Marshall','KY','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KY,Martin','Martin','KY','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KY,Mason','Mason','KY','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KY,McCracken','McCracken','KY','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KY,McCreary','McCreary','KY','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KY,McLean','McLean','KY','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KY,Meade','Meade','KY','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KY,Menifee','Menifee','KY','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KY,Mercer','Mercer','KY','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KY,Metcalfe','Metcalfe','KY','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KY,Monroe','Monroe','KY','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KY,Montgomery','Montgomery','KY','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KY,Morgan','Morgan','KY','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KY,Muhlenberg','Muhlenberg','KY','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KY,Nelson','Nelson','KY','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KY,Nicholas','Nicholas','KY','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KY,Ohio','Ohio','KY','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KY,Oldham','Oldham','KY','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KY,Owen','Owen','KY','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KY,Owsley','Owsley','KY','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KY,Pendleton','Pendleton','KY','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KY,Perry','Perry','KY','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KY,Pike','Pike','KY','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KY,Powell','Powell','KY','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KY,Pulaski','Pulaski','KY','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KY,Robertson','Robertson','KY','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KY,Rockcastle','Rockcastle','KY','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KY,Rowan','Rowan','KY','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KY,Russell','Russell','KY','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KY,Scott','Scott','KY','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KY,Shelby','Shelby','KY','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KY,Simpson','Simpson','KY','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KY,Spencer','Spencer','KY','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KY,Taylor','Taylor','KY','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KY,Todd','Todd','KY','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KY,Trigg','Trigg','KY','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KY,Trimble','Trimble','KY','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KY,Union','Union','KY','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KY,Warren','Warren','KY','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KY,Washington','Washington','KY','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KY,Wayne','Wayne','KY','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KY,Webster','Webster','KY','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KY,Whitley','Whitley','KY','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KY,Wolfe','Wolfe','KY','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'KY,Woodford','Woodford','KY','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'LA,Acadia','Acadia Parish','LA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'LA,Allen','Allen Parish','LA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'LA,Ascension','Ascension Parish','LA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'LA,Assumption','Assumption Parish','LA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'LA,Avoyelles','Avoyelles Parish','LA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'LA,Beauregard','Beauregard Parish','LA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'LA,Bienville','Bienville Parish','LA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'LA,Bossier','Bossier Parish','LA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'LA,Caddo','Caddo Parish','LA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'LA,Calcasieu','Calcasieu Parish','LA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'LA,Caldwell','Caldwell Parish','LA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'LA,Cameron','Cameron Parish','LA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'LA,Catahoula','Catahoula Parish','LA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'LA,Claiborne','Claiborne Parish','LA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'LA,Concordia','Concordia Parish','LA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'LA,DeSoto','DeSoto Parish','LA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'LA,E. Baton Rouge','East Baton Rouge Parish','LA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'LA,E. Carroll','East Carroll Parish','LA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'LA,E. Feliciana','East Feliciana Parish','LA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'LA,Evangeline','Evangeline Parish','LA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'LA,Franklin','Franklin Parish','LA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'LA,Grant','Grant Parish','LA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'LA,Iberia','Iberia Parish','LA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'LA,Iberville','Iberville Parish','LA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'LA,Jackson','Jackson Parish','LA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'LA,Jefferson Davis','Jefferson Davis Parish','LA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'LA,Jefferson','Jefferson Parish','LA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'LA,La Salle','La Salle Parish','LA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'LA,Lafayette','Lafayette Parish','LA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'LA,Lafourche','Lafourche Parish','LA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'LA,Lincoln','Lincoln Parish','LA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'LA,Livingston','Livingston Parish','LA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'LA,Madison','Madison Parish','LA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'LA,Morehouse','Morehouse Parish','LA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'LA,Natchitoches','Natchitoches Parish','LA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'LA,Orleans','Orleans Parish','LA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'LA,Ouachita','Ouachita Parish','LA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'LA,Plaquemines','Plaquemines Parish','LA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'LA,Pointe Coupee','Pointe Coupee Parish','LA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'LA,Rapides','Rapides Parish','LA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'LA,Red River','Red River Parish','LA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'LA,Richland','Richland Parish','LA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'LA,Sabine','Sabine Parish','LA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'LA,St. Bernard','St. Bernard Parish','LA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'LA,St. Charles','St. Charles Parish','LA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'LA,St. Helena','St. Helena Parish','LA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'LA,St. James','St. James Parish','LA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'LA,St. John the Baptist','St. John the Baptist Parish','LA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'LA,St. Landry','St. Landry Parish','LA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'LA,St. Martin','St. Martin Parish','LA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'LA,St. Mary','St. Mary Parish','LA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'LA,St. Tammany','St. Tammany Parish','LA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'LA,Tangipahoa','Tangipahoa Parish','LA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'LA,Tensas','Tensas Parish','LA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'LA,Terrebonne','Terrebonne Parish','LA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'LA,Union','Union Parish','LA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'LA,Vermilion','Vermilion Parish','LA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'LA,Vernon','Vernon Parish','LA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'LA,Washington','Washington Parish','LA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'LA,Webster','Webster Parish','LA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'LA,W. Baton Rouge','West Baton Rouge Parish','LA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'LA,W. Carroll','West Carroll Parish','LA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'LA,W. Feliciana','West Feliciana Parish','LA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'LA,Winn','Winn Parish','LA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MD,Allegany','Allegany','MD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MD,Anne Arundel','Anne Arundel','MD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MD,Baltimore City','Baltimore City','MD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MD,Baltimore','Baltimore','MD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MD,Calvert','Calvert','MD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MD,Caroline','Caroline','MD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MD,Carroll','Carroll','MD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MD,Cecil','Cecil','MD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MD,Charles','Charles','MD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MD,Dorchester','Dorchester','MD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MD,Frederick','Frederick','MD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MD,Garrett','Garrett','MD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MD,Harford','Harford','MD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MD,Howard','Howard','MD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MD,Kent','Kent','MD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MD,Montgomery','Montgomery','MD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MD,Prince George''s','Prince George''s','MD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MD,Queen Anne''s','Queen Anne''s','MD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MD,Somerset','Somerset','MD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MD,St. Mary''s','St. Mary''s','MD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MD,Talbot','Talbot','MD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MD,Washington','Washington','MD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MD,Wicomico','Wicomico','MD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MD,Worcester','Worcester','MD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'ME,Androscoggin','Androscoggin','ME','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'ME,Aroostook','Aroostook','ME','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'ME,Cumberland','Cumberland','ME','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'ME,Franklin','Franklin','ME','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'ME,Hancock','Hancock','ME','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'ME,Kennebec','Kennebec','ME','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'ME,Knox','Knox','ME','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'ME,Lincoln','Lincoln','ME','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'ME,Oxford','Oxford','ME','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'ME,Penobscot','Penobscot','ME','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'ME,Piscataquis','Piscataquis','ME','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'ME,Sagadahoc','Sagadahoc','ME','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'ME,Somerset','Somerset','ME','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'ME,Waldo','Waldo','ME','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'ME,Washington','Washington','ME','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'ME,York','York','ME','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MI,Alcona','Alcona','MI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MI,Alger','Alger','MI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MI,Allegan','Allegan','MI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MI,Alpena','Alpena','MI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MI,Antrim','Antrim','MI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MI,Arenac','Arenac','MI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MI,Baraga','Baraga','MI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MI,Barry','Barry','MI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MI,Bay','Bay','MI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MI,Benzie','Benzie','MI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MI,Berrien','Berrien','MI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MI,Branch','Branch','MI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MI,Calhoun','Calhoun','MI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MI,Cass','Cass','MI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MI,Charlevoix','Charlevoix','MI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MI,Cheboygan','Cheboygan','MI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MI,Chippewa','Chippewa','MI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MI,Clare','Clare','MI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MI,Clinton','Clinton','MI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MI,Crawford','Crawford','MI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MI,Delta','Delta','MI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MI,Dickinson','Dickinson','MI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MI,Eaton','Eaton','MI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MI,Emmet','Emmet','MI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MI,Genesee','Genesee','MI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MI,Gladwin','Gladwin','MI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MI,Gogebic','Gogebic','MI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MI,Grand Traverse','Grand Traverse','MI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MI,Gratiot','Gratiot','MI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MI,Hillsdale','Hillsdale','MI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MI,Houghton','Houghton','MI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MI,Huron','Huron','MI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MI,Ingham','Ingham','MI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MI,Ionia','Ionia','MI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MI,Iosco','Iosco','MI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MI,Iron','Iron','MI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MI,Isabella','Isabella','MI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MI,Jackson','Jackson','MI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MI,Kalamazoo','Kalamazoo','MI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MI,Kalkaska','Kalkaska','MI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MI,Kent','Kent','MI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MI,Keweenaw','Keweenaw','MI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MI,Lake','Lake','MI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MI,Lapeer','Lapeer','MI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MI,Leelanau','Leelanau','MI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MI,Lenawee','Lenawee','MI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MI,Livingston','Livingston','MI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MI,Luce','Luce','MI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MI,Mackinac','Mackinac','MI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MI,Macomb','Macomb','MI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MI,Manistee','Manistee','MI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MI,Marquette','Marquette','MI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MI,Mason','Mason','MI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MI,Mecosta','Mecosta','MI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MI,Menominee','Menominee','MI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MI,Midland','Midland','MI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MI,Missaukee','Missaukee','MI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MI,Monroe','Monroe','MI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MI,Montcalm','Montcalm','MI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MI,Montmorency','Montmorency','MI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MI,Muskegon','Muskegon','MI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MI,Newaygo','Newaygo','MI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MI,Oakland','Oakland','MI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MI,Oceana','Oceana','MI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MI,Ogemaw','Ogemaw','MI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MI,Ontonagon','Ontonagon','MI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MI,Osceola','Osceola','MI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MI,Oscoda','Oscoda','MI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MI,Otsego','Otsego','MI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MI,Ottawa','Ottawa','MI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MI,Presque Isle','Presque Isle','MI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MI,Roscommon','Roscommon','MI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MI,Saginaw','Saginaw','MI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MI,Sanilac','Sanilac','MI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MI,Schoolcraft','Schoolcraft','MI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MI,Shiawassee','Shiawassee','MI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MI,St. Clair','St. Clair','MI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MI,St. Joseph','St. Joseph','MI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MI,Tuscola','Tuscola','MI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MI,Van Buren','Van Buren','MI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MI,Washtenaw','Washtenaw','MI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MI,Wayne','Wayne','MI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MI,Wexford','Wexford','MI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MN,Aitkin','Aitkin','MN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MN,Anoka','Anoka','MN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MN,Becker','Becker','MN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MN,Beltrami','Beltrami','MN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MN,Benton','Benton','MN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MN,Big Stone','Big Stone','MN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MN,Blue Earth','Blue Earth','MN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MN,Brown','Brown','MN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MN,Carlton','Carlton','MN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MN,Carver','Carver','MN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MN,Cass','Cass','MN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MN,Chippewa','Chippewa','MN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MN,Chisago','Chisago','MN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MN,Clay','Clay','MN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MN,Clearwater','Clearwater','MN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MN,Cook','Cook','MN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MN,Cottonwood','Cottonwood','MN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MN,Crow Wing','Crow Wing','MN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MN,Dakota','Dakota','MN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MN,Dodge','Dodge','MN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MN,Douglas','Douglas','MN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MN,Faribault','Faribault','MN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MN,Fillmore','Fillmore','MN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MN,Freeborn','Freeborn','MN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MN,Goodhue','Goodhue','MN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MN,Grant','Grant','MN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MN,Hennepin','Hennepin','MN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MN,Houston','Houston','MN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MN,Hubbard','Hubbard','MN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MN,Isanti','Isanti','MN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MN,Itasca','Itasca','MN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MN,Jackson','Jackson','MN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MN,Kanabec','Kanabec','MN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MN,Kandiyohi','Kandiyohi','MN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MN,Kittson','Kittson','MN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MN,Koochiching','Koochiching','MN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MN,Lac qui Parle','Lac qui Parle','MN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MN,Lake of the Woods','Lake of the Woods','MN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MN,Lake','Lake','MN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MN,Le Sueur','Le Sueur','MN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MN,Lincoln','Lincoln','MN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MN,Lyon','Lyon','MN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MN,Mahnomen','Mahnomen','MN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MN,Marshall','Marshall','MN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MN,Martin','Martin','MN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MN,McLeod','McLeod','MN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MN,Meeker','Meeker','MN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MN,Mille Lacs','Mille Lacs','MN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MN,Morrison','Morrison','MN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MN,Mower','Mower','MN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MN,Murray','Murray','MN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MN,Nicollet','Nicollet','MN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MN,Nobles','Nobles','MN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MN,Norman','Norman','MN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MN,Olmsted','Olmsted','MN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MN,Otter Tail','Otter Tail','MN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MN,Pennington','Pennington','MN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MN,Pine','Pine','MN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MN,Pipestone','Pipestone','MN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MN,Polk','Polk','MN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MN,Pope','Pope','MN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MN,Ramsey','Ramsey','MN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MN,Red Lake','Red Lake','MN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MN,Redwood','Redwood','MN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MN,Renville','Renville','MN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MN,Rice','Rice','MN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MN,Rock','Rock','MN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MN,Roseau','Roseau','MN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MN,Scott','Scott','MN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MN,Sherburne','Sherburne','MN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MN,Sibley','Sibley','MN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MN,St. Louis','St. Louis','MN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MN,Stearns','Stearns','MN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MN,Steele','Steele','MN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MN,Stevens','Stevens','MN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MN,Swift','Swift','MN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MN,Todd','Todd','MN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MN,Traverse','Traverse','MN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MN,Wabasha','Wabasha','MN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MN,Wadena','Wadena','MN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MN,Waseca','Waseca','MN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MN,Washington','Washington','MN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MN,Watonwan','Watonwan','MN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MN,Wilkin','Wilkin','MN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MN,Winona','Winona','MN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MN,Wright','Wright','MN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MN,Yellow Medicine','Yellow Medicine','MN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MO,Adair','Adair','MO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MO,Andrew','Andrew','MO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MO,Atchison','Atchison','MO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MO,Audrain','Audrain','MO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MO,Barry','Barry','MO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MO,Barton','Barton','MO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MO,Bates','Bates','MO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MO,Benton','Benton','MO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MO,Bollinger','Bollinger','MO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MO,Boone','Boone','MO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MO,Buchanan','Buchanan','MO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MO,Butler','Butler','MO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MO,Caldwell','Caldwell','MO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MO,Callaway','Callaway','MO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MO,Camden','Camden','MO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MO,Cape Girardeau','Cape Girardeau','MO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MO,Carroll','Carroll','MO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MO,Carter','Carter','MO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MO,Cass','Cass','MO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MO,Cedar','Cedar','MO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MO,Chariton','Chariton','MO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MO,Christian','Christian','MO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MO,Clark','Clark','MO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MO,Clay','Clay','MO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MO,Clinton','Clinton','MO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MO,Cole','Cole','MO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MO,Cooper','Cooper','MO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MO,Crawford','Crawford','MO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MO,Dade','Dade','MO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MO,Dallas','Dallas','MO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MO,Daviess','Daviess','MO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MO,Dent','Dent','MO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MO,DeKalb','DeKalb','MO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MO,Douglas','Douglas','MO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MO,Dunklin','Dunklin','MO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MO,Franklin','Franklin','MO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MO,Gasconade','Gasconade','MO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MO,Gentry','Gentry','MO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MO,Greene','Greene','MO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MO,Grundy','Grundy','MO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MO,Harrison','Harrison','MO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MO,Henry','Henry','MO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MO,Hickory','Hickory','MO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MO,Holt','Holt','MO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MO,Howard','Howard','MO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MO,Howell','Howell','MO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MO,Iron','Iron','MO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MO,Jackson','Jackson','MO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MO,Jasper','Jasper','MO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MO,Jefferson','Jefferson','MO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MO,Johnson','Johnson','MO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MO,Knox','Knox','MO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MO,Laclede','Laclede','MO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MO,Lafayette','Lafayette','MO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MO,Lawrence','Lawrence','MO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MO,Lewis','Lewis','MO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MO,Lincoln','Lincoln','MO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MO,Linn','Linn','MO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MO,Livingston','Livingston','MO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MO,Macon','Macon','MO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MO,Madison','Madison','MO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MO,Maries','Maries','MO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MO,Marion','Marion','MO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MO,McDonald','McDonald','MO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MO,Mercer','Mercer','MO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MO,Miller','Miller','MO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MO,Mississippi','Mississippi','MO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MO,Moniteau','Moniteau','MO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MO,Monroe','Monroe','MO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MO,Montgomery','Montgomery','MO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MO,Morgan','Morgan','MO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MO,New Madrid','New Madrid','MO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MO,Newton','Newton','MO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MO,Nodaway','Nodaway','MO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MO,Oregon','Oregon','MO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MO,Osage','Osage','MO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MO,Ozark','Ozark','MO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MO,Pemiscot','Pemiscot','MO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MO,Perry','Perry','MO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MO,Pettis','Pettis','MO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MO,Phelps','Phelps','MO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MO,Pike','Pike','MO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MO,Platte','Platte','MO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MO,Polk','Polk','MO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MO,Pulaski','Pulaski','MO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MO,Putnam','Putnam','MO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MO,Ralls','Ralls','MO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MO,Randolph','Randolph','MO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MO,Ray','Ray','MO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MO,Reynolds','Reynolds','MO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MO,Ripley','Ripley','MO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MO,Saline','Saline','MO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MO,Schuyler','Schuyler','MO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MO,Scotland','Scotland','MO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MO,Scott','Scott','MO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MO,Shannon','Shannon','MO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MO,Shelby','Shelby','MO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MO,St. Charles','St. Charles','MO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MO,St. Clair','St. Clair','MO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MO,St. Francois','St. Francois','MO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MO,St. Louis City','St. Louis City','MO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MO,St. Louis','St. Louis','MO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MO,Ste. Genevieve','Ste. Genevieve','MO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MO,Stoddard','Stoddard','MO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MO,Stone','Stone','MO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MO,Sullivan','Sullivan','MO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MO,Taney','Taney','MO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MO,Texas','Texas','MO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MO,Vernon','Vernon','MO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MO,Warren','Warren','MO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MO,Washington','Washington','MO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MO,Wayne','Wayne','MO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MO,Webster','Webster','MO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MO,Worth','Worth','MO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MO,Wright','Wright','MO','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MS,Adams','Adams','MS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MS,Alcorn','Alcorn','MS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MS,Amite','Amite','MS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MS,Attala','Attala','MS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MS,Benton','Benton','MS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MS,Bolivar','Bolivar','MS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MS,Calhoun','Calhoun','MS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MS,Carroll','Carroll','MS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MS,Chickasaw','Chickasaw','MS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MS,Choctaw','Choctaw','MS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MS,Claiborne','Claiborne','MS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MS,Clarke','Clarke','MS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MS,Clay','Clay','MS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MS,Coahoma','Coahoma','MS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MS,Copiah','Copiah','MS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MS,Covington','Covington','MS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MS,DeSoto','DeSoto','MS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MS,Forrest','Forrest','MS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MS,Franklin','Franklin','MS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MS,George','George','MS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MS,Greene','Greene','MS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MS,Grenada','Grenada','MS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MS,Hancock','Hancock','MS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MS,Harrison','Harrison','MS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MS,Hinds','Hinds','MS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MS,Holmes','Holmes','MS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MS,Humphreys','Humphreys','MS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MS,Issaquena','Issaquena','MS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MS,Itawamba','Itawamba','MS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MS,Jackson','Jackson','MS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MS,Jasper','Jasper','MS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MS,Jefferson','Jefferson','MS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MS,Jefferson Davis','Jefferson Davis','MS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MS,Jones','Jones','MS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MS,Kemper','Kemper','MS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MS,Lafayette','Lafayette','MS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MS,Lamar','Lamar','MS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MS,Lauderdale','Lauderdale','MS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MS,Lawrence','Lawrence','MS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MS,Leake','Leake','MS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MS,Lee','Lee','MS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MS,Leflore','Leflore','MS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MS,Lincoln','Lincoln','MS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MS,Lowndes','Lowndes','MS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MS,Madison','Madison','MS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MS,Marion','Marion','MS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MS,Marshall','Marshall','MS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MS,Monroe','Monroe','MS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MS,Montgomery','Montgomery','MS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MS,Neshoba','Neshoba','MS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MS,Newton','Newton','MS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MS,Noxubee','Noxubee','MS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MS,Oktibbeha','Oktibbeha','MS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MS,Panola','Panola','MS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MS,Pearl River','Pearl River','MS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MS,Perry','Perry','MS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MS,Pike','Pike','MS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MS,Pontotoc','Pontotoc','MS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MS,Prentiss','Prentiss','MS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MS,Quitman','Quitman','MS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MS,Rankin','Rankin','MS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MS,Scott','Scott','MS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MS,Sharkey','Sharkey','MS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MS,Simpson','Simpson','MS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MS,Smith','Smith','MS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MS,Stone','Stone','MS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MS,Sunflower','Sunflower','MS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MS,Tallahatchie','Tallahatchie','MS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MS,Tate','Tate','MS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MS,Tippah','Tippah','MS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MS,Tishomingo','Tishomingo','MS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MS,Tunica','Tunica','MS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MS,Union','Union','MS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MS,Walthall','Walthall','MS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MS,Warren','Warren','MS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MS,Washington','Washington','MS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MS,Wayne','Wayne','MS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MS,Webster','Webster','MS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MS,Wilkinson','Wilkinson','MS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MS,Winston','Winston','MS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MS,Yalobusha','Yalobusha','MS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MS,Yazoo','Yazoo','MS','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MT,Beaverhead','Beaverhead','MT','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MT,Big Horn','Big Horn','MT','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MT,Blaine','Blaine','MT','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MT,Broadwater','Broadwater','MT','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MT,Carbon','Carbon','MT','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MT,Carter','Carter','MT','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MT,Cascade','Cascade','MT','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MT,Chouteau','Chouteau','MT','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MT,Custer','Custer','MT','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MT,Daniels','Daniels','MT','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MT,Dawson','Dawson','MT','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MT,Deer Lodge','Deer Lodge','MT','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MT,Fallon','Fallon','MT','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MT,Fergus','Fergus','MT','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MT,Flathead','Flathead','MT','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MT,Gallatin','Gallatin','MT','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MT,Garfield','Garfield','MT','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MT,Glacier','Glacier','MT','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MT,Golden Valley','Golden Valley','MT','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MT,Granite','Granite','MT','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MT,Hill','Hill','MT','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MT,Jefferson','Jefferson','MT','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MT,Judith Basin','Judith Basin','MT','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MT,Lake','Lake','MT','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MT,Lewis & Clark','Lewis & Clark','MT','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MT,Liberty','Liberty','MT','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MT,Lincoln','Lincoln','MT','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MT,Madison','Madison','MT','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MT,McCone','McCone','MT','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MT,Meagher','Meagher','MT','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MT,Mineral','Mineral','MT','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MT,Missoula','Missoula','MT','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MT,Musselshell','Musselshell','MT','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MT,Park','Park','MT','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MT,Petroleum','Petroleum','MT','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MT,Phillips','Phillips','MT','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MT,Pondera','Pondera','MT','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MT,Powder River','Powder River','MT','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MT,Powell','Powell','MT','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MT,Prairie','Prairie','MT','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MT,Ravalli','Ravalli','MT','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MT,Richland','Richland','MT','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MT,Roosevelt','Roosevelt','MT','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MT,Rosebud','Rosebud','MT','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MT,Sanders','Sanders','MT','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MT,Sheridan','Sheridan','MT','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MT,Silver Bow','Silver Bow','MT','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MT,Stillwater','Stillwater','MT','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MT,Sweet Grass','Sweet Grass','MT','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MT,Teton','Teton','MT','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MT,Toole','Toole','MT','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MT,Treasure','Treasure','MT','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MT,Valley','Valley','MT','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MT,Wheatland','Wheatland','MT','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MT,Wibaux','Wibaux','MT','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'MT,Yellowstone','Yellowstone','MT','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NC,Alamance','Alamance','NC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NC,Alexander','Alexander','NC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NC,Alleghany','Alleghany','NC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NC,Anson','Anson','NC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NC,Ashe','Ashe','NC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NC,Avery','Avery','NC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NC,Beaufort','Beaufort','NC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NC,Bertie','Bertie','NC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NC,Bladen','Bladen','NC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NC,Brunswick','Brunswick','NC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NC,Buncombe','Buncombe','NC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NC,Burke','Burke','NC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NC,Cabarrus','Cabarrus','NC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NC,Caldwell','Caldwell','NC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NC,Camden','Camden','NC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NC,Carteret','Carteret','NC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NC,Caswell','Caswell','NC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NC,Catawba','Catawba','NC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NC,Chatham','Chatham','NC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NC,Cherokee','Cherokee','NC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NC,Chowan','Chowan','NC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NC,Clay','Clay','NC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NC,Cleveland','Cleveland','NC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NC,Columbus','Columbus','NC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NC,Craven','Craven','NC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NC,Cumberland','Cumberland','NC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NC,Currituck','Currituck','NC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NC,Dare','Dare','NC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NC,Davidson','Davidson','NC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NC,Davie','Davie','NC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NC,Duplin','Duplin','NC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NC,Durham','Durham','NC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NC,Edgecombe','Edgecombe','NC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NC,Forsyth','Forsyth','NC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NC,Franklin','Franklin','NC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NC,Gaston','Gaston','NC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NC,Gates','Gates','NC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NC,Graham','Graham','NC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NC,Granville','Granville','NC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NC,Greene','Greene','NC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NC,Guilford','Guilford','NC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NC,Halifax','Halifax','NC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NC,Harnett','Harnett','NC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NC,Haywood','Haywood','NC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NC,Henderson','Henderson','NC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NC,Hertford','Hertford','NC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NC,Hoke','Hoke','NC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NC,Hyde','Hyde','NC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NC,Iredell','Iredell','NC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NC,Jackson','Jackson','NC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NC,Johnston','Johnston','NC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NC,Jones','Jones','NC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NC,Lee','Lee','NC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NC,Lenoir','Lenoir','NC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NC,Lincoln','Lincoln','NC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NC,Macon','Macon','NC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NC,Madison','Madison','NC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NC,Martin','Martin','NC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NC,McDowell','McDowell','NC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NC,Mecklenburg','Mecklenburg','NC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NC,Mitchell','Mitchell','NC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NC,Montgomery','Montgomery','NC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NC,Moore','Moore','NC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NC,Nash','Nash','NC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NC,New Hanover','New Hanover','NC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NC,Northampton','Northampton','NC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NC,Onslow','Onslow','NC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NC,Orange','Orange','NC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NC,Pamlico','Pamlico','NC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NC,Pasquotank','Pasquotank','NC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NC,Pender','Pender','NC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NC,Perquimans','Perquimans','NC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NC,Person','Person','NC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NC,Pitt','Pitt','NC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NC,Polk','Polk','NC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NC,Randolph','Randolph','NC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NC,Richmond','Richmond','NC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NC,Robeson','Robeson','NC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NC,Rockingham','Rockingham','NC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NC,Rowan','Rowan','NC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NC,Rutherford','Rutherford','NC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NC,Sampson','Sampson','NC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NC,Scotland','Scotland','NC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NC,Stanly','Stanly','NC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NC,Stokes','Stokes','NC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NC,Surry','Surry','NC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NC,Swain','Swain','NC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NC,Transylvania','Transylvania','NC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NC,Tyrrell','Tyrrell','NC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NC,Union','Union','NC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NC,Vance','Vance','NC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NC,Wake','Wake','NC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NC,Warren','Warren','NC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NC,Washington','Washington','NC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NC,Watauga','Watauga','NC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NC,Wayne','Wayne','NC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NC,Wilkes','Wilkes','NC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NC,Wilson','Wilson','NC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NC,Yadkin','Yadkin','NC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NC,Yancey','Yancey','NC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'ND,Adams','Adams','ND','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'ND,Barnes','Barnes','ND','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'ND,Benson','Benson','ND','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'ND,Billings','Billings','ND','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'ND,Bottineau','Bottineau','ND','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'ND,Bowman','Bowman','ND','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'ND,Burke','Burke','ND','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'ND,Burleigh','Burleigh','ND','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'ND,Cass','Cass','ND','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'ND,Cavalier','Cavalier','ND','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'ND,Dickey','Dickey','ND','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'ND,Divide','Divide','ND','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'ND,Dunn','Dunn','ND','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'ND,Eddy','Eddy','ND','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'ND,Emmons','Emmons','ND','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'ND,Foster','Foster','ND','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'ND,Golden Valley','Golden Valley','ND','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'ND,Grand Forks','Grand Forks','ND','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'ND,Grant','Grant','ND','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'ND,Griggs','Griggs','ND','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'ND,Hettinger','Hettinger','ND','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'ND,Kidder','Kidder','ND','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'ND,LaMoure','LaMoure','ND','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'ND,Logan','Logan','ND','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'ND,McHenry','McHenry','ND','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'ND,McIntosh','McIntosh','ND','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'ND,McKenzie','McKenzie','ND','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'ND,McLean','McLean','ND','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'ND,Mercer','Mercer','ND','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'ND,Morton','Morton','ND','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'ND,Mountrail','Mountrail','ND','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'ND,Nelson','Nelson','ND','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'ND,Oliver','Oliver','ND','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'ND,Pembina','Pembina','ND','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'ND,Pierce','Pierce','ND','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'ND,Ramsey','Ramsey','ND','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'ND,Ransom','Ransom','ND','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'ND,Renville','Renville','ND','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'ND,Richland','Richland','ND','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'ND,Rolette','Rolette','ND','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'ND,Sargent','Sargent','ND','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'ND,Sheridan','Sheridan','ND','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'ND,Sioux','Sioux','ND','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'ND,Slope','Slope','ND','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'ND,Stark','Stark','ND','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'ND,Steele','Steele','ND','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'ND,Stutsman','Stutsman','ND','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'ND,Towner','Towner','ND','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'ND,Traill','Traill','ND','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'ND,Walsh','Walsh','ND','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'ND,Ward','Ward','ND','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'ND,Wells','Wells','ND','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'ND,Williams','Williams','ND','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NE,Adams','Adams','NE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NE,Antelope','Antelope','NE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NE,Arthur','Arthur','NE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NE,Banner','Banner','NE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NE,Blaine','Blaine','NE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NE,Boone','Boone','NE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NE,Box Butte','Box Butte','NE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NE,Boyd','Boyd','NE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NE,Brown','Brown','NE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NE,Buffalo','Buffalo','NE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NE,Burt','Burt','NE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NE,Butler','Butler','NE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NE,Cass','Cass','NE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NE,Cedar','Cedar','NE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NE,Chase','Chase','NE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NE,Cherry','Cherry','NE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NE,Cheyenne','Cheyenne','NE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NE,Clay','Clay','NE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NE,Colfax','Colfax','NE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NE,Cuming','Cuming','NE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NE,Custer','Custer','NE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NE,Dakota','Dakota','NE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NE,Dawes','Dawes','NE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NE,Dawson','Dawson','NE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NE,Deuel','Deuel','NE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NE,Dixon','Dixon','NE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NE,Dodge','Dodge','NE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NE,Douglas','Douglas','NE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NE,Dundy','Dundy','NE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NE,Fillmore','Fillmore','NE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NE,Franklin','Franklin','NE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NE,Frontier','Frontier','NE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NE,Furnas','Furnas','NE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NE,Gage','Gage','NE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NE,Garden','Garden','NE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NE,Garfield','Garfield','NE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NE,Gosper','Gosper','NE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NE,Grant','Grant','NE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NE,Greeley','Greeley','NE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NE,Hall','Hall','NE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NE,Hamilton','Hamilton','NE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NE,Harlan','Harlan','NE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NE,Hayes','Hayes','NE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NE,Hitchcock','Hitchcock','NE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NE,Holt','Holt','NE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NE,Hooker','Hooker','NE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NE,Howard','Howard','NE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NE,Jefferson','Jefferson','NE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NE,Johnson','Johnson','NE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NE,Kearney','Kearney','NE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NE,Keith','Keith','NE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NE,Keya Paha','Keya Paha','NE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NE,Kimball','Kimball','NE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NE,Knox','Knox','NE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NE,Lancaster','Lancaster','NE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NE,Lincoln','Lincoln','NE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NE,Logan','Logan','NE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NE,Loup','Loup','NE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NE,Madison','Madison','NE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NE,McPherson','McPherson','NE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NE,Merrick','Merrick','NE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NE,Morrill','Morrill','NE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NE,Nance','Nance','NE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NE,Nemaha','Nemaha','NE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NE,Nuckolls','Nuckolls','NE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NE,Otoe','Otoe','NE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NE,Pawnee','Pawnee','NE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NE,Perkins','Perkins','NE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NE,Phelps','Phelps','NE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NE,Pierce','Pierce','NE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NE,Platte','Platte','NE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NE,Polk','Polk','NE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NE,Red Willow','Red Willow','NE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NE,Richardson','Richardson','NE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NE,Rock','Rock','NE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NE,Saline','Saline','NE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NE,Sarpy','Sarpy','NE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NE,Saunders','Saunders','NE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NE,Scotts Bluff','Scotts Bluff','NE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NE,Seward','Seward','NE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NE,Sheridan','Sheridan','NE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NE,Sherman','Sherman','NE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NE,Sioux','Sioux','NE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NE,Stanton','Stanton','NE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NE,Thayer','Thayer','NE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NE,Thomas','Thomas','NE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NE,Thurston','Thurston','NE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NE,Valley','Valley','NE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NE,Washington','Washington','NE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NE,Wayne','Wayne','NE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NE,Webster','Webster','NE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NE,Wheeler','Wheeler','NE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NE,York','York','NE','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NH,Belknap','Belknap','NH','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NH,Carroll','Carroll','NH','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NH,Cheshire','Cheshire','NH','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NH,Coos','Coos','NH','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NH,Grafton','Grafton','NH','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NH,Hillsborough','Hillsborough','NH','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NH,Merrimack','Merrimack','NH','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NH,Rockingham','Rockingham','NH','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NH,Strafford','Strafford','NH','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NH,Sullivan','Sullivan','NH','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NM,Cibola','Cibola','NM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NM,Bernalillo','Bernalillo','NM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NM,Catron','Catron','NM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NM,Chaves','Chaves','NM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NM,Colfax','Colfax','NM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NM,Curry','Curry','NM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NM,De Baca','De Baca','NM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NM,Dona Ana','Dona Ana','NM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NM,Eddy','Eddy','NM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NM,Grant','Grant','NM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NM,Guadalupe','Guadalupe','NM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NM,Harding','Harding','NM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NM,Hidalgo','Hidalgo','NM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NM,Lea','Lea','NM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NM,Lincoln','Lincoln','NM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NM,Los Alamos','Los Alamos','NM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NM,Luna','Luna','NM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NM,McKinley','McKinley','NM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NM,Mora','Mora','NM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NM,Otero','Otero','NM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NM,Quay','Quay','NM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NM,Rio Arriba','Rio Arriba','NM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NM,Roosevelt','Roosevelt','NM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NM,San Juan','San Juan','NM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NM,San Miguel','San Miguel','NM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NM,Sandoval','Sandoval','NM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NM,Santa Fe','Santa Fe','NM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NM,Sierra','Sierra','NM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NM,Socorro','Socorro','NM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NM,Taos','Taos','NM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NM,Torrance','Torrance','NM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NM,Union','Union','NM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NM,Valencia','Valencia','NM','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NV,Churchill','Churchill','NV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NV,Clark','Clark','NV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NV,Douglas','Douglas','NV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NV,Elko','Elko','NV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NV,Esmeralda','Esmeralda','NV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NV,Eureka','Eureka','NV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NV,Humboldt','Humboldt','NV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NV,Lander','Lander','NV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NV,Lincoln','Lincoln','NV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NV,Lyon','Lyon','NV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NV,Mineral','Mineral','NV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NV,Nye','Nye','NV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NV,Pershing','Pershing','NV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NV,Storey','Storey','NV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NV,Washoe','Washoe','NV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'NV,White Pine','White Pine','NV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OH,Adams','Adams','OH','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OH,Allen','Allen','OH','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OH,Ashland','Ashland','OH','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OH,Ashtabula','Ashtabula','OH','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OH,Athens','Athens','OH','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OH,Auglaize','Auglaize','OH','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OH,Belmont','Belmont','OH','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OH,Brown','Brown','OH','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OH,Butler','Butler','OH','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OH,Carroll','Carroll','OH','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OH,Champaign','Champaign','OH','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OH,Clark','Clark','OH','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OH,Clermont','Clermont','OH','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OH,Clinton','Clinton','OH','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OH,Columbiana','Columbiana','OH','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OH,Coshocton','Coshocton','OH','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OH,Crawford','Crawford','OH','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OH,Cuyahoga','Cuyahoga','OH','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OH,Darke','Darke','OH','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OH,Defiance','Defiance','OH','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OH,Delaware','Delaware','OH','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OH,Erie','Erie','OH','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OH,Fairfield','Fairfield','OH','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OH,Fayette','Fayette','OH','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OH,Franklin','Franklin','OH','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OH,Fulton','Fulton','OH','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OH,Gallia','Gallia','OH','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OH,Geauga','Geauga','OH','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OH,Greene','Greene','OH','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OH,Guernsey','Guernsey','OH','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OH,Hamilton','Hamilton','OH','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OH,Hancock','Hancock','OH','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OH,Hardin','Hardin','OH','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OH,Harrison','Harrison','OH','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OH,Henry','Henry','OH','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OH,Highland','Highland','OH','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OH,Hocking','Hocking','OH','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OH,Holmes','Holmes','OH','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OH,Huron','Huron','OH','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OH,Jackson','Jackson','OH','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OH,Jefferson','Jefferson','OH','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OH,Knox','Knox','OH','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OH,Lake','Lake','OH','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OH,Lawrence','Lawrence','OH','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OH,Licking','Licking','OH','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OH,Logan','Logan','OH','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OH,Lorain','Lorain','OH','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OH,Lucas','Lucas','OH','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OH,Madison','Madison','OH','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OH,Mahoning','Mahoning','OH','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OH,Marion','Marion','OH','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OH,Medina','Medina','OH','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OH,Meigs','Meigs','OH','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OH,Mercer','Mercer','OH','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OH,Miami','Miami','OH','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OH,Monroe','Monroe','OH','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OH,Montgomery','Montgomery','OH','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OH,Morgan','Morgan','OH','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OH,Morrow','Morrow','OH','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OH,Muskingum','Muskingum','OH','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OH,Noble','Noble','OH','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OH,Ottawa','Ottawa','OH','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OH,Paulding','Paulding','OH','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OH,Perry','Perry','OH','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OH,Pickaway','Pickaway','OH','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OH,Pike','Pike','OH','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OH,Portage','Portage','OH','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OH,Preble','Preble','OH','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OH,Putnam','Putnam','OH','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OH,Richland','Richland','OH','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OH,Ross','Ross','OH','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OH,Sandusky','Sandusky','OH','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OH,Scioto','Scioto','OH','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OH,Seneca','Seneca','OH','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OH,Shelby','Shelby','OH','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OH,Stark','Stark','OH','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OH,Summit','Summit','OH','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OH,Trumbull','Trumbull','OH','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OH,Tuscarawas','Tuscarawas','OH','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OH,Union','Union','OH','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OH,Van Wert','Van Wert','OH','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OH,Vinton','Vinton','OH','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OH,Warren','Warren','OH','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OH,Washington','Washington','OH','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OH,Wayne','Wayne','OH','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OH,Williams','Williams','OH','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OH,Wood','Wood','OH','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OH,Wyandot','Wyandot','OH','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OK,Adair','Adair','OK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OK,Alfalfa','Alfalfa','OK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OK,Atoka','Atoka','OK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OK,Beaver','Beaver','OK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OK,Beckham','Beckham','OK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OK,Blaine','Blaine','OK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OK,Bryan','Bryan','OK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OK,Caddo','Caddo','OK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OK,Canadian','Canadian','OK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OK,Carter','Carter','OK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OK,Cherokee','Cherokee','OK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OK,Choctaw','Choctaw','OK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OK,Cimarron','Cimarron','OK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OK,Cleveland','Cleveland','OK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OK,Coal','Coal','OK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OK,Comanche','Comanche','OK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OK,Cotton','Cotton','OK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OK,Craig','Craig','OK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OK,Creek','Creek','OK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OK,Custer','Custer','OK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OK,Delaware','Delaware','OK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OK,Dewey','Dewey','OK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OK,Ellis','Ellis','OK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OK,Garfield','Garfield','OK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OK,Garvin','Garvin','OK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OK,Grady','Grady','OK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OK,Grant','Grant','OK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OK,Greer','Greer','OK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OK,Harmon','Harmon','OK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OK,Harper','Harper','OK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OK,Haskell','Haskell','OK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OK,Hughes','Hughes','OK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OK,Jackson','Jackson','OK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OK,Jefferson','Jefferson','OK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OK,Johnston','Johnston','OK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OK,Kay','Kay','OK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OK,Kingfisher','Kingfisher','OK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OK,Kiowa','Kiowa','OK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OK,Latimer','Latimer','OK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OK,Le Flore','Le Flore','OK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OK,Lincoln','Lincoln','OK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OK,Logan','Logan','OK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OK,Love','Love','OK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OK,Major','Major','OK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OK,Marshall','Marshall','OK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OK,Mayes','Mayes','OK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OK,McClain','McClain','OK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OK,McCurtain','McCurtain','OK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OK,McIntosh','McIntosh','OK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OK,Murray','Murray','OK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OK,Muskogee','Muskogee','OK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OK,Noble','Noble','OK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OK,Nowata','Nowata','OK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OK,Okfuskee','Okfuskee','OK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OK,Oklahoma','Oklahoma','OK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OK,Okmulgee','Okmulgee','OK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OK,Osage','Osage','OK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OK,Ottawa','Ottawa','OK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OK,Pawnee','Pawnee','OK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OK,Payne','Payne','OK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OK,Pittsburg','Pittsburg','OK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OK,Pontotoc','Pontotoc','OK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OK,Pottawatomie','Pottawatomie','OK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OK,Pushmataha','Pushmataha','OK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OK,Roger Mills','Roger Mills','OK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OK,Rogers','Rogers','OK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OK,Seminole','Seminole','OK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OK,Sequoyah','Sequoyah','OK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OK,Stephens','Stephens','OK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OK,Texas','Texas','OK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OK,Tillman','Tillman','OK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OK,Tulsa','Tulsa','OK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OK,Wagoner','Wagoner','OK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OK,Washington','Washington','OK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OK,Washita','Washita','OK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OK,Woods','Woods','OK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OK,Woodward','Woodward','OK','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OR,Baker','Baker','OR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OR,Benton','Benton','OR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OR,Clackamas','Clackamas','OR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OR,Clatsop','Clatsop','OR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OR,Columbia','Columbia','OR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OR,Coos','Coos','OR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OR,Crook','Crook','OR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OR,Curry','Curry','OR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OR,Deschutes','Deschutes','OR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OR,Douglas','Douglas','OR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OR,Gilliam','Gilliam','OR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OR,Grant','Grant','OR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OR,Harney','Harney','OR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OR,Hood River','Hood River','OR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OR,Jackson','Jackson','OR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OR,Jefferson','Jefferson','OR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OR,Josephine','Josephine','OR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OR,Klamath','Klamath','OR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OR,Lake','Lake','OR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OR,Lane','Lane','OR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OR,Lincoln','Lincoln','OR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OR,Linn','Linn','OR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OR,Malheur','Malheur','OR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OR,Marion','Marion','OR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OR,Morrow','Morrow','OR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OR,Multnomah','Multnomah','OR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OR,Polk','Polk','OR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OR,Sherman','Sherman','OR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OR,Tillamook','Tillamook','OR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OR,Umatilla','Umatilla','OR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OR,Union','Union','OR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OR,Wallowa','Wallowa','OR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OR,Wasco','Wasco','OR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OR,Washington','Washington','OR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OR,Wheeler','Wheeler','OR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'OR,Yamhill','Yamhill','OR','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'RI,Bristol','Bristol','RI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'RI,Kent','Kent','RI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'RI,Newport','Newport','RI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'RI,Providence','Providence','RI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'RI,Washington','Washington','RI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'SC,Abbeville','Abbeville','SC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'SC,Aiken','Aiken','SC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'SC,Allendale','Allendale','SC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'SC,Anderson','Anderson','SC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'SC,Bamberg','Bamberg','SC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'SC,Barnwell','Barnwell','SC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'SC,Beaufort','Beaufort','SC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'SC,Berkeley','Berkeley','SC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'SC,Calhoun','Calhoun','SC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'SC,Charleston','Charleston','SC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'SC,Cherokee','Cherokee','SC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'SC,Chester','Chester','SC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'SC,Chesterfield','Chesterfield','SC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'SC,Clarendon','Clarendon','SC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'SC,Colleton','Colleton','SC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'SC,Darlington','Darlington','SC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'SC,Dillon','Dillon','SC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'SC,Dorchester','Dorchester','SC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'SC,Edgefield','Edgefield','SC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'SC,Fairfield','Fairfield','SC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'SC,Florence','Florence','SC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'SC,Georgetown','Georgetown','SC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'SC,Greenville','Greenville','SC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'SC,Greenwood','Greenwood','SC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'SC,Hampton','Hampton','SC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'SC,Horry','Horry','SC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'SC,Jasper','Jasper','SC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'SC,Kershaw','Kershaw','SC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'SC,Lancaster','Lancaster','SC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'SC,Laurens','Laurens','SC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'SC,Lee','Lee','SC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'SC,Lexington','Lexington','SC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'SC,Marion','Marion','SC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'SC,Marlboro','Marlboro','SC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'SC,McCormick','McCormick','SC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'SC,Newberry','Newberry','SC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'SC,Oconee','Oconee','SC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'SC,Orangeburg','Orangeburg','SC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'SC,Pickens','Pickens','SC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'SC,Richland','Richland','SC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'SC,Saluda','Saluda','SC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'SC,Spartanburg','Spartanburg','SC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'SC,Sumter','Sumter','SC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'SC,Union','Union','SC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'SC,Williamsburg','Williamsburg','SC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'SC,York','York','SC','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'SD,Aurora','Aurora','SD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'SD,Beadle','Beadle','SD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'SD,Bennett','Bennett','SD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'SD,Bon Homme','Bon Homme','SD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'SD,Brookings','Brookings','SD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'SD,Brown','Brown','SD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'SD,Brule','Brule','SD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'SD,Buffalo','Buffalo','SD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'SD,Butte','Butte','SD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'SD,Campbell','Campbell','SD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'SD,Charles Mix','Charles Mix','SD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'SD,Clark','Clark','SD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'SD,Clay','Clay','SD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'SD,Codington','Codington','SD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'SD,Corson','Corson','SD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'SD,Custer','Custer','SD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'SD,Davison','Davison','SD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'SD,Day','Day','SD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'SD,Deuel','Deuel','SD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'SD,Dewey','Dewey','SD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'SD,Douglas','Douglas','SD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'SD,Edmunds','Edmunds','SD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'SD,Fall River','Fall River','SD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'SD,Faulk','Faulk','SD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'SD,Grant','Grant','SD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'SD,Gregory','Gregory','SD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'SD,Haakon','Haakon','SD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'SD,Hamlin','Hamlin','SD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'SD,Hand','Hand','SD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'SD,Hanson','Hanson','SD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'SD,Harding','Harding','SD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'SD,Hughes','Hughes','SD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'SD,Hutchinson','Hutchinson','SD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'SD,Hyde','Hyde','SD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'SD,Jackson','Jackson','SD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'SD,Jerauld','Jerauld','SD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'SD,Jones','Jones','SD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'SD,Kingsbury','Kingsbury','SD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'SD,Lake','Lake','SD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'SD,Lawrence','Lawrence','SD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'SD,Lincoln','Lincoln','SD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'SD,Lyman','Lyman','SD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'SD,Marshall','Marshall','SD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'SD,McCook','McCook','SD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'SD,McPherson','McPherson','SD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'SD,Meade','Meade','SD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'SD,Mellette','Mellette','SD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'SD,Miner','Miner','SD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'SD,Minnehaha','Minnehaha','SD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'SD,Moody','Moody','SD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'SD,Pennington','Pennington','SD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'SD,Perkins','Perkins','SD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'SD,Potter','Potter','SD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'SD,Roberts','Roberts','SD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'SD,Sanborn','Sanborn','SD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'SD,Spink','Spink','SD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'SD,Stanley','Stanley','SD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'SD,Sully','Sully','SD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'SD,Todd','Todd','SD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'SD,Tripp','Tripp','SD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'SD,Turner','Turner','SD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'SD,Union','Union','SD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'SD,Walworth','Walworth','SD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'SD,Yankton','Yankton','SD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'SD,Ziebach','Ziebach','SD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'SD,Oglala Lakota','Oglala Lakota','SD','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TN,Anderson','Anderson','TN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TN,Bedford','Bedford','TN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TN,Benton','Benton','TN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TN,Bledsoe','Bledsoe','TN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TN,Blount','Blount','TN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TN,Bradley','Bradley','TN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TN,Campbell','Campbell','TN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TN,Cannon','Cannon','TN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TN,Carroll','Carroll','TN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TN,Carter','Carter','TN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TN,Cheatham','Cheatham','TN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TN,Chester','Chester','TN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TN,Claiborne','Claiborne','TN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TN,Clay','Clay','TN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TN,Cocke','Cocke','TN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TN,Coffee','Coffee','TN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TN,Crockett','Crockett','TN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TN,Cumberland','Cumberland','TN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TN,Davidson','Davidson','TN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TN,Decatur','Decatur','TN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TN,DeKalb','DeKalb','TN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TN,Dickson','Dickson','TN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TN,Dyer','Dyer','TN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TN,Fayette','Fayette','TN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TN,Fentress','Fentress','TN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TN,Franklin','Franklin','TN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TN,Gibson','Gibson','TN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TN,Giles','Giles','TN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TN,Grainger','Grainger','TN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TN,Greene','Greene','TN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TN,Grundy','Grundy','TN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TN,Hamblen','Hamblen','TN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TN,Hamilton','Hamilton','TN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TN,Hancock','Hancock','TN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TN,Hardeman','Hardeman','TN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TN,Hardin','Hardin','TN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TN,Hawkins','Hawkins','TN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TN,Haywood','Haywood','TN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TN,Henderson','Henderson','TN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TN,Henry','Henry','TN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TN,Hickman','Hickman','TN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TN,Houston','Houston','TN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TN,Humphreys','Humphreys','TN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TN,Jackson','Jackson','TN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TN,Jefferson','Jefferson','TN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TN,Johnson','Johnson','TN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TN,Knox','Knox','TN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TN,Lake','Lake','TN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TN,Lauderdale','Lauderdale','TN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TN,Lawrence','Lawrence','TN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TN,Lewis','Lewis','TN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TN,Lincoln','Lincoln','TN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TN,Loudon','Loudon','TN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TN,Macon','Macon','TN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TN,Madison','Madison','TN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TN,Marion','Marion','TN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TN,Marshall','Marshall','TN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TN,Maury','Maury','TN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TN,McMinn','McMinn','TN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TN,McNairy','McNairy','TN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TN,Meigs','Meigs','TN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TN,Monroe','Monroe','TN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TN,Montgomery','Montgomery','TN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TN,Moore','Moore','TN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TN,Morgan','Morgan','TN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TN,Obion','Obion','TN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TN,Overton','Overton','TN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TN,Perry','Perry','TN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TN,Pickett','Pickett','TN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TN,Polk','Polk','TN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TN,Putnam','Putnam','TN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TN,Rhea','Rhea','TN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TN,Roane','Roane','TN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TN,Robertson','Robertson','TN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TN,Rutherford','Rutherford','TN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TN,Scott','Scott','TN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TN,Sequatchie','Sequatchie','TN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TN,Sevier','Sevier','TN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TN,Shelby','Shelby','TN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TN,Smith','Smith','TN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TN,Stewart','Stewart','TN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TN,Sullivan','Sullivan','TN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TN,Sumner','Sumner','TN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TN,Tipton','Tipton','TN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TN,Trousdale','Trousdale','TN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TN,Unicoi','Unicoi','TN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TN,Union','Union','TN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TN,Van Buren','Van Buren','TN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TN,Warren','Warren','TN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TN,Washington','Washington','TN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TN,Wayne','Wayne','TN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TN,Weakley','Weakley','TN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TN,White','White','TN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TN,Williamson','Williamson','TN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'TN,Wilson','Wilson','TN','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'UT,Beaver','Beaver','UT','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'UT,Box Elder','Box Elder','UT','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'UT,Cache','Cache','UT','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'UT,Carbon','Carbon','UT','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'UT,Daggett','Daggett','UT','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'UT,Davis','Davis','UT','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'UT,Duchesne','Duchesne','UT','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'UT,Emery','Emery','UT','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'UT,Garfield','Garfield','UT','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'UT,Grand','Grand','UT','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'UT,Iron','Iron','UT','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'UT,Juab','Juab','UT','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'UT,Kane','Kane','UT','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'UT,Millard','Millard','UT','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'UT,Morgan','Morgan','UT','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'UT,Piute','Piute','UT','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'UT,Rich','Rich','UT','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'UT,Salt Lake','Salt Lake','UT','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'UT,San Juan','San Juan','UT','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'UT,Sanpete','Sanpete','UT','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'UT,Sevier','Sevier','UT','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'UT,Summit','Summit','UT','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'UT,Tooele','Tooele','UT','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'UT,Uintah','Uintah','UT','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'UT,Utah','Utah','UT','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'UT,Wasatch','Wasatch','UT','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'UT,Washington','Washington','UT','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'UT,Wayne','Wayne','UT','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'UT,Weber','Weber','UT','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'VA,Accomack','Accomack','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'VA,Albemarle','Albemarle','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'VA,Alleghany','Alleghany','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'VA,Amelia','Amelia','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'VA,Amherst','Amherst','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'VA,Appomattox','Appomattox','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'VA,Arlington','Arlington','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'VA,Augusta','Augusta','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'VA,Bath','Bath','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'VA,Bedford','Bedford','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'VA,Bland','Bland','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'VA,Botetourt','Botetourt','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'VA,Brunswick','Brunswick','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'VA,Buchanan','Buchanan','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'VA,Buckingham','Buckingham','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'VA,Campbell','Campbell','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'VA,Caroline','Caroline','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'VA,Carroll','Carroll','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'VA,Charles City','Charles City','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'VA,Charlotte','Charlotte','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'VA,Chesterfield','Chesterfield','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'VA,Clarke','Clarke','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'VA,Craig','Craig','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'VA,Culpeper','Culpeper','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'VA,Cumberland','Cumberland','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'VA,Dickenson','Dickenson','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'VA,Dinwiddie','Dinwiddie','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'VA,Essex','Essex','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'VA,Fairfax','Fairfax','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'VA,Fauquier','Fauquier','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'VA,Floyd','Floyd','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'VA,Fluvanna','Fluvanna','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'VA,Franklin','Franklin','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'VA,Frederick','Frederick','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'VA,Giles','Giles','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'VA,Gloucester','Gloucester','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'VA,Goochland','Goochland','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'VA,Grayson','Grayson','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'VA,Greene','Greene','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'VA,Greensville','Greensville','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'VA,Halifax','Halifax','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'VA,Hanover','Hanover','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'VA,Henrico','Henrico','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'VA,Henry','Henry','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'VA,Highland','Highland','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'VA,Isle of Wight','Isle of Wight','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'VA,James City','James City','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'VA,King and Queen','King and Queen','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'VA,King George','King George','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'VA,King William','King William','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'VA,Lancaster','Lancaster','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'VA,Lee','Lee','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'VA,Loudoun','Loudoun','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'VA,Louisa','Louisa','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'VA,Lunenburg','Lunenburg','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'VA,Madison','Madison','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'VA,Mathews','Mathews','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'VA,Mecklenburg','Mecklenburg','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'VA,Middlesex','Middlesex','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'VA,Montgomery','Montgomery','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'VA,Nelson','Nelson','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'VA,New Kent','New Kent','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'VA,Northampton','Northampton','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'VA,Northumberland','Northumberland','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'VA,Nottoway','Nottoway','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'VA,Orange','Orange','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'VA,Page','Page','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'VA,Patrick','Patrick','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'VA,Pittsylvania','Pittsylvania','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'VA,Powhatan','Powhatan','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'VA,Prince Edward','Prince Edward','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'VA,Prince George','Prince George','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'VA,Prince William','Prince William','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'VA,Pulaski','Pulaski','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'VA,Rappahannock','Rappahannock','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'VA,Richmond','Richmond','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'VA,Roanoke','Roanoke','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'VA,Rockbridge','Rockbridge','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'VA,Rockingham','Rockingham','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'VA,Russell','Russell','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'VA,Scott','Scott','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'VA,Shenandoah','Shenandoah','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'VA,Smyth','Smyth','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'VA,Southampton','Southampton','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'VA,Spotsylvania','Spotsylvania','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'VA,Stafford','Stafford','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'VA,Surry','Surry','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'VA,Sussex','Sussex','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'VA,Tazewell','Tazewell','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'VA,Warren','Warren','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'VA,Washington','Washington','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'VA,Westmoreland','Westmoreland','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'VA,Wise','Wise','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'VA,Wythe','Wythe','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'VA,York','York','VA','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'VT,Addison','Addison','VT','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'VT,Bennington','Bennington','VT','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'VT,Caledonia','Caledonia','VT','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'VT,Chittenden','Chittenden','VT','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'VT,Essex','Essex','VT','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'VT,Franklin','Franklin','VT','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'VT,Grand Isle','Grand Isle','VT','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'VT,Lamoille','Lamoille','VT','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'VT,Orange','Orange','VT','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'VT,Orleans','Orleans','VT','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'VT,Rutland','Rutland','VT','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'VT,Washington','Washington','VT','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'VT,Windham','Windham','VT','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'VT,Windsor','Windsor','VT','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'WI,Adams','Adams','WI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'WI,Ashland','Ashland','WI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'WI,Barron','Barron','WI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'WI,Bayfield','Bayfield','WI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'WI,Brown','Brown','WI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'WI,Buffalo','Buffalo','WI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'WI,Burnett','Burnett','WI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'WI,Calumet','Calumet','WI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'WI,Chippewa','Chippewa','WI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'WI,Clark','Clark','WI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'WI,Columbia','Columbia','WI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'WI,Crawford','Crawford','WI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'WI,Dane','Dane','WI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'WI,Dodge','Dodge','WI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'WI,Door','Door','WI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'WI,Douglas','Douglas','WI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'WI,Dunn','Dunn','WI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'WI,Eau Claire','Eau Claire','WI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'WI,Florence','Florence','WI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'WI,Fond du Lac','Fond du Lac','WI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'WI,Forest','Forest','WI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'WI,Grant','Grant','WI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'WI,Green','Green','WI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'WI,Green Lake','Green Lake','WI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'WI,Iowa','Iowa','WI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'WI,Iron','Iron','WI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'WI,Jackson','Jackson','WI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'WI,Jefferson','Jefferson','WI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'WI,Juneau','Juneau','WI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'WI,Kenosha','Kenosha','WI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'WI,Kewaunee','Kewaunee','WI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'WI,La Crosse','La Crosse','WI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'WI,Lafayette','Lafayette','WI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'WI,Langlade','Langlade','WI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'WI,Lincoln','Lincoln','WI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'WI,Manitowoc','Manitowoc','WI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'WI,Marathon','Marathon','WI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'WI,Marinette','Marinette','WI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'WI,Marquette','Marquette','WI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'WI,Menominee','Menominee','WI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'WI,Milwaukee','Milwaukee','WI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'WI,Monroe','Monroe','WI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'WI,Oconto','Oconto','WI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'WI,Oneida','Oneida','WI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'WI,Outagamie','Outagamie','WI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'WI,Ozaukee','Ozaukee','WI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'WI,Pepin','Pepin','WI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'WI,Pierce','Pierce','WI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'WI,Polk','Polk','WI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'WI,Portage','Portage','WI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'WI,Price','Price','WI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'WI,Racine','Racine','WI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'WI,Richland','Richland','WI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'WI,Rock','Rock','WI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'WI,Rusk','Rusk','WI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'WI,Sauk','Sauk','WI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'WI,Sawyer','Sawyer','WI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'WI,Shawano','Shawano','WI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'WI,Sheboygan','Sheboygan','WI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'WI,St. Croix','St. Croix','WI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'WI,Taylor','Taylor','WI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'WI,Trempealeau','Trempealeau','WI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'WI,Vernon','Vernon','WI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'WI,Vilas','Vilas','WI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'WI,Walworth','Walworth','WI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'WI,Washburn','Washburn','WI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'WI,Washington','Washington','WI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'WI,Waukesha','Waukesha','WI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'WI,Waupaca','Waupaca','WI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'WI,Waushara','Waushara','WI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'WI,Winnebago','Winnebago','WI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'WI,Wood','Wood','WI','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'WV,Barbour','Barbour','WV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'WV,Berkeley','Berkeley','WV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'WV,Boone','Boone','WV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'WV,Braxton','Braxton','WV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'WV,Brooke','Brooke','WV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'WV,Cabell','Cabell','WV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'WV,Calhoun','Calhoun','WV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'WV,Clay','Clay','WV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'WV,Doddridge','Doddridge','WV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'WV,Fayette','Fayette','WV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'WV,Gilmer','Gilmer','WV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'WV,Grant','Grant','WV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'WV,Greenbrier','Greenbrier','WV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'WV,Hampshire','Hampshire','WV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'WV,Hancock','Hancock','WV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'WV,Hardy','Hardy','WV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'WV,Harrison','Harrison','WV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'WV,Jackson','Jackson','WV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'WV,Jefferson','Jefferson','WV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'WV,Kanawha','Kanawha','WV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'WV,Lewis','Lewis','WV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'WV,Lincoln','Lincoln','WV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'WV,Logan','Logan','WV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'WV,Marion','Marion','WV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'WV,Marshall','Marshall','WV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'WV,Mason','Mason','WV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'WV,McDowell','McDowell','WV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'WV,Mercer','Mercer','WV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'WV,Mineral','Mineral','WV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'WV,Mingo','Mingo','WV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'WV,Monongalia','Monongalia','WV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'WV,Monroe','Monroe','WV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'WV,Morgan','Morgan','WV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'WV,Nicholas','Nicholas','WV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'WV,Ohio','Ohio','WV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'WV,Pendleton','Pendleton','WV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'WV,Pleasants','Pleasants','WV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'WV,Pocahontas','Pocahontas','WV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'WV,Preston','Preston','WV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'WV,Putnam','Putnam','WV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'WV,Raleigh','Raleigh','WV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'WV,Randolph','Randolph','WV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'WV,Ritchie','Ritchie','WV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'WV,Roane','Roane','WV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'WV,Summers','Summers','WV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'WV,Taylor','Taylor','WV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'WV,Tucker','Tucker','WV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'WV,Tyler','Tyler','WV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'WV,Upshur','Upshur','WV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'WV,Wayne','Wayne','WV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'WV,Webster','Webster','WV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'WV,Wetzel','Wetzel','WV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'WV,Wirt','Wirt','WV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'WV,Wood','Wood','WV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'WV,Wyoming','Wyoming','WV','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'WY,Albany','Albany','WY','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'WY,Big Horn','Big Horn','WY','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'WY,Campbell','Campbell','WY','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'WY,Carbon','Carbon','WY','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'WY,Converse','Converse','WY','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'WY,Crook','Crook','WY','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'WY,Fremont','Fremont','WY','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'WY,Goshen','Goshen','WY','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'WY,Hot Springs','Hot Springs','WY','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'WY,Johnson','Johnson','WY','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'WY,Laramie','Laramie','WY','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'WY,Lincoln','Lincoln','WY','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'WY,Natrona','Natrona','WY','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'WY,Niobrara','Niobrara','WY','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'WY,Park','Park','WY','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'WY,Platte','Platte','WY','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'WY,Sheridan','Sheridan','WY','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'WY,Sublette','Sublette','WY','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'WY,Sweetwater','Sweetwater','WY','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'WY,Teton','Teton','WY','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'WY,Uinta','Uinta','WY','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'WY,Washakie','Washakie','WY','');
INSERT INTO adif_enum_secondary_subdivision VALUES (291,'WY,Weston','Weston','WY','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'01079','Futami','#01','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'0104','Hakodate (city)','#01','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'0103','Otaru (city)','#01','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'0102','Asahikawa (city)','#01','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'0101','Sapporo (city)','#01','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'010106','Sapporo, Minami Ku','#01','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'01001','Akan','#01','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'01002','Ashoro','#01','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'01003','Atsukeshi','#01','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'01004','Atsuta (deleted)','#01','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'01005','Abashiri','#01','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'01006','Abuta(Shiribeshi)','#01','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'01007','Abuta(Iburi)','#01','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'01008','Ishikari','#01','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'01064','Horoizumi','#01','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'01072','Yubari','#01','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'01065','Horobetsu (deleted)','#01','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'01066','Mashike','#01','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'01067','Matsumae','#01','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'01068','Mitsuishi (deleted)','#01','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'01069','Menashi','#01','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'010103','Sapporo, Higashi Ku','#01','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'01071','Yamakoshi','#01','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'010104','Sapporo, Shiroishi Ku','#01','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'01073','Yufutsu(Iburi)','#01','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'01074','Yufutsu(Kamikawa)','#01','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'01075','Yoichi','#01','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'01076','Rishiri','#01','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'01077','Rumoi','#01','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'0107','Obihiro (city)','#01','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'01070','Mombetsu','#01','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'0130','Noboribetsu (city)','#01','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'0105','Muroran (city)','#01','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'0124','Chitose (city)','#01','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'0125','Takikawa (city)','#01','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'0126','Sunagawa (city)','#01','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'0127','Utashinai (city)','#01','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'0122','Mikasa (city)','#01','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'0129','Furano (city)','#01','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'0121','Nayoro (city)','#01','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'0131','Eniwa (city)','#01','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'0132','Kameda (city, deleted)','#01','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'0133','Date (city)','#01','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'0134','Kitahiroshima (city)','#01','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'0135','Ishikari (city)','#01','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'0136','Hokuto (city)','#01','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'0128','Fukagawa (city)','#01','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'0114','Wakkanai (city)','#01','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'01078','Rebun','#01','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'0108','Kitami (city)','#01','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'0109','Yubari (city)','#01','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'0110','Iwamizawa (city)','#01','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'0111','Abashiri (city)','#01','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'0123','Nemuro (city)','#01','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'0113','Tomakomai (city)','#01','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'0106','Kushiro (city)','#01','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'0115','Bibai (city)','#01','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'0116','Ashibetsu (city)','#01','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'0117','Ebetsu (city)','#01','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'0118','Akabira (city)','#01','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'0119','Mombetsu (city)','#01','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'0120','Shibetsu (city)','#01','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'0112','Rumoi (city)','#01','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'01029','Sapporo (deleted)','#01','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'01063','Furubira','#01','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'01021','Kamiiso','#01','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'01022','Kamikawa(Tokachi)','#01','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'01023','Kamikawa(Kamikawa)','#01','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'01024','Kameda','#01','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'01025','Kayabe','#01','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'01026','Kawakami','#01','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'01019','Kato','#01','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'01028','Kudo','#01','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'01018','Kasai','#01','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'01030','Samani','#01','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'01031','Saru','#01','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'01032','Shizunai (deleted)','#01','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'01033','Shibetsu','#01','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'01034','Shimamaki','#01','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'01080','Hidaka','#01','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'01035','Shakotan','#01','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'01027','Kushiro','#01','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'010101','Sapporo, Chuo Ku','#01','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'010107','Sapporo, Nishi Ku','#01','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'010108','Sapporo, Atsubetsu Ku','#01','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'010109','Sapporo, Teine Ku','#01','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'010105','Sapporo, Toyohira Ku','#01','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'010110','Sapporo, Kiyota Ku','#01','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'01081','Uryu(Kamikawa)','#01','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'01009','Isoya','#01','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'01020','Kabato','#01','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'010102','Sapporo, Kita Ku','#01','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'01036','Shari','#01','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'01011','Usu','#01','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'01012','Utasutsu (deleted)','#01','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'01013','Urakawa','#01','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'01014','Uryu(Sorachi)','#01','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'01015','Esashi','#01','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'01016','Okushiri','#01','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'01017','Oshoro (deleted)','#01','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'01010','Iwanai','#01','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'01056','Hanasaki (deleted)','#01','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'01052','Niikappu','#01','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'01047','Tokachi','#01','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'01062','Furuu','#01','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'01061','Futoro (deleted)','#01','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'01060','Hiroo','#01','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'01059','Hiyama','#01','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'01045','Teshio(Rumoi)','#01','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'01057','Hamamasu (deleted)','#01','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'01044','Chitose (deleted)','#01','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'01055','Notsuke','#01','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'01054','Nemuro (deleted)','#01','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'01053','Nishi','#01','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'01049','Tomamae','#01','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'01050','Nakagawa(Kamikawa)','#01','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'01051','Nakagawa(Tokachi)','#01','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'01058','Bikuni (deleted)','#01','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'01037','Shiraoi','#01','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'01043','Sorachi(Kamikawa)','#01','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'01042','Sorachi(Sorachi)','#01','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'01041','Soya','#01','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'01040','Setana','#01','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'01039','Suttsu','#01','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'01038','Shiranuka','#01','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'01048','Tokoro','#01','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'01046','Teshio(Soya)','#01','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'0210','Hirakawa (city)','#02','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'0209','Tsugaru (city)','#02','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'0208','Mutsu (city)','#02','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'0207','Misawa (city)','#02','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'0206','Towada (city)','#02','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'0205','Goshogawara (city)','#02','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'0204','Kuroishi (city)','#02','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'0202','Hirosaki (city)','#02','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'0201','Aomori (city)','#02','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'0203','Hachinohe (city)','#02','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'02008','Minamitsugaru','#02','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'02001','Kamikita','#02','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'02002','Kitatsugaru','#02','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'02003','Sannohe','#02','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'02004','Shimokita','#02','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'02005','Nakatsugaru','#02','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'02006','Nishitsugaru','#02','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'02007','Higashitsugaru','#02','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'0304','Ichinoseki (city)','#03','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'0313','Ninohe (city)','#03','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'0312','Esashi (city, deleted)','#03','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'0311','Rikuzentakata (city)','#03','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'0310','Tono (city)','#03','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'0309','Kuji (city)','#03','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'0308','Kitakami (city)','#03','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'0307','Hanamaki (city)','#03','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'0303','Miyako (city)','#03','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'0315','Oshu (city)','#03','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'03009','Nishiiwai','#03','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'0314','Hachimantai (city)','#03','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'0305','Ofunato (city)','#03','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'0306','Mizusawa (city, deleted)','#03','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'03007','Shimohei','#03','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'0302','Kamaishi (city)','#03','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'03010','Ninohe','#03','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'03012','Higashiiwai (deleted)','#03','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'03011','Hienuki (deleted)','#03','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'03008','Shiwa','#03','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'03006','Kesen','#03','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'03005','Kunohe','#03','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'03004','Kamihei','#03','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'03003','Esashi (deleted)','#03','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'03002','Iwate','#03','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'03001','Isawa','#03','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'0301','Morioka (city)','#03','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'03013','Waga','#03','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'0406','Oga (city)','#04','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'0414','Nikaho (city)','#04','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'0415','Senboku (city)','#04','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'0413','Kitaakita (city)','#04','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'0411','Katagami (city)','#04','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'0410','Yurihonjo (city)','#04','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'04001','Ogachi','#04','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'04009','Yuri (deleted)','#04','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'04008','Yamamoto','#04','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'04007','Minamiakita','#04','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'04006','Hiraka (deleted)','#04','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'04005','Semboku','#04','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'04004','Kitaakita','#04','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'0408','Omagari (city, deleted)','#04','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'04002','Kaduno','#04','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'0409','Kazuno (city)','#04','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'0401','Akita (city)','#04','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'0402','Noshiro (city)','#04','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'0403','Odate (city)','#04','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'0404','Yokote (city)','#04','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'0405','Honjo (city, deleted)','#04','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'0407','Yuzawa (city)','#04','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'04003','Kawabe (deleted)','#04','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'0412','Daisen (city)','#04','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'0506','Sagae (city)','#05','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'0501','Yamagata (city)','#05','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'0513','Nan''yo (city)','#05','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'0502','Yonezawa (city)','#05','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'0503','Tsuruoka (city)','#05','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'0505','Shinjo (city)','#05','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'05011','Mogami','#05','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'05010','Minamimurayama (deleted)','#05','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'05009','Minamiokitama (deleted)','#05','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'05008','Higashimurayama','#05','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'05007','Higashitagawa','#05','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'0508','Murayama (city)','#05','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'0511','Higashine (city)','#05','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'0504','Sakata (city)','#05','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'05006','Higashiokitama','#05','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'0509','Nagai (city)','#05','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'0507','Kaminoyama (city)','#05','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'0512','Obanazawa (city)','#05','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'05001','Akumi','#05','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'05002','Kitamurayama','#05','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'05003','Nishiokitama','#05','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'05004','Nishitagawa (deleted)','#05','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'05005','Nishimurayama','#05','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'0510','Tendo (city)','#05','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'0602','Ishinomaki (city)','#06','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'0609','Tagajo (city)','#06','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'0612','Tome (city)','#06','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'0611','Iwanuma (city)','#06','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'0610','Izumi (city, deleted)','#06','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'0608','Kakuda (city)','#06','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'0607','Natori (city)','#06','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'0606','Shiroishi (city)','#06','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'0605','Kesennuma (city)','#06','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'0603','Shiogama (city)','#06','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'0601','Sendai (city)','#06','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'06013','Miyagi','#06','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'0604','Furukawa (city, deleted)','#06','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'06007','Shida (deleted)','#06','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'0616','Tomiya (city)','#06','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'060101','Sendai, Aoba Ku','#06','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'060102','Sendai, Miyagino Ku','#06','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'060103','Sendai, Wakabayashi Ku','#06','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'060104','Sendai, Taihaku Ku','#06','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'060105','Sendai, Izumi Ku','#06','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'06001','Igu','#06','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'06002','Oshika','#06','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'06003','Katta','#06','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'06004','Kami','#06','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'06015','Monou (deleted)','#06','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'06006','Kurokawa','#06','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'0613','Kuruhara (city)','#06','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'06008','Shibata','#06','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'06009','Tamadukuri (deleted)','#06','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'06010','Toda','#06','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'06011','Tome (deleted)','#06','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'06012','Natori (deleted)','#06','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'06014','Motoyoshi','#06','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'06016','Watari','#06','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'0615','Osaki (city)','#06','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'0614','Higashimatsushima (city)','#06','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'06005','Kurihara (deleted)','#06','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'0716','Wakamatsu (city, deleted)','#07','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'0711','Soma (city)','#07','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'0719','Date (city)','#07','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'0718','Minamisoma (city)','#07','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'0717','Tamura (city)','#07','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'0715','Iwaki (city)','#07','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'0703','Koriyama (city)','#07','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'0714','Nihonmatsu (city)','#07','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'0712','Uchigo (city, deleted)','#07','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'0704','Taira (city, deleted)','#07','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'0710','Iwaki (city, deleted)','#07','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'0709','Joban (city, deleted)','#07','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'0708','Kitakata (city)','#07','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'0707','Sukagawa (city)','#07','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'0706','Haramachi (city, deleted)','#07','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'0705','Shirakawa (city)','#07','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'0713','Nakoso (city, deleted)','#07','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'07012','Tamura','#07','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'07001','Asaka (deleted)','#07','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'07002','Adachi','#07','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'07003','Ishikawa','#07','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'07005','Iwase','#07','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'07007','Kawanuma','#07','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'07008','Kitaaidu (deleted)','#07','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'07009','Shinobu (deleted)','#07','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'07017','Yama','#07','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'07011','Date','#07','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'0702','Aizuwakamatsu (city)','#07','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'07013','Nishishirakawa','#07','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'07014','Higashishirakawa','#07','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'07015','Futaba','#07','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'07006','Onuma','#07','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'07016','Minamiaidu','#07','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'0720','Motomiya (city)','#07','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'0701','Fukushima (city)','#07','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'07010','Soma','#07','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'07004','Ishiki (deleted)','#07','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'0802','Nagaoka (city)','#08','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'0810','Tokamachi (city)','#08','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'0809','Kamo (city)','#08','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'0808','Ojiya (city)','#08','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'0807','Niitsu (city), deleted','#08','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'0806','Shibata (city)','#08','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'0805','Kashiwazaki (city)','#08','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'080108','Niigata, Nishikan Ku','#08','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'0803','Takada (city, deleted)','#08','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'0813','Tsubame (city)','#08','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'0801','Niigata (city)','#08','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'080102','Niigata, Higashi Ku','#08','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'080104','Niigata, Konan Ku','#08','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'080105','Niigata, Akiha Ku','#08','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'080106','Niigata, Minami Ku','#08','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'080107','Niigata, Nishi Ku','#08','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'0804','Sanjo (city)','#08','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'0820','Shirone (city, deleted)','#08','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'0828','Tainai (city)','#08','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'0827','Myoko (city)','#08','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'0826','Minamiuonuma (city)','#08','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'0825','Uonuma (city)','#08','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'0824','Sado (city)','#08','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'0823','Agano (city)','#08','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'0811','Mitsuke (city)','#08','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'0821','Toyosaka (city, deleted)','#08','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'0812','Murakami (city)','#08','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'0819','Ryotsu (city, deleted)','#08','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'0818','Gosen (city, deleted)','#08','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'0817','Arai (city, deleted)','#08','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'0816','Itoigawa (city, deleted)','#08','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'0815','Tochio (city, deleted)','#08','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'0814','Naoetsu (city, deleted)','#08','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'080103','Niigata, Chuo Ku','#08','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'0822','Joetsu (city)','#08','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'08007','Santo','#08','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'08015','Minamiuonuma','#08','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'08014','Higashikubiki (deleted)','#08','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'08012','Nishikubiki (deleted)','#08','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'080101','Niigata, Kita Ku','#08','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'08011','Nishikambara','#08','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'08010','Nakakubiki (deleted)','#08','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'08016','Minamikambara','#08','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'08008','Nakauonuma','#08','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'08013','Higashikambara','#08','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'08006','Sado (deleted)','#08','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'08005','Koshi (deleted)','#08','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'08004','Kitakambara','#08','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'08003','Kitauonuma (deleted)','#08','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'08002','Kariwa','#08','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'08001','Iwafune','#08','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'08009','Nakakambara (deleted)','#08','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'0909','Ina (city)','#09','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'09014','Hanishina','#09','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'09013','(Reserved)','#09','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'09006','Kitasaku','#09','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'09011','Suwa','#09','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'09009','Shimotakai','#09','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'09008','Shimoina','#09','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'0901','Nagano (city)','#09','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'0902','Matsumoto (city)','#09','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'0903','Ueda (city)','#09','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'0904','Okaya (city)','#09','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'0905','Iida (city)','#09','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'0906','Suwa (city)','#09','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'09015','Higashichikuma','#09','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'0908','Komoro (city)','#09','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'09012','Chiisagata','#09','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'0910','Komagane (city)','#09','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'0911','Nakano (city)','#09','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'0912','Omachi (city)','#09','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'0913','Iiyama (city)','#09','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'0914','Chino (city)','#09','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'0915','Shiojiri (city)','#09','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'0916','Shinonoi (city, deleted)','#09','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'0917','Koshoku (city, deleted)','#09','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'0918','Saku (city)','#09','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'0919','Chikuma (city)','#09','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'0920','Tomi (city)','#09','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'0921','Azumino (city)','#09','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'0907','Suzaka (city)','#09','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'09005','Kitaazumi','#09','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'09016','Minamiazumi (deleted)','#09','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'09001','Kamiina','#09','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'09002','Kamitakai','#09','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'09003','Kamiminochi','#09','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'09010','Shimominochi','#09','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'09004','Kiso','#09','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'09007','Sarashina (deleted)','#09','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'09017','Minamisaku','#09','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1014','Higashimurayama (city)','#10','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1015','Kokubunji (city)','#10','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1019','Fussa (city)','#10','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1013','Hino (city)','#10','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1012','Kodaira (city)','#10','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1011','Koganei (city)','#10','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'10003','Minamitama (deleted)','#10','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1010','Machida (city)','#10','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1009','Chofu (city)','#10','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1017','Hoya (city, deleted)','#10','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1016','Kunitachi (city)','#10','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1025','Tama (city)','#10','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'10002','Nishitama','#10','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'10004','Oshima','#10','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'10005','Miyake','#10','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1030','Nishitokyo (city)','#10','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1029','Akiruno (city)','#10','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1028','Hamura (city)','#10','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'10007','Ogawasara','#10','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1026','Inagi (city)','#10','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1018','Tanashi (city, deleted)','#10','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1024','Musashimurayama (city)','#10','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1023','Higashikurume (city)','#10','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1022','Kiyose (city)','#10','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1021','Higashiyamato (city)','#10','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1020','Komae (city)','#10','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'10006','Hachijo','#10','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1027','Akigawa (city, deleted)','#10','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'100114','Tokyo, Nakano','#10','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'100103','Tokyo, Minato','#10','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'100104','Tokyo, Shinjuku','#10','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'100106','Tokyo, Taito','#10','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'100107','Tokyo, Sumida','#10','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'100108','Tokyo, Koto','#10','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'100109','Tokyo, Shinagawa','#10','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'100110','Tokyo, Meguro','#10','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'100111','Tokyo, Ota','#10','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'100102','Tokyo, Chuo','#10','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'100113','Tokyo, Shibuya','#10','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'100105','Tokyo, Bunkyo','#10','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'100115','Tokyo, Suginami','#10','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'100116','Tokyo, Toshima','#10','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'100117','Tokyo, Kita','#10','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'100118','Tokyo, Arakawa','#10','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'100119','Tokyo, Itabashi','#10','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'100120','Tokyo, Nerima','#10','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'100121','Tokyo, Adachi','#10','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'100122','Tokyo, Katsushika','#10','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'100123','Tokyo, Edogawa','#10','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'100112','Tokyo, Setagaya','#10','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1008','Akishima (city)','#10','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1001','Tokyo 23-wards (city, deleted)','#10','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1002','Hachioji (city)','#10','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1003','Tachikawa (city)','#10','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1004','Musashino (city)','#10','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1005','Mitaka (city)','#10','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1006','Ome (city)','#10','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'10001','Kitatama (deleted)','#10','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1007','Fuchu (city)','#10','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'100101','Tokyo, Chiyoda','#10','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'110301','Kawasaki, Kawasaki Ku','#11','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'110304','Kawasaki, Takatsu Ku','#11','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'110306','Kawasaki, Miyamae Ku','#11','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'110302','Kawasaki, Saiwai Ku','#11','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'110118','Yokohama, Tsuzuki Ku','#11','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'110117','Yokohama, Aoba Ku','#11','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'110116','Yokohama, Izumi Ku','#11','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'110115','Yokohama, Sakae Ku','#11','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'110114','Yokohama, Seya Ku','#11','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'110113','Yokohama, Midori Ku','#11','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'110112','Yokohama, Asahi Ku','#11','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'111001','Sagamihara, Midori Ku','#11','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'110303','Kawasaki, Nakahara Ku','#11','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'111002','Sagamihara, Chuou Ku','#11','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1105','Kamakura (city)','#11','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1104','Hiratsuka (city)','#11','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1103','Kawasaki (city)','#11','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1102','Yokosuka (city)','#11','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1101','Yokohama (city)','#11','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'110307','Kawasaki, Asao Ku','#11','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1107','Odawara (city)','#11','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1109','Zushi (city)','#11','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1119','Ayase (city)','#11','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1110','Sagamihara (city)','#11','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1118','Minamiashigara (city)','#11','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'110305','Kawasaki, Tama Ku','#11','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1111','Miura (city)','#11','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1112','Hadano (city)','#11','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1113','Atsugi (city)','#11','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1114','Yamato (city)','#11','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1115','Isehara (city)','#11','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1116','Ebina (city)','#11','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1117','Zama (city)','#11','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1106','Fujisawa (city)','#11','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'11003','Ashigarashimo','#11','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'110111','Yokohama, Konan Ku','#11','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1108','Chigasaki (city)','#11','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'11007','Miura','#11','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'11006','Naka','#11','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'11004','Koza','#11','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'11002','Ahigarakami','#11','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'11001','Aiko','#11','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'110101','Yokohama, Tsurumi Ku','#11','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'110102','Yokohama, Kanagawa Ku','#11','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'110107','Yokohama, Isogo Ku','#11','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'110104','Yokohama, Naka Ku','#11','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'110105','Yokohama, Minami Ku','#11','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'110109','Yokohama, Kohoku Ku','#11','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'11005','Tsukui (deleted)','#11','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'110106','Yokohama, Hodogaya Ku','#11','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'110103','Yokohama, Nishi Ku','#11','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'110110','Yokohama, Totsuka Ku','#11','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'111003','Sagamihara, Minami Ku','#11','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'110108','Yokohama, Kanazawa Ku','#11','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1221','Yachiyo (city)','#12','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1219','Ichihara (city)','#12','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1218','Katsuura (city)','#12','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1223','Kamogawa (city)','#12','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1216','Narashino (city)','#12','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1229','Sodegaura (city)','#12','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1215','Asahi (city)','#12','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1214','Yokaichiba (city, deleted)','#12','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1213','Togane (city)','#12','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1212','Sakura (city)','#12','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1211','Narita (city)','#12','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1210','Mobara (city)','#12','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1217','Kashiwa (city)','#12','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1230','Yachimata (city)','#12','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1237','Sanmu (city)','#12','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1236','Katori (city)','#12','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1235','Sosa (city)','#12','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1234','Minamiboso (city)','#12','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1233','Tomisato (city)','#12','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1227','Urayasu (city)','#12','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1231','Inzai (city)','#12','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1224','Kimitsu (city)','#12','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1220','Nagareyama (city)','#12','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1209','Sawara (city, deleted)','#12','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1238','Isumi (city)','#12','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1228','Yotsukaido (city)','#12','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1226','Futtu (city)','#12','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1225','Kamagaya (city)','#12','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1232','Shiroi (city)','#12','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'120103','Chiba, Inage Ku','#12','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1222','Abiko (city)','#12','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1208','Noda (city)','#12','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'120102','Chiba, Hanamigawa Ku','#12','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'120104','Chiba, Wakaba Ku','#12','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'120105','Chiba, Midori Ku','#12','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'120106','Chiba, Mihama Ku','#12','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'12001','Awa','#12','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'12002','Isumi','#12','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'12003','Ichihara (deleted)','#12','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'12004','Inba','#12','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'12005','Kaijo (deleted)','#12','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'12006','Katori','#12','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'12007','Kimitsu (deleted)','#12','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1202','Choshi (city)','#12','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'120101','Chiba, Chuo Ku','#12','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1206','Kisarazu (city)','#12','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1207','Matsudo (city)','#12','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'12008','Sambu','#12','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1203','Ichikawa (city)','#12','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1205','Tateyama (city)','#12','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1201','Chiba (city)','#12','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1239','Oamishirasato (city)','#12','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'12012','Higashikatsushika (deleted)','#12','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'12011','Chosei','#12','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'12010','Chiba (deleted)','#12','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'12009','Sosa (deleted)','#12','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1204','Funabashi (city)','#12','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1333','Kitamoto (city)','#13','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1332','Kuki (city)','#13','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1331','Okegawa (city)','#13','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1330','Niiza (city)','#13','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1329','Wako (city)','#13','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1328','Shiki (city)','#13','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1334','Yashio (city)','#13','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1326','Hatogaya (city, deleted)','#13','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1340','Satte (city)','#13','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1325','Iruma (city)','#13','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1327','Asaka (city)','#13','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1335','Kamifukuoka (city, deleted)','#13','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1336','Fujimi (city)','#13','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1337','Misato (city)','#13','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1339','Sakado (city)','#13','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1341','Tsurugashima (city)','#13','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1342','Hidaka (city)','#13','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1343','Yoshikawa (city)','#13','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1344','Saitama (city)','#13','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1324','Toda (city)','#13','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1306','Gyoda (city)','#13','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1338','Hasuda (city)','#13','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1311','Honjo (city)','#13','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1318','Fukaya (city)','#13','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1345','Fujimino (city)','#13','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1346','Shiraoka (city)','#13','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1301','Urawa (city, deleted)','#13','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1302','Kawagoe (city)','#13','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1303','Kumagaya (city)','#13','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1304','Kawaguchi (city)','#13','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1305','Omiya (city, deleted)','#13','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1307','Chichibu (city)','#13','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1308','Tokorozawa (city)','#13','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1310','Kazo (city)','#13','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1323','Warabi (city)','#13','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1312','Higashimatsuyama (city)','#13','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1313','Iwatsuki (city, deleted)','#13','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1314','Kasukabe (city)','#13','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1315','Sayama (city)','#13','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1317','Konosu (city)','#13','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1319','Ageo (city)','#13','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1320','Yono (city, deleted)','#13','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1321','Soka (city)','#13','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1322','Koshigaya (city)','#13','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1309','Hanno (city)','#13','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'13009','Minamisaitama','#13','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'134403','Saitama, Omiya Ku','#13','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'134404','Saitama, Minuma Ku','#13','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'134402','Saitama, Kita Ku','#13','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'134405','Saitama, Chuo Ku','#13','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'134406','Saitama, Sakura Ku','#13','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'134407','Saitama, Urawa Ku','#13','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'134410','Saitama, Iwatsuki Ku','#13','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'134408','Saitama, Minami Ku','#13','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1316','Hanyu (city)','#13','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'13008','Hiki','#13','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'13007','Chichibu','#13','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'13006','Kodama','#13','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'13005','Kitasaitama (deleted)','#13','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'13004','Kitakatsushika','#13','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'13003','Kitaadachi','#13','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'13002','Osato','#13','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'13001','Iruma','#13','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'134401','Saitama, Nishi Ku','#13','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'134409','Saitama, Midori Ku','#13','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'14006','Taga (deleted)','#14','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'14013','Makabe (deleted)','#14','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'14014','Yuki','#14','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'14012','Higashiibaraki','#14','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'14011','Nishiibaraki (deleted)','#14','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'14010','Niihari (deleted)','#14','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'14009','Namegata (deleted)','#14','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'14007','Tsukuba (deleted)','#14','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1435','Joso (city)','#14','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'14005','Sashima','#14','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'14004','Kuji','#14','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'14003','Kitasoma','#14','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'14002','Kashima (deleted)','#14','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'14001','Inashiki','#14','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'14008','Naka','#14','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1419','Ushiku (city)','#14','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1437','Omitama (city)','#14','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1412','Hitachiota (city)','#14','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1413','Katsuta (city, deleted)','#14','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1414','Takahagi (city)','#14','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1415','Kitaibaraki (city)','#14','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1416','Kasama (city)','#14','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1410','Shimotsuma (city)','#14','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1418','Iwai (city, deleted)','#14','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1409','Nakaminato (city, deleted)','#14','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1420','Tsukuba (city)','#14','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1421','Hitachinaka (city)','#14','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1422','Kashima (city)','#14','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1423','Itako (city)','#14','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1424','Moriya (city)','#14','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1425','Hitachiomiya (city)','#14','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1426','Naka (city)','#14','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1417','Toride (city)','#14','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1401','Mito (city)','#14','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1427','Chikusei (city)','#14','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1434','Hokota (city)','#14','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1433','Namegata (city)','#14','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1432','Kamisu (city)','#14','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1431','Sakuragawa (city)','#14','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1430','Kasumigaura (city)','#14','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1411','Mitsukaido (city, deleted)','#14','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1428','Bandou (city)','#14','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1436','Tsukubamirai (city)','#14','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1402','Hitachi (city)','#14','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1403','Tsuchiura (city)','#14','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1404','Koga (city)','#14','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1405','Ishioka (city)','#14','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1406','Shimodate (city, deleted)','#14','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1407','Yuki (city)','#14','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1408','Ryugasaki (city)','#14','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1429','Inashiki (city)','#14','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'15001','Ashikaga (deleted)','#15','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'15002','Aso (deleted)','#15','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'15003','Kamitsuga (deleted)','#15','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'15004','Kawachi','#15','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'15005','Shioya','#15','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'15008','Haga','#15','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'15007','Nasu','#15','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'15006','Shimotsuga','#15','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1516','Shimotsuke (city)','#15','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1503','Tochigi (city)','#15','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1504','Sano (city)','#15','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1505','Kanuma (city)','#15','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1506','Nikko (city)','#15','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1507','Imaichi (city, deleted)','#15','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1508','Oyama (city)','#15','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1509','Mooka (city)','#15','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1510','Otawara (city)','#15','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1511','Yaita (city)','#15','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1512','Kuroiso (city, deleted)','#15','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1513','Nasushiobara (city)','#15','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1514','Sakura (city)','#15','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1501','Utsunomiya (city)','#15','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1515','Nasukarasuyama (city)','#15','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1502','Ashikaga (city)','#15','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'16003','Ora','#16','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1602','Takasaki (city)','#16','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1603','Kiryu (city)','#16','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1604','Isesaki (city)','#16','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1605','Ota (city)','#16','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1606','Numata (city)','#16','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1607','Tatebayashi (city)','#16','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1608','Shibukawa (city)','#16','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1609','Fujioka (city)','#16','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1610','Tomioka (city)','#16','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1611','Annaka (city)','#16','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1612','Midori (city)','#16','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1601','Maebashi (city)','#16','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'16010','Tone','#16','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'16001','Agatsuma (deleted)','#16','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'16011','Nitta (deleted)','#16','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'16009','Tano','#16','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'16008','Seta (deleted)','#16','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'16007','Sawa','#16','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'16006','Gumma (deleted)','#16','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'16005','Kitagumma','#16','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'16004','Kanra','#16','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'16002','Usui (deleted)','#16','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'16012','Yamada (deleted)','#16','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'17003','Nakakoma','#17','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1707','Nirasaki (city)','#17','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1701','Kofu (city)','#17','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1702','Fujiyoshida (city)','#17','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1703','Enzan (city, deleted)','#17','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1704','Tsuru (city)','#17','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1705','Yamanashi (city)','#17','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1706','Otsuki (city)','#17','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'17008','Minamitsuru','#17','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'17007','Minamikoma','#17','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'17006','Higashiyamanashi (deleted)','#17','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1709','Hokuto (city)','#17','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'17004','Nishiyatsushiro','#17','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1710','Kai (city)','#17','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'17002','Kitatsuru','#17','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'17001','Kitakoma (deleted)','#17','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'17005','Higashiyatsushiro (deleted)','#17','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1711','Fuehuki (city)','#17','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1713','Koshu (city)','#17','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1708','Minami-Alps (city)','#17','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1712','Uenohara (city)','#17','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1714','Chuo (city)','#17','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1808','Ito (city)','#18','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'18004','Iwata (deleted)','#18','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'18003','Ihara (deleted)','#18','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'18002','Inasa (deleted)','#18','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'18001','Abe (deleted)','#18','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1809','Shimada (city)','#18','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'18007','Shida (deleted)','#18','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1807','Fujinomiya (city)','#18','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1806','Mishima (city)','#18','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1805','Atami (city)','#18','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1804','Shimizu (city, deleted)','#18','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1803','Numadu (city)','#18','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1802','Hamamatsu (city)','#18','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1810','Yoshiwara (city, deleted)','#18','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'18012','Hamana (deleted)','#18','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'180209','Hamamatsu, Hamana Ku','#18','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'18013','Fuji (deleted)','#18','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'180208','Hamamatsu, Chuo Ku','#18','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1819','Hamakita (city, deleted)','#18','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1818','Tenryu (city, deleted)','#18','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1817','Fukuroi (city)','#18','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1816','Gotemba (city)','#18','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'18005','Ogasa (deleted)','#18','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1815','Fujieda (city)','#18','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'18006','Kamo','#18','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'18011','Haibara','#18','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'18010','Tagata','#18','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1814','Kakegawa (city)','#18','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1813','Fuji (city)','#18','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1812','Yaidu (city)','#18','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'18009','Sunto','#18','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'18008','Shuchi','#18','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1820','Shimoda (city)','#18','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'180201','Hamamatsu, Naka Ku (deleted)','#18','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1801','Shizuoka (city)','#18','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1811','Iwata (city)','#18','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1827','Makinohara (city)','#18','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1826','Izunokuni (city)','#18','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1825','Kikugawa (city)','#18','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1824','Omaezaki (city)','#18','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1823','Izu (city)','#18','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1821','Susono (city)','#18','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'180202','Hamamatsu, Higashi Ku (deleted)','#18','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'180203','Hamamatsu, Nishi Ku (deleted)','#18','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'180204','Hamamatsu, Minami Ku (deleted)','#18','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'180205','Hamamatsu, Kita Ku (deleted)','#18','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'180206','Hamamatsu, Hamakita Ku (deleted)','#18','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'180101','Shizuoka, Aoi Ku','#18','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'180103','Shizuoka, Shimizu Ku','#18','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'180102','Shizuoka, Suruga Ku','#18','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1822','Kosai (city)','#18','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'180207','Hamamatsu, Tenryu Ku','#18','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'19014','Mugi (deleted)','#19','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'19018','Yoshiki (deleted)','#19','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1920','Gero (city)','#19','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'19017','Yoro','#19','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'19016','Yamagata (deleted)','#19','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1919','Gujo (city)','#19','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'19001','Ampachi','#19','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'19002','Inaba (deleted)','#19','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'19003','Ibi','#19','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'19004','Ena (deleted)','#19','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'19012','Fuwa','#19','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'19005','Ono','#19','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'19011','Hashima','#19','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1918','Motosu (city)','#19','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'19010','Toki (deleted)','#19','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1921','Kaizu (city)','#19','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'19009','Gujo (deleted)','#19','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'19008','Kamo','#19','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'19007','Kani','#19','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'19006','Kaizu (deleted)','#19','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'19013','Mashita (deleted)','#19','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1912','Toki (city)','#19','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1909','Hashima (city)','#19','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1908','Mizunami (city)','#19','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1907','Mino (city)','#19','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1906','Nakatsugawa (city)','#19','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1904','Tajimi (city)','#19','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1917','Hida (city)','#19','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1905','Seki (city)','#19','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1901','Gifu (city)','#19','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1915','Yamagata (city)','#19','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1911','Minokamo (city)','#19','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1916','Mizuho (city)','#19','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1913','Kakamigahara (city)','#19','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1903','Takayama (city)','#19','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1914','Kani (city)','#19','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'19015','Motosu','#19','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1902','Ogaki (city)','#19','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'1910','Ena (city)','#19','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'20016','Hoi (deleted)','#20','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'20017','Minamishitara (deleted)','#20','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'20018','Yana (deleted)','#20','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2038','Miyoshi (city)','#20','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2039','Ama (city)','#20','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2017','Tokoname (city)','#20','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2019','Konan (city)','#20','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2029','Owariasahi (city)','#20','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2020','Bisai (city, deleted)','#20','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2021','Komaki (city)','#20','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'200116','Nagoya, Tenpaku Ku','#20','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'200115','Nagoya, Meito Ku','#20','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'200114','Nagoya, Midori Ku','#20','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2018','Moriyama (city, deleted)','#20','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2030','Iwakura (city)','#20','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'20015','Hekikai (deleted)','#20','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2025','Obu (city)','#20','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2026','Chita (city)','#20','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2027','Takahama (city)','#20','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2028','Chiryu (city)','#20','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2024','Tokai (city)','#20','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2031','Toyoake (city)','#20','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2022','Inazawa (city)','#20','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'200113','Nagoya, Moriyama Ku','#20','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2032','Nissin (city)','#20','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2033','Tahara (city)','#20','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2034','Aisai (city)','#20','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2035','Kiyosu (city)','#20','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2036','Kitanagoya (city)','#20','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2037','Yatomi (city)','#20','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'200112','Nagoya, Minami Ku','#20','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2023','Shinshiro (city)','#20','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2001','Nagoya (city)','#20','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'20014','Higashikamo (deleted)','#20','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2014','Nishio (city)','#20','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2002','Toyohashi (city)','#20','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2012','Toyota (city)','#20','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2011','Kariya (city)','#20','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2010','Hekinan (city)','#20','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2009','Tsushima (city)','#20','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2008','Toyokawa (city)','#20','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2007','Kasugai (city)','#20','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2006','Handa (city)','#20','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2005','Seto (city)','#20','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2016','Inuyama (city)','#20','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2003','Okazaki (city)','#20','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2015','Gamagori (city)','#20','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'200101','Nagoya, Chikusa Ku','#20','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'200102','Nagoya, Higashi Ku','#20','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'200103','Nagoya, Kita Ku','#20','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'200104','Nagoya, Nishi Ku','#20','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'200105','Nagoya, Nakamura Ku','#20','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'200106','Nagoya, Naka Ku','#20','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'200107','Nagoya, Showa Ku','#20','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'200108','Nagoya, Mizuho Ku','#20','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'200109','Nagoya, Atsuta Ku','#20','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'200110','Nagoya, Nakagawa Ku','#20','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'200111','Nagoya, Minato Ku','#20','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2004','Ichinomiya (city)','#20','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'20001','Aichi','#20','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'20006','Nakashima (deleted)','#20','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'20005','Chita','#20','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'20007','Nishikasugai','#20','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'20004','Kitashitara','#20','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'20008','Nishikamo (deleted)','#20','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'20009','Niwa','#20','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'20010','Nukata','#20','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2013','Anjo (city)','#20','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'20002','Atsumi (deleted)','#20','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2040','Nagakute (city)','#20','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'20013','Higashikasugai (deleted)','#20','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'20012','Hazu (deleted)','#20','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'20011','Haguri (deleted)','#20','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'20003','Ama','#20','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'21014','Mie','#21','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2116','Shima (city)','#21','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2117','Iga (city)','#21','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'21001','Age (deleted)','#21','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'21015','Minamimuro','#21','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'21002','Ano (deleted)','#21','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'21004','Iinan (deleted)','#21','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'21003','Ayama (deleted)','#21','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'21013','Naga (deleted)','#21','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'21012','Taki','#21','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2113','Hisai (city, deleted)','#21','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'21011','Suzuka (deleted)','#21','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'21016','Watarai','#21','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'21010','Shima (deleted)','#21','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2109','Owase (city)','#21','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2108','Nabari (city)','#21','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2107','Suzuka (city)','#21','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2106','Ueno (city, deleted)','#21','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2105','Kuwana (city)','#21','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2104','Matsusaka (city)','#21','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2103','Ise (city)','#21','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2102','Yokkaichi (city)','#21','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2101','Tsu (city)','#21','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2115','Inabe (city)','#21','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2112','Kumano (city)','#21','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2114','Ujiyamada (city, deleted)','#21','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'21009','Kuwana','#21','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'21008','Kitamuro','#21','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'21007','Kawage (deleted)','#21','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'21006','Inabe','#21','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'21005','Ichishi (deleted)','#21','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2110','Kameyama (city)','#21','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2111','Toba (city)','#21','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'22009','Takeno (deleted)','#22','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'22008','Soraku','#22','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'22010','Tsuduki','#22','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'22007','Kumano (deleted)','#22','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'22012','Funai','#22','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'22013','Minamikuwada (deleted)','#22','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'22011','Naka (deleted)','#22','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2201','Kyoto (city)','#22','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2202','Fukuchiyama (city)','#22','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2203','Maiduru (city)','#22','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2204','Ayabe (city)','#22','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2205','Uji (city)','#22','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'22014','Yoza','#22','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2207','Kameoka (city)','#22','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2206','Miyazu (city)','#22','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2209','Nagaokakyo (city)','#22','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2210','Muko (city)','#22','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2211','Yawata (city)','#22','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2212','Kyotanabe (city)','#22','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2213','Kyotango (city)','#22','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2214','Nantan (city)','#22','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2215','Kizugawa (city)','#22','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'220108','Kyoto, Ukyo Ku','#22','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'22006','Kuze','#22','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'220106','Kyoto, Shimogyo Ku','#22','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'22004','Kasa (deleted)','#22','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'22003','Otokuni','#22','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'22002','Ikaruga (deleted)','#22','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'22001','Amada (deleted)','#22','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'220109','Kyoto, Fushimi Ku','#22','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'220110','Kyoto, Yamashina Ku','#22','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'220111','Kyoto, Nishikyo Ku','#22','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'220101','Kyoto, Kita Ku','#22','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'220102','Kyoto, Kamigyo Ku','#22','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'220103','Kyoto, Sakyo Ku','#22','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2208','Joyo (city)','#22','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'220105','Kyoto, Higashiyama Ku','#22','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'220107','Kyoto, Minami Ku','#22','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'22005','Kitakuwada (deleted)','#22','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'220104','Kyoto, Nakagyo Ku','#22','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2301','Otsu (city)','#23','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2305','Yokaichi (city, deleted)','#23','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2306','Kusatsu (city)','#23','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2307','Moriyama (city)','#23','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2308','Ritto (city)','#23','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2309','Koka (city)','#23','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2310','Yasu (city)','#23','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2311','Konan (city)','#23','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2312','Takashima (city)','#23','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2313','Higashioumi (city)','#23','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2314','Maibara (city)','#23','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2304','Omihachiman (city)','#23','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'23004','Gamou','#23','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'23007','Koka (deleted)','#23','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2303','Nagahama (city)','#23','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2302','Hikone (city)','#23','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'23005','Kanzaki (deleted)','#23','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'23003','Echi','#23','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'23002','Inukamo','#23','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'23001','Ika (deleted)','#23','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'23009','Shiga (deleted)','#23','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'23010','Takashima (deleted)','#23','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'23011','Higashiazai (deleted)','#23','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'23012','Yasu (deleted)','#23','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'23008','Sakata (deleted)','#23','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'23006','Kurita (deleted)','#23','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2401','Nara (city)','#24','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2409','Ikoma (city)','#24','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'24002','Uda','#24','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2412','Uda (city)','#24','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2411','Katsuragi (city)','#24','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2410','Kashiba (city)','#24','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2408','Gose (city)','#24','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2407','Gojo (city)','#24','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2406','Sakurai (city)','#24','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2405','Kashihara (city)','#24','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2404','Tenri (city)','#24','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2402','Yamatotakada (city)','#24','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2403','Yamatokoriyama (city)','#24','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'24004','Kitakatsuragi','#24','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'24001','Ikoma','#24','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'24010','Yoshino','#24','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'24009','Yamabe','#24','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'24008','Minamikatsuragi (deleted)','#24','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'24003','Uchi (deleted)','#24','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'24005','Shiki','#24','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'24007','Takaichi','#24','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'24006','Soekami (deleted)','#24','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'250122','Osaka, Nishinari Ku','#25','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'25007','Minamikawachi','#25','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'250117','Osaka, Asahi Ku','#25','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'250118','Osaka, Joto Ku','#25','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'250119','Osaka, Abeno Ku','#25','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'25006','Mishima','#25','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'250121','Osaka, Higashisumiyoshi Ku','#25','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'25001','Kitakawachi (deleted)','#25','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'250101','Osaka, Kita Ku','#25','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'250207','Sakai, Mihara Ku','#25','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'250206','Sakai, Kita Ku','#25','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'250205','Sakai, Minami Ku','#25','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'250204','Sakai, Nishi Ku','#25','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'250203','Sakai, Higashi Ku','#25','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'250202','Sakai, Naka Ku','#25','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'250120','Osaka, Sumiyoshi Ku','#25','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'250106','Osaka, Nishi Ku','#25','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'250115','Osaka, Higashinari Ku','#25','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'250114','Osaka, Higashiyodogawa Ku','#25','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'250113','Osaka, Nishiyodogawa Ku','#25','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'250112','Osaka, Oyodo (deleted)','#25','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'250111','Osaka, Naniwa Ku','#25','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'250110','Osaka, Minami Ku (deleted)','#25','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'250109','Osaka, Tennoji Ku','#25','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'25003','Senboku','#25','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'250107','Osaka, Minato Ku','#25','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'25005','Nakakawachi (deleted)','#25','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'250105','Osaka, Higashi Ku (deleted)','#25','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'250104','Osaka, Konohana Ku','#25','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'250103','Osaka, Fukushima Ku','#25','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'250102','Osaka, Miyakojima Ku','#25','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'25002','Sennan','#25','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'250116','Osaka, Ikuno Ku','#25','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'25004','Toyono','#25','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'250108','Osaka, Taisho Ku','#25','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2506','Ikeda (city)','#25','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2516','Tondabayashi (city)','#25','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2515','Izumisano (city)','#25','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2514','Yao (city)','#25','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2513','Ibaraki (city)','#25','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2512','Hirakata (city)','#25','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2511','Moriguchi (city)','#25','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2510','Kaiduka (city)','#25','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2509','Takatsuki (city)','#25','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'250201','Sakai, Sakai Ku','#25','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2507','Suita (city)','#25','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2519','Hiraoka (city, deleted)','#25','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2505','Fuse (city, deleted)','#25','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2504','Toyonaka (city)','#25','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2503','Kishiwada (city)','#25','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2502','Sakai (city)','#25','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2501','Osaka (city)','#25','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'250125','Osaka, Suminoe Ku','#25','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'250126','Osaka, Hirano Ku','#25','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'250127','Osaka, Chuo Ku','#25','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2508','Izumiotsu (city)','#25','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2528','Settsu (city)','#25','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'250124','Osaka, Tsurumi Ku','#25','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2536','Hannan (city)','#25','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2535','Osakasayama (city)','#25','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2534','Katano (city)','#25','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2533','Shijonawate (city)','#25','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2532','Sennan (city)','#25','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2531','Higashiosaka (city)','#25','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2517','Neyagawa (city)','#25','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2529','Fujiidera (city)','#25','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2518','Kawachinagano (city)','#25','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2527','Kadoma (city)','#25','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2526','Habikino (city)','#25','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2525','Kashiwara (city)','#25','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2524','Mino (city)','#25','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2523','Izumi (city)','#25','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2522','Daito (city)','#25','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2521','Matsubara (city)','#25','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2520','Kawachi (city, deleted)','#25','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'250123','Osaka, Yodogawa Ku','#25','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2530','Takaishi (city)','#25','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2605','Gobo (city)','#26','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2602','Shingu (city)','#26','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'26007','Hidaka','#26','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'26006','Higashimuro','#26','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'26005','Nishimuro','#26','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2607','Arida (city)','#26','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2601','Wakayama (city)','#26','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'26001','Arida','#26','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2603','Kainan (city)','#26','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2604','Tanabe (city)','#26','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2606','Hashimoto (city)','#26','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2608','Kinokawa (city)','#26','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2609','Iwade (city)','#26','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'26003','Kaiso','#26','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'26002','Ito','#26','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'26004','Naga (deleted)','#26','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2724','Tanba (city)','#27','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2729','Kato (city)','#27','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2727','Awaji (city)','#27','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2726','Asago (city)','#27','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2725','Minamiawaji (city)','#27','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2730','Tatsuno (city)','#27','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'270108','Kobe, Chuo Ku','#27','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'270109','Kobe, Nishi Ku','#27','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'270110','Kobe, Fukiai Ku (deleted)','#27','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'270111','Kobe, Ikuta Ku (deleted)','#27','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2701','Kobe (city)','#27','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2702','Himeji (city)','#27','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2703','Amagasaki (city)','#27','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2704','Akashi (city)','#27','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2705','Nishinomiya (city)','#27','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2706','Sumoto (city)','#27','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2707','Ashiya (city)','#27','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2708','Itami (city)','#27','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2712','Tatsuno (city, deleted)','#27','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2710','Toyooka (city)','#27','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2723','Yabu (city)','#27','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2711','Kakogawa (city)','#27','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'27001','Akou','#27','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2713','Ako (city)','#27','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2714','Nishiwaki (city)','#27','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2715','Takaraduka (city)','#27','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2716','Miki (city)','#27','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2717','Takasago (city)','#27','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2718','Kawanishi (city)','#27','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2719','Ono (city)','#27','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2720','Sanda (city)','#27','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2721','Kasai (city)','#27','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2722','Sasayama (city, deleted)','#27','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2709','Aioi (city)','#27','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'27018','Tsuna (deleted)','#27','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'27010','Kawabe','#27','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'27011','Kanzaki','#27','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'27012','Kinosaki (deleted)','#27','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'27013','Sayo','#27','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2728','Shiso (city)','#27','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'27021','Mino (deleted)','#27','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'27002','Asago (deleted)','#27','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'27009','Kato (deleted)','#27','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'27017','Taki (deleted)','#27','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'27014','Shikama (deleted)','#27','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'27019','Hikami (deleted)','#27','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'27008','Kasai (deleted)','#27','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2731','Tanbasasayama (city)','#27','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'27020','Mikata','#27','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'27024','Yabu (deleted)','#27','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'27023','Muko (deleted)','#27','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'27022','Mihara (deleted)','#27','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'27016','Taka','#27','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'27005','Ibo','#27','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'270105','Kobe, Suma Ku','#27','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'270107','Kobe, Kita Ku','#27','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'270104','Kobe, Nagata Ku','#27','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'270103','Kobe, Hyogo Ku','#27','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'270102','Kobe, Nada Ku','#27','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'27003','Arima (deleted)','#27','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'270101','Kobe, Higashinada Ku','#27','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'27004','Izushi (deleted)','#27','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'27007','Kako','#27','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'27006','Innami (deleted)','#27','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'27015','Shiso (deleted)','#27','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'270106','Kobe, Tarumi Ku','#27','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'28007','Himi (deleted)','#28','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'28006','Nei (deleted)','#28','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'28005','Nishitonami (deleted)','#28','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'28004','Nakaniikawa','#28','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'28003','Shimoniikawa','#28','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'28001','Imizu (deleted)','#28','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2801','Toyama (city)','#28','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'28002','Kaminiikawa (deleted)','#28','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'28008','Higashitonami (deleted)','#28','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2806','Namerikawa (city)','#28','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2805','Himi (city)','#28','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2802','Takaoka (city)','#28','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2803','Shinminato (city, deleted)','#28','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2804','Uodu (city)','#28','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2808','Tonami (city)','#28','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2809','Oyabe (city)','#28','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2810','Nanto (city)','#28','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2811','Imizu (city)','#28','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2807','Kurobe (city)','#28','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'29004','Ono (deleted)','#29','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2910','Sakai (city)','#29','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2901','Fukui (city)','#29','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2902','Tsuruga (city)','#29','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2903','Takefu (city, deleted)','#29','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2904','Obama (city)','#29','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2905','Ono (city)','#29','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2906','Katsuyama (city)','#29','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2907','Sabae (city)','#29','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'29002','Imadate','#29','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2909','Echizen (city)','#29','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'29001','Asuwa (deleted)','#29','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'29012','Mikatakaminaka','#29','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'29011','Yoshida','#29','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'29010','Mikata','#29','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'29008','Nanjo','#29','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'29006','Sakai (deleted)','#29','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'29005','Onyu (deleted)','#29','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'29007','Tsuruga (deleted)','#29','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'29003','Oi','#29','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'2908','Awara (city)','#29','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'29009','Nyuu','#29','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'3004','Wajima (city)','#30','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'30001','Ishikawa (deleted)','#30','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'30002','Enuma (deleted)','#30','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'30004','Kahoku','#30','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'30005','Suzu (deleted)','#30','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'30008','Fugeshi (deleted)','#30','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'30009','Housu','#30','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'3001','Kanazawa (city)','#30','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'30006','Nomi','#30','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'3003','Komatsu (city)','#30','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'30007','Hakui','#30','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'3005','Suzu (city)','#30','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'3006','Kaga (city)','#30','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'3007','Hakui (city)','#30','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'3008','Matsuto (city, deleted)','#30','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'3009','Kahoku (city)','#30','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'3010','Hakusan (city)','#30','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'3011','Nomi (city)','#30','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'3002','Nanao (city)','#30','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'3012','Nonoichi (city)','#30','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'30003','Kashima','#30','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'3103','Tsuyama (city)','#31','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'3102','Kurashiki (city)','#31','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'3101','Okayama (city)','#31','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'31006','Oda','#31','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'31007','Katsuta','#31','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'31001','Aida','#31','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'31009','Kibi (deleted)','#31','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'3106','Tamashima (city, deleted)','#31','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'31010','Kume','#31','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'31011','Kojima (deleted)','#31','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'31013','Jodo (deleted)','#31','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'31014','Jobo (deleted)','#31','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'31015','Tsukubo','#31','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'31008','Kawakami (deleted)','#31','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'3105','Kojima (city, deleted)','#31','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'31004','Atetsu (deleted)','#31','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'31005','Oku (deleted)','#31','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'3118','Asakuchi (city)','#31','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'3117','Mimasaka (city)','#31','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'3116','Maniwa (city)','#31','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'3115','Akaiwa (city)','#31','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'3114','Setouchi (city)','#31','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'3113','Bizen (city)','#31','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'3112','Niimi (city)','#31','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'3111','Takahashi (city)','#31','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'3110','Soja (city)','#31','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'3109','Ibara (city)','#31','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'3108','saidaiji (city, deleted)','#31','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'3104','Tamano (city)','#31','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'310104','Okayama, Minami Ku','#31','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'31012','Shitsuki (deleted)','#31','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'310101','Okayama, Kita Ku','#31','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'31016','Tomada','#31','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'31003','Asakuchi','#31','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'31002','Akaiwa (deleted)','#31','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'310102','Okayama, Naka Ku','#31','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'310103','Okayama, Higashi Ku','#31','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'3107','Kasaoka (city)','#31','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'31017','Maniwa','#31','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'31018','Mitsu (deleted)','#31','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'31019','Wake','#31','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'31020','Kaga','#31','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'3203','Izumo (city)','#32','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'32014','Nogi (deleted)','#32','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'32013','Nima','#32','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'32010','Chibu (deleted)','#32','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'32001','Ano (deleted)','#32','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'32015','Hikawa (deleted)','#32','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'32011','Naka (deleted)','#32','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'32012','Nita','#32','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'3201','Matsue (city)','#32','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'3202','Hamada (city)','#32','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'32002','Ama (deleted)','#32','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'3204','Masuda (city)','#32','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'3205','Oda (city)','#32','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'3206','Yasugi (city)','#32','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'3208','Hirata (city, deleted)','#32','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'32009','Suki (deleted)','#32','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'32017','Yatsuka (deleted)','#32','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'32003','Iishi','#32','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'32004','Ochi','#32','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'32005','Ohara (deleted)','#32','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'32006','Oki','#32','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'32007','Ochi (deleted)','#32','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'32008','Kanoashi','#32','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'3209','Unnan (city)','#32','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'32016','Mino (deleted)','#32','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'3207','Gotsu (city)','#32','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'3314','Shinnan''yo (city, deleted)','#33','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'33010','Mine (deleted)','#33','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'33001','Asa (deleted)','#33','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'3301','Yamaguchi (city)','#33','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'33002','Abu','#33','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'3302','Shimonoseki (city)','#33','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'33003','Oshima','#33','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'33009','Toyoura (deleted)','#33','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'33004','Otsu (deleted)','#33','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'33005','Kuga','#33','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'33006','Kumage','#33','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'33007','Saba (deleted)','#33','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'33011','Yoshiki (deleted)','#33','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'3315','Shunan (city)','#33','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'3313','Mine (city)','#33','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'3312','Yanai (city)','#33','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'3311','Nagato (city)','#33','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'3310','Hikari (city)','#33','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'3309','Onoda (city, deleted)','#33','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'3308','Iwakuni (city)','#33','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'3307','Kudamatsu (city)','#33','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'3306','Hofu (city)','#33','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'3305','Tokuyama (city, deleted)','#33','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'3304','Hagi (city)','#33','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'3303','Ube (city)','#33','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'33008','Tsuno (deleted)','#33','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'3316','San''yoonoda (city)','#33','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'3403','Yonago (city)','#34','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'34001','Iwami','#34','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'34002','Ketaka (deleted)','#34','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'34003','Saihaku','#34','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'34004','Tohaku','#34','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'3404','Sakaiminato (city)','#34','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'3402','Kurayoshi (city)','#34','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'3401','Tottori (city)','#34','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'34006','Yazu','#34','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'34005','Hino','#34','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'350103','Hiroshima, Minami Ku','#35','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'350108','Hiroshima, Saeki Ku','#35','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'35001','Aki (deleted)','#35','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'350107','Hiroshima, Aki Ku','#35','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'350106','Hiroshima, Asakita Ku','#35','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'350104','Hiroshima, Nishi Ku','#35','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'350102','Hiroshima, Higashi Ku','#35','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'350101','Hiroshima, Naka Ku','#35','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'350105','Hiroshima, Asaminami Ku','#35','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'35011','Numakuma (deleted)','#35','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'35003','Ashina (deleted)','#35','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'3516','Etajima (city)','#35','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'3515','Akitakata (city)','#35','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'3514','Hatsukaichi (city)','#35','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'3513','Higashihiroshima (city)','#35','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'35016','Yamagata (deleted)','#35','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'35015','Mitsugi (deleted)','#35','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'35014','Futami (deleted)','#35','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'35012','Hiba (deleted)','#35','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'35010','Toyota','#35','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'35009','Takata (deleted)','#35','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'35008','Sera','#35','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'35007','Jinseki','#35','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'35006','Saeki (deleted)','#35','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'3507','Matsunaga (city, deleted)','#35','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'3504','Mihara (city)','#35','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'3505','Onomichi (city)','#35','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'3502','Kure (city)','#35','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'3501','Hiroshima (city)','#35','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'35013','Fukayasu (deleted)','#35','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'35005','Konu (deleted)','#35','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'3503','Takehara (city)','#35','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'3506','Innoshima (city, deleted)','#35','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'3508','Fukuyama (city)','#35','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'3509','Fuchu (city)','#35','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'3510','Miyoshi (city)','#35','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'3511','Syoubara (city)','#35','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'3512','Otake (city)','#35','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'35002','Asa (deleted)','#35','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'35004','Kamo (deleted)','#35','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'36003','Kagawa','#36','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'3606','Sanuki (city)','#36','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'3604','Zentsuji (city)','#36','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'3607','Higashikagawa (city)','#36','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'3602','Marugame (city)','#36','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'3605','Kan''onji (city)','#36','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'36007','Mitoyo (deleted)','#36','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'36006','Nakatado','#36','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'36004','Kita','#36','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'36002','Okawa (deleted)','#36','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'36001','Ayauta','#36','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'3608','Mitoyo (city)','#36','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'3603','Sakaide (city)','#36','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'36005','Syozu','#36','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'3601','Takamatsu (city)','#36','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'3704','Anan (city)','#37','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'37010','Miyoshi','#37','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'3701','Tokushima (city)','#37','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'3703','Komatsushima (city)','#37','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'3705','Yoshinogawa (city)','#37','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'3706','Awa (city)','#37','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'3707','Mima (city)','#37','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'3708','Miyoshi (city)','#37','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'37001','Awa (deleted)','#37','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'37008','Myodo','#37','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'37007','Myozai','#37','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'37006','Naka','#37','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'37005','Katsuura','#37','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'37004','Kaifu','#37','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'37003','Oe (deleted)','#37','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'37002','Itano','#37','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'37009','Mima','#37','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'3702','Naruto (city)','#37','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'38011','Higashiuwa (deleted)','#38','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'38010','Nishiuwa','#38','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'38009','Nii','#38','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'38008','Shuso (deleted)','#38','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'38007','Kitauwa','#38','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'38006','Kita','#38','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'38005','Kamiukena','#38','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'38004','Onsen (deleted)','#38','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'38002','Uma (deleted)','#38','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'38012','Minamiuwa','#38','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'38003','Ochi','#38','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'3810','Iyo (city)','#38','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'3807','Ozu (city)','#38','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'3801','Matsuyama (city)','#38','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'3811','Hojo (city, deleted)','#38','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'3812','Toyo (city, deleted)','#38','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'3813','Shikokuchuo (city)','#38','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'3814','Seiyo (city)','#38','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'3815','Toon (city)','#38','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'3803','Uwajima (city)','#38','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'3804','Yawatahama (city)','#38','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'3802','Imabari (city)','#38','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'3806','Saijo (city)','#38','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'3808','Iyomishima (city, deleted)','#38','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'3809','Kawanoe (city, deleted)','#38','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'38001','Iyo','#38','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'3805','Niihama (city)','#38','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'3909','Nankoku (city)','#39','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'3907','Sukumo (city)','#39','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'3908','Tosashimizu (city)','#39','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'39006','Nagaoka','#39','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'3910','Shimanto (city)','#39','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'3912','Kami (city)','#39','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'39001','Agawa','#39','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'3911','Konan (city)','#39','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'39004','Takaoka','#39','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'39002','Aki','#39','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'39005','Tosa','#39','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'39007','Hata','#39','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'3906','Nakamura (city, deleted)','#39','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'3905','Susaki (city)','#39','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'3904','Tosa (city)','#39','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'3903','Aki (city)','#39','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'3902','Muroto (city)','#39','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'3901','Kochi (city)','#39','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'39003','Kami (deleted)','#39','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'4021','Kitakyushu (city)','#40','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'4016','Chikugo (city)','#40','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'4017','Okawa (city)','#40','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'4018','Yukuhashi (city)','#40','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'4019','Buzen (city)','#40','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'4020','Nakama (city)','#40','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'4022','Ogoori (city)','#40','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'4023','Kasuga (city)','#40','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'4024','Chikushino (city)','#40','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'4025','Onojo (city)','#40','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'40001','Asakura','#40','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'4027','Dazaifu (city)','#40','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'4008','Omuta (city)','#40','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'4026','Munakata (city)','#40','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'4015','Yame (city)','#40','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'4014','Yamada (city, deleted)','#40','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'4013','Amagi (city, deleted)','#40','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'4012','Yanagawa (city)','#40','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'4011','Tagawa (city)','#40','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'4009','Noogata (city)','#40','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'4028','Maebaru (city, deleted)','#40','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'4007','Kurume (city)','#40','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'4006','Wakamatsu (city, deleted)','#40','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'4005','Tobata (city, deleted)','#40','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'4004','Yahata (city, deleted)','#40','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'4003','Moji (city, deleted)','#40','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'4002','Kokura (city, deleted)','#40','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'4001','Fukuoka (city, deleted)','#40','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'4010','Iizuka (city)','#40','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'40012','Mii','#40','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'40005','Kasuya','#40','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'40007','Kurate','#40','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'40009','Tagawa','#40','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'40010','Chikushi (deleted)','#40','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'4037','Nakagawa (city)','#40','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'4032','Miyawaka (city)','#40','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'4033','Kama (city)','#40','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'4034','Asakura (city)','#40','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'40004','Onga','#40','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'40011','Chikujo','#40','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'40006','Kaho','#40','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'40013','Miike (deleted)','#40','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'40014','Mizuma','#40','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'40015','Miyako','#40','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'40016','Munakata (deleted)','#40','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'40017','Yamato (deleted)','#40','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'40018','Yame','#40','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'40003','Ukiha (deleted)','#40','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'400101','Fukuoka, Higashi Ku','#40','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'4035','Miyama (city)','#40','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'402103','Kitakyushu, Tobata Ku','#40','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'400102','Fukuoka, Hakata Ku','#40','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'400103','Fukuoka, Chuo Ku','#40','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'400104','Fukuoka, Minami Ku','#40','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'400105','Fukuoka, Nishi Ku','#40','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'400106','Fukuoka, Jonan Ku','#40','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'400107','Fukuoka, Sawara Ku','#40','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'40008','Sawara (deleted)','#40','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'402102','Kitakyushu, Wakamatsui Ku','#40','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'40002','Itoshima (deleted)','#40','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'402104','Kitakyushu, Kokurakitai Ku','#40','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'4031','Ukiha (city)','#40','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'402106','Kitakyushu, Yahatahigashii Ku','#40','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'402107','Kitakyushu, Yahatanishii Ku','#40','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'402108','Kitakyushu, Yahata Ku (deleted)','#40','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'402109','Kitakyushu, Kokura Ku (deleted)','#40','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'4029','Koga (city)','#40','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'4030','Fukutsu (city)','#40','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'402105','Kitakyushu, Kokuraminamii Ku','#40','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'4036','Itoshima (city)','#40','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'402101','Kitakyushu, Mojii Ku','#40','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'41006','Higashimatsuura','#41','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'41007','Fujitsu','#41','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'41004','Saga (deleted)','#41','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'41003','Kishima','#41','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'41002','Kanzaki','#41','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'41008','Miyaki','#41','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'41001','Ogi (deleted)','#41','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'4108','Ogi (city)','#41','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'4109','Ureshino (city)','#41','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'4107','Kashima (city)','#41','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'4106','Takeo (city)','#41','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'4105','Imari (city)','#41','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'4104','Taku (city)','#41','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'4103','Tosu (city)','#41','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'4102','Karatsu (city)','#41','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'41005','Nishimatsuura','#41','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'4101','Saga (city)','#41','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'4110','Kanzaki (city)','#41','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'4202','Sasebo (city)','#42','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'42008','Minamitakaki (deleted)','#42','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'42007','Higashisonogi','#42','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'42006','Nishisonogi','#42','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'42005','Shimoagata (deleted)','#42','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'42004','Kitamatsuura','#42','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'42003','Kitatakaki (deleted)','#42','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'42002','Kamiagata (deleted)','#42','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'42001','Iki (deleted)','#42','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'4206','Fukue (city, deleted)','#42','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'4205','Omura (city)','#42','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'4203','Shimabara (city)','#42','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'4201','Nagasaki (city)','#42','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'42009','Minamimatsuura','#42','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'4214','Minamishimabara (city)','#42','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'4208','Matsuura (city)','#42','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'4213','Unzen (city)','#42','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'4212','Saikai (city)','#42','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'4211','Goto (city)','#42','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'4209','Tsushima (city)','#42','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'4207','Hirado (city)','#42','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'4204','Isahaya (city)','#42','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'4210','Iki (city)','#42','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'4307','Hondo (city, deleted)','#43','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'430104','Minami','#43','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'430103','Nishi','#43','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'430102','Higashi','#43','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'430101','Chuo','#43','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'4301','Kumamoto (city)','#43','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'4302','Yatsushiro (city)','#43','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'4303','Hitoyoshi (city)','#43','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'4304','Arao (city)','#43','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'430105','Kita','#43','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'4306','Tamana (city)','#43','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'4311','Uto (city)','#43','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'4308','Yamaga (city, deleted)','#43','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'4309','Ushibuka (city, deleted)','#43','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'4310','Kikuchi (city)','#43','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'4312','Kamiamakusa (city)','#43','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'4314','Aso (city)','#43','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'4315','Amakusa (city)','#43','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'4316','Koshi (city)','#43','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'4305','Minamata (city)','#43','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'43008','Kuma','#43','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'43012','Yatsushiro','#43','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'43011','Hotaku (deleted)','#43','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'43010','Tamana','#43','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'43001','Ashikita','#43','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'43009','Shimomashiki','#43','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'43007','Kikuchi','#43','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'43006','Kamoto (deleted)','#43','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'43005','Kamimashiki','#43','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'43004','Uto (deleted)','#43','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'43003','Amakusa','#43','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'43002','Aso','#43','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'4313','Uki (city)','#43','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'4401','Oita (city)','#44','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'44004','Kitaamabe (deleted)','#44','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'44001','Usa (deleted)','#44','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'4408','Taketa (city)','#44','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'4415','Kunisaki (city)','#44','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'4414','Yufu (city)','#44','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'4413','Bungoono (city)','#44','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'4412','Usa (city)','#44','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'4411','Kitsuki (city)','#44','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'44002','Oita (deleted)','#44','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'4409','Tsurusaki (city, deleted)','#44','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'44003','Ono (deleted)','#44','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'4406','Usuki (city)','#44','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'44010','Higashikunisaki','#44','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'4404','Hita (city)','#44','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'4403','Nakatsu (city)','#44','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'4402','Beppu (city)','#44','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'4405','Saiki (city)','#44','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'4410','Bungotakada (city)','#44','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'44005','kusu','#44','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'44009','Hayami','#44','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'44008','Nishikunisaki (deleted)','#44','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'44006','Shimoge (deleted)','#44','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'44012','Minamiamabe (deleted)','#44','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'44011','Hita (deleted)','#44','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'4407','Tsukumi (city)','#44','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'44007','Naoiri (deleted)','#44','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'45007','Minaminaka (deleted)','#45','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'45006','Higashimorokata','#45','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'45005','Higashiusuki','#45','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'45004','Nishimorokata','#45','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'45003','Nishiusuki','#45','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'45002','Koyu','#45','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'4509','Ebino (city)','#45','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'4503','Nobeoka (city)','#45','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'45008','Miyazaki (deleted)','#45','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'4507','Kushima (city)','#45','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'4506','Hyuga (city)','#45','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'4505','Kobayashi (city)','#45','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'4504','Nichinan (city)','#45','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'4501','Miyazaki (city)','#45','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'4502','Miyakonojo (city)','#45','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'45001','Kitamorokata','#45','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'4508','Saito (city)','#45','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'4614','Nishinoomote (city)','#46','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'4604','Makurazaki (city)','#46','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'4605','Kushikino (city, deleted)','#46','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'4606','Akune (city)','#46','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'4607','Izumi (city)','#46','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'4608','Naze (city, deleted)','#46','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'46001','Aira (deleted)','#46','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'4610','Ibusuki (city)','#46','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'4603','Kanoya (city)','#46','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'4613','Taniyama (city, deleted)','#46','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'4609','Oguchi (city, deleted)','#46','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'4615','Tarumizu (city)','#46','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'4616','Satsumasendai (city)','#46','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'4617','Hioki (city)','#46','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'4618','Soo (city)','#46','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'4619','Kirishima (city)','#46','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'4620','Ichikikushikino (city)','#46','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'4621','Minamisatsuma (city)','#46','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'4622','Shibushi (city)','#46','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'4623','Amami (city)','#46','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'4612','Kokubu (city, deleted)','#46','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'46005','Oshima','#46','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'46002','Isa (deleted)','#46','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'4611','Kaseda (city, deleted)','#46','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'46004','Ibusuki (deleted)','#46','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'4602','Sendai (city, deleted)','#46','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'46006','Kagoshima','#46','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'46007','Kawanabe (deleted)','#46','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'46008','Kimotsuki','#46','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'46009','Kumage','#46','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'4626','Aira (city)','#46','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'4625','Isa (city)','#46','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'4601','Kagoshima (city)','#46','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'46010','Satsuma','#46','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'46003','Izumi','#46','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'46011','Soo','#46','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'46012','Hioki (deleted)','#46','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'4624','Minamikyushu (city)','#46','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'4708','Nago (city)','#47','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'4709','Urasoe (city)','#47','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'4710','Itoman (city)','#47','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'4707','Gushikawa (city, deleted)','#47','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'4712','Tomigusuku (city)','#47','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'4703','Hirara (city, deleted)','#47','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'4713','Uruma (city)','#47','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'4714','Miyakojima (city)','#47','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'4715','Nanjo (city)','#47','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'4711','Okinawa (city)','#47','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'4706','Ginowan (city)','#47','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'4704','Ishigaki (city)','#47','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'4702','Ishikawa (city, deleted)','#47','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'4701','Naha (city)','#47','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'47002','Shimajiri','#47','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'47003','Nakagami','#47','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'47004','Miyako','#47','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'47001','Kunigami','#47','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'47005','Yaeyama','#47','');
INSERT INTO adif_enum_secondary_subdivision VALUES (339,'4705','Koza (city, deleted)','#47','');
